<?php
/**
 * @name Model_adm
 * @description Classe modèle de la partie d'administration d'Esia
 *		Toutes les méthodes sont déclarée en 'static' ainsi on garde les Objets PDO pour les programmeurs extérieurs et pour faire des modifications, ils ne sont pas /
 *		obligé de connaitre la hierachie de classe d'ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 10/04/2013
 */
include_once DEVICE_CLASS;
include_once MVCMODEL;
class Model_gesaStat extends Model_esia
{
	/*@NAME: countServiceByNodeGroup
	 *@IN: $dbconnection l'objet de connexion PDO
	*	$user (optionnel) id de l'utilisateur
	*@OUT: le nombre de service
	*@TREAT: count le nombre de service tournant dans l'application  si le paramètre user est défini les droit d'accès (device view) est vérifié afin que le nombre de
	*	service corresponde à ce que l'utilisateur à accès
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	static function countServiceByNodeGroup($dbconnection,$user,$group=null)
	{
		if($group==null)
		{
			$query= $dbconnection->prepare("SELECT device_group_name, count(DISTINCT service_id) as services
						FROM esia_services
						JOIN esia_device ON service_device_id = device_id
						JOIN esia_group_device_access ON  device_id = access_device_id
						JOIN esia_groups ON access_group_id = group_id
						JOIN esia_memberof ON member_group_id = group_id
						WHERE service_delete=FALSE AND service_name <> 'GESA_HEALTH' AND device_ip_addr<>2130706433 AND member_user_id =? AND service_name <> 'CHECK_HTTPS_CERTIFICATE_EXPIRE'
						GROUP BY device_group_name;");
			$query = Model_esia::executeStatement($query,array($user));
		}
		else
		{
			$query= $dbconnection->prepare("SELECT device_group_name,count(DISTINCT service_id) as services
						FROM esia_services
						JOIN esia_device ON service_device_id = device_id
						JOIN esia_group_device_access ON  device_id = access_device_id
						JOIN esia_groups ON access_group_id = group_id
						JOIN esia_memberof ON member_group_id = group_id
						WHERE service_delete=FALSE AND service_name <> 'GESA_HEALTH' AND device_ip_addr<>2130706433 AND member_user_id =? AND device_group_name=? AND service_name <> 'CHECK_HTTPS_CERTIFICATE_EXPIRE'
						GROUP BY device_group_name;");
			$query = Model_esia::executeStatement($query,array($user,$group));
		}
		$arr= array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$service = $row->services;
			if(empty($row->device_group_name))
				$arr['noGroup']=$service;
			else
				$arr[$row->device_group_name]=$service;
		}
		return $arr;
	}	
	/*@NAME count_service_execute
	 *@IN: $dbconnection un objet PDO
	*	$id = id de la gesa
	*@OUT: le nombre de service executé
	*@TREAT: compte le nombre de service lié à la GESA
	*@EXCEPTION: EsiaDbException si une erreur survient
	*/
	public static function count_service_execute($dbconnection,$id)
	{
		$query=$dbconnection->prepare("SELECT COUNT(*) AS nbr
				FROM esia_services
				JOIN esia_device ON device_id = service_device_id
				JOIN gesa_connected ON device_id = connected_device_id
				WHERE connected_gesa_id = ? AND service_delete=FALSE AND service_name <> 'GESA_HEALTH' AND device_ip_addr<>2130706433" );
		$query = parent::executeStatement($query,array($id));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->nbr;
	}
}
?>
