<?php
//HTTP Exception
abstract class HTTPException extends Exception{
	public static $codeslist = array(
            400 => 'Bad Request',
			401 => 'Unauthorized',
			403 => 'Forbidden',
			404 => 'Not Found',
			405 => 'Method Not Allowed',
			408 => 'Request Time-out',
			500 => 'Internal Server Error',
			501 => 'Not Implemented',
	);

	protected $httpErrorCode;

	public function getHttpErrorCode(){
		return $this->httpErrorCode;
	}

	public function getHttpErrorName(){
		return HTTPException::$codeslist[$this->httpErrorCode];
	}

	public function getHttpErrorFull(){
		return $this->httpErrorCode.' '.HTTPException::$codeslist[$this->httpErrorCode];
	}

	public static function makeException($httpErrorCode,$apiCode,$message=NULL){
		$elist[400][0]='HttpBadRequestException';

		$elist[401][0]='HttpUnauthorizedException';
		$elist[401][1]='BadUserPassException';
		$elist[401][2]='NotAuthenticatedException';

		$elist[403][0]='HttpForbiddenException';
		$elist[403][1]='AccessDeniedException';

		$elist[404][0]='HttpNotFoundException';
		$elist[404][1]='NotFoundResourceException';
		$elist[404][2]='NotFoundMethodException';
		$elist[404][3]='NotFoundIdException';
		$elist[404][4]='InvalidParametersException';

		$elist[405][0]='HttpMethodNotAllowedException';
		$elist[405][1]='BadFormatException';
		$elist[405][2]='BadVersionNumberException';

		$elist[408][0]='HttpRequestTimeoutException';
		$elist[408][1]='TimeoutException';

		$elist[500][0]='HttpInternalServerErrorException';
		$elist[500][1]='DatabaseConnectionException';
		$elist[500][2]='DatabaseErrorException';
		$elist[500][3]='APIResourceErrorException';

		try{
			if($apiCode != 0 && isset($elist[$httpErrorCode][$apiCode])){
				$ex = new $elist[$httpErrorCode][$apiCode]($message);
			}
			else if(isset($elist[$httpErrorCode][0])){
				if(empty($message)){
					if( isset(self::$codeslist[$httpErrorCode]) ) $message = self::$codeslist[$httpErrorCode];
				}
				$ex = new $elist[$httpErrorCode][0]($message);
			}
			else{
				$ex = new HttpClientException("Unknown or not implemented HttpErrorCode : $message");
			}
		}
		catch(Exception $ex){
		}
		return $ex;
	}
}

//Intermediates HTTP Exception
class HttpBadRequestException extends HTTPException {
	public function __construct($message = NULL, $code = 0, Exception $previous = null){
		$this->httpErrorCode = 400;
		parent::__construct($message,$code,$previous);
	}
}

class HttpUnauthorizedException extends HTTPException {
	public function __construct($message = NULL, $code = 0, Exception $previous = null){
		$this->httpErrorCode = 401;
		parent::__construct($message,$code,$previous);
	}
}

class HttpForbiddenException extends HTTPException {
	public function __construct($message = NULL, $code = 0, Exception $previous = null){
		$this->httpErrorCode = 403;
		parent::__construct($message,$code,$previous);
	}
}

class HttpNotFoundException extends HTTPException {
	public function __construct($message = NULL, $code = 0, Exception $previous = null){
		$this->httpErrorCode = 404;
		parent::__construct($message,$code,$previous);
	}
}

class HttpMethodNotAllowedException extends HTTPException {
	public function __construct($message = NULL, $code = 0, Exception $previous = null){
		$this->httpErrorCode = 405;
		parent::__construct($message,$code,$previous);
	}
}

class HttpRequestTimeoutException extends HTTPException {
	public function __construct($message = NULL, $code = 0, Exception $previous = null){
		$this->httpErrorCode = 408;
		parent::__construct($message,$code,$previous);
	}
}

class HttpInternalServerErrorException extends HTTPException {
	public function __construct($message = NULL, $code = 0, Exception $previous = null){
		$this->httpErrorCode = 500;
		parent::__construct($message,$code,$previous);
	}
}

////API Exceptions
//HttpUnauthorizedException
class BadUserPassException extends  HttpUnauthorizedException{
	public function __construct($message = "Login or pass incorrect",Exception $previous = null){
		parent::__construct($message,1,$previous);
	}
}

class NotAuthenticatedException extends HttpUnauthorizedException {
	public function __construct($message = "Authentication required before",Exception $previous = null){
		parent::__construct($message,2,$previous);
	}
}

//HttpNotFoundException
class NotFoundResourceException extends HttpNotFoundException{
	public function __construct($message = "Resource not found",Exception $previous = null){
		parent::__construct($message,1,$previous);
	}
}

class NotFoundMethodException extends HttpNotFoundException{
	public function __construct($message = "Method not available for this resource",Exception $previous = null){
		parent::__construct($message,2,$previous);
	}
}

class NotFoundIdException extends HttpNotFoundException{
	public function __construct($message = "Incorrect or not found ressource ID",Exception $previous = null){
		parent::__construct($message,3,$previous);
	}
}

class InvalidParametersException extends  HttpNotFoundException{
	public function __construct($message = "Incorrect or missing parameter",Exception $previous = null){
		parent::__construct($message,4,$previous);
	}
}

//HttpMethodNotAllowedException
class BadFormatException extends HttpMethodNotAllowedException{
	public function __construct($message = "Unknown format",Exception $previous = null){
		parent::__construct($message,1,$previous);
	}
}

class BadVersionNumberException extends HttpMethodNotAllowedException{
	public function __construct($message = "Unknown version number",Exception $previous = null){
		parent::__construct($message,2,$previous);
	}
}

//HttpForbiddenException
class AccessDeniedException extends  HttpForbiddenException {
	public function __construct($message = "Access denied",Exception $previous = null){
		parent::__construct($message,1,$previous);
	}
}

class InvalidLicenseException extends  HttpForbiddenException {
	public function __construct($message = "Access denied",Exception $previous = null){
		parent::__construct($message,2,$previous);
	}
}

class ExpiredLicenseException extends  HttpForbiddenException {
	public function __construct($message = "Access denied",Exception $previous = null){
		parent::__construct($message,3,$previous);
	}
}

//HttpRequestTimeoutException
class TimeoutException extends HttpRequestTimeoutException {
	public function __construct($message = "Exec time too long",Exception $previous = null){
		parent::__construct($message,1,$previous);
	}
}

//HttpInternalServerErrorException
class DatabaseConnectionException extends  HttpInternalServerErrorException{
	public function __construct($message = "API can not connect to database",Exception $previous = null){
		parent::__construct($message,1,$previous);
	}
}

class DatabaseErrorException extends  HttpInternalServerErrorException{
	public function __construct($message = "Database has returned SQL error to API",Exception $previous = null){
		parent::__construct($message,2,$previous);
	}
}

class APIResourceErrorException extends  HttpInternalServerErrorException{
	public function __construct($message = "Resource API no correctly implemented",Exception $previous = null){
		parent::__construct($message,3,$previous);
	}
}
//------------------------------------------------
class HttpCurlException extends HTTPException {
	public function __construct($message = NULL, $code = 0, Exception $previous = null){
		$this->httpErrorCode = 0;
		parent::__construct($message,$code,$previous);
	}
}

class HttpClientException extends HTTPException {
	public function __construct($message = NULL, $code = 0, Exception $previous = null){
		$this->httpErrorCode = 0;
		parent::__construct($message,$code,$previous);
	}
}
class InternFileAccessException extends  HttpInternalServerErrorException{
	public function __construct($message = "Resource API no correctly implemented",Exception $previous = null){
		parent::__construct($message,3,$previous);
	}
}
?>
