<?php
/**
 * @name Device
 * @description Classe principal de la  gestion des noeuds
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 16/11/2012
 */
include_once SNMP_CLASS;
include_once LIB_SNMP;
include_once TELNET_CLASS;
include_once SSH_CLASS;
include_once INTERFACE_CLASS;
class Device
{
	protected $id;
	protected $name;
	protected $type;
	protected $description;
	protected $ip;
	protected $ip_version;
	protected $login;
	protected $pwd;
	protected $typeConnection;
	protected $group;
	protected $groupid;
	protected $list_group;
	protected $snmpReq;
	protected $manufacturer;
	protected $manufacturer_logo;
	protected $model;
	protected $os;
	protected $os_version;
	protected $interfaces;
	protected $rdp_software;
	protected $rdp_soft_id;
	const VERSION_1 = 1;
	const VERSION_2C= 2;
	const VERSION_3 = 3;
	const TELNET = 'TELNET';
	const SSH = 'SSH';
	/*@NAME: __construct
	 *@IN: (Optionnel) si on passe un objet 'Device', le nouvelle objet sera une copie du précédent
	*@OUT: /
	*@TREAT: Constructeur par défaut de la classe Device.
	*@EXCEPTION: 	DeviceException si l'objet $dev n'est pas un objet device
	*/
	public function __construct ($dev=NULL)
	{
		if(is_object($dev))
		{
			if(get_class($dev)=='Device')
			{
				$this->constructDevice($dev);
			}
			else
			{
				throw DeviceException('$dev is not a Device object');
			}
		}
		else
		{
			$this->snmpReq = new SnmpRequest();
			$this->ip_version=4;
			$this->interfaces = array();
			$this->list_group = array();
		}
	}
	/*@NAME: __clone
	 *@IN:/
	*@OUT: /
	*@TREAT: Clone l'objet device
	*@EXCEPTION: /
	*/
	public function __clone()
	{
		$this->constructDevice($self);
	}
	/*@NAME: constructDevice
	 *@IN: $dev un objet Device
	*@OUT: /
	*@TREAT: constructeur de "copie"
	*@EXCEPTION: 	/
	*/
	public function constructDevice(Device $dev)
	{
		$this->id = $dev->getId();
		$this->name = $dev->getName();
		$this->description = $dev->getDescription();
		$this->type = $dev->getType();
		$this->ip = $dev->getIPAddress();
		$this->ip_version = $dev->getIPVersion();
		$this->login = $dev->getLogin();
		$this->pwd = $dev->getPassword();
		$this->typeConnection = $dev->getTypeConnection();
		$this->group = $dev->getGroup();
		$this->groupid = $dev->getGroupId();
		$this->snmpReq = $dev->getSNMPRequest();
		$this->interfaces = $dev->getInterfaces();
		$this->list_group = $dev->getGroupList();
		$this->rdp_software = $dev->getRdpSoftware();
		$this->rdp_sof_id = $dev->getRdpSoftId();
	}
	/*@NAME:setId
	 *@IN: $n id du device
	*@OUT: /
	*@TREAT: set l'id du device
	*@EXCEPTION: 	/
	*/
	public function setId($n)
	{
		$this->id=$n;
	}
	/*@NAME:setName
	 *@IN: $n nom du device
	*@OUT: /
	*@TREAT: set le nom du device
	*@EXCEPTION: 	/
	*/
	public function setName($n)
	{
		$this->name=$n;
	}
	/*@NAME:setType
	 *@IN: $n type du device
	*@OUT: /
	*@TREAT: set le type du device
	*@EXCEPTION: 	/
	*/
	public function setType($n)
	{
		$this->type=$n;
	}
	/*@NAME:setDescription
	 *@IN: $n description du device
	*@OUT: /
	*@TREAT: set la description du device
	*@EXCEPTION: 	/
	*/
	public function setDescription($n)
	{
		$this->description=$n;
	}
	/*@NAME:setIPv4Adress
	 *@IN: $tip un adresse ipv4 au format XXX.XXX.XXX.XXX
	*@OUT: TRUE si l'ip est au bon format sinon FALSE
	*@TREAT: set l'IP du device
	*@EXCEPTION: 	/
	*/
	public function setIPv4Adress($tip)
	{
		$tip = preg_replace('#\s+#','',$tip);
		if(filter_var($tip, FILTER_VALIDATE_IP,FILTER_FLAG_IPV4))
		{
			$this->ip = $tip;
			return TRUE;
		}
		else
        {
			return FALSE;
        }
	}
	/*@NAME:setIPAddress
	 *@IN: $tip un adresse IP valide
	*@OUT: TRUE si l'ip est au bon format sinon FALSE
	*@TREAT: set l'IP du device
	*@EXCEPTION: 	/
	*/
	public function setIPAddress($tip)
	{
		$tip = preg_replace('#\s+#','',$tip);
		if(filter_var($tip, FILTER_VALIDATE_IP))
		{
			$this->ip = $tip;
			return TRUE;
		}
		else
        {
			return FALSE;
        }
	}
	/*@NAME:setIPv4DecimalAdress
	 *@IN: $tip un adresse ipv4 au format décimal
	*@OUT: /
	*@TREAT: set l'IP du device
	*@EXCEPTION: 	/
	*/
	public function setIPv4DecimalAdress($tip)
	{
		$tmp = long2ip($tip);
        if(filter_var($tmp, FILTER_VALIDATE_IP,FILTER_FLAG_IPV4))
        {
		    $this->ip = long2ip($tip);
        }
        else
        {
            $this->ip = null;
        }
	}
	/*@NAME:setSNMPVersion
	 *@IN: $v la version SNMP
	*@OUT: /
	*@TREAT:la version SNMP du devicel'IP du device
	*@EXCEPTION: 	/
	*/
	public function setSNMPVersion($v)
	{
		switch($v)
		{
			case 1: $this->snmpReq->setVersion(SnmpRequest::VERSION_1); break;
			case 2: $this->snmpReq->setVersion(SnmpRequest::VERSION_2C);break;
			case 3: $this->snmpReq->setVersion(SnmpRequest::VERSION_3); break;
			default: return FALSE; break;
		}
		return TRUE;
	}
	/*@setSNMPTimeout
	 *@IN: $v timer snmp en us
	 *@OUT: /
	 *@TREAT: fixe le timer SNMP du device
	 *@EXCEPTION: 	/
	 */
	public function setSNMPTimeout($v)
	{
		$this->snmpReq->setTimeout($v);
	}
	/*@setSNMPPort
	 *@IN: $v le port distant SNMP 
	 *@OUT: /
	 *@TREAT: fixe le port SNMP du device
	 *@EXCEPTION: 	/
	 */
	public function setSNMPPort($v)
	{
		$this->snmpReq->setRemotePort($v);
	}
	/*@NAME:setSNMPCommunity
	 *@IN: $v la communauté SNMP
	*@OUT: /
	*@TREAT:la version SNMP du devicel'IP du device
	*@EXCEPTION: 	/
	*/
	public function setSNMPCommunity($c)
	{
		$this->snmpReq->setCommunity($c);
	}
	/*@NAME:setSNMP3Parameter
	 *@IN:$user le nom d'utilisateur SNMPv3
	*    $sec_lvl niveau de sécurité ( authNoPriv ou authPriv)
	*    $authprot protocaol authentification (md5 ou sha1)
	*    $authkey clé d'authentification
	*    $privprot (optionnel) protocol de chiffrement
	*    $privpass (optionnel) clé de chiffrement
	*@OUT: /
	*@TREAT:la version SNMP du devicel'IP du device
	*@EXCEPTION: 	/
	*/
	public function setSNMP3Parameter($user,$sec_lvl,$authprot,$authkey,$privprot=null,$privpass=null)
	{
		$this->snmpReq->setUser($user);
		//echo "seclvl: ".$sec_lvl."<br />";
		if($sec_lvl=='authNoPriv')
			$this->snmpReq->setSecurityLevel(SnmpRequest::AUTHNOPRIV);
		elseif($sec_lvl=='authPriv')
		$this->snmpReq->setSecurityLevel(SnmpRequest::AUTHPRIV);
		if($authprot=='md5')
			$this->snmpReq->setAuthentificationProtocol(SnmpRequest::MD5);
		elseif($authprot=='sha')
		$this->snmpReq->setAuthentificationProtocol(SnmpRequest::SHA);
		if($privprot=='aes')
			$this->snmpReq->setPrivacyProtocol(SnmpRequest::AES);
		elseif($privprot=='des')
		$this->snmpReq->setPrivacyProtocol(SnmpRequest::DES);
		if(!empty($privpass))
			$this->snmpReq->setPrivacyKey($privpass);
		$this->snmpReq->setAuthentificationKey($authkey);
	}
	/*@NAME:setConnectionID
	 *@IN: $l le login $p le mot de passe
	*@OUT: /
	*@TREAT: set les identifiants pour la connexion telnet /ssh
	*@EXCEPTION: 	/
	*/
	public function setConnectionID($l,$p)
	{
		$this->login=$l;
		$this->pwd=$p;
	}
	/*@NAME:setGroup
	 *@IN: $g le nom du groupe
	*@OUT: /
	*@TREAT: set le nom du groupe du device
	*@EXCEPTION: 	/
	*/
	public function setGroup($g)
	{
		$this->group=$g;
	}
	/*@NAME:setGroupId
	 *@IN: $g l'id du groupe
	*@OUT: /
	*@TREAT: set l'id du groupe du device
	*@EXCEPTION: 	/
	*/
	public function setGroupID($g)
	{
		$this->groupid=$g;
	}
	/*@setGroupList
	 *@IN: $g un tableau de groupe 
	*@OUT: /
	*@TREAT: set l'id des groupes auquelle le noeud appartient
	*@EXCEPTION: 	/
	*/
	public function setGroupList($g)
	{
		$this->list_group=$g;
	}	
	/*@NAME:setGroup
	 *@IN: $p Device::TELNET ou Device::SSH
	*@OUT: /
	*@TREAT: set le type de connexion utilisée
	*@EXCEPTION: 	/
	*/
	public function setTypeConnection($p)
	{
		if($p==Device::TELNET)
			$this->typeConnection=$p;
		elseif($p==Device::SSH)
			$this->typeConnection=$p;
		elseif($p=='API')
			$this->typeConnection=$p;
	}
	/*@NAME: setManufacturer
	 *@IN: $man le nom du fabricant
	*@OUT:  /
	*@TREAT: set le nom du fabricant
	*@EXCEPTION: /
	*/
	public function setManufacturer($man)
	{
		$this->manufacturer=$man;
	}
	/*@NAME: setManufacturerLogo
	 *@IN: $logo le chemin 'apache' vers le fichier du logo
	*@OUT:  /
	*@TREAT: set le logo du fabricant
	*@EXCEPTION: /
	*/
	public function setManufacturerLogo($logo)
	{
		$this->manufacturer_logo=$logo;
	}
	/*@NAME: setOS
	 *@IN: $os l'OS du switch
	*@OUT:  /
	*@TREAT: set l'OS du switch
	*@EXCEPTION: /
	*/
	public function setOS($os)
	{
		$this->os=$os;
	}
	/*@NAME: setOSVersion
	 *@IN: $os la version de l'OS du switch
	*@OUT:  /
	*@TREAT: set la version de l'OS du switch
	*@EXCEPTION: /
	*/
	public function setOSVersion($os)
	{
		$this->os_version=$os;
	}
	/*@NAME: setModel
	 *@IN: $man le modèle du switch
	*@OUT:  /
	*@TREAT: set le modèle du switch
	*@EXCEPTION: /
	*/
	public function setModel($man)
	{
		$this->model=$man;
	}
	/*@NAME:setRdpSoftware
	 *@IN: $r software RDP 
	 *@OUT: /
	 *@TREAT: set le logiciel de prise en main à distance du device
	 *@EXCEPTION: 	/
	 */
	public function setRdpSoftware($r)
	{
		$this->rdp_software=$r;
	}
	/*@NAME:setRdpSoftId
	 *@IN: $r id software RDP
	 *@OUT: /
	 *@TREAT: set l'id pour le logiciel de prise en main à distance du device
	 *@EXCEPTION: 	/
	 */
	public function setRdpSoftId($r)
	{
		$this->rdp_soft_id=$r;
	}
	/*@NAME:getIPv4Adress
	 *@IN: /
	*@OUT: L'adresse IP au format xxx.xxx.xxx.xxx
	*@TREAT: retourne l'adresse IP
	*@EXCEPTION: 	/
	*/
	public function getIPAddress()
	{
		return $this->ip;
	}
	/*@NAME:getIPv4Adress
	 *@IN: /
	*@OUT: L'adresse IP au format xxx.xxx.xxx.xxx
	*@TREAT: retourne l'adresse IP
	*@EXCEPTION: 	/
	*/
	public function getIPv4Adress()
	{
        if(filter_var($this->ip, FILTER_VALIDATE_IP,FILTER_FLAG_IPV4))
        {
		    return $this->ip;
        }
        else
        {
            return null;
        }
	}
	/*@NAME:getIPv4DecimalAdress
	 *@IN: /
	*@OUT: L'adresse IP au format décimal
	*@TREAT: retourne l'adresse IP
	*@EXCEPTION: 	/
	*/
	public function getIPv4DecimalAdress()
	{
        if(filter_var($this->ip, FILTER_VALIDATE_IP,FILTER_FLAG_IPV4))
        {
		    return sprintf("%u\n", ip2long($this->ip));
        }
        else
        {
            return null;
        }
	}
	/*@NAME:getIPVersion
	 *@IN: /
	*@OUT: la version de l'adresse IP
	*@TREAT: retourne la version de l'adresse IP
	*@EXCEPTION: 	/
	*/
	public function getIPVersion()
	{
		return $this->ip_version;
	}
	/*@NAME:getId
	 *@IN: /
	*@OUT: l'identifiant du noeud ou null si il n'est pas définit
	*@TREAT: retourne l'identifiant
	*@EXCEPTION: 	/
	*/
	public function getId()
	{
		if(isset($this->id) && !empty ($this->id))
			return $this->id;
		else
			return null;
	}
	/*@NAME:getDescription
	 *@IN: /
	*@OUT: la description du device
	*@TREAT: retourne la description
	*@EXCEPTION: 	/
	*/
	public function getDescription()
	{
		return $this->description;
	}
	/*@NAME:getType
	 *@IN: /
	*@OUT: le type du device
	*@TREAT: retourne le type
	*@EXCEPTION: 	/
	*/
	public function getType()
	{
		return $this->type;
	}
	/*@NAME:getName
	 *@IN: /
	*@OUT: le nom du device
	*@TREAT: retourne le nom
	*@EXCEPTION: 	/
	*/
	public function getName()
	{
		return $this->name;
	}
	/*@NAME:getSNMPVersion
	 *@IN: /
	*@OUT: la version SNMP
	*@TREAT: retourne la version SNMP
	*@EXCEPTION: 	/
	*/
	public function getSNMPVersion()
	{
		return $this->snmpReq->getVersion();
	}
	/*@NAME: getSNMPTimeout
	 *@IN: /
	*@OUT: le timer snmp
	*@TREAT: retournee timer snmp
	*@EXCEPTION: 	/
	*/
	public function getSNMPTimeout()
	{
		return $this->snmpReq->getTimeout();
	}
	/*@getSNMPPort
	 *@IN: /
	 *@OUT: retourne le port snmp
	 *@TREAT: renvoi le port SNMP du device
	 *@EXCEPTION: 	/
	 */
	public function getSNMPPort()
	{
		return $this->snmpReq->getRemotePort();
	}
	/*@getSNMPCommunity
	 *@IN: /
	 *@OUT: retourne la communauté snmp
	 *@TREAT: renvoi la communauté SNMP du device
	 *@EXCEPTION: 	/
	 */
	public function getSNMPCommunity()
	{
		return $this->snmpReq->getCommunity();
	}
	/*@NAME:getSNMPRequest
	 *@IN: /
	*@OUT: un objet SNMPRequest
	*@TREAT: retourne  un objet SNMPRequest
	*@EXCEPTION: 	/
	*/
	public function getSNMPRequest()
	{
		return $this->snmpReq;
	}
	/*@NAME:getLogin
	 *@IN: /
	*@OUT: le login du noeud
	*@TREAT: retourne  le login
	*@EXCEPTION: 	/
	*/
	public function getLogin()
	{
		return $this->login;
	}
	/*@NAME:getPassword
	 *@IN: /
	*@OUT: le mot de passe du noeud
	*@TREAT: retourne  le mot de passe
	*@EXCEPTION: 	/
	*/
	public function getPassword()
	{
		return $this->pwd;
	}
	/*@NAME:getDNSName
	 *@IN: /
	*@OUT: le nom DNS du noeud si sa reussi sinon FALSE
	*@TREAT:effectue une requête DNS afin de faire la correspondance IP ==> DNS
	*@EXCEPTION: 	/
	*/
	public function getDNSName()
	{
		$nomdns = @gethostbyaddr($this->ip);
		if($nomdns!=$this->ip)
			return $nomdns;
		else
			return FALSE;
	}
	/*@NAME:getTypeConnection
	 *@IN: /
	*@OUT: le type de connexion utilisée
	*@TREAT: retourne le type de connexion utilisée par le device
	*@EXCEPTION: 	/
	*/
	public function getTypeConnection()
	{
		return $this->typeConnection;
	}
	/*@NAME:getGroup
	 *@IN: /
	*@OUT: le nom du groupe
	*@TREAT: retourne le nom du groupe
	*@EXCEPTION: 	/
	*/
	public function getGroup()
	{
		return $this->group;
	}
	/*@NAME: getGroupId
	 *@IN: /
	*@OUT: le'id du groupe
	*@TREAT: retourne l'id du groupe
	*@EXCEPTION: 	/
	*/
	public function getGroupId()
	{
		return $this->groupid;
	}
	/*@NAME: getGroupList
	 *@IN: /
	*@OUT: $g un tableau de groupe
	*@TREAT: retourne un tableau avec les id des groupes auquelle le noeud appartient
	*@EXCEPTION: 	/
	*/
	public function getGroupList()
	{
		return $this->list_group;
	}
	/*@getManufacturer
	 *@IN: /
	*@OUT: le nom du fabricant
	*@TREAT: retourne le nom du fabricant
	*@EXCEPTION: 	/
	*/
	public function getManufacturer()
	{
		return $this->manufacturer;
	}
	/*@NAME:getGroup
	 *@IN: /
	*@OUT: le nom du groupe
	*@TREAT: retourne le nom du groupe
	*@EXCEPTION: 	/
	*/
	public function getModel()
	{
		return $this->model;
	}
	/*@getOS
	 *@IN: /
	*@OUT: le nom du de l'OS
	*@TREAT: retourne le nom du de l'OS
	*@EXCEPTION: 	/
	*/
	public function getOS()
	{
		return $this->os;
	}
	/*@getOSversion
	 *@IN: /
	*@OUT: la version de l'OS
	*@TREAT: retourne la version de l'OS
	*@EXCEPTION: 	/
	*/
	public function getOSversion()
	{
		return $this->os_version;
	}
	/*@getManufacturerLogo
	 *@IN: /
	*@OUT: le chemin apache du logo du fabricant
	*@TREAT: retourne le chemin apache du logo du fabricant
	*@EXCEPTION: 	/
	*/
	public function getManufacturerLogo()
	{
		return $this->manufacturer_logo;
	}
	/*@getInterfaces
	 *@IN: /
	*@OUT: Un tableau d'objet interface
	*@TREAT: retourne un tableau d'objet interface
	*@EXCEPTION: 	/
	*/
	public function getInterfaces()
	{
		return $this->interfaces;
	}
	
	/*@NAME:getHostAliveBySNMP
	 *@IN: /
	*@OUT: TRUE Si sa le test reussi sinon une Exception est levée
	*@TREAT: Effectue une requête SNMP sur le noeud (oid du sysUptime) si le noeud répond TRUE est retourné sinon  une exception est levée
	*@EXCEPTION: DeviceNotAliveSNMPException si une erreur survient
	*/
	public function getHostAliveBySNMP()
	{
		try
		{
			$uptime =$this->snmpget(".1.3.6.1.2.1.1.3.0");
		}
		catch (DeviceSNMPException $e)
		{
			throw new DeviceNotAliveSNMPException('Device SNMP not respond',1,$e);
		}
		return true;
	}
	/*@NAME:getUpTime
	 *@IN: /
	*@OUT: un tableau associatif contenant les clés 'day','hours','minut' et 'second'
	*@TREAT: Effectue une requête SNMP sur le noeud (oid du sysUptime  => .1.3.6.1.2.1.1.3.0)
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function getUpTime()
	{
		$uptime =$this->snmpget(".1.3.6.1.2.1.1.3.0");
		return TimeTicksToSec($uptime);
	}
	/*@NAME:getSysDescr
	 *@IN: /
	*@OUT: le sysDescr du noeud
	*@TREAT: Effectue une requête SNMP sur le noeud (oid du sysDescription => .1.3.6.1.2.1.1.1.0)
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function getSysDescr()
	{
		$desc =$this->snmpget(".1.3.6.1.2.1.1.1.0");
		return formatString($desc);
	}
	/*@NAME:getSysLocation
	 *@IN: /
	*@OUT: la localisation du noeud
	*@TREAT: Effectue une requête SNMP sur le noeud (oid du sysLocation => .1.3.6.1.2.1.1.6.0)
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function getSysLocation()
	{
		$desc =$this->snmpget(".1.3.6.1.2.1.1.6.0");
		return formatString($desc);
	}
	/*@NAME:getSysContact
	 *@IN: /
	*@OUT: le contact du noeud
	*@TREAT: Effectue une requête SNMP sur le noeud (oid du sysContact => .1.3.6.1.2.1.1.4.0)
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function getSysContact()
	{
		$desc =$this->snmpget(".1.3.6.1.2.1.1.4.0");
		return formatString($desc);
	}
	
	/*@NAME:getSysName
	 *@IN: /
	*@OUT: le sysName du noeud
	*@TREAT: Effectue une requête SNMP sur le noeud (oid du sysContact => .1.3.6.1.2.1.1.5.0)
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function getSysName()
	{
		$desc =$this->snmpget(".1.3.6.1.2.1.1.5.0");
		return formatString($desc);
	}
	
	
	/*@NAME:getSysObjectId
	 *@IN: /
	*@OUT: le type du noeud
	*@TREAT: Effectue une requête SNMP sur le noeud (oid du sysContact => .1.3.6.1.2.1.1.2.0)
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	protected $sysObjectId = null;
	public function getSysObjectId()
	{
		if( !is_array($this->sysObjectId)  ) $this->sysObjectId = $this->snmpwalk(".1.3.6.1.2.1.1.2");
		$tmp_walk = $this->sysObjectId;
		$arr=array();
		foreach($tmp_walk as $oid)
		{
			$tmp_oid  =explode(":",$oid);
			$tmp_oid  = trim($tmp_oid[1]);
			$arr[]=$tmp_oid;
		}
		return $arr;	
	}
	
	/*@NAME:snmpget
	 *@IN: $oid à interroger
	*@OUT: le résultat de la reqête
	*@TREAT: Effectue une requête SNMP GET sur le noeud
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function snmpget($oid)
	{
		try
		{
			return $this->snmpReq->snmpget($this->ip,$oid);
		}
		catch(SnmpException $e)
		{
			throw new DeviceSNMPException($e->getMessage(),1,$e);
		}
	}
	
	public function detectSNMPVersion($community){
		$snmp = new SnmpRequest();
		$snmp->setTimeout(1000000);
		
		$snmp->setVersion(SnmpRequest::VERSION_2C);
		$snmp->setCommunity($community); //La community doit être settée après la version
		try{
			$this->sysObjectId = $snmp->snmpwalk($this->getIPAddress(),".1.3.6.1.2.1.1.2");
			return SnmpRequest::VERSION_2C;
		}
		catch(Exception $ex){}
		
		try{
			$snmp = new SnmpRequest();
			$snmp->setVersion(SnmpRequest::VERSION_1);
			$snmp->setCommunity($community); //La community doit être settée après la version
	
			$this->sysObjectId = $snmp->snmpwalk($this->getIPAddress(),".1.3.6.1.2.1.1.2");
			return SnmpRequest::VERSION_1;
		}
		catch(Exception $ex){}
		return null;
	}
	
	/*@NAME:snmpwalk
	 *@IN: $oid à interroger
	*@OUT: le résultat de la reqête
	*@TREAT: Effectue une requête SNMP WALK sur le noeud
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function snmpwalk($oid)
	{
		try
		{
			return $this->snmpReq->snmpwalk($this->ip,$oid);
		}
		catch(SnmpException $e)
		{
			throw new DeviceSNMPException($e->getMessage(),1,$e);
		}
	}
	/*@NAME:snmpwalkoid
	 *@IN: $oid à interroger
	*@OUT: le résultat de la reqête
	*@TREAT: Effectue une requête SNMP WALK sur le noeud
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function snmpwalkoid($oid)
	{
		try
		{
			return $this->snmpReq->snmpwalkoid($this->ip,$oid);
		}
		catch(SnmpException $e)
		{
			throw new DeviceSNMPException($e->getMessage(),1,$e);
		}
	}
	/*@NAME:getSNMPObject
	 *@IN: /
	*@OUT: l'OBJET SNMPREquest
	*@TREAT: retourne l'objet SNMP Request du noeud
	*@EXCEPTION: /
	*/
	public function getSNMPObject()
	{
		return $this->snmpReq;
	}
	/*@NAME:getRdpSoftware
	 *@IN: /
	 *@OUT: le logiciel RDP 
	 *@TREAT: retourne  le nom du logiciel
	 *@EXCEPTION: 	/
	 */
	public function getRdpSoftware()
	{
		return $this->rdp_software;
	}
	/*@NAME:getRdpSoftId
	 *@IN: /
	 *@OUT: l'identifiant pour le logiciel RDP
	 *@TREAT: retourne  l'identifiant utilisé pour le logiciel RDP
	 *@EXCEPTION: 	/
	 */
	public function getRdpSoftId()
	{
		return $this->rdp_soft_id;
	}
	/*@NAME:checkConnection
	 *@IN: /
	*@OUT: /
	*@TREAT: test la connexion du noeud
	*@EXCEPTION: DeviceException si une erreur survient
	*/
	public function checkConnection($lien='')
	{
		if($this->typeConnection==Device::TELNET)
		{
			try
			{
				$t = new Telnet($this->ip,23,1);
				$t->login($this->login,$this->pwd);
				$t->close();
			}
			catch(TelnetException $e)
			{
				throw new DeviceException($e->getMessage(),2,$e);
			}
		}
		elseif($this->typeConnection==Device::SSH)
		{
			try
			{
				$t = new SSH($this->ip,22);
				$t->login($this->login,$this->pwd);
				$t->close();
			}
			catch(SSHException $e)
			{
				throw new DeviceException($e->getMessage(),2,$e);
			}
		}
		else
		{
			throw new DeviceException('Not connection protocol specified');
		}
	}
}
class DeviceException extends Exception{}
class DeviceSNMPException  extends DeviceException{}
class DeviceNotAliveException  extends DeviceException{}
class DeviceNotAliveSNMPException extends DeviceSNMPException{}
class DeviceNotAlivePingException extends DeviceNotAliveException{}
?>
