<?php
/**
 * @name SnmpRequest
 * @description Classe gérant les requètes SNMP du projet ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 07/11/2012
 */
class SnmpRequest
{
	//Constante pour les versions SNMP
	const VERSION_1 = 1;
	const VERSION_2C= 2;
	const VERSION_3 = 3;
	//Niveau de sécurité
	const AUTHNOPRIV ='authNoPriv';
	const AUTHPRIV   ='authpriv';
	//Protocol d'authentification
	const SHA ='sha';
	const MD5 ='md5';
	//protocol de chiffrement
	const AES ='aes';
	const DES ='des';
	//Constante des différents type.
	const TYPE_MIB='=';
	const TYPE_INT='i';
	const TYPE_INT64_UNSIGNED='U';
	const TYPE_INT64='I';
	const TYPE_INT_UNSIGNED='u';
	const TYPE_FLOAT='F';
	const TYPE_DOUBLE='D';
	const TYPE_STRING='s';
	const TYPE_HEXSTRING='x';
	const TYPE_DECSTRING='d';
	const TYPE_NULLOBJ='n';
	const TYPE_OBJID='o';
	const TYPE_TIMETICKS='t';
	const TYPE_IPADDRESS='a';
	const TYPE_BITS='b';
	const TYPE_GAUGE32='u';

	// public $count=0;
	private $snmp_timer;
	private $snmp_retrie;
	private $snmp_version;
	private $snmp_community;
	private $snmp_port;
	//variable SNMPv3
	private $snmp_user;
	private $snmp_auth_key;
	private $snmp_auth_protocol;
	private $snmp_priv_key;
	private $snmp_priv_protocol;
	private $sec_level;
    //variable de session
    private $session;
	/*@NAME: __construct
	 *@IN: /
	*@OUT: /
	*@TREAT: Constructeur par défaut de la classe SnmpRequest.
	*@EXCEPTION: 	/
	*/
	public function __construct ()
	{
		ini_set('track_errors', 'On');
		$this->snmp_timer=1000000;
		$this->snmp_retrie=1;
		$this->snmp_port=161;
        $this->session=null;
		/*$this->snmp_version=SnmpRequest::VERSION_1;*/
	}
	/*@NAME: setRemotePort
	 *@IN: $v 'integer' port utiliser (défaut: 161);
	 *@OUT: /
	 *@TREAT: set le port SNMP utilisé
	 *@EXCEPTION: 	/
	 */
	public function setRemotePort($v)
	{
		if(is_numeric($v))
		{
			$this->snmp_port=$v;
		}
		else 
		{
			throw new SnmpException("SNMP Error Port is not numeric $v");
		}
	}
	/*@NAME: setVersion
	 *@IN: $v qui prend comme constante SnmpRequest::VERSION_1,SnmpRequest::VERSION_2C,SnmpRequest::VERSION_3
	*@OUT: /
	*@TREAT: set la version SNMP utilisée
	*@EXCEPTION: 	/
	*/
	public function setVersion($v)
	{
		$flag=true;
		switch($v)
		{

			case SnmpRequest::VERSION_1: $this->snmp_version=SnmpRequest::VERSION_1; break;
			case SnmpRequest::VERSION_2C: $this->snmp_version=SnmpRequest::VERSION_2C; break;
			case SnmpRequest::VERSION_3: $this->snmp_version=SnmpRequest::VERSION_3; break;
			default:  $flag=false; break;
		}
		if($flag==false)
		{
			switch($v)
			{
				case "1": $this->snmp_version=VERSION_1; break;
				case "2c": $this->snmp_version=VERSION_2C;break;
				case "3": $this->snmp_version=VERSION_3; break;
				default: $flag=false; break;
			}
		}
		return TRUE;
	}
	/*@NAME: setCommunity
	 *@IN: $c le nom de la communauté SNMP attention bien "setter" la version AVANT de mettre la communauté
	*@OUT: /
	*@TREAT: set la version SNMP utilisée
	*@EXCEPTION: 	/
	*/
	public function setCommunity($c)
	{
		if($this->snmp_version==SnmpRequest::VERSION_1||$this->snmp_version==SnmpRequest::VERSION_2C)
		{
			$this->snmp_community=$c;
		}
	}
	/*@NAME: setTimeout
	 *@IN: $t temps en ms
	*@OUT: /
	*@TREAT: fixe la durée du timer en Microsecondes
	*@EXCEPTION: 	/
	*/
	public function setTimeout($t)
	{
		$this->snmp_timer=$t;
	}
	/*@NAME: setUser
	 *@IN: $u le nom de l'utilisateur
	*@OUT: /
	*@TREAT: fixe le nom de l'utilisateur pour l'authentification SNMPv3
	*@EXCEPTION: 	/
	*/
	public function setUser($u)
	{
		$this->snmp_user=$u;
	}
	/*@NAME: setAuthentificationKey
	 *@IN: $u la clé d'authentification
	*@OUT: /
	*@TREAT: fixe la clé d'authentification (SNMPv3)
	*@EXCEPTION: 	/
	*/
	public function setAuthentificationKey($u)
	{
		$this->snmp_auth_key=$u;
	}
	/*@NAME: setAuthentificationProtocol
	 *@IN: $v le protocol que l'on veut utiliser (SnmpRequest::SHA ou SnmpRequest::MD5)
	*@OUT: /
	*@TREAT: fixe la méthode de hashage qui sera utilisée pour hashé la clé d'authentification
	*@EXCEPTION: 	/
	*/
	public function setAuthentificationProtocol($v)
	{
		switch($v)
		{
			case SnmpRequest::SHA :$this->snmp_auth_protocol='sha'; break;
			case SnmpRequest::MD5: $this->snmp_auth_protocol='md5'; break;
			default: return FALSE; break;
		}
	}
	/*@NAME: setPrivacyKey
	 *@IN: $u la clé de chiffrement
	*@OUT: /
	*@TREAT: fixe la clé qui sera utilisée pour chiffrer le message SNMP
	*@EXCEPTION: 	/
	*/
	public function setPrivacyKey($u)
	{
		$this->snmp_priv_key=$u;
	}
	/*@NAME: setPrivacyProtocol
	 *@IN: $v le protocol que l'on veut utiliser (SnmpRequest::AES ou SnmpRequest::DES)
	*@OUT: /
	*@TREAT: fixe le protocol de chiffrement
	*@EXCEPTION: 	/
	*/
	public function setPrivacyProtocol($v)
	{
		switch($v)
		{
			case SnmpRequest::AES :$this->snmp_priv_protocol='aes'; break;
			case SnmpRequest::DES :$this->snmp_priv_protocol='des'; break;
			default: return FALSE; break;
		}
	}
	/*@NAME: setSecurityLevel
	 *@IN: $v le niveau de sécurité que l'on veut utiliser (SnmpRequest::AUTHNOPRIV ou SnmpRequest::AUTHPRIV)
	*@OUT: /
	*@TREAT: fixe le niveau de sécurité (snmpv3)
	*@EXCEPTION: 	/
	*/
	public function setSecurityLevel($v)
	{
		switch($v)
		{
			case SnmpRequest::AUTHNOPRIV : $this->sec_level='authNoPriv'; break;
			case SnmpRequest::AUTHPRIV: $this->sec_level='authPriv'; break;
			default: return FALSE; break;
		}
	}
	//Getter des propriétés
	public function getCommunity()
	{
		return $this->snmp_community;
	}
	public function getVersion()
	{
		if($this->snmp_version==SnmpRequest::VERSION_1)
			return SnmpRequest::VERSION_1;
		elseif($this->snmp_version==SnmpRequest::VERSION_2C)
		return SnmpRequest::VERSION_2C;
		elseif($this->snmp_version==SnmpRequest::VERSION_3)
		return SnmpRequest::VERSION_3;
	}
	/*@NAME: getTimeout
	 *@IN: /
	*@OUT: le timer un microseconde
	*@TREAT: retourne la durée du timer en microsecondes
	*@EXCEPTION: 	/
	*/
	public function getTimeout()
	{
		return $this->snmp_timer;
	}
	/*@NAME: getRemotePort
	 *@IN: /
	 *@OUT: integer le port snmp utilisé
	 *@TREAT: retourne le port SNMP utilisé
	 *@EXCEPTION: 	/
	 */
	public function getRemotePort()
	{
		return $this->snmp_port;
	}
	public function getSNMP3User()
	{
		return $this->snmp_user;
	}
	public function getSNMP3SecLevel()
	{
		return $this->sec_level;
	}
	public function getSNMP3AuthProtocol()
	{
		return $this->snmp_auth_protocol;
	}
	public function getSNMP3AuthKey()
	{
		return $this->snmp_auth_key;
	}
	public function getPrivacyKey()
	{
		return $this->snmp_priv_key;
	}
	public function getPrivacyProtocol()
	{
		return $this->snmp_priv_protocol;
	}
	/*@NAME: snmpget
	 *@IN: $ip de l'agent SNMP à intérroger, $oid: OID de la MIB à récupérer
	*@OUT: une chaine de caractère, contenant les informations demandées
	*@TREAT: effectue une requête SNMP (v1,v2 ou v3) et retourne le résultat
	*@EXCEPTION: 	SnmpNoResponseException: si l'agent n'a pas répondu
	*		SnmpNoSuchOID: si l'oid demandé n'existe pas
	*		SnmpUnknownVersionException: Si la version est inconnue
	*		Snmp3AuthorizationException: si une erreur d'autorisation apparait (SNMPv3)
	*		SnmpException: pour les autres erreurs (toute les exceptions citée hérite de SnmpException)
	*/
	public function snmpget($ip,$oid)
	{
    
        $value="";
        try
        {
            $this->instanceObject($ip);
            $value=$this->session->get($oid);
        }
        catch(SnmpUnknownVersionException $e)
        {
        	throw new SnmpException($e->getMessage());
        }
        catch(Exception $e)
        {
        	throw new SnmpException("SNMP Error".$this->session->getError());
        }
		return $this->check_error($value);
	}
	/*@NAME: snmpwalk
	 *@IN: $ip de l'agent SNMP à intérroger, $oid: OID de la MIB à récupérer
	*@OUT: Retourne un tableau de valeurs de l'objet SNMP
	*@TREAT: effectue une requête SNMP walk (v1,v2 ou v3) et retourne le résultat
	*@EXCEPTION: 	SnmpNoResponseException: si l'agent n'a pas répondu
	*		SnmpNoSuchOID: si l'oid demandé n'existe pas
	*		SnmpUnknownVersionException: Si la version est inconnue
	*		Snmp3AuthorizationException: si une erreur d'autorisation apparait (SNMPv3)
	*		SnmpException: pour les autres erreurs (toute les exceptions citée hérite de SnmpException)
	*/
	public function snmpwalk($ip,$oid)
	{
        try
        {
            $this->instanceObject($ip);
            $value=$this->session->walk($oid);
        }
        catch(SnmpUnknownVersionException $e)
        {
        	throw new SnmpException($e->getMessage());
        }
        catch(Exception $e)
        {
            throw new SnmpException("SNMP Error".$this->session->getError());
        }
		return $this->check_error($value);
	}
	/*@NAME: snmpwalkoid
	 *@IN: $ip de l'agent SNMP à intérroger, $oid: OID de la MIB à récupérer
	*@OUT: Retourne un tableau associatif d'identifiants d'objets SNMP ainsi que leurs valeurs
	*@TREAT: effectue une requête SNMP walk (v1,v2 ou v3) et retourne le résultat
	*@EXCEPTION: 	SnmpNoResponseException: si l'agent n'a pas répondu
	*		SnmpNoSuchOID: si l'oid demandé n'existe pas
	*		SnmpUnknownVersionException: Si la version est inconnue
	*		Snmp3AuthorizationException: si une erreur d'autorisation apparait (SNMPv3)
	*		SnmpException: pour les autres erreurs (toute les exceptions citée hérite de SnmpException)
	*/
	public function snmpwalkoid($ip,$oid)
	{
		$value=FALSE;
        try
        {
            $this->instanceObject($ip);
            $value=$this->session->walk($oid);
        }
        catch(SnmpUnknownVersionException $e)
        {
        	throw new SnmpException($e->getMessage());
        }
        catch(Exception $e)
        {
           throw new SnmpException("SNMP Error".$this->session->getError());
        }
		return $this->check_error($value);

	}
	/*@NAME: snmpset
	 *@IN: 	$ip de l'agent SNMP à intérroger
	*	$oid: OID de la MIB
	*	$value: valeur à donner
	*	$type: type de la valeur (voir les constantes de la clase)
	*@OUT: /
	*@TREAT: effectue une requête SNMP set (v1,v2 ou v3). Emet une exception si une erreur survient
	*@EXCEPTION: 	SnmpNoResponseException: si l'agent n'a pas répondu
	*		SnmpNoSuchOID: si l'oid demandé n'existe pas
	*		SnmpUnknownVersionException: Si la version est inconnue
	*		Snmp3AuthorizationException: si une erreur d'autorisation apparait (SNMPv3)
	*		SnmpException: pour les autres erreurs (toute les exceptions citée hérite de SnmpException)
	*/
	public function snmpset($ip,$oid,$value,$type=SnmpRequest::TYPE_MIB)
	{
        try
        {
            $this->instanceObject($ip);
            $value=$this->session->set($oid,$type,$value);
        }
        catch(SnmpUnknownVersionException $e)
        {
        	throw new SnmpException($e->getMessage());
        }
        catch(Exception $e)
        {
            throw new SnmpException("SNMP Error".$this->session->getError());
        }
	}
	/*@NAME: check_error
	 *@IN: 	$value le resultat d'une requête SNMP
	*@OUT: la valeur inchangée si OK sinon une exception
	*@TREAT: Parse le résultat d'une requête SNMP et émet une exception si une erreur est survenue
	*@EXCEPTION: 	SnmpNoResponseException: si l'agent n'a pas répondu
	*		SnmpNoSuchOID: si l'oid demandé n'existe pas
	*		SnmpUnknownVersionException: Si la version est inconnue
	*		Snmp3AuthorizationException: si une erreur d'autorisation apparait (SNMPv3)
	*		SnmpException: pour les autres erreurs (toute les exceptions citée hérite de SnmpException)
	*/
	private function check_error($value)
	{
		if($value === FALSE )
		{
			if(isset($php_errormsg))
			{
				if(preg_match('#No response#',$php_errormsg))
					throw new SnmpNoResponseException($php_errormsg);
				elseif(preg_match('#Timeout#',$php_errormsg))
				throw new SnmpNoResponseException($php_errormsg);
				elseif(preg_match('#This name does not exist#',$php_errormsg))
				throw new SnmpNoSuchOID($php_errormsg);
				elseif(preg_match('#An error occurred#',$php_errormsg))
				throw new SnmpException($php_errormsg);
				elseif(preg_match('#Invalid security level#',$php_errormsg))
				throw new AuthorizationException($php_errormsg);
				elseif(preg_match('#authorizationError#',$php_errormsg))
				throw new AuthorizationException($php_errormsg);
				elseif(preg_match('#Error in packet\. Reason: \(badValue\)#',$php_errormsg))
				throw new SnmpException($php_errormsg);
				elseif(preg_match('#Could not add variable#',$php_errormsg))
				throw new SnmpException($php_errormsg);
				else
					throw new SnmpException("Unknown Error".$php_errormsg);
			}
			else
			{
				throw new SnmpException(" php_errormsg is not set, you don't trace SnmpException");
			}
		}
		else
		{
			if(is_string($value))
			{
				if(preg_match('#No Such Instance currently exists at this OID#',$value))
					throw new SnmpNoSuchOID($value);
				if(preg_match('#No Such Object available on this agent at this OID#',$value))
					throw new SnmpNoSuchOID($value);
			}
		}
		return $value;
	}
    private function instanceObject($ip) 
    {
        if($this->session == null)
        {
        	if($this->snmp_port !=161)
        	{
        		$ip=$ip.':'.$this->snmp_port;
        	}
            if(filter_var($ip, FILTER_VALIDATE_IP,FILTER_FLAG_IPV6))
            {
                $ip="[$ip]";
            }
		    if($this->snmp_version==SnmpRequest::VERSION_1)
		    {
			    $this->session= new SNMP(SNMP::VERSION_1, $ip, $this->snmp_community,$this->snmp_timer,$this->snmp_retrie);
		    }
		    elseif($this->snmp_version==SnmpRequest::VERSION_2C)
		    {
                $this->session= new SNMP(SNMP::VERSION_2C, $ip, $this->snmp_community,$this->snmp_timer,$this->snmp_retrie);
		    }
		    elseif($this->snmp_version==SnmpRequest::VERSION_3)
		    {
                $this->session= new SNMP(SNMP::VERSION_3, $ip, $this->snmp_user,$this->snmp_timer,$this->snmp_retrie);
               // echo $this->sec_level .' '.$this->snmp_auth_protocol.' '. $this->snmp_auth_key.' '. $this->snmp_priv_protocol.' '. $this->snmp_priv_key."\n";
                $this->session->setSecurity($this->sec_level, $this->snmp_auth_protocol, $this->snmp_auth_key, $this->snmp_priv_protocol, $this->snmp_priv_key);
		    }
		    else
		    {
			    throw new SnmpUnknownVersionException('Unknown SNMP version');
		    }
        }
    }
}
class SnmpNoResponseException extends SnmpException {}
class SnmpNoSuchOID extends SnmpException {}
class SnmpUnknownVersionException extends SnmpException {}
class Snmp3AuthorizationException extends SnmpException {}

?>
