<?php
/**
 * @name Telnet
 * @description Classe SSH du projet ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2010
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 31/10/2012
 */
class ssh
{
	protected $fp;
	protected $usleep;
	/*@NAME: __construct
	 *@IN: $host: adresse IP ou nom de l'hôte
	$port: port de connexion par défaut 23
	*@OUT: /
	*@TREAT: Constructeur par défaut de la classe SSH, initialise la connexion.
	*@EXCEPTION: 	SSHUnableToConnectException si une erreur survient
	*/
	public function __construct ($host,$port=22)
	{
		if(!is_numeric($port))
		 throw new SSHUnableToConnectException('Port is not numeric');
		$this->usleep=100000;
		$this->fp=@ssh2_connect($host,$port);
		if($this->fp===false)
		{
			throw new SSHUnableToConnectException('Unable to connect '.$host.':'.$port.' '.$php_errormsg);
		}
		@ssh2_fingerprint($this->fp);
		usleep($this->usleep);
	}
	public function __destruct ()
	{
	    fclose($this->fp);
	}
	/*@NAME: login
	 *@IN: $log: login de l'utilisateur
	$pwd: mot de passe de l'utilisateur
	*@OUT: /
	*@TREAT: Entre le login et le mots de passe de l'utilisateur
	*@EXCEPTION: SSHLoginException si une erreur survient
	*/
	public function login($log, $pwd)
	{
		if(@ssh2_auth_password($this->fp,$log,$pwd))
		{
			return true;
		}
		else
		{
			throw new SSHLoginException('Login failed');
		}
	}
	/*@NAME: cmd
	 *@IN: $cmd: la commande a uploader sur l'hôte
	*@OUT: retourne le résultat de la commande
	*@TREAT: upload une commande sur l'hôte
	*@EXCEPTION: /
	*/
	public function cmd ($cmd)
	{
		$stream = ssh2_exec($this->fp, $cmd);
		stream_set_blocking($stream, true);
		$output=stream_get_contents($stream);
		fclose($stream);
		return $output;
	}
	/*@NAME: close
	 *@IN: /
	*@OUT: /
	*@TREAT: Ferme la socket de connexion
	*@EXCEPTION: /
	*/
	public function close()
	{
		fclose($this->fp);
	}
	/*@NAME: setTimer
	 *@IN: $t la durée du timer en microseconde
	*@OUT: /
	*@TREAT: set le timer
	*@EXCEPTION: /
	*/
	public function setTimer($t)
	{
		$this->usleep=$t;
	}
}
class SSHException extends Exception {}
class SSHUnableToConnectException extends SSHException {}
class SSHLoginException extends SSHException {}
