/**
 * @name jquery.esia.bar.js
 * @description Propose des fonctions d'affichage de bares pour les pourcentages et des listes de messages. Regénérer ke fichier  jquery.esia.bar.min.js après modifications
 * @author George Julien <j.george@esia-sa.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 25/02/2015
 */


/*@NAME: bar_view
*@IN:	elm 	l'élement qui contiendra la barre, peut être un id html, un élément DOM HTML javascript ou un selecteur jquery
*		val 	le pourcentage qui sera affiché dans la barre
*		text	si différent de null, à la place du pourcentage, la barre affichera le text
*		color	la couleur de la barre ou l'url de l'image d'arrière plan. Si null, la barre sera verte (<80%), jaune(<90%) ou rouge.
*		tooltip si non null, est le message, au format html qui sera affiché en tooltip lors du survol de la barre avec la souris.
@OUT: /
@TREAT: Affiche une barre affichant un pourcentage.
@EXCEPTION: 	/
*/
function bar_view(elm,val,text,color,tooltip){
        if( elm instanceof HTMLElement         ){ elm = $(elm); }
        if( typeof elm == "string"        ){ elm = $("#"+elm); }
        
        var colors = ['/theme/default-image/ok.png','/theme/default-image/warning.png', '/theme/default-image/critical.png'];
                if( color == null &&val > 0 && val < 101){
                if        ( val < 80 )        color=colors[0];
                else if        ( val < 90 )        color=colors[1];
                else                         color=colors[2];
        }
        if( text == null ) { text=val+"%"; }
        
        elm.html('<div class="progress-label">'+text+"</div>");
        elm.progressbar({ value: val });                
            var regexp = new RegExp(".*\.png|\.jpg|\.gif$");
        if (regexp.test(color))     { 
        	elm.find( ".ui-progressbar-value" ).css({"background-image":"url("+color+")"        });
        	elm.find( ".ui-progressbar-value" ).css({"background-repeat":"repeat"        });
        }
        else                        { elm.find( ".ui-progressbar-value" ).css({"background": color                        });}
        if( tooltip != null ){
                elm.tooltip();       
                elm.attr('title', tooltip);         
                elm.tooltip("option", tooltip );
                elm.tooltip( "option", "position", { my: "left+40 center", at: "center center" });        
        }
}

function namedrow_view(elm,title,content){
	if( elm instanceof HTMLElement         ){ elm = $(elm); }
        if( typeof elm == "string"        ){ elm = $("#"+elm); }

	if(elm.is("table")){
		var tbody=document.createElement("tbody");
		elm.get(0).style.width = "100%";
		elm.append(tbody);
		namedrow_view(tbody,title,content);
	}
	else if(elm.is("tbody")){
		var tr=document.createElement("tr");
		elm.append(tr);
		namedrow_view(tr,title,content);
	}
	else if(elm.is("tr")){
	       	var td_header=document.createElement("td");
		var td_content=document.createElement("td");
		        	
		elm.empty();        
		elm.append(td_header);
                elm.append(td_content);
		
		td_header.innerHTML="<strong>"+title+" :</strong>";
                td_header.style.width="1px";
                td_header.style.whiteSpace = "nowrap";

		td_content.appendChild(content);
	}	
	else if(elm.is("td")){
		namedrow_view(elm.parent(),title,content);
	}
	else {
		var table=document.createElement("table");                
                elm.append(table);
                namedrow_view(table,title,content);
	}
}

/*@NAME: namedbar_view
*@IN:	elm 	l'élement qui contiendra la barre, peut être un id html, un élément DOM HTML javascript ou un selecteur jquery
*		val 	le pourcentage qui sera affiché dans la barre
*		title	le texte qui sera affiché comme label
*		color	la couleur de la barre ou l'url de l'image d'arrière plan. Si null, la barre sera verte (<80%), jaune(<90%) ou rouge.
*		tooltip si non null, est le message, au format html qui sera affiché en tooltip lors du survol de la barre avec la souris.
@OUT: /
@TREAT: Affiche une barre affichant un pourcentage précédé d'un label. Si l'élément est une <table>, <tr> ou <td>, la barre sera ajoutée au tableau. (sans le label si l'élément est un td).
@EXCEPTION: 	/
*/
function namedbar_view(elm,val,title,color,tooltip){
        if( elm instanceof HTMLElement         ){ elm = $(elm); }
        if( typeof elm == "string"        ){ elm = $("#"+elm); }

	var div=document.createElement("div");
	namedrow_view(elm,title,div);
	bar_view(div,val,null,color,tooltip);
}

/*@NAME: namedtext_view
*@IN:	elm 	l'élement qui contiendra le namedtext. Peut être un id html, un élément DOM HTML javascript ou un selecteur jquery
*		text 	le contenu du namedtext
*		title	le texte qui sera affiché comme label
*		tooltip si non null, est le message, au format html qui sera affiché en tooltip lors du survol du namedtext avec la souris.
@OUT: /
@TREAT: Affiche un texte précédé d'un label. Si l'élément est une <table>, <tr> ou <td>, le namedtext sera ajoutée au tableau. (sans le label si l'élément est un td).
@EXCEPTION: 	/
*/
function namedtext_view(elm,text,title,tooltip){
	if( elm instanceof HTMLElement         ){ elm = $(elm); }
        if( typeof elm == "string"        ){ elm = $("#"+elm); }

	var div=document.createElement("div");
	namedrow_view(elm,title,div);

	div=$(div);

	div.text(text);
        if( tooltip != null ){
        	div.tooltip();       
                div.attr('title', tooltip);         
                div.tooltip("option", tooltip );
                div.tooltip( "option", "position", { my: "left+20 center+40", at: "left center" });        
        }
}

/*@NAME: list_view
*@IN:	elm 	l'élement qui contiendra le namedtext. Peut être un id html, un élément DOM HTML javascript ou un selecteur jquery
*		arr 	un tableau de tableaux associatifs contenant les paramétres de plusieurs namedtext et namedbar : [ {text : "mon texte", title : "le titre", tooltip : "un tooltip"}, {value : 5, title : "le titre", color : './img/ok.png', tooltip : "un tooltip"}]
@OUT: /
@TREAT: Affiche à partir de arr une liste de namedtext et namedbar. La fonction reconnait la namedbar si le champs "value" existe. 
* 		Si l'élément est un <div>, la fonction créera une table pour la liste. Si c'est une <table>, elle ajoutera des lignes, avec un <tr>, tout sera sur la même ligne.	
@EXCEPTION: 	/
*/
function list_view(elm,arr){
        if( elm instanceof HTMLElement         ){ elm = $(elm); }
        if( typeof elm == "string"        ){ elm = $("#"+elm); }
        
        if( elm.is("div") ){                                        //Pour que tout les éléments d'un div soient dans la même table
            var table=document.createElement("table");                
            elm.append(table);
            elm=table;
        }
        
        if( elm.is("td") ){
        	elm=elm.parent();
        	elm.empty();
        }
        
        for(idx in arr){
                item=arr[idx];
                
                if(elm.is("tr")){
                	if(idx>0){
                		elm.after("<tr></tr>");
                		elm=elm.next();
                	}
                }
                
                if( item["value"] != undefined ) namedbar_view(elm,item["value"],item["title"],item["color"],item["tooltip"]);
                else namedtext_view(elm,item["text"],item["title"],item["tooltip"]);
        }
}