<?php
/**
 * @name EsiaLog
 * @description Classe permettant de créer et de gérer un fichier de log
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2013
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 29/04/2013
 */
class EsiaLog {
	protected $filename;
	protected $username;
	/*@NAME: __construct
	 *@IN: $file chemin d'accès vers le fichier
	 *		$user 'optionnel' l'utilisateur qui est à l'origine de l'action
	*@OUT: /
	*@TREAT: Constructeur par défaut de la classe EsiaLog
	*@EXCEPTION:  /
	*/
	function __construct($file,$user='')
	{
		$this->filename=$file;
		$this->username=$user;
	}
	/*@NAME: write
	 *@IN: $plugin le plugin où l'on se trouve
	*	   $message le message à écrire
	*@OUT: /
	*@TREAT: écrit une ligne dans le fichier de log
	*@EXCEPTION: 	FileLogException si une erreur survient
	*/
	function write($plugin,$message)
	{
		$fp = @fopen($this->filename,"a+");
		if($fp!==FALSE)
		{
			$t = date("d/m/Y H:i:s");
			fwrite($fp,"($t) $plugin:".$this->username.":$message\n");
			fclose($fp);
		}
		else 
		{
			throw new FileLogException("Can't open file:".$this->filename);
		}
	}
	/*@NAME: getEsiaLog
	 *@IN: /
	*@OUT:  $tab[]["Type"]  
	*		$tab[]["Date"]		date 
	*		$tab[]["System"]	si c'est Esia ou un plugin à l'origine de la ligne
	*		$tab[]["User"]		Utilisateur
	*		$tab[]["Message"]	Message
	*@TREAT: retourne un tableau associatif contenant les diverse ligne du fichier de log
	*@EXCEPTION: 	FileLogException si une erreur survient
	*/
	function getEsiaLog()
	{
		$tab=array();
		$file = file_get_contents($this->filename);
		if($fp===FALSE)
		{
			throw new FileLogException("Can't open file:".$this->filename);
		}
		$out = explode("\n",$file);
		$nbrelem = count($out);
		for($i=0;$i<$nbrelem;$i++)
		{
			$ligne=$out[$i];
			if(preg_match("#\(([0-9]{1,2}\/[0-9]{1,2}\/[0-9]{4} [0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})\)#",$ligne,$regs))
			{
				$date = $regs[1];
				preg_match("#\)([a-zA-Z-\s]*):([a-zA-Z0-9-\.@\\\-_\s]*):(.*)#",$ligne,$reg);
				$tab[$i]=array('type'=>'none',
						'date'=>$date,
						'system'=>$reg[1],
						'user'=>$reg[2],
						'message'=>$reg[3]);
			}
			else
			{
				$tab[$i-1]['message']=$tab[$i-1]['message']."\n".$ligne;
			}
		}

		//$pattern = "#\(([0-9]{1,2}\/[0-9]{1,2}\/[0-9]{4} [0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})\)([a-zA-Z-\s]*):([a-zA-Z0-9-]*):(.*)#";
		//preg_match_all($pattern,$file, $matches, PREG_SET_ORDER);
		//echo '<pre>'.print_r($matches,true).'</pre>';
		/*while(!feof($fp))
		{
			// On récupère une ligne
			$ligne = fgets($fp);
			if(!feof($fp))
			{
				//selection de la date
				preg_match("#\(([0-9]{1,2}\/[0-9]{1,2}\/[0-9]{4} [0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})\)#",$ligne,$regs);
				$date = $regs[1];
				preg_match("#\)([a-zA-Z-\s]*):([a-zA-Z0-9-]*):(.*)#",$ligne,$reg);
				$tab[]=array('type'=>'none',
							'date'=>$date,
							'system'=>$reg[1],
							'user'=>$reg[2],
							'message'=>$reg[3]);
			}
		}
		fclose($fp);*/
		return $tab;
	}
}
class FileLogException extends Exception {}
?>
