<?php
/**
 * @name gesa.class.php
 * @description Classe de gestion des GESA du plugin "gesa" du projet ESIA. 
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2014
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date ${date}
 */ 

include_once SNMP_CLASS;
include_once DEVICE_CLASS;
include_once LIB_SNMP;

class GesaConnectionFailed extends Exception{
	public function __construct($message, $code = 0, Exception $previous = null) {
		    parent::__construct($message, $code, $previous);
	}

  	public function __toString() {
    		return __CLASS__ . ": [{$this->code}]: {$this->message}\n";
  	}	
}

class GESA extends Device
{
	public const GESA_TYPE = array(
		'01'=>'Unity',
		'86'=>'Unity x86 virtual',
		'10'=>'Egide Box',
		'90'=>'Egide x86 virtual'
	);
	protected $port;
	protected $sn;
	protected $maxSlot;
	protected $version_base;
	protected $version_nrpe;
	protected $version_plugin;
	protected $version_web;
	protected $repo_global;
	protected $repo_gesa;
	protected $last_update;
	protected $type;
	protected $rsbp;
	protected $localip;
	protected $publicip;
	protected $macaddress;
	protected $gesatype;
	protected $gesa_status;
	/*@NAME: __construct
	 *@IN: /
	*@OUT: /
	*@TREAT: constructeur de base 
	*@EXCEPTION: /
	*/
	public function __construct ()
	{
		parent::__construct();
		$this->sn='';
		$this->maxSlot=30;
		$this->port=5666;
	}
	/*@NAME: SetSerialNumber
	 *@IN: le numÃ©ros de sÃ©rie
	*@OUT: /
	*@TREAT: set le numÃ©ros de sÃ©rie 
	*@EXCEPTION:
	*/
	public function SetSerialNumber($serial)
	{
		if(preg_match('#([0-9]{8})\-([0-9]{2})\-([0-9]*)#', $serial, $regs))
		{
			$this->sn=$regs[1].$regs[2].$regs[3];
		}
		else
		{
			$this->sn = $serial;
		}
		$snTag = substr($this->sn, 8, 2);
		$this->gesatype = $snTag;
	}
	/*@NAME: getSerialNumber
	 *@IN: /
	*@OUT:  le numÃ©ros de sÃ©rie ou null s'il n'y en a pas
	*@TREAT: formate le numÃ©ros de sÃ©rie et le renvoie
	*@EXCEPTION:
	*/
	public function getSerialNumber()
	{
		$serial=null;
		return $this->sn;
	}
	/*@NAME: getMaxSlot
	 *@IN: /
	*@OUT:  Nombre maximun de slot 5 min disponible
	*@TREAT: retourne le nombre maximun de slot
	*@EXCEPTION:
	*/
	public function getMaxSlot()
	{
		return $this->maxSlot;
	}
	/*@NAME: setMaxSlot
	 *@IN:  $max = Nombre maximun de slot 5 min disponible
	*@OUT:  /
	*@TREAT: set le nombre maximun de slot
	*@EXCEPTION:
	*/
	public function setMaxSlot($max)
	{
		$this->maxSlot= $max;
	}

	/**
	 * retourne le port utilisé par la GESA
	 * @return number 
	 */
	public function getPort()
	{
		return $this->port;
	}
	/**
	 * set le port TCP de la GESA
	 * @param integer $port
	 */
	public function setPort($port)
	{
		$this->port= $port;
	}
	/**
	 * retourne la version du paque gesa-base
	 * @return string version du paquet
	 */
	public function getVersionBase()
	{
		return $this->version_base;
	}
	/**
	 * set la version du paquet gesa-base
	 * @param string $version
	 */
	public function setVersionBase($version)
	{
		$this->version_base= $version;
	}
	/**
	 * retourne la version du paque gesa-nrpe
	 * @return string version du paquet
	 */
	public function getVersionNRPE()
	{
		return $this->version_nrpe;
	}
	/**
	 * set la version du paquet gesa-nrpe
	 * @param string $version
	 */
	public function setVersionNRPE($version)
	{
		$this->version_nrpe= $version;
	}
	/**
	 * retourne la version du paque gesa-nrpe-plugins
	 * @return string version du paquet
	 */
	public function getVersionPlugins()
	{
		return $this->version_plugin;
	}
	/**
	 * set la version du paquet gesa-nrpe-plugins
	 * @param string $version
	 */
	public function setVersionPlugins($version)
	{
		$this->version_plugin= $version;
	}
	/**
	 * retourne la version du paque gesa-web-interface
	 * @return string version du paquet
	 */
	public function getVersionWeb()
	{
		return $this->version_web;
	}
	/**
	 * set la version du paquet gesa-web-interface
	 * @param string $version
	 */
	public function setVersionWeb($version)
	{
		$this->version_web= $version;
	}
	/**
	 * retourne l'url du repository global
	 * @return string version du paquet
	 */
	public function getRepositoryGlobal()
	{
		return $this->repo_global;
	}
	/**
	 * set  l'url du repository global
	 * @param string $repo
	 */
	public function setRepositoryGlobal($repo)
	{
		$this->repo_global= $repo;
	}	
	/**
	 * retourne l'url du repository GESA
	 * @return string version du paquet
	 */
	public function getRepositoryGESA()
	{
		return $this->repo_gesa;
	}
	/**
	 * set  l'url du repositiry GESA
	 * @param string $repo
	 */
	public function setRepositoryGESA($repo)
	{
		$this->repo_gesa= $repo;
	}	
	/**
	 * retourne la date du dernier update
	 * @return string version du paquet
	 */
	public function getLastUpdate()
	{
		return $this->last_update;
	}
	/**
	 * set la date de dernière update
	 * @param string $date
	 */
	public function setLastUpdate($date)
	{
		$this->last_update= $date;
	}
	/**
	 * retourne la version de la carte mère
	 * @return string la version de la carte mère
	 */
	public function getMotherBoard()
	{
		return $this->rsbp;
	}
	/**
	 * set la version de la carte mère
	 * @param string $motherboard
	 */
	public function setMotherBoard($motherboard)
	{
		$this->rsbp= $motherboard;
	}
	/**
	 * @brief set l'adresse ip local
	 * @param string $adresse : l'adresse ip locale de la gesa
	 */
	public function setLocalIPaddress($adresse)
	{
		$this->localip= $adresse;
	}
	/**
	 * @brief get l'adresse ip local
	 * @retval string l'adresse ip locale de la gesa
	 */
	public function getLocalIPaddress()
	{
		return $this->localip;
	}
	/**
	 * @brief set l'adresse ip publique
	 * @param string $adresse : l'adresse ip publique de la gesa
	 */
	public function setPublicIPaddress($adresse)
	{
		$this->publicip= $adresse;
	}
	/**
	 * @brief get l'adresse ip publique
	 * @retval string l'adresse ip publique de la gesa
	 */
	public function getPublicIPaddress()
	{
		return $this->publicip;
	}
	/**
	 * @brief set l'adresse mac de la unity
	 * @param string $adresse : l'adresse mac de la gesa
	 */
	public function setMacAddress($adresse)
	{
		$this->macaddress= $adresse;
	}
	/**
	 * @brief get l'adresse mac
	 * @retval string l'adresse mac de la gesa
	 */
	public function getMacAddress()
	{
		return $this->macaddress;
	}
	public function getGesaType()
	{
		return GESA::GESA_TYPE[$this->gesatype];
	}
	public function getGesaTypeNum()
	{
		return $this->gesatype;
	}
	/**
	 * brief TRUE si la GESA est accessible par le serveur ESIA, FALSE si non connectée
	 * @param integer $serial : le numero de serie de la gesa
	 * @return boolean 
	 */
	public static function getIsConnected($serial)
	{
		$isConnected = FALSE;

		if(!empty($serial))
		{
			// Use check_gesa plugin to get gesa list
			$cmd = '/usr/local/esia/plugins/check_gesa -C check-unity-state -G ' . $serial;
			exec($cmd, $output, $code);
			if($code == 0)
			{
				// If cmd succeed check if we found our gesa serial in the list
				foreach($output as $row)
				{
					// Get SN from list rows (sn=public_ip)
					$rowSerial = explode('=', trim($row))[0];
					if($rowSerial == $serial)
					{
						$isConnected = TRUE;
						break;
					}
				}
			}
		}
		
		return $isConnected;
	}
	/**
	 * retourne le status de la gesa (new, ok, critical, warning, unknown)
	 * @return string version du paquet
	 */
	public function getGesaStatus()
	{
		$gesaStatus = $this->gesa_status;
		if(empty($gesaStatus))
		{
			$gesaStatus = 'unknown';
		}

		return $gesaStatus;
	}
	/**
	 * set le status de la gesa (new, ok, critical, warning, unknown)
	 * @param string $status
	 */
	public function setGesaStatus($status)
	{
		$this->gesa_status = $status;
	}
	/*@NAME: check_nrpe
	 *@IN: $host : la machine que la gesa doit checker, peut Ãªtre une chaine vide (""), un objet Node ou une ip 
	*	$command : la commande nrpe a exÃ©cuter
	*	$args (optionel) : des arguments a passer a la commande nrpe
	*	$timeout (optionel) : timeout en secondes
	*@OUT: un tableau associatif organiser comme suit:
	* 	$arr["status"]=END signigie que toutes les donnÃ©es on Ã©tÃ© transmises, MORE signifie qu'il reste des donnÃ©es.
	*	$arr["size"]=Nbre de lignes transmise, si ["status"]=MORE, correspond Ã  la ligne suivante Ã  transmettre.
	* 	$arr["data"]=Tableau retournant la totalitÃ© de l'output de checkn_nrpe, [0] correspondant a la premiÃ¨re ligne
	*@TREAT: ExÃ©cute un check_nrpe et retourne un tableau avec les rÃ©sultats.
	*@EXCEPTION: GesaConnectionFailed en cas d'erreur de NRPE.
	*/
	public function check_nrpe($host,$command,$args="",$timeout=50){
		if(empty($args))$args="";
		if(empty($timeout))$timeout=50;
		
		$link = $this->name;
		if(gethostbyname($this->name)==$this->name)
		{
			//$link = $this->publicip;
			$link = $this->ip;
		}
		
		if($host instanceof Device){
			if(strcmp($host->getIPAddress(),"0.0.0.0")==0) $host=$host->getName(); 
			else $host=$host->getIPAddress();
		}
		//echo $command." ".$args.'<br />';
		//echo NRPE_PATH.' -H '.$link.' -p '.$this->port.' -c '.$command.' -t '.$timeout.' -a '.$host.' '.$args.'<br />';

		$sn = $this->getSerialNumber();
		if(empty($sn)){
			exec(NRPE_PATH.' -H '.$link.' -p '.$this->port.' -c '.$command.' -t '.$timeout.' -a '.$host.' '.$args,$output,$returnexec);
		}
		else{
			exec(NRPE_PATH.' -S '.$sn.' -H '.$link.' -p '.$this->port.' -c '.$command.' -t '.$timeout.' -a '.$host.' '.$args,$output,$returnexec);
		}
		
		if(empty($output[0]))  throw new GesaConnectionFailed("GESA UNREACHEABLE ".$host,0);
		if(preg_match('#UNKNOWN\s+ERROR#',$output[0]) || $returnexec==3 ) throw new GesaConnectionFailed($output[0],1);
		if(preg_match("#NRPE: Command $command not defined#",$output[0])) throw new GesaConnectionFailed($output[0],2); 

		//echo "<pre>".print_r($output,true)."</pre>";
		preg_match("#<(\w*)>#",end($output),$matches);
		if(empty($matches[1]))  return array("status"=>"CHECK",count($output),"data"=>$output);

		array_pop($output);
		
		$data=array();
		foreach($output as $key => $value){
			$data[$key]=str_getcsv ( $value , ",", '"','"');
		}
	
		return array("status"=>$matches[1],"size"=>count($output),"data"=>$data);
	}
	/*@NAME: check_ecatp
	 *@IN: $host : la machine que la gesa doit checker, peut Ãªtre une chaine vide (""), un objet Node ou une ip
	 *	$command : la commande nrpe a exÃ©cuter
	 *	$args (optionel) : des arguments a passer a la commande nrpe
	 *	$timeout (optionel) : timeout en secondes
	 *@OUT: un tableau associatif organiser comme suit:
	 * 	$arr["status"]=END signigie que toutes les donnÃ©es on Ã©tÃ© transmises, MORE signifie qu'il reste des donnÃ©es.
	 *	$arr["size"]=Nbre de lignes transmise, si ["status"]=MORE, correspond Ã  la ligne suivante Ã  transmettre.
	 * 	$arr["data"]=Tableau retournant la totalitÃ© de l'output de checkn_nrpe, [0] correspondant a la premiÃ¨re ligne
	 *@TREAT: ExÃ©cute un check ecatp et retourne un tableau avec les rÃ©sultats.
	 *@EXCEPTION: GesaConnectionFailed en cas d'erreur.
	 */
	public function check_ecatp($host,$command,$args="",$timeout=50,$accept_unknown=FALSE)
	{
                if(empty($args))$args="";
                if(empty($timeout))$timeout=50;

				$host=$host->getIPAddress();
                return $this->execByECATP($command,"-H $host $args",$timeout,$accept_unknown);
	}
	/*@NAME: execByECATP
	 *@IN: $command : la commande nrpe a exÃ©cuter
	 *	$args (optionel) : des arguments a passer a la commande nrpe
	 *	$timeout (optionel) : timeout en secondes
	 *@OUT: un tableau associatif organiser comme suit:
	 * 	$arr["status"]=END signigie que toutes les donnÃ©es on Ã©tÃ© transmises, MORE signifie qu'il reste des donnÃ©es.
	 *	$arr["size"]=Nbre de lignes transmise, si ["status"]=MORE, correspond Ã  la ligne suivante Ã  transmettre.
	 * 	$arr["data"]=Tableau retournant la totalitÃ© de l'output de checkn_nrpe, [0] correspondant a la premiÃ¨re ligne
	 *@TREAT: ExÃ©cute un check ecatp et retourne un tableau avec les rÃ©sultats.
	 *@EXCEPTION: GesaConnectionFailed en cas d'erreur.
	 */
    public function execByECATP($command,$args="",$timeout=50,$accept_unknown=FALSE)
    {
		$sn = $this->getSerialNumber();
       	exec('/usr/local/esia/plugins/check_gesa -G '. $sn . ' -C '. $command. ' -t '.$timeout." -A '$args'",$output,$returnexec);
       //	echo '/usr/local/esia/plugins/check_gesa -G '. $sn . ' -C '. $command. ' -t '.$timeout." -A '$args'\n";
       	if(empty($output[0]))
       	{
       		throw new GesaConnectionFailed("GESA UNREACHEABLE ",0);
       	}
       	if(!$accept_unknown && preg_match('#UNKNOWN\s+ERROR#',$output[0]) || $returnexec==3 ) 
       	{
       		throw new GesaConnectionFailed($output[0],1);
       	}
       	if(preg_match("#NRPE: Command $command not defined#",$output[0]))
       	{
       		throw new GesaConnectionFailed($output[0],2);
       	}
       	//echo "<pre>".print_r($output,true)."</pre>";
       	//check if json
       	$firstchar = substr($output[0],0,1);
       	if($firstchar=='{')
       	{
       		//on est en mode JSON
       		$json="";
       		foreach($output as $line)
       		{
       			$json.=$line;
       		}
       	//	echo($json);
       		$json = @json_decode($json, true);
       		if(is_array($json))
       		{
       			$data=array();
       			foreach($json as $key => $value)
       			{
       				$data[$key]=$value;
       			}
       			return array("status"=>'END',"size"=>count($json),"data"=>$data);
       		}
       		else 
       		{
       			throw new GesaConnectionFailed("BAD output return ",2);
       		}
       	}
       	else 
       	{
	       	preg_match("#<(\w*)>#",end($output),$matches);
	       	if(empty($matches[1]))
	       	{
	       		return array("status"=>"CHECK",count($output),"data"=>$output);
	       	}
	       	
	       	array_pop($output);
	       	
	       	$data=array();
	       	foreach($output as $key => $value)
	       	{
	       		$data[$key]=str_getcsv ( $value , ",", '"','"');
	       	}
	       	return array("status"=>$matches[1],"size"=>count($output),"data"=>$data);
       	}
	}
	
	/*@NAME: line_to_array
	 *@IN:	$headers	: Un array contenant les noms des colonnes
	*		$line 	 	: Un array contenant chaque champs d'une ligne 
	*@OUT: Un tableau associatif
	*@TREAT: Utilise $headers pour nommer les keys de $line et retourner un tableau associatif de $line
	*@EXCEPTION:
	*/
	public static function line_to_array($headers, $line){
		$ret=array();
		for($i=0;$i < count($headers);++$i){
			if(isset($line[$i]))	$ret[ $headers[$i] ] = $line[$i];
			else $ret[ $headers[$i] ]= "";
		}
		return $ret;
	}
	
	/*@NAME: data_to_arrays
	 *@IN:	$data	: Un tableau de tableaux dont le tableau 0 contient les headers des lignes suivantes
	*@OUT: Un tableau de tableaus associatifs
	*@TREAT: Utilise la fonction line_to_array pour construire un tableau donc chaque index y compris le 0 contient un tableau associatif d'une ligne csv.
	*@EXCEPTION:
	*/
	public static function data_to_arrays($data){
		$ret=array();
		$headers=array_shift($data);
		foreach($data as $value){
			array_push($ret,GESA::line_to_array($headers,$value));
		}
		return $ret;
	}
}
?>
