<?php
/**
 * @name interfaces
 * @description Classe Interfaces du plugin "switch" du projet ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.00
 * @date 25/04/2013
 */
include_once SNMP_CLASS;
class interfaces
{
	//valeur DB
	protected $id;
	protected $description;
	protected $plug;
	protected $critical;
	protected $userLastChange;
	protected $timeLastChange;
	
	protected $ifIndex;
	protected $ifDescr;
	protected $ifType;
	protected $ifMtu;
	protected $ifSpeed;  //ifSpeed ou ifHighSpeed
	protected $ifPhysAddress; //
	protected $ifAdminStatus; //
	protected $ifOperStatus; //
	protected $ifLastChange; //
	//trafic en entrée
	protected $ifInOctets;	//Octet en entrée  ifInOctet  ifHCInOctets
	protected $ifInUcastPkts; //paquets unicast
	protected $ifInNUcastPkts; //paquets multicast et broadcast
	protected $ifInDiscards; //paquets qui ont été jeté, une raison possible est un manque de place dans le buffer
	protected $ifInErrors;  //paquets en erreur (ex: CRC)
	
	//trafic en sortie
	protected $ifOutOctets;	//Octet en sortie
	protected $ifOutUcastPkts; //paquets unicast
	protected $ifOutNUcastPkts; //paquets multicast et broadcast
	protected $ifOutDiscards; //paquets qui ont été jeté, une raison possible est un manque de place dans le buffer
	protected $ifOutErrors;  //paquets en erreur
	//IFX MIB
	protected $ifName;
	protected $ifAlias;
	protected $ifCounterDiscontinuityTime;
	
	protected $ip;
	protected $ipMask;
	protected $ipGateway;
	
	//
	protected $vlan;
	protected $mode;
	protected $reset;
	/*@NAME: __construct
	 *@IN: Un nombre X d'argument (2,6 ou 7)
	*@OUT: /
	*@TREAT: Constructeur par défaut de la classe Interfaces, il appelle au méthode __construct2,__construct6 ou __construct7 suivant le nombre de paramètre
	*@EXCEPTION: 	/
	*/
	public function __construct ()
	{
		$reset=0;
		$critical=0;
		$a = func_get_args();
		$i = func_num_args();
		if($i==2)
			call_user_func_array(array($this,'__construct2'),$a);
		elseif($i==6)
		call_user_func_array(array($this,'__construct6'),$a);
		elseif($i==7)
		call_user_func_array(array($this,'__construct7'),$a);
	}
	/*@NAME: __construct2
	 *@IN: $ip IP de l'interface, $snmp un object SnmpRequest (voir SNMP.CLASS.php)
	*@OUT: /
	*@TREAT: Constructeur prenant 2 paramètres.
	*@EXCEPTION: 	/
	*/
	public function __construct2 ($snmp,$ip)
	{
		$this->ip = $ip;
		$this->snmpReq = $snmp;
	}
	/*@NAME: __construct6
	 *@IN: $if ifindex de l'interface, $t le type d'interface, $name son nom, $snmp un object SnmpRequest (voir SNMP.CLASS.php), $man présent pour des raison de compatibilité. $ip ipd e l'interface ou du noeud relié.
	*@OUT: /
	*@TREAT: Constructeur prenant 6 paramètres.
	*@EXCEPTION: 	/
	*/
	public function __construct6 ($if,$t,$name,$snmp,$man,$ip)
	{
		$this->ifindex = $if;
		$this->type = $t;
		$this->ifName = $name;
		//$this->manufacturer = $man;
		$this->ip = $ip;
		$this->snmpReq = $snmp;
		// echo "<br />".$this->name."|".$this->ifindex."|".$this->snmpReq->getVersion()."|".$this->snmpReq->getCommunity();
	}
	/*@NAME: __construct7
	 *@IN: $if ifindex de l'interface, $t le type d'interface, $name son nom, $ver versionn SNMP (voir SNMP.CLASS.php),$com la communauté SNMP $man présent pour des raison de compatibilité. $ip ipd e l'interface ou du noeud relié.
	*@OUT: /
	*@TREAT: Constructeur prenant 7 paramètres.
	*@EXCEPTION: 	/
	*/
	public function __construct7 ($if,$t,$name,$ver,$com,$man,$ip)
	{
		$this->ifindex = $if;
		$this->type = $t;
		$this->ifName = $name;
		//$this->manufacturer = $man;
		$this->ip = $ip;

		$this->snmpReq = new SnmpRequest();
		switch($ver)
		{
			case 1 : $this->snmpReq->setVersion(SnmpRequest::VERSION_1); break;
			case 2 : $this->snmpReq->setVersion(SnmpRequest::VERSION_2C); break;
			case 3 : $this->snmpReq->setVersion(SnmpRequest::VERSION_3); break;
			default: break;
		}
		$this->snmpReq->setCommunity($com);
		// echo "<br />".$this->name."|".$this->ifindex."|".$this->snmpReq->getVersion()."|".$this->snmpReq->getCommunity();
	}
	/*@NAME: setSNMP3Parameter
	 *@IN: $user utilisateur (snmpv3, $sec_lvl nivau de sécurité (SnmpRequest::AUTHNOPRIV,SnmpRequest::AUTHPRIV), $authpro méthode de hashage pour l'authentification  (SnmpRequest::MD5,SnmpRequest::SHA) $authkey clé d'authentification ,
	 *@OUT: /
	 *@TREAT:set les valeurs pour une requête SNMP v3
	 *@EXCEPTION: 	/
	 */
	public function setSNMP3Parameter($user,$sec_lvl,$authprot,$authkey)
	{
		$this->snmpReq->setUser($user);
		if($sec_lvl=='authNoPriv')
			$this->snmpReq->setSecurityLevel(SnmpRequest::AUTHNOPRIV);
		elseif($sec_lvl=='authPriv')
		$this->snmpReq->setSecurityLevel(SnmpRequest::AUTHPRIV);
		if($authprot=='md5')
			$this->snmpReq->setAuthentificationProtocol(SnmpRequest::MD5);
		elseif($authprot=='sha')
		$this->snmpReq->setAuthentificationProtocol(SnmpRequest::SHA);
		$this->snmpReq->setAuthentificationKey($authkey);
	}
	/*@NAME: Methode magique __get()
	 *@IN: (string) $property nom de la propriété 
	*@OUT: la valeur de la propriété demandé
	*@TREAT:set l'id de l'interface
	*@EXCEPTION: 	/
	*/
	public function __get($property) 
	{
			return $this->$property;
	}
	/*@NAME: Methode magique __set()
	 *@IN: (string) $property nom de la propriété (string) $value valeur de la clé
	*@OUT: /
	*@TREAT:set la propriété avec la valeur demandée
	*@EXCEPTION: 	/
	*/	
	public function __set($property,$value) 
	{
			$this->$property =  $value;
	}
	/*@NAME: snmpGet
	 *@IN: $oid, l'oid à requeter
	*@OUT: retourne le résultat de la requête
	*@TREAT: effectue un SNMP "get" 
	*@EXCEPTION: 	interfaceSNMPException si une erreur SNMP survient.
	*/
	public function snmpGet($oid)
	{
		try
		{
		 return $this->snmpReq->snmpget($this->ip,$oid);
		}
		catch(SnmpException $e)
		{
			throw new interfaceSNMPException($e->getMessage(),0,$e);
		}
	}
	/*@NAME: getIfNameBySNMP
	 *@IN: /
	*@OUT: retourne une chaine de caratère, contenant la description de l'interface
	*@TREAT: Effectue un requete SNMP sur l'OID .1.3.6.1.2.1.2.2.1.2.<ifindex> (IF-MIB) et effectue un formatage de la chaine retournée (FormatString)

	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getIfNameBySNMP()
	{
		$oid = ".1.3.6.1.2.1.31.1.1.1.1.".$this->ifindex;
		$desc= $this->snmpGet($oid);
		$this->name = formatString($desc);
		$this->ifName = $this->name ;
		return $this->name;
	}
	/*@NAME: getIfAliasBySNMP
	 *@IN: /
	*@OUT: retourne une chaine de caratère, contenant l'alias de l'interface
	*@TREAT: Effectue un requete SNMP sur l'OID .1.3.6.1.2.1.31.1.1.1.18.<ifindex> (IF-MIB) et effectue un formatage de la chaine retournée (FormatString)
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getIfAliasBySNMP()
	{
		$oid = ".1.3.6.1.2.1.31.1.1.1.18.".$this->ifindex;
		$desc= $this->snmpGet($oid);
		$this->ifAlias =  formatString($desc);
		return $this->ifAlias;
	}
	/*@NAME: getTypeBySNMP
	 *@IN: /
	*@OUT: Un nombre correspondant au type d'interface. un tableau peut-être obtenu ici
	http://tools.cisco.com/Support/SNMP/do/BrowseOID.do?objectInput=1.3.6.1.2.1.2.2.1.3&translate=Translate&submitValue=SUBMIT&submitClicked=true
	*@TREAT: Effectue un requete SNMP sur l'OID .1.3.6.1.2.1.2.2.1.3.<ifindex> (ifType de la IF-MIB) et effectue un formatage de la chaine retournée (FormatString) ainsi que set la propriété "description"
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getTypeBySNMP()
	{
		$oid = "1.3.6.1.2.1.2.2.1.3.".$this->ifindex;
		$desc= $this->snmpGet($oid);
		$this->ifType = formatInteger($desc);
		return $this->ifType;
	}
	/*@NAME: getDescriptionBySNMP
	 *@IN: /
	*@OUT: retourne une chaine de caratère, contenant la description de l'interface
	*@TREAT: Effectue un requete SNMP sur l'OID .1.3.6.1.2.1.2.2.1.2.<ifindex> (IF-MIB) et effectue un formatage de la chaine retournée (FormatString)
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getDescriptionBySNMP()
	{
		$oid = ".1.3.6.1.2.1.2.2.1.2.".$this->ifindex;
		$desc= $this->snmpGet($oid);
		$this->description = formatString($desc);
		$this->ifDescr = $this->description;
		return $this->description;
	}
	/*@NAME: getModeBySNMP

	 *@IN: /
	*@OUT: une chaine de caractère
	*@TREAT: DOIT ÊTRE REDEFINIE dans les classes filles. Retourne le mode de fonctionnement de l'interface mais c'est défini par le fabricant
	*@EXCEPTION: interfaceUnsuportedException, si l'interface n'est pas supportée
	*/
	public function getModeBySNMP()
	{
		throw new interfaceUnsuportedException('switch unsuported');
	}
	/*@NAME: getVlanBySNMP
	 *@IN: /
	*@OUT: Le vlanID d'une interface
	*@TREAT: DOIT ÊTRE REDEFINIE dans les classes filles. Retourne le vlan id de l'interface mais c'est défini par le fabricant
	*@EXCEPTION: interfaceUnsuportedException, si l'interface n'est pas supportée
	*/
	public function getVlanBySNMP()
	{
		throw new interfaceUnsuportedException('switch unsuported');
	}
	/*@NAME: getVoiceVlanBySNMP
	 *@IN: /
	*@OUT: Le ID du voiceVlan d'une interface
	*@TREAT: DOIT ÊTRE REDEFINIE dans les classes filles. Retourne l'id du voicevlan de l'interface mais c'est défini par le fabricant
	*@EXCEPTION: interfaceUnsuportedException, si l'interface n'est pas supportée
	*/
	public function getVoiceVlanBySNMP()
	{
		throw new interfaceUnsuportedException('switch unsuported');
	}
	/*@NAME: getIfTrapLinkUpDownEnable
	 *@IN: /
	*@OUT: retourne 1(enable) ou 2 (disabled)
	*@TREAT: Effectue un requete SNMP sur l'OID .1.3.6.1.2.1.31.1.1.1.14.<ifindex> (IF-MIB) pour vérifier si l'envoit des traps LinkUp/Down est activé sur l'interface
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getIfTrapLinkUpDownEnable()
	{
		$oid =".1.3.6.1.2.1.31.1.1.1.14.".$this->ifindex;
		$a = $this->snmpGet($oid);
		return formatInteger ($a);
	}
	/*@NAME: getIfMacNotificationEnable
	 *@IN: /
	*@OUT: retourne 1(enable) ou 2 (disabled)
	*@TREAT: DOIT ÊTRE REDEFINIE dans les classes filles. Vérifie si l'envoit des traps "MacNotification) est activée ou non sur l'interface
	*@EXCEPTION: interfaceUnsuportedException, si l'interface n'est pas supportée
	*/
	public function getIfMacNotificationEnable()
	{
		throw new interfaceUnsuportedException('switch unsuported');
	}
	/*@NAME: getOperStatusBySNMP
	 *@IN: /
	*@OUT: retourne 1(up),2 (down),3 (testing), 4 (unknown), 5 (dormant), 6(notPresent), 7(lowerLayerDown)
	*@TREAT: Effectue un requete SNMP sur l'OID .1.3.6.1.2.1.2.2.1.8.<ifindex> (IF-MIB) et retourne le résultat. L'OperStatus est le status réel de l'interface.
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getOperStatusBySNMP()
	{
		//.iso.org.dod.internet.mgmt.mib-2.interfaces.ifTable.ifEntry.ifOperStatus
		$oid = ".1.3.6.1.2.1.2.2.1.8.".$this->ifindex;
		$this->ifOperStatus =  formatInteger($this->snmpGet($oid));
		return $this->ifOperStatus;
	}
	/*@NAME: getAdminStatusBySNMP
	 *@IN: /
	*@OUT: retourne 1(up),2 (down),3 (testing)
	*@TREAT: Effectue un requete SNMP sur l'OID .1.3.6.1.2.1.2.2.1.7.<ifindex> (IF-MIB) et retourne le résultat. L'adminStatus est le status mis par l'administrateur sur une interface donnée.
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getAdminStatusBySNMP()
	{
		//.iso.org.dod.internet.mgmt.mib-2.interfaces.ifTable.ifEntry.ifAdminStatus
		$oid = ".1.3.6.1.2.1.2.2.1.7.".$this->ifindex;
		$this->ifAdminStatus = formatInteger($this->snmpGet($oid));
		return $this->ifAdminStatus;
	}
	/*@NAME: getPhysAddressBySNMP
	 *@IN: /
	*@OUT: la mac adresse de l'interface au format XX XX XX XX XX XX XX ou une chaine vide si l'interface n'a pas de mac adresse
	*@TREAT: Effectue un requete SNMP sur l'OID .1.3.6.1.2.1.2.2.1.6.<ifindex> (IF-MIB) et retourne le résultat. Recupérer la mac adresse de l'interface
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getPhysAddressBySNMP()
	{
		//.iso.org.dod.internet.mgmt.mib-2.interfaces.ifTable.ifEntry.ifPhysAddress
		$oid = ".1.3.6.1.2.1.2.2.1.6.".$this->ifindex;
		return formatHexString($this->snmpGet($oid));
	}
	/*@NAME: getSpeedSNMP
	 *@IN: /
	*@OUT: la vitesse d'une interface en bit/s
	*@TREAT: Effectue un requete SNMP et retourne la vitesse en bits/s
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getSpeedSNMP()
	{
		$speed=0;
		try
		{
			//.iso.org.dod.internet.mgmt.mib-2.interfaces.ifTable.ifEntry.ifHighSpeed
			$oid = ".1.3.6.1.2.1.31.1.1.1.15.".$this->ifindex; //si l'interface est récente
			$speed = formatGauge32($this->snmpGet($oid));
			$speed = $speed*1000000 ; //on passe des MB/s en des bit/s
		}
		catch(interfaceSNMPException $e)
		{
			//si l'interface est vielle et ne supporte pas l'oid ifHighSpeed
			$oid = ".1.3.6.1.2.1.2.2.1.5".$this->ifindex;
			$speed = formatGauge32($this->snmpGet($oid));

		}
		$this->ifSpeed = $speed;
		return $speed;
	}
	/*@NAME: getLastOutputBySNMP
	 *@IN: /
	*@OUT: la valeur du sysUptime(1.3.6.1.2.1.1.3) arrondis à la seconde, lors du changement d'état de l'interface
	*@TREAT: Effectue une requete SNMP sur l'OID 1.3.6.1.2.1.2.2.1.9.<ifindex> (ifLastChange de la IF-MIB)
	*        Si on souhaite connaitre le temp écoulé depuis le dernier changement d'état. Il faut Récupérer le sysUptime (1.3.6.1.2.1.1.3) du noeud et soustraire la valeur
	*	  du ifLastChange. Attention, la valeur est un TimeTick et pour des raisons de lisibilité, cette méthode arrondit à la seconde.
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getLastOutputBySNMP()
	{
		 //Lastoutput de chez Cisco
		//  $oid = ".1.3.6.1.4.1.9.2.2.1.1.4.".$this->ifindex;
		$oid = "1.3.6.1.2.1.2.2.1.9.".$this->ifindex;
		$lastout = $this->snmpGet($oid);
		
		return TimeTicksToSec($lastout);
	}
	/*@NAME: getOutErrorsBySNMP
	 *@IN: /
	*@OUT: retourne le nombre d'erreur en sortie d'une interface
	*@TREAT: Effectue un requete SNMP sur l'OID 1.3.6.1.2.1.2.2.1.20.<ifindex> (ifOutErrors de la IF-MIB) et retourne le nombre d'erreur en sortie
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getOutErrorsBySNMP()
	{
		//.iso.org.dod.internet.mgmt.mib-2.interfaces.ifTable.ifEntry.ifOutErrors
		$oid = "1.3.6.1.2.1.2.2.1.20.".$this->ifindex;
		$this->ifOutErrors =  formatCounter($this->snmpGet($oid));
		return $this->ifOutErrors;
	}
	/*@NAME: getInErrorsBySNMP
	 *@IN: /
	*@OUT: retourne le nombre d'erreur en sortie d'une interface
	*@TREAT: Effectue un requete SNMP sur l'OID 1.3.6.1.2.1.2.2.1.14.<ifindex> (ifInErrors de la IF-MIB) et retourne le nombre d'erreur en entrée
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getInErrorsBySNMP()
	{
		//.iso.org.dod.internet.mgmt.mib-2.interfaces.ifTable.ifEntry.ifInErrors
		$oid = "1.3.6.1.2.1.2.2.1.14.".$this->ifindex;
		$this->ifInErrors =  formatCounter($this->snmpGet($oid));
		return $this->ifInErrors;
	}

	/*@NAME: getOutOctetsBySNMP
	 *@IN: /
	*@OUT: retourne le nombre octet transmit
	*@TREAT: Effectue un requete SNMP sur l'OID 1.3.6.1.2.1.2.2.1.16.<ifindex> (ifOutOctets de la IF-MIB) t retourne le nombre d'octet transmit
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getOutOctetsBySNMP()
	{ 	
		$inoctet=0;
		try
		{
			//	ifHCOutOctets
			$oid = "1.3.6.1.2.1.31.1.1.1.10.".$this->ifindex;
			$inoctet = $this->snmpGet($oid);	
		}
		catch(interfaceSNMPException $e)
		{
			//si l'interface est vielle et ne supporte pas l'oid ifHCOutOctets
			//.iso.org.dod.internet.mgmt.mib-2.interfaces.ifTable.ifEntry.ifOutOctets
			$oid = "1.3.6.1.2.1.2.2.1.16.".$this->ifindex;
			$inoctet = $this->snmpGet($oid);
		
		}

		$this->ifOutOctets =  formatCounter($inoctet);
		return  $this->ifOutOctets;
	}
	/*@NAME: getInOctetsBySNMP
	 *@IN: /
	*@OUT: retourne le nombre d'octet reçut
	*@TREAT: Effectue un requete SNMP sur l'OID 1.3.6.1.2.1.2.2.1.10.<ifindex> (ifInOctets de la IF-MIB) et retourne le nombre d'octet reçut
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getInOctetsBySNMP()
	{
		$inoctet=0;
		try
		{
			//	ifHCInOctets
			$oid = "1.3.6.1.2.1.31.1.1.1.6.".$this->ifindex;
			$inoctet = $this->snmpGet($oid);
		}
		catch(interfaceSNMPException $e)
		{
				//.iso.org.dod.internet.mgmt.mib-2.interfaces.ifTable.ifEntry.ifInOctets
				//si l'interface est vielle et ne supporte pas l'oid ifHCInOctets
				$oid = ".1.3.6.1.2.1.2.2.1.10.".$this->ifindex;
				$inoctet = $this->snmpGet($oid);	
		}
		$this->ifInOctets =  formatCounter($inoctet);
		return $this->ifInOctets;
	}
	/*@NAME: getMTUBySNMP
	 *@IN: /
	*@OUT: retourne le MTU de l'interface
	*@TREAT: Effectue un requete SNMP sur l'OID 1.3.6.1.2.1.2.2.1.4.<ifindex> (ifMtu de la IF-MIB) et retourne la taille du MTU
	*@EXCEPTION: interfaceSNMPException si une erreur SNMP survient.
	*/
	public function getMTUBySNMP()
	{
		//.iso.org.dod.internet.mgmt.mib-2.interfaces.ifTable.ifEntry.ifInErrors
		$oid = "1.3.6.1.2.1.2.2.1.4.".$this->ifindex;
		$this->ifMtu = formatInteger($this->snmpGet($oid));
		return $this->ifMtu;
	}

	/*@DEPRECATED
	 *@NAME: CommandShow
	*@IN: /
	*@OUT: retourne le nom la commande pour avoir des informations
	*@TREAT: retourne le nom la commande pour avoir des informations
	*@EXCEPTION:
	*/
	public function CommandShow()
	{
		return 'show';
	}
}
class interfaceException extends Exception {}
class interfaceNoVLanException extends interfaceException {}
class interfaceNoModeException extends interfaceException {}
class interfaceNoVoiceVlanException extends interfaceException {}
class interfaceUnsuportedException extends interfaceException {}
class interfaceSNMPException extends interfaceException {}
?>