<?php
/**
 * @name navbar
 * @description Classe de gestion de la barre de navigation. Doit avoir accès au variable de session ainsi qu'a la classe MVCMODEL
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 16/11/2012
 */
include_once MVCMODEL;
class navbar
{
	private $_dbconnection;
	private $_name;
	private $_linkgrpnode;
	private $_icongrp;
	private $_lgroup;
	/*@NAME: __construct
	 *@IN: $dbconnection objet PDO de connexion à une base de donnée ESIA
	 *		$name nom de la variable de session
	*@OUT: /
	*@TREAT: Constructeur par défaut de la classe navbar.
	*@EXCEPTION: /
	*/
	public function __construct ($dbconnection,$name,$lgroup,$linkgrp,$icongrp)
	{
		$this->_name=$name;
		$_SESSION[$this->_name]=array();
		$this->_dbconnection = $dbconnection;
		$this->_linkgrpnode = $linkgrp;
		$this->_icongrp = $icongrp;
		$this->_lgroup=$lgroup;
	}
	/*@NAME: addElement
	 *@IN: $name, nom du lien dans la navbar
	*		$link URL d'accès
	*		$image (optionnelle) icon de la barre de navigation
	*@OUT: /
	*@TREAT: Ajoute un élément à la NAVBAR
	*@EXCEPTION: /
	*/
	public function addElement($name,$link,$image=null)
	{
		if(isset($image))
		{
			array_push($_SESSION[$this->_name],array('name'=>$name,'link'=>$link,'img'=>$image));
		}
		else
		{
			array_push($_SESSION[$this->_name],array('name'=>$name,'link'=>$link));
		}
	}
	/*@NAME: addNode
	 *@IN: $node objet device
	 *		$icon (optionnelle) icon de la barre de navigation
	 *@OUT: /
	 *@TREAT: Ajoute un élément à la NAVBAR
	 *@EXCEPTION: /
	 */
	public function addNode(Device $node,$icon=null)
	{		
		$this->addNodeGroups($node->getGroupId());
		$pagecontrol = Model_esia::getPageControl($this->_dbconnection,$node->getType());
		$pagecontrol = Model_esia::replacePageControlVariable($pagecontrol,(array) $node);
		$tmpid = $node->getGroupId();
		if(isset($tmpid)&&!empty($tmpid))
		{
			$tmplink = preg_replace('#:id#', $node->getGroupId(), $this->_linkgrpnode);
			$this->addElement($node->getGroup(), $tmplink,$this->_icongrp);
		}
		$this->addElement($node->getName(), $pagecontrol,$icon);
	}
	public function getNavbarGroupNodeOpen()
	{
	    $treeview_opengroups = array();
	    foreach($_SESSION[$this->_name] as $group)
	    {
	        if(preg_match('#node_index.php\?nodegroup\=([0-9]*)#', $group['link'],$regs))
	        {
	            $treeview_opengroups[]= $regs[1];
	        }
	    }
	    return $treeview_opengroups;
	}
	/*@NAME: addElement
	 *@IN: $nodegroup, id du groupe 
	*		$link URL d'accès (le :id est remplacé automatiquement par id du groupe
	*		$icons (optionnelle) icone de la barre de navigation
	*@OUT: /
	*@TREAT: Ajoute tous les groupes parents d'un groupe donnée à la barre de navigation
	*@EXCEPTION: /
	*/
	public function addNodeGroups($nodegroup)
	{
		$this->addElement($this->_lgroup,'/node_index.php',$this->_icongrp);
		$arr = $this->searchParentNode($nodegroup);
		//on commence par la fin du tableau afin de ne pas inverser l'ordre
		if($arr != null)
		{
			$i=(count($arr)-1);
			while($i>=0)
			{
				
				$tmplink = preg_replace('#:id#', $arr[$i]['id'], $this->_linkgrpnode);
				$this->addElement($arr[$i]['name'], $tmplink,$this->_icongrp);
				$i--;
			}
		}
	}
	
	private function searchParentNode($nodegroup)
	{
		$gr = Model_esia::getParrentGroup($this->_dbconnection,$nodegroup);
		if(!empty($gr))
		{
			$arr[]=array('id'=>$gr[0]['id'],'name'=>$gr[0]['name'] );
			$tmp = $this->searchParentNode($gr[0]['id']);
			if(isset($tmp))
				$arr = array_merge($arr, $tmp);
			return $arr;
		}
		return null;
	}

	
}
?>