<?php
/**
 * @name Telnet
 * @description Classe telnet du projet ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2010
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 31/10/2012
 */
class Telnet
{
	protected $fp;
	protected $usleep;
	/*@NAME: __construct
	 *@IN: $host: adresse IP ou nom de l'hôte
	$port: port de connexion par défaut 23
	$time: timeout en milliseconde 
	*@OUT: /
	*@TREAT: Constructeur par défaut de la classe Telnet, initialise la connexion.
	*@EXCEPTION: 	TelnetUnableToConnectException si une erreur survient
	*/
	public function __construct ($host,$port=23,$time=1)
	{
		if(!is_numeric($port))
			throw new TelnetUnableToConnectException("Port is not numeric");
		if(!is_numeric($time))
			throw new TelnetUnableToConnectException("stream timeout is not numeric");
		elseif($time<1)
			throw new TelnetUnableToConnectException("stream timeout < 1");
		$this->fp=@fsockopen($host,$port,$errno,$errstr);
		if($this->fp===false)
			throw new TelnetUnableToConnectException($errstr);
		$second = (int)$time/1000;
		$micro = $time*1000;
		stream_set_timeout($this->fp,$second,$micro);
		$this->usleep=10000;
		# sending the Telnet header
		fputs($this->fp,chr(0xFF).chr(0xFB).chr(0x1F).chr(0xFF).chr(0xFB).
				chr(0x20).chr(0xFF).chr(0xFB).chr(0x18).chr(0xFF).chr(0xFB).
				chr(0x27).chr(0xFF).chr(0xFD).chr(0x01).chr(0xFF).chr(0xFB).
				chr(0x03).chr(0xFF).chr(0xFD).chr(0x03).chr(0xFF).chr(0xFC).
				chr(0x23).chr(0xFF).chr(0xFC).chr(0x24).chr(0xFF).chr(0xFA).
				chr(0x1F).chr(0x00).chr(0x50).chr(0x00).chr(0x18).chr(0xFF).
				chr(0xF0).chr(0xFF).chr(0xFA).chr(0x20).chr(0x00).chr(0x33).
				chr(0x38).chr(0x34).chr(0x30).chr(0x30).chr(0x2C).chr(0x33).
				chr(0x38).chr(0x34).chr(0x30).chr(0x30).chr(0xFF).chr(0xF0).
				chr(0xFF).chr(0xFA).chr(0x27).chr(0x00).chr(0xFF).chr(0xF0).
				chr(0xFF).chr(0xFA).chr(0x18).chr(0x00).chr(0x58).chr(0x54).
				chr(0x45).chr(0x52).chr(0x4D).chr(0xFF).chr(0xF0));
		usleep($this->usleep);

		fputs($this->fp,chr(0xFF).chr(0xFC).chr(0x01).chr(0xFF).chr(0xFC).
				chr(0x22).chr(0xFF).chr(0xFE).chr(0x05).chr(0xFF).chr(0xFC).chr(0x21));
		usleep($this->usleep);
	}
	/*@NAME: login
	 *@IN: $log: login de l'utilisateur
	$pwd: mot de passe de l'utilisateur
	*@OUT: /
	*@TREAT: Entre le login et le mots de passe de l'utilisateur
	*@EXCEPTION: TelnetLoginException si une erreur survient
	*/
	public function login($log, $pwd)
	{
		fputs($this->fp,"$log\r");
		usleep($this->usleep);
		fputs($this->fp,"$pwd\r");
		usleep($this->usleep);
		usleep($this->usleep);
		do
		{
			$output.=fread($this->fp, 80);
			// read line by line, or at least small chunks
			$stat=socket_get_status($this->fp);
		}
		while($stat["unread_bytes"]);
		//check login
		$this->checkLogin($output);
	}
	/*@NAME: cmd
	 *@IN: $cmd: la commande a uploader sur l'hôte
	*@OUT: retourne le résultat de la commande
	*@TREAT: upload une commande sur l'hôte
	*@EXCEPTION: /
	*/
	public function cmd ($cmd)
	{
		fputs($this->fp,"$cmd\r");
		usleep($this->usleep);
		do
		{
			$buff=fgets($this->fp);
			// read line by line, or at least small chunks
			$stat=socket_get_status($this->fp);
			$output.=$buff;
		}
		while($stat["unread_bytes"]);
		$this->checkLogin($output);
		return $output;
	}
	/*@NAME: close
	 *@IN: /
	*@OUT: /
	*@TREAT: Ferme la socket de connexion
	*@EXCEPTION: /
	*/
	public function close()
	{
		fclose($this->fp);
	}
	/*@NAME: setTimer
	 *@IN: $t la durée du timer en microseconde
	*@OUT: /
	*@TREAT: set le timer
	*@EXCEPTION: /
	*/
	public function setTimer($t)
	{
		$this->usleep=$t;
	}
	protected function checkLogin($output)
	{
		if(stristr($output,'Login invalid'))
			throw new TelnetLoginException('Login failed');
		elseif(stristr($output,'Authentication failed'))
			throw new TelnetLoginException('Login failed');
		elseif(stristr($output,'incorrect password'))
			throw new TelnetLoginException('Login failed');
		elseif(stristr($output,'Password required, but none set'))
			throw new TelnetLoginException('Login failed');
	}
}
class TelnetException extends Exception {}
class TelnetUnableToConnectException extends TelnetException {}
class TelnetLoginException extends TelnetException {}
?>
