<?php
/**
 * @name User
 * @description Classe gérant les utilisateurs enregistré au sein d'Esia (droit d'accès,etc).
 * 		Il est nécessaire que la variable MVCMODEL contenant le chemin d'accès à la classe "Model_esia" soit définie!!!!!
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 06/11/2012
 */
include_once MVCMODEL;
class User {
	//Chaine de connexion à la base de donnée
	private $dbconnect;
	//tableau à 2 dimensions contenant les droits de chaque utilisateurs
	//$list[$i][0]='id du groupe';
	//$list[$i][1]='nom du groupe';
	//$list[$i][2]='nom du droit';
	//$list[$i][3]='1= accès autorisé, 0= refusé';
	//$list[$i][4]='si le droit se réfère à un noeud ou un droit sur l'application';
	private $listdroit;
	//id de l'utilisateur
	private $id;
	//nom de l'utilisateur
	private $name;
	//langue préférée
	private $lang;
	//email
	private $email;
	//mot de passe
	private $password;
	//id de session
	private $session_id;
	//timestamp de l'inscription (format Postgresql)
	private $inscription_time;
	//requête préparée pour les test d'accès
	private $query_access;
	//thème préféré
	private $theme;
	//Société de l'utilisateur n'existe que pour le mode SaaS
	private $company;
	//array pour l'indication de tuto
	private $tuto;
	//pour l'indication de la page d'accueil
	private $home_page;
	// tableau des identifiants des groups utilisateur
	private $listgroup;
	// Source de l'authentification
	private $src;
	/*@NAME: __construct
	 *@IN: $dbconnection chaine de connexion à la base de donnée (objet PDO)
	*@OUT: /
	*@TREAT: Constructeur par défaut de la classe User.
	*@EXCEPTION: 	/
	*/
	public function __construct($dbconnection)
	{
		$this->dbconnect =$dbconnection;
		$this->query_access =Model_esia::PrepareQueryCheckAccess($this->dbconnect);
		$this->listdroit = array(array());
	}
	/*@NAME: connect
	 *@IN: $username login de l'utilisateur
	* 	$pwd mot de passe de l'utilisateur
	*@OUT: /
	*@TREAT: Vérifie si l'utilisateur existe et si il est authentifié
	*@EXCEPTION: 	UserInvalidePasswordException, si l'utilisateur n'existe pas ou si le mot de passe est invalide
	*/
	public function connect($username,$pwd)
	{
		$_SESSION['session_id']['time']=time();
		$u =  Model_esia::InstanceUser($this->dbconnect,$username);
		$str = $username.$pwd.$u->getInscriptionTime();
		$str = sha1($str);
		if ($str==$u->getHashPassword())
		{
			$this->name=$u->getName();
			$this->lang = $u->getLang();
			$this->email= $u->getEmail();
			$this->id= $u->getId();
			$this->listdroit=$u->getAllAccess();
			$this->listgroup=$u->getAllGroupUser();
			$this->inscription_time=$u->getInscriptionTime();
			$this->password = $u->getHashPassword();
			$this->home_page = $u->getHomePage();
            $this->src = $u->getAuthSource();
		}
		else
		{
			throw new UserInvalidePasswordException("Invalide login/password");
		}
	}
	/*@NAME: generateSessionID
	 *@IN: /
	*@OUT: Un ID de session
	*@TREAT: Génère un id de session pour l'utilisateur, Il utilise les paramètre de session $_SESSION['session_id'] contenant un tableau associatif ayant pour clé
	*	id= id de la session
	*	name= nom de l'utilisateur
	* 	time= timestamp de la génération de l'id
	*@EXCEPTION: 	/
	*/
	public function generateSessionID()
	{
		if(isset($_SESSION['session_id']))
		{
			unset($_SESSION['session_id']);
		}
		$sesID = $this->name.time().$this->random(20);
		$sesID = sha1($sesID);
		$_SESSION['session_id']=array('id'=>$sesID,
				'name'=>$this->name,
				'time'=>time());
		return $sesID;
	}
	/*@NAME: generateHashPassword
	 *@IN: $user: login de l'utilisateur
	*     $password: mot de passe
	*     $time=timestamp de l'inscription de l'utilisateur
	*@OUT: un HASH SHA1 du login,mot de passe et de la date d'inscription
	*@TREAT: Génère un HASH SHA1.
	*@EXCEPTION: 	/
	*/
	public function generateHashPassword($user,$password,$time)
	{
		$str = $user.$password.$time;
		$this->password = sha1($str);
		return $this->password;
	}
	/*@NAME: sessionIDisValid
	*@IN: $during durée de la session (par défaut: 20 min)
	*@OUT: /
	*@TREAT: Vérifie si l'id de session est encore valide (durée par défaut 20min), si la session est valide, on instancie l'utilisateur correspondant
	*@EXCEPTION: 	UserInvalideSessionException si la session n'est plus valide
	*/
	public function sessionIDisValid($during = 20)
	{
		if(isset($_SESSION['session_id'])&&!empty($_SESSION['session_id']))
		{
			if(isset($_SESSION['session_id']['name'])&& !empty($_SESSION['session_id']['name']) ) 
			{
				if($_SESSION['session_id']['time']<(time()-($during*60)))
				{
					unset($_SESSION['session_id']);
					throw new UserInvalideSessionException('Session invalid: session during '.$during.' min');
				}
				$_SESSION['session_id']['time']=time();
			
				$u =  Model_esia::InstanceUser($this->dbconnect,$_SESSION['session_id']['name']);
				$this->name=$u->getName();
				$this->lang = $u->getLang();
				$this->email= $u->getEmail();
				$this->id= $u->getId();
				$this->listdroit=$u->getAllAccess();
				$this->listgroup=$u->getAllGroupUser();
				$this->inscription_time=$u->getInscriptionTime();
				$this->theme = $u->getTheme();
				$this->tuto = $u->getTutorial();
				$this->home_page= $u->getHomePage();
                $this->src = $u->getAuthSource();
			}
			else
			{
				if($_SERVER['REQUEST_URI']!="/login.php")
				{
					unset($_SESSION['session_id']);
					throw new UserInvalideSessionException('Session unset not name');
				}
			}
		}
		else
		{
			if($_SERVER['REQUEST_URI']!="/login.php")
			{
				unset($_SESSION['session_id']);
				throw new UserInvalideSessionException('Session unset empty');
			}
		}
	}
	/*@NAME: disconnect
	 *@IN: /
	*@OUT: /
	*@TREAT: Déconnecte l'utilisateur en supprimant l'id de session
	*@EXCEPTION: /
	*/
	public function disconnect()
	{
		if(isset($_SESSION['session_id'])&&!empty($_SESSION['session_id']))
		{
			unset($_SESSION['session_id']);
		}
	}
	/*@NAME: checkEmail
	 *@IN: $adresse adresse mail à tester
	*@OUT: TRUE si l'adresse mail est valide sinon false
	*@TREAT: Test si une adresse mail à un format valide
	*@EXCEPTION: /
	*/
	public function checkEmail($adresse)
	{
		$Syntaxe='#^[\w.-]+@[\w.-]+\.[a-zA-Z]{2,6}$#';
		if(preg_match($Syntaxe,$adresse))
		{
			$this->email=$adresse;
			return true;
		}
		else
		{
			return false;
		}
		return true;

	}
	/*@NAME: generatePassword
	 *@IN: $car nombre de caractère composant le mot de passe
	*@OUT: le mot de passe généré
	*@TREAT: Génère un mot de passe aléatoire, de X caractère
	*@EXCEPTION: /
	*/
	public function generatePassword($car)
	{
		$string = "";
		$chaine = "abcdefghijklmnpqrstuvwxyABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789*/#";
		for($i=0; $i<$car; $i++)
		{
			$string .= $chaine[rand()%strlen($chaine)];
		}
		return $string;

	}
	/*@NAME: setTimeConnection
	 *@IN: $time l'heure de connexion
	*@OUT: /
	*@TREAT: set l'heure de connexion
	*@EXCEPTION: /
	*/
	public function setTimeConnection($time)
	{
		$this->heureconnection = $time;
	}
	/*@NAME: setSessionDuration
	 *@IN: $time la durée maximun d'inactivité
	*@OUT: /
	*@TREAT: set la durée maximun d'inactivité
	*@EXCEPTION: /
	*/
	public function setSessionDuration($time)
	{
		$this->dureeconnection = $time;
	}
	/*@NAME: getDroits
	 *@IN: $dbconnection: chaine de connexion à la base de donnée
	*	$droitname= nom du droit à tester
	*	$node(optionnel) nom du noeud au se rapporte le droit
	*@OUT: 1 si l'utilisateur à bien l'autorisation d'accès
	*	0 si non précisé
	*	-1 si accès refusé
	*@TREAT: test si l'utilisateur à les droit d'accès passé en paramètre
	*@EXCEPTION: /
	*/
	public function getDroits($dbconnection,$droitname,$node='')
	{
		$test=0;
		foreach($this->listdroit as $ligne)
		{
			if($ligne[2]==$droitname )
			{
				if($ligne[3]==1)
				{
					if($node==''&&$ligne[4]==1)
						$test=1;
					else
						$test=2;
				}
				else
				{
					return -1;
				}
			}
		}
		if($test==0)
		{
			return 0;
		}
		if($node!='')
		{
		   // print_r($this->listgroup);
			$test=0;
			$this->query_access->execute(array($node));
			while($row=$this->query_access->fetch(PDO::FETCH_OBJ))
			{
			    if(in_array($row->access_group_id,$this->listgroup))
				{
					$test=1;
					break;
				}
			}
		}
		return $test;
	}
	/*@NAME: getAccess alias de le fonction getDroits
	 *@IN: $dbconnection: chaine de connexion à la base de donnée
	*	$droitname= nom du droit à tester
	*	$node(optionnel) nom du noeud au se rapporte le droit
	*@OUT: 1 si l'utilisateur à bien l'autorisation d'accès
	*	0 si non précisé
	*	-1 si accès refusé
	*@TREAT: test si l'utilisateur à les droit d'accès passé en paramètre
	*@EXCEPTION: /
	*/
	public function getAccess($dbconnection,$droitname,$node='')
	{
		if($node=='')
		{
			return $this->getDroits($dbconnection,$droitname);
		}
		else
		{
			return $this->getDroits($dbconnection,$droitname,$node);
		}
	}	
	
	/*@NAME: random
	 *@IN: $car nom de caractère
	*@OUT: la chaine générée
	*@TREAT: génère une chaine aléatoire
	*@EXCEPTION: /
	*/
	public function random($car)
	{
		$string = "";
		$chaine = "abcdefghijklmnpqrstuvwxyABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
		for($i=0; $i<$car; $i++)
		{
			$string .= $chaine[rand()%strlen($chaine)];
		}
		return $string;
	}

	
	//
	//Getter et setter de la classe
	//
	public function setName($u)
	{
		$this->name=$u;
	}
	public function setHashPassword($u)
	{
		$this->password=$u;
	}
	public function setId($u)
	{
		$this->id=$u;
	}
	public function setLang($u)
	{
		$this->lang=$u;
	}
	public function setEmail($u)
	{
		$this->email=$u;
	}
	public function setAccess($u)
	{
		$this->listdroit=$u;
	}
	public function setGroupUser($u)
	{
		$this->listgroup=$u;
	}
	public function setInscriptionTime($u)
	{
		$this->inscription_time=$u;
	}
	public function setTheme($u)
	{
		$this->theme=$u;
	}
	public function setCompany($u)
	{
		$this->company=$u;
	}
	public function setHomePage($u)
	{
		$this->home_page=$u;
	}
	public function setAuthSource($u)
	{
		$this->src=$u;
	}
	/**
	 * @brief set les tuto vu/pas vu
	 * @param array $tu un tableau de tuto
	 */
	public function setTutorial($tu)
	{
		$this->tuto = $tu;
	}
	public function getName()
	{
		return $this->name;
	}
	public function getId()
	{
		return $this->id;
	}
	public function getLang()
	{
		return $this->lang;
	}
	public function getEmail()
	{
		return $this->email;
	}
	public function getHomePage()
	{
		return $this->home_page;
	}
	public function getAllAccess()
	{
		return $this->listdroit;
	}
	public function getAllGroupUser()
	{
		return $this->listgroup;
	}
	public function getInscriptionTime()
	{
		return $this->inscription_time;
	}
	public function getHashPassword()
	{
		return $this->password;
	}
	public function getTheme()
	{
		return $this->theme;
	}
	public function getCompany()
	{
		return $this->company;
	}
	public function isCompanyPartner()
	{
		return $this->company->isPartner;
	}
	public function getTutorial() 
	{
		return $this->tuto;
	}
	public function getAuthSource()
	{
		return $this->src;
	}
	public function isFinishingTutoriel($id)
	{
		if(isset($this->tuto[$id]))
		{
			if($this->tuto[$id] == 1)
			{
				return true;
			}
			else 
			{
				return false;
			}
		}
		else 
		{
			return false;
		}
	}
	public function finishingTutoriel($id)
	{
		$this->tuto[$id] = 1;
	}
	public function resetTutoriel()
	{
		$this->tuto = array(0);
	}
}
class UserException extends Exception{}
class UserInvalidePasswordException extends UserException{}
class UserInvalideSessionException extends UserException{}
?>
