<?php
/**
 * @name Model_esia
 * @description Classe modèle d'Esia
 *		Toutes les méthodes sont déclarée en 'static' ainsi on garde les Objets PDO pour les programmeurs extérieurs et pour faire des modifications, ils ne sont pas /
 *		obligé de connaitre la hierachie de classe d'ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 16/11/2012
 */


class Model_TableSort
{
	/**
	 * Vérifie que les paramètres filters, order, size et paging sont bien conforme au format attendu sinon une exception est levée
	 * @param array $params tableau associatif envoyé par le tablesorter (format: Array(
	 *		    [filters] => Array // numéros de colonne et le filtre appliqué
	 *		        (
	 *					[4] => critique
	 *			    ),
	 *			[order] => Array // numéros de colonne qui trie en ascendant ou descendant
	 *			        (
	 *			            [8] => 1
	 *			        )
	 *			[size] => 25, //Nombre d'élement afficher dans la page
	 *			[paging] => 0) //Numéros de page
	 * @throws EsiaDbException
	 * @retval la chaine de caractère à mettre dans le WHERE
	 */
	static function ts_checkParams($params)
	{
		if(!array_key_exists('filters', $params))
		{
			throw new EsiaDbException("TS check params filters array not found in \$params");
		}
		if(!array_key_exists('order', $params))
		{
			throw new EsiaDbException("TS check params order array not found in \$params");
		}
		if(!array_key_exists('size', $params))
		{
			throw new EsiaDbException("TS check params size not found in \$params");
		}
		else
		{
			if(!is_numeric($params['size']))
			{
				throw new EsiaDbException("TS check params size is not integer \$params");
			}
		}
		if(!array_key_exists('paging', $params))
		{
			throw new EsiaDbException("TS check params paging not found in \$params");
		}
		else
		{
			if(!is_numeric($params['paging']))
			{
				throw new EsiaDbException("TS check params paging is not integer \$params");
			}
		}

	}
	/**
	 * Fait la correspondance entre le paramètre "cols" du tablesorter et les champs de la base de donnée afin de réaliser l'ORDER BY
	 * @param array $dataname Un tableau référençant la position de la colonne du tableSorter et le nom de la colonne en base de donnée
	 * @param array $cols le paramètre cols envoyé par le tableSorter en GET
	 * @throws EsiaDbException
	 * @retval la chaine de caractère à mettre dans l'ORDER BY
	 */
	static function ts_orderAllCols($dataname,$cols)
	{
		$sql="";
		foreach($cols as $key => $col)
		{
			if(array_key_exists($key, $dataname))
			{
				if($dataname[$key]['type'] == 'date')
				{
					$sql.=$dataname[$key]['data'];
				}
				else
				{
					$sql.=$dataname[$key]['col'];
				}
				if($col == 1)
				{
					$sql.=" ASC,";
				}
				else
				{
					$sql.=" DESC,";
				}
			}
			else
			{
				throw new EsiaDbException("key ($key) not found in \$dataname");
			}
		}
		$sql = rtrim($sql,',');
		return $sql;
	}
	/**
	 * Gère la pagination et retourne les donnée pour le LIMIT / OFFSET SQL
	 * @param integer $size nombre de donnée à afficher
	 * @param integer $page numéros de la page
	 * @throws EsiaDbException
	 * @retval la chaine de caractère à mettre après l'ORDER BY (LIMIT / OFFSET)
	 */
	static function ts_paging($size,$page)
	{
		$sql="";
		if(is_numeric($size) && is_numeric($page))
		{
			$sql='LIMIT '.$size.' OFFSET '.($page*$size);
			//echo "OFFSET ".$page*$size."<br />";
		}
		return $sql;
	}
	/**
	 * Fait la correspondance entre le paramètre "cols" du tablesorter et les champs de la base de donnée afin de réaliser l'ORDER BY
	 * @param array $dataname Un tableau référençant la position de la colonne du tableSorter et le nom de la colonne en base de donnée
	 * @param array $filters le filtre à appliquer
	 * @throws EsiaDbException
	 * @retval array array('sql'=>$sql,'params'=>$params); $sql le sql à ajouter à la requête et $params les paramètres à ajouter au tableau PDO de l'executeStatement
	 */
	static function ts_orderAllFilter($dataname,$filters)
	{
		$sql="";
		$params = array();
		foreach($filters as $key => $filter)
		{
			if(array_key_exists($key, $dataname))
			{
				$colsname = $dataname[$key]['col'];
				$fil = array($filter);
				$flag_or = FALSE;
				if(strstr($filter, '||'))
				{
					$fil = explode('||', $filter);
					$flag_or = TRUE;
				}
				elseif(strstr($filter, '&&'))
				{
					$fil = explode('&&', $filter);
				}
				$sub_sql="";
				$i = 0;
				foreach ($fil as $finalFilter)
				{
					$flag_not=FALSE;
					if(preg_match('#^\!.*#',$finalFilter))
					{
						$finalFilter = preg_replace('#\!#','',$finalFilter);
						$flag_not=TRUE;
					}

					//protège d'une erreur 500 si on a envoyé que un '!'
					if(!empty($finalFilter))
					{
						if( $dataname[$key]['type'] == 'string' || $dataname[$key]['type'] == 'date')
						{
							$paramname = preg_replace('#\.#','',$colsname);
							$paramname.=$i;
							$i++;
							if($flag_not)
							{
								$sub_sql.= ' '.$colsname." NOT ILIKE :".$paramname;
							}
							else
							{
								$sub_sql.= ' '.$colsname." ILIKE :".$paramname;
							}

							$params[':'.$paramname]='%'.$finalFilter.'%';
						}
						elseif( $dataname[$key]['type'] == 'array')
						{
							$ret = Model_TableSort::ts_orderArrayFilter($dataname[$key],$colsname,$finalFilter,$flag_not,$params,$i);
							$sub_sql.=$ret;
						}
						elseif( $dataname[$key]['type'] == 'arrayAndString')
						{
							$ret = Model_TableSort::ts_orderArrayFilter($dataname[$key],$colsname,$finalFilter,$flag_not,$params,$i);
							$sql_string="";
							if($flag_not)
							{
								$sql_string.= ' '.$colsname." NOT ILIKE :".$colsname.$i;
							}
							else
							{
								$sql_string.= ' '.$colsname." ILIKE :".$colsname.$i;
							}
							$params[':'.$colsname.$i]='%'.$finalFilter.'%';
							$i++;
							//echo '('.$ret.' OR '.$sql_string.')';
							if($flag_not)
							{
								$sub_sql.='('.$ret.' AND '.$sql_string.')' ;
							}
							else
							{
								$sub_sql.='('.$ret.' OR '.$sql_string.')' ;
							}
						}
						if($flag_or)
						{
							$sub_sql.=" OR";
						}
						else
						{
							$sub_sql.=" AND";
						}
					}
				}
				$sub_sql = rtrim($sub_sql,'OR');
				$sub_sql = rtrim($sub_sql,'AND');
				if(!empty($sub_sql))
				{
					$sql.=" ($sub_sql) ";
				}
			}
			else
			{
				throw new EsiaDbException("key ($key) not found in \$dataname");
			}
			$sql.=" AND";
		}
		$sql = rtrim($sql,'AND');
		$sql = trim($sql);
		return array('sql'=>$sql,'params'=>$params);
	}
	/**
	 * Execute la requête SQL demandée afin de retourner le nombre de ligne
	 * @param PDO $dbconnection objet de connexion à la base de donnée
	 * @param string $sql la requête sql a exécuter sans le SELECT. On commence le SQL avec le from
	 * @param array $params le tableau des paramètres
	 * @throws EsiaDbException
	 * @retval
	 */
	private static function ts_getRowsNumber($dbconnection,$sql,$params,$begin="")
	{
		$total = 0;
		$queryCount = $dbconnection->prepare($begin.' SELECT count(*) as nbr '.$sql);
		$queryCount = Model_esia::executeStatement($queryCount,$params);
		$row = $queryCount->fetch(PDO::FETCH_OBJ);
		if(!empty($row))
		{
			$total = $row->nbr;
		}
		return $total;

	}

	/**
	 * Fait la correspondance entre le paramètre "cols" du tablesorter et les champs de la base de donnée afin de réaliser l'ORDER BY
	 * @param array $dataname Un tableau référençant la position de la colonne du tableSorter et le nom de la colonne en base de donnée
	 * @param string $cols le nom de la colonne dans la table
	 * @param string $finalFilter le filtre que l'utilisateur recherche
	 * @param boolean $flag_not on indique si la recherche est un not ou pas
	 * @param array &$params attention passage par référence, le tableau des paramètres est modifier dans la fonction pour ajouter les paramètres de $dataname['data']
	 * @param integer &$i  attention passage par référence, la valeur de $i est incrémentée en fonction du nombre d'insertion qui est faite dans le tableau de paramètre
	 * @throws EsiaDbException
	 * @retval la chaine de caractère à mettre dans le WHERE
	 */
	protected static function ts_orderArrayFilter($dataname,$colsname,$finalFilter,$flag_not,&$params,&$i)
	{
		$sql_prepare_arr='';
		$flag_found = FALSE;
		$sub_sql="";
		foreach($dataname['data'] as $db => $value)
		{
			if(stripos($value,$finalFilter) !==FALSE)
			{
				$params[':'.$colsname.$i]=$db;
				$sql_prepare_arr.=':'.$colsname.$i.',';
				$i++;
				$flag_found = TRUE;
			}
		}
		$sql_prepare_arr= rtrim($sql_prepare_arr,',');
		if($flag_found == TRUE)
		{
			if($flag_not)
			{
				$sub_sql.= ' '.$colsname." NOT IN ($sql_prepare_arr)";
			}
			else
			{
				$sub_sql.= ' '.$colsname." IN ($sql_prepare_arr)";
			}
		}
		else
		{
			if($flag_not)
			{
				$sub_sql.= ' '.$colsname." NOT IN ('DBNOTDATA')";
			}
			else
			{
				$sub_sql.= ' '.$colsname." IN ('DBNOTDATA')";
			}
		}
		return $sub_sql;
	}
	/**
	 *  retourne le tableau de correspondance entre le paramètre "cols" du tablesorter et les champs de la base de donnée et le type de tri. Il retourne uniquemejnt les colonnes pour les erreurs active
	 * @param array $lang_error_type tableau associatif faisant la correspondance entre le nom technique des erreurs (error_type) et les variables de langue.
	 * 		format FR: Array ( [warning] => Alerte,    [critical] => Critique,    [unknown] => Inconnu,    [maintenance] => Maintenance)
	 * @param array $lang_plugin tableau associatif faisant la correspondance entre le nom technique des plugins et le nom traduit
	 * 		format FR: Array ( [CHECK_SNMP_LOAD] => Processeur)
	 * @retval array  Tableau de correspondance entre le SQL et le tableSorter
	 */
	private static function ts_getActiveErrors_columns($lang_error_type,$lang_plugin)
	{
		//l'index de la column doit être égale à l'index dans le table sorter
		$columns[0]=array('col'=>'device_name','type'=>'string');
		$columns[1]=array('col'=>'device_ip_address','type'=>'string');
		$columns[2]=array('col'=>'device_type','type'=>'string');
		$columns[3]=array('col'=>'device_description','type'=>'string');
		$columns[4]=array('col'=>'device_group','type'=>'string');
		$columns[5]=array('col'=>'error_type','type'=>'array','data'=>$lang_error_type);
		$columns[6]=array('col'=>'service_name','type'=>'arrayAndString','data'=>$lang_plugin);
		$columns[7]=array('col'=>'service_name','type'=>'string');
		$columns[8]=array('col'=>'service_last_stdout','type'=>'string');
		$columns[9]=array('col'=>'happen_time','type'=>'date','data'=>'happen_timestamp');
		return $columns;
	}
	/**
	 *  retourne le tableau de correspondance entre le paramètre "cols" du tablesorter et les champs de la base de donnée et le type de tri. Il retourne uniquemejnt les colonnes pour les erreurs active
	 * @param array $lang_error_type tableau associatif faisant la correspondance entre le nom technique des erreurs (error_type) et les variables de langue.
	 * 		format FR: Array ( [warning] => Alerte,    [critical] => Critique,    [unknown] => Inconnu,    [maintenance] => Maintenance)
	 * @param array $lang_plugin tableau associatif faisant la correspondance entre le nom technique des plugins et le nom traduit
	 * 		format FR: Array ( [CHECK_SNMP_LOAD] => Processeur)
	 * @retval array  Tableau de correspondance entre le SQL et le tableSorter
	 */
	private static function ts_getOKService_columns($lang_error_type,$lang_plugin)
	{
		//l'index de la column doit être égale à l'index dans le table sorter
		$columns[0]=array('col'=>'device_name','type'=>'string');
		$columns[1]=array('col'=>'device_ip_address','type'=>'string');
		$columns[2]=array('col'=>'device_type','type'=>'string');
		$columns[3]=array('col'=>'device_description','type'=>'string');
		$columns[4]=array('col'=>'device_group','type'=>'string');
		$columns[5]=array('col'=>'error_type','type'=>'string');
		$columns[6]=array('col'=>'service_name','type'=>'arrayAndString','data'=>$lang_plugin);
		$columns[7]=array('col'=>'service_name','type'=>'string');
		$columns[8]=array('col'=>'service_last_stdout','type'=>'string');
		$columns[9]=array('col'=>'happen_time','type'=>'date','data'=>'service_last_exec_time');
		return $columns;
	}
	/**
	 *  gère l'application des filtres, du tri et de la pagination pour les erreurs active. FONCTION A POINTEUR
	 * @param array $columns Tableau de correspondance entre le SQL et le tableSorter
	 * @param array $TSParams tableau associatif envoyé par le tablesorter (format: Array(
	 *		    [filters] => Array // numéros de colonne et le filtre appliqué
	 *		        (
	 *					[4] => critique
	 *			    ),
	 *			[order] => Array // numéros de colonne qui trie en ascendant ou descendant
	 *			        (
	 *			            [8] => 1
	 *			        )
	 *			[size] => 25, //Nombre d'élement afficher dans la page
	 *			[paging] => 0) //Numéros de page
	 * @param string &$sql POINTEUR modifie le SQL afin d'y ajouter les filtres
	 * @param array &$param POINTEUR modifie le tableau de correspondance PDO
	 * @param string &$sql_order POINTEUR modifie le SQL afin d'y ajouter le ORDER BY, LIMIT et OFFSET
	 * @throws EsiaDbException
	 * @retval boolean false si aucune donnée n'a été modifiée sinon true
	 */
	protected static function ts_getActiveErrors_filter($columns,$TSParams,&$sql,&$param,&$sql_order)
	{
		$flag_modified = false;
		if(!empty($TSParams))
		{
			Model_TableSort::ts_checkParams($TSParams);
			if(!empty($TSParams['filters']))
			{
				$tmp = Model_TableSort::ts_orderAllFilter($columns, $TSParams['filters'] );
				if(!empty($tmp['sql']))
				{
					$sql .= " WHERE ".$tmp['sql'];
				}
				$flag_modified = true;
				$param  = array_merge($param ,$tmp['params']);
			}
			if(!empty($TSParams['order']))
			{
				$sql_order = " ORDER BY ".Model_TableSort::ts_orderAllCols($columns, $TSParams['order'] );
				$flag_modified = true;
			}
			if(isset($TSParams['paging']))
			{
				$sql_order .= " ".Model_TableSort::ts_paging($TSParams['size'],$TSParams['paging'] );
				$flag_modified = true;
			}
		}
		return $flag_modified;
	}
	/**
	 *  gère le tri et de la pagination pour les erreurs active. FONCTION A POINTEUR
	 * @param array $columns Tableau de correspondance entre le SQL et le tableSorter
	 * @param array $TSParams tableau associatif envoyé par le tablesorter (format: Array(
	 *		    [filters] => Array // numéros de colonne et le filtre appliqué
	 *		        (
	 *					[4] => critique
	 *			    ),
	 *			[order] => Array // numéros de colonne qui trie en ascendant ou descendant
	 *			        (
	 *			            [8] => 1
	 *			        )
	 *			[size] => 25, //Nombre d'élement afficher dans la page
	 *			[paging] => 0) //Numéros de page
	 * @param string &$sql_order POINTEUR modifie le SQL afin d'y ajouter le ORDER BY, LIMIT et OFFSET
	 * @throws EsiaDbException
	 * @retval boolean false si aucune donnée n'a été modifiée sinon true
	 */
	private static function ts_paginationOrder($columns,$TSParams,&$sql_order)
	{
		$flag_modified = false;
		if(!empty($TSParams))
		{
			if(!empty($TSParams['order']))
			{
				$sql_order = " ORDER BY ".Model_TableSort::ts_orderAllCols($columns, $TSParams['order'] );
				$flag_modified = true;
			}
			if(!empty($TSParams['paging']))
			{
				$sql_order .= " ".Model_TableSort::ts_paging($TSParams['size'],$TSParams['paging'] );
				$flag_modified = true;
			}
		}
		return $flag_modified;
	}
	/**
	 * Format la sortie pour les fonctions ts_getActiveErrors et ts_getOKservice
	 * @param object $row un objet PDO ayant les paramètres suivants:
	 * 		'type_error' => $row->error_type,
	 'time' => $row->happen_time,
	 'name' => $row->device_name,
	 'id' => $row->device_id,
	 'ip_addr' => $row->device_ip_address,
	 'service_name' => $row->service_name,
	 'check_plugin_name' => $row->plugin_name,
	 'page_control' => $page_control,
	 'description' => $row->device_description,
	 'last_stdout' => $row->service_last_stdout
	 * @param array $nodetype = Model_esia::getWebPluginByNodeType($dbconnection,NULL,Model_esia::NODE_TYPE_FORMAT_TYPE_URL);
	 * @retval array 	array('type_error' => $row->error_type,
	 'time' => $row->happen_time,
	 'name' => $row->device_name,
	 'id' => $row->device_id,
	 'ip_addr' => $row->device_ip_address,
	 'service_name' => $row->service_name,
	 'check_plugin_name' => $row->plugin_name,
	 'page_control' => $page_control,
	 'description' => $row->device_description,
	 'last_stdout' => $row->service_last_stdout
	 */
	private function ts_getFormatRowError($row,$nodetype)
	{
		if(array_key_exists($row->device_type,$nodetype))
		{
			$page_control = Model_esia::replacePageControlVariable($nodetype[$row->device_type],$row);
		}
		else
		{
			$page_control = '/node_view.php?nodeid='.$row->device_id;
		}
		$currentError = array('type_error' => $row->error_type,
			'time' => $row->happen_time,
			'name' => $row->device_name,
			'id' => $row->device_id,
			'ip_addr' => $row->device_ip_address,
			'type' => $row->device_type,
			'device_group' => $row->device_group,
			'service_name' => $row->service_name,
			'check_plugin_name' => $row->plugin_name,
			'page_control' => $page_control,
			'description' => $row->device_description,
			'last_stdout' => $row->service_last_stdout

		);
		return $currentError;
	}
	/**
	 * Format la sortie pour les fonctions ts_getHistoricError et ts_getPendingAcquitteErrors
	 * @param object $row un objet PDO ayant les paramètres suivants:
			 'type_error' => $row->error_type,
			 'time' => $row->happen_time,
			 'name' => $row->device_name,
			 'id' => $row->device_id,
			 'ip_addr' => $row->device_ip_address,
			 'service_name' => $row->service_name,
			 'check_plugin_name' => $row->plugin_name,
			 'page_control' => $page_control,
			 'description' => $row->device_description,
			 'last_stdout' => $row->service_last_stdout,
			 'time_acquitte'=>$row->happen_acquitte
	 * @param array $nodetype = Model_esia::getWebPluginByNodeType($dbconnection,NULL,Model_esia::NODE_TYPE_FORMAT_TYPE_URL);
	 * @retval array 	'type_error' => $row->error_type,
						 'time' => $row->happen_time,
						 'name' => $row->device_name,
						 'id' => $row->device_id,
						 'ip_addr' => $row->device_ip_address,
						 'service_name' => $row->service_name,
						 'check_plugin_name' => $row->plugin_name,
						 'page_control' => $page_control,
						 'description' => $row->device_description,
						 'last_stdout' => $row->service_last_stdout,
						 'time_acquitte'=>$row->happen_acquitte
	 */
	private function ts_getFormatRowErrorHistoric($row,$nodetype)
	{
		if(array_key_exists($row->device_type,$nodetype))
		{
			$page_control = Model_esia::replacePageControlVariable($nodetype[$row->device_type],$row);
		}
		else
		{
			$page_control = '/node_view.php?nodeid='.$row->device_id;
		}
		$arr = array('type_error' => $row->error_type,
			'time' => $row->happen_time,
			'name' => $row->device_name,
			'type'=> $row->device_type,
			'id' => $row->device_id,
			'ip_addr' => $row->device_ip_address,
			'device_group' => $row->device_group,
			'service_name' => $row->service_name,
			'check_plugin_name' => $row->plugin_name,
			'page_control' => $page_control,
			'description' => $row->device_description,
			'last_stdout' => $row->service_last_stdout,
			'time_acquitte'=>$row->happen_acquitte
		);
		return $arr;
	}

	/**
	 * Récupère les erreurs en cours dans la base de donnée
	 * @param integer $user identifiant utilisateur
	 * @param array $TSParams tableau associatif envoyé par le tablesorter (format: Array(
	 *		    [filters] => Array // numéros de colonne et le filtre appliqué
	 *		        (
	 *					[4] => critique
	 *			    ),
	 *			[order] => Array // numéros de colonne qui trie en ascendant ou descendant
	 *			        (
	 *			            [8] => 1
	 *			        )
	 *			[size] => 25, //Nombre d'élement afficher dans la page
	 *			[paging] => 0) //Numéros de page
	 * @param array $lang_error_type tableau associatif faisant la correspondance entre le nom technique des erreurs (error_type) et les variables de langue.
	 * 		format FR: Array ( [warning] => Alerte,    [critical] => Critique,    [unknown] => Inconnu,    [maintenance] => Maintenance)
	 * @param array $lang_plugin tableau associatif faisant la correspondance entre le nom technique des plugins et le nom traduit
	 * 		format FR: Array ( [CHECK_SNMP_LOAD] => Processeur)
	 * @throws EsiaDbException
	 * @retval array array('total'=>$total,'rows'=>$arr); Total est le nombre d'erreur présente et le rows sont les données en tenant compte de la pagination
	 */
	static function ts_getActiveErrors($dbconnection,$user=NULL,$TSParams=NULL,$lang_error_type=NULL,$lang_plugin=NULL,$hiera =FALSE)
	{
		$columns = Model_TableSort::ts_getActiveErrors_columns($lang_error_type,$lang_plugin);
		$sql_begin="";
		$sql = " FROM (SELECT device_id,device_name,device_type, host(device_ip_address) as device_ip_address, device_description,
						error_type, service_name, service_last_stdout,
						to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') AS happen_time,plugin_name,happen_timestamp,device_group_path AS device_group
				FROM esia_services
				JOIN esia_device ON device_id = service_device_id
				JOIN esia_error_happen ON service_id = happen_service_id
				JOIN esia_plugins ON service_plugin_id = plugin_id
				JOIN esia_errors ON error_id = happen_error_id
				LEFT JOIN esia_device_group_in  ON group_in_device_id=device_id
				LEFT JOIN esia_device_group AS TGroupe ON group_in_device_group_id=device_group_id";
		$param = array();
		if(isset($user))
		{
			if($hiera !==FALSE)
			{
				$sql_begin="WITH servicePriority
				AS (
					SELECT service_device_id as devid, min(service_priority) as priority
						FROM public.esia_services
						JOIN esia_error_happen ON service_id = happen_service_id
						WHERE service_device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND happen_acquitte=0
						GROUP BY service_device_id
				)";
				$sql .= " JOIN servicePriority ON service_device_id = servicePriority.devid
						 WHERE service_priority=servicePriority.priority AND happen_acquitte=0";
			}
			else
			{
				$sql .= " WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND happen_acquitte=0";
			}
			$param[':user'] = $user;
		}
		else
		{
			if($hiera !==FALSE)
			{
				$sql_begin="WITH servicePriority
				AS (
					SELECT service_device_id as devid, min(service_priority) as priority
						FROM public.esia_services
						GROUP BY service_device_id
				)";
				$sql .= " JOIN servicePriority ON service_device_id = servicePriority.devid
						 WHERE service_priority=servicePriority.priority AND happen_acquitte=0";
			}
			else
			{
				$sql .= " WHERE happen_acquitte=0";
			}
		}
		$sql .= ") as tabletmp ";
		$sql_order = "";
		$flag = Model_TableSort::ts_getActiveErrors_filter($columns,$TSParams,$sql,$param,$sql_order);
		$total = Model_TableSort::ts_getRowsNumber($dbconnection,$sql,$param,$sql_begin);
		if($total < ($TSParams['size'] * $TSParams['paging']))
		{
			$TSParams['paging']=0;
			$sql_order = "";
			$flag = Model_TableSort::ts_paginationOrder($columns,$TSParams,$sql_order);
		}
		if($flag === FALSE)
		{
			//il n'y a pas de modification effectuée et donc, on met la valeur par défaut;
			$sql_order = " ORDER BY happen_timestamp DESC";
		}
		//echo $sql_begin.' SELECT * '.$sql.' '.$sql_order;
		  //           echo $sql_begin.' SELECT * '.$sql.' '.$sql_order;
                    //         print_r($param);

		$query = $dbconnection->prepare($sql_begin.' SELECT * '.$sql.' '.$sql_order);
		$query = Model_esia::executeStatement($query,$param);
		$nodetype = Model_esia::getWebPluginByNodeType($dbconnection,NULL,Model_esia::NODE_TYPE_FORMAT_TYPE_URL);
		$currentError = array();
		$listid = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!in_array($row->device_id,$listid))
			{
				$listid[]=$row->device_id;
			}
			$currentError[] = Model_TableSort::ts_getFormatRowError($row,$nodetype);
		}
		return array('total'=>$total,'rows'=>$currentError,'listid'=>$listid);
	}
	/**
	 * Récupère les services actuellement en OK dans la base de donnée
	 * @param integer $user identifiant utilisateur
	 * @param array $TSParams tableau associatif envoyé par le tablesorter (format: Array(
	 *		    [filters] => Array // numéros de colonne et le filtre appliqué
	 *		        (
	 *					[4] => critique
	 *			    ),
	 *			[order] => Array // numéros de colonne qui trie en ascendant ou descendant
	 *			        (
	 *			            [8] => 1
	 *			        )
	 *			[size] => 25, //Nombre d'élement afficher dans la page
	 *			[paging] => 0) //Numéros de page
	 * @param array $lang_error_type tableau associatif faisant la correspondance entre le nom technique des erreurs (error_type) et les variables de langue.
	 * 		format FR: Array ( [warning] => Alerte,    [critical] => Critique,    [unknown] => Inconnu,    [maintenance] => Maintenance)
	 * @param array $lang_plugin tableau associatif faisant la correspondance entre le nom technique des plugins et le nom traduit
	 * 		format FR: Array ( [CHECK_SNMP_LOAD] => Processeur)
	 * @throws EsiaDbException
	 * @retval array array('total'=>$total,'rows'=>$arr); Total est le nombre d'erreur présente et le rows sont les données en tenant compte de la pagination
	 */
	static function ts_getOKService($dbconnection,$user=NULL,$TSParams=NULL,$lang_error_type=NULL,$lang_plugin=NULL)
	{
		$columns = Model_TableSort::ts_getOKService_columns($lang_error_type,$lang_plugin);
		$sql = "FROM (SELECT device_id,device_name,device_type, host(device_ip_address) as device_ip_address, device_description,
						'OK' as error_type, service_name, service_last_stdout,device_group_path AS device_group,
						to_char(service_last_exec_time,'DD/MM/YYYY HH24:MI:SS') AS happen_time,plugin_name,service_last_exec_time
				FROM esia_device as t2
				JOIN esia_services ON service_device_id = device_id
				JOIN esia_plugins ON service_plugin_id = plugin_id
				LEFT JOIN esia_device_group_in  ON group_in_device_id=device_id
				LEFT JOIN esia_device_group AS TGroupe ON group_in_device_group_id=device_group_id";
		$param = array();
		if(isset($user))
		{
			$sql .= " WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND service_previous_state=0";
			$param[':user'] = $user;
		}
		else
		{
			$sql .= " WHERE service_previous_state=0";
		}
		$sql .= ") as tabletmp ";
		$sql_order = "";

		$flag = Model_TableSort::ts_getActiveErrors_filter($columns,$TSParams,$sql,$param,$sql_order);
		if($flag === FALSE)
		{
			//il n'y a pas de modification effectuée et donc, on met la valeur par défaut;
			$sql_order = " ORDER BY service_last_exec_time DESC";
		}
		$total = 0;
		$total = Model_TableSort::ts_getRowsNumber($dbconnection,$sql,$param);
		$query = $dbconnection->prepare('SELECT * '.$sql.' '.$sql_order);
		$query = Model_esia::executeStatement($query,$param);
		$nodetype = Model_esia::getWebPluginByNodeType($dbconnection,NULL,Model_esia::NODE_TYPE_FORMAT_TYPE_URL);
		$currentError = array();
		$listid = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!in_array($row->device_id,$listid))
			{
				$listid[]=$row->device_id;
			}
			$currentError[] = Model_TableSort::ts_getFormatRowError($row,$nodetype);
		}
		return array('total'=>$total,'rows'=>$currentError,'listid'=>$listid);
	}
	/**
	 * Récupère l'historique des erreurs dans la base de donnée
	 * @param integer $user identifiant utilisateur
	 * @param integer $time (default:1) nombre de jour à prendre en compte
	 * @param array $TSParams tableau associatif envoyé par le tablesorter (format: Array(
	 *		    [filters] => Array // numéros de colonne et le filtre appliqué
	 *		        (
	 *					[4] => critique
	 *			    ),
	 *			[order] => Array // numéros de colonne qui trie en ascendant ou descendant
	 *			        (
	 *			            [8] => 1
	 *			        )
	 *			[size] => 25, //Nombre d'élement afficher dans la page
	 *			[paging] => 0) //Numéros de page
	 * @param array $lang_error_type tableau associatif faisant la correspondance entre le nom technique des erreurs (error_type) et les variables de langue.
	 * 		format FR: Array ( [warning] => Alerte,    [critical] => Critique,    [unknown] => Inconnu,    [maintenance] => Maintenance)
	 * @param array $lang_plugin tableau associatif faisant la correspondance entre le nom technique des plugins et le nom traduit
	 * 		format FR: Array ( [CHECK_SNMP_LOAD] => Processeur)
	 * @throws EsiaDbException
	 * @retval array array('total'=>$total,'rows'=>$arr); Total est le nombre d'erreur présente et le rows sont les données en tenant compte de la pagination
	 */
	static function ts_getHistoricErrors($dbconnection,$user=NULL,$time=1, $TSParams=NULL,$lang_error_type=NULL,$lang_plugin=NULL)
	{
		//l'index de la column doit être égale à l'index dans le table sorter
		$columns = Model_TableSort::ts_getActiveErrors_columns($lang_error_type,$lang_plugin);
		$columns[9]=array('col'=>'happen_acquitte','type'=>'date','data'=>'happen_time_acquitte');

		$columns_for_select = "
			device_id,device_name,device_type, host(device_ip_address) as device_ip_address,
			device_description, error_type, service_name, service_last_stdout,plugin_name,
			to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') AS happen_time,happen_timestamp,
			to_char(happen_time_acquitte,'DD/MM/YYYY HH24:MI:SS') AS happen_acquitte,happen_time_acquitte,device_group_path AS device_group
		";

		$sql = "SELECT $columns_for_select
				FROM esia_device as t2
				JOIN esia_group_device_access ON device_id=access_device_id
				JOIN esia_groups ON access_group_id = group_id
				JOIN esia_memberof ON member_group_id = group_id
				JOIN esia_error_happen ON device_id = happen_device_id
				JOIN esia_services ON service_id = happen_service_id
				JOIN esia_plugins ON service_plugin_id = plugin_id
				JOIN esia_errors ON error_id = happen_error_id
				LEFT JOIN esia_device_group_in  ON group_in_device_id=device_id
				LEFT JOIN esia_device_group AS TGroupe ON group_in_device_group_id=device_group_id
				";

		$param = array();
		if(isset($user))
		{
			$sql .= " WHERE member_user_id = :user AND (happen_timestamp AT TIME ZONE 'Europe/Brussels' ) > (CURRENT_TIMESTAMP - :timeuser::interval)";
			$param[':user'] = $user;
		}
		else
		{
			$sql .= " WHERE (happen_timestamp AT TIME ZONE 'Europe/Brussels' ) > (CURRENT_TIMESTAMP - :timeuser::interval)";
		}
		$param[':timeuser'] = $time.' day';

		$drop_table_sql = "DROP TABLE IF EXISTS temp_nodeHistory;";
		$query = $dbconnection->prepare($drop_table_sql);
		$query = Model_esia::executeStatement($query);

		$create_table = "CREATE TEMPORARY TABLE temp_nodeHistory AS ($sql)";
		$query = $dbconnection->prepare($create_table,$param);
		$query = Model_esia::executeStatement($query,$param);

		$sql = "FROM temp_nodeHistory";
		$param = array();

		$sql_order = "";
		$flag = Model_TableSort::ts_getActiveErrors_filter($columns,$TSParams,$sql,$param,$sql_order);
		if($flag === FALSE)
		{
			//il n'y a pas de modification effectuée et donc, on met la valeur par défaut;
			$sql_order = " ORDER BY happen_timestamp DESC";
		}


		$total = Model_TableSort::ts_getRowsNumber($dbconnection,$sql,$param);

		$query = $dbconnection->prepare('SELECT * '.$sql.' '.$sql_order);
		$query = Model_esia::executeStatement($query,$param);
		$nodetype = Model_esia::getWebPluginByNodeType($dbconnection,NULL,Model_esia::NODE_TYPE_FORMAT_TYPE_URL);
		$arr = array();
		$listid = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!in_array($row->device_id,$listid))
			{
				$listid[]=$row->device_id;
			}
			$arr[]= Model_TableSort::ts_getFormatRowErrorHistoric($row,$nodetype);
		}
		return array('total'=>$total,'rows'=>$arr,'listid'=>$listid);
	}
	/**
	 * Récupère les erreur acquittée ou les services en maintenance dans base de donnée
	 * @param integer $user identifiant utilisateur
	 * @param integer $time (default:1) nombre de jour à prendre en compte
	 * @param array $TSParams tableau associatif envoyé par le tablesorter (format: Array(
	 *		    [filters] => Array // numéros de colonne et le filtre appliqué
	 *		        (
	 *					[4] => critique
	 *			    ),
	 *			[order] => Array // numéros de colonne qui trie en ascendant ou descendant
	 *			        (
	 *			            [8] => 1
	 *			        )
	 *			[size] => 25, //Nombre d'élement afficher dans la page
	 *			[paging] => 0) //Numéros de page
	 * @param array $lang_error_type tableau associatif faisant la correspondance entre le nom technique des erreurs (error_type) et les variables de langue.
	 * 		format FR: Array ( [warning] => Alerte,    [critical] => Critique,    [unknown] => Inconnu,    [maintenance] => Maintenance)
	 * @param array $lang_plugin tableau associatif faisant la correspondance entre le nom technique des plugins et le nom traduit
	 * 		format FR: Array ( [CHECK_SNMP_LOAD] => Processeur)
	 * @throws EsiaDbException
	 * @retval array array('total'=>$total,'rows'=>$arr); Total est le nombre d'erreur présente et le rows sont les données en tenant compte de la pagination
	 */
	static function ts_getPendingAcquittErrors($dbconnection,$user=NULL, $TSParams=NULL,$lang_error_type=NULL,$lang_plugin=NULL)
	{
		//l'index de la column doit être égale à l'index dans le table sorter
		$columns = Model_TableSort::ts_getActiveErrors_columns($lang_error_type,$lang_plugin);
		$columns[9]=array('col'=>'happen_acquitte','type'=>'date','data'=>'happen_time_acquitte');
		$cols_group = "";
		$sql_group ="";
		if(isset($TSParams['filters'][4]) || isset($TSParams['order'][4]) )
		{
			$sql_group =" JOIN esia_device_group_in  ON group_in_device_id=device_id
					 JOIN esia_device_group AS TGroupe ON group_in_device_group_id=device_group_id";
			$cols_group=", TGroupe.device_group_name AS device_group";
		}
		$sql_select ="SELECT device_id,device_name,device_type, host(device_ip_address) as device_ip_address,
						device_description, service_name, service_last_stdout,plugin_name,
						to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') AS happen_time,happen_timestamp,
						to_char(happen_time_acquitte,'DD/MM/YYYY HH24:MI:SS') AS happen_acquitte,happen_time_acquitte";
		//préparation reqûete de maintenance
		$sql_maintenance = "$sql_select,'maintenance' as error_type $cols_group
				FROM esia_device
				JOIN esia_error_happen ON device_id = happen_device_id
				JOIN esia_services ON service_id = happen_service_id
				JOIN esia_plugins ON service_plugin_id = plugin_id
				JOIN esia_errors ON error_id = happen_error_id $sql_group
				WHERE error_type='maintenance' AND happen_acquitte = 0";
		//préparation reqûete des acquittement
		$sql_acquitte = "$sql_select,'acquitte' as error_type $cols_group
				FROM esia_device as t2
				JOIN esia_services as tservice ON device_id = service_device_id
				JOIN esia_plugins ON service_plugin_id = plugin_id
				JOIN (SELECT DISTINCT ON(happen_service_id) happen_service_id ,happen_timestamp,happen_time_acquitte
					 FROM esia_error_happen
					 JOIN esia_services ON service_id = happen_service_id
					 WHERE service_previous_state <>0
					 	AND happen_service_id NOT IN (SELECT happen_service_id FROM view_actual_error)
						AND service_last_stdout IS NOT NULL ";
		$param = array();
		if(isset($user))
		{
			$sql_acquitte .= " AND service_device_id IN ( SELECT * FROM getUserAccessNode(:user0)) ";
			$param[':user0'] = $user;
			$sql_maintenance .=" AND device_id IN ( SELECT * FROM getUserAccessNode(:user1)) ";
			$param[':user1'] = $user;
		}
		$sql_acquitte .= ")
					AS thappen ON thappen.happen_service_id = tservice.service_id
					$sql_group";

					$sql = "FROM (
				$sql_acquitte
				UNION
				$sql_maintenance
				)
				as tabletmp ";
				$sql_order = "";
				$flag = Model_TableSort::ts_getActiveErrors_filter($columns,$TSParams,$sql,$param,$sql_order);
				if($flag === FALSE)
				{
					//il n'y a pas de modification effectuée et donc, on met la valeur par défaut;
					$sql_order = " ORDER BY happen_timestamp DESC";
				}
				$total = Model_TableSort::ts_getRowsNumber($dbconnection,$sql,$param);
				//echo 'SELECT * '.$sql.' '.$sql_order;
				//print_r($param);
				$query = $dbconnection->prepare('SELECT * '.$sql.' '.$sql_order);
				$query = Model_esia::executeStatement($query,$param);

				$nodetype = Model_esia::getWebPluginByNodeType($dbconnection,NULL,Model_esia::NODE_TYPE_FORMAT_TYPE_URL);
				$arr = array();
				$listid = array();
				while($row = $query->fetch(PDO::FETCH_OBJ))
				{
					if(!in_array($row->device_id,$listid))
					{
						$listid[]=$row->device_id;
					}
					if(array_key_exists($row->device_type,$nodetype))
					{
						$page_control = Model_esia::replacePageControlVariable($nodetype[$row->device_type],$row);
					}
					else
					{
						$page_control = '/node_view.php?nodeid='.$row->device_id;
					}
					$arr[]= Model_TableSort::ts_getFormatRowErrorHistoric($row,$nodetype);
				}
				return array('total'=>$total,'rows'=>$arr,'listid'=>$listid);
	}
}
?>
