<?php
/**
 * @name model_ts_adm
 * @description Classe modèle d'Esia pour les tablesorter ajax de l'administration
 *		Toutes les méthodes sont déclarée en 'static' ainsi on garde les Objets PDO pour les programmeurs extérieurs et pour faire des modifications, ils ne sont pas /
 *		obligé de connaitre la hierachie de classe d'ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 16/11/2012
 */
$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/model_tablesort.php';

class Model_Adm_TableSort extends Model_TableSort
{
    /**
	 * @brief Get all services from it's plugin id
	 * @param PDO $dbconnection
	 * @param integer $id of the plugin to fetch services from
	 * @retval array['rows'] = list of services, array['total'] = number of rows
	 * @exception EsiaDbException
	 */
	public static function ts_getServicesByPlugin($dbconnection, $id, $TSParams=NULL)
	{
        if(empty($id) || !is_numeric($id))
        {
            throw new EsiaDbException('Plugin id must be numeric');
        }

		$sql = "
            WITH service_pattern AS (
                SELECT pattern_name, included_id AS pattern_service_included_id, included_parameter AS pattern_plugin_parameter
                FROM esia_service_patterns
                LEFT JOIN esia_service_pattern_included ON esia_service_pattern_included.included_pattern_id = esia_service_patterns.pattern_id
            )
            SELECT plugin_id, plugin_name, plugin_default_parameter,
            service_id, service_name, service_parameter, service_previous_state,
            device_id, device_name, device_type,
            pattern_name, pattern_plugin_parameter
            FROM esia_plugins
            JOIN esia_services ON plugin_id = service_plugin_id
            JOIN esia_device ON service_device_id = device_id
            LEFT JOIN service_pattern ON pattern_service_included_id = service_included_id
        ";

        $params = array();
		$columns = array(
            array('col' => 'service_id', 'type' => 'integer'), // Unsortable col checkbox
			array('col' => 'device_name', 'type' => 'string'),
            array('col' => 'service_name', 'type' => 'string'),
            array('col' => 'pattern_name', 'type' => 'string'),
            array('col' => 'service_previous_state', 'type' => 'integer'),
            array('col' => 'service_parameter', 'type' => 'string'),
		);
		$sql_order = "";

		$flag = Model_TableSort::ts_getActiveErrors_filter($columns, $TSParams, $sql, $params, $sql_order);
		if($flag === FALSE)
		{
			// If not sql update set default value for order
			$sql_order = " ORDER BY device_name";
		}
		if(preg_match('#WHERE#', $sql))
		{
			$sql .= " AND ";
		}
		else
		{
			$sql .= " WHERE ";
		}

        $sql .= "plugin_id=:plugin_id AND service_delete=FALSE";
        $params[':plugin_id'] = $id;

        $query = $dbconnection->prepare($sql . $sql_order);
        $query = Model_esia::executeStatement($query, $params);
        $total = $query->rowCount();

        $nodeTypesUrl = Model_esia::getWebPluginByNodeType($dbconnection, NULL, Model_esia::NODE_TYPE_FORMAT_TYPE_URL);
        $services = array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
		{
            // Get node link url
            if(array_key_exists($row->device_type, $nodeTypesUrl))
			{
				$page_control = Model_esia::replacePageControlVariable($nodeTypesUrl[$row->device_type], array('id' => $row->device_id, 'name' => $row->device_name));
			}
			else
			{
				$page_control = '/node_view.php?nodeid=' . $row->device_id;
			}

			$services[] = array(
                'id' => $row->service_id,
                'name' => $row->service_name,
                'parameter' => $row->service_parameter,
                'state' => $row->service_previous_state,
                'plugin_id' => $row->plugin_id,
                'plugin_name' => $row->plugin_name,
                'plugin_parameter' => $row->plugin_default_parameter,
                'device_name' => $row->device_name,
                'pattern_name' => empty($row->pattern_name) ? '' : $row->pattern_name,
                'pattern_plugin_parameter' => empty($row->pattern_plugin_parameter) ? '' : $row->pattern_plugin_parameter,
                'page_control' => $page_control,
			);
		}

        return array('total' => $total, 'rows' => $services);
	}
}
?>
