#!/bin/bash

# Location to place backups.
backup_dir="/root/backup/"
#nombre de backup à garder
nbr_backup=5
#base de donnée à sauvegarder
database="esia"
#dossier temporaire de backup
tmp_dir="/var/tmp/esiabackup/"
esia_dir_plugin="/usr/local/esia/plugins"
esia_dir_html="/usr/local/esia/public_html/"
esia_conf="/etc/esia/"
db_conf="/etc/postgresql/11/main/"
#esia_log="/var/log/esia/"
esia_spool="/var/spool/esia/"
esia_init="/etc/init.d/esia"
#Nombre de jour où l'on souhaite garder les backups
number_of_days=30

#On prend la date pour le nom du fichier
backup_date=`date +%Y-%m-%d:%H:%M:%S`
#test si le répertoire par défaut existe
if [ -d $tmp_dir ]; then
	# si oui on le supprime (backup précédent qui n'a pas fini)
	rm -R $tmp_dir  2>&1 > /dev/null
fi

var=$(mkdir $tmp_dir 2>&1 > /dev/null)
if [ "$?" -ne 0 ]; then
	echo "[CRITICAL] $var"
	exit 2
fi
chmod -R 777 $tmp_dir

var=$(mkdir $tmp_dir/application 2>&1 > /dev/null)
if [ "$?" -ne 0 ]; then
	echo "[CRITICAL] $var"
	exit 2
fi

#creation de l'arborescence 
mkdir $tmp_dir/application/plugins
mkdir $tmp_dir/application/public_html
mkdir $tmp_dir/configuration
mkdir $tmp_dir/spool
mkdir $tmp_dir/log
mkdir $tmp_dir/init

#sauvegarde de la DB
var=$( su - postgres -c "pg_dump --create $database > $tmp_dir$database\_$backup_date" 2>&1 > /dev/null)
if [ "$?" -ne 0 ]; then
	echo "CRITICAL: $var"
	exit 2
fi
#copie des fichiers
var=$(cp -R $esia_dir_plugin* $tmp_dir/application/plugins 2>&1 > /dev/null)
if [ "$?" -ne 0 ]; then
	echo "CRITICAL: $var"
	exit 2
fi
var=$(cp -R $esia_dir_html* $tmp_dir/application/public_html/ 2>&1 > /dev/null)
if [ "$?" -ne 0 ]; then
	echo "CRITICAL: $var"
	exit 2
fi
cp -R $esia_conf* $tmp_dir/configuration/ 
cp -R $db_conf/postgresql.conf $tmp_dir/configuration/ 
cp -R $db_conf/pg_hba.conf $tmp_dir/configuration/ 
cp -R $esia_spool* $tmp_dir/spool/ 2> /dev/null
cp -R $esia_init* $tmp_dir/init/

mkdir -p $backup_dir

#compression et finalisation du fichier de backup
var=$(tar -jcf $backup_dir/esia\_$backup_date.tar.bz2 $tmp_dir  2>&1 > /dev/null)
if [ "$?" -ne 0 ]; then
	echo "CRITICAL: $var"
	exit 2
fi
#suppression des temporaires
var=$(rm -R $tmp_dir  2>&1 > /dev/null)
if [ "$?" -ne 0 ]; then
	echo "CRITICAL: $var"
	exit 1
fi
echo "OK: $backup_date backup successfull DIR: $backup_dir\esia_$backup_date.tar.gz"
#suppression des anciennes archives
counter=0
returncode=0
for i in $( ls -t $backup_dir ); do
	if [ $counter -ge $nbr_backup ]; then
		rm $backup_dir/$i
		if [ "$?" -ne 0 ]; then
			echo "[WARNING] rm $backup_dir/$i"
			returncode=1
		fi
	fi
	let counter=counter+1
done
echo "OK: PURGE OLD BACKUP"
#for i in $( ls -t $backup_dir ); do
#	echo item: $i
#done
exit $returncode





