<?php
/**
 * @name Model_esia
 * @description Classe modèle d'Esia
 *		Toutes les méthodes sont déclarée en 'static' ainsi on garde les Objets PDO pour les programmeurs extérieurs et pour faire des modifications, ils ne sont pas /
 *		obligé de connaitre la hierachie de classe d'ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 16/11/2012
 */

include_once DEVICE_CLASS;
include_once REMOTE_DEVICE_CLASS;
include_once GESA_CLASS;
include_once $_SERVER['DOCUMENT_ROOT'].'../keystore/scripts/keystore.lib.php';
include_once 'model_tablesort.php';

class Model_esia
{
	const NODE_ASSOC = 1;
	const NODE_INSTANCE = 2;
	const NODE_COUNT_SERVICE = 3;
	const NODE_COUNT_SERVICE_STATE = 4;
	const LOG_DAEMON_CHECK = 5;
	const LOG_DAEMON_TRAP = 6;
	const LOG_LAST_CHECK_EXEC = 7;
	const RRD_GRAPH_TYPES_TO_C3 = array( // RRD types in DB (rrd_plugins) to c3.js types
		'AREA' => 'area-spline',
		'AREA2' => 'area',
		'LINE1' => 'spline',
		'LINE2' => 'line',
	);

	/**
	 * exécute une requete préparée et traite l'erreur
	 * @param PDOStatement $query un objet PDOStatement (requête préparée)
	 * @param array  $param $param les paramètres à passer à la requête (voir PDOStatement::execute())
	 * @throws EsiaDbException
	 * @retval PDOStatement un objet PDOStatement qui a été exécuté
	 */
	static function executeStatement(PDOStatement $query,$param=NULL)
	{
		$ret = $query->execute($param);
		if($ret === FALSE)
		{
			$err = $query->errorInfo();
			throw new EsiaDbException($err[2]);
		}
		return $query;
	}
	/**
	 * utilisé pour débugger, affiche un tableau PHP dans une page WEB
	 * @param array $arr
	 */
	static function printHTMLarray($arr)
	{
		echo '<pre>';
		print_r($arr);
		echo '</pre>';
	}
	/**
	 * retourne le nom du thême par défaut d'esia
	 * @param PDO $dbconnection
	 * @retval string le nom tu theme par défaut
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getApplicationTheme($dbconnection)
	{
		$query = $dbconnection->prepare("SELECT config_value
				FROM esia_configuration
				WHERE config_key='DEFAULT_THEME';");
		$query = Model_esia::executeStatement($query);
		$theme = $query->fetch(PDO::FETCH_OBJ);

		$query = $dbconnection->prepare("SELECT theme_name
				FROM esia_theme
				WHERE theme_id = ?;");
		$query = Model_esia::executeStatement($query,array($theme->config_value));
		$row = $query->fetch(PDO::FETCH_OBJ);
		return $row->theme_name;
	}
	/**
	 * retourne la durée de session d'Esia
	 * @param PDO $dbconnection
	 * @retval integer la durée de session en minute
	 */
	static function getDuringSession($dbconnection)
	{
		$query = $dbconnection->prepare("SELECT config_value
				FROM esia_configuration
				WHERE config_key='DEFAULT_SESSION_TIME';");
		$query = Model_esia::executeStatement($query);
		$row = $query->fetch(PDO::FETCH_OBJ);
		return $row->config_value;
	}
	/**
	 * retourne les clé de configuration
	 * @param PDO $dbconnection
	 * @param string $key
	 * @retval array un tableau associatif
	 *					$arr[<Key>]['value'] valeur de la clé
	 *					$arr[<Key>]['description'] description de la clé
	 *					$arr[<Key>]['type'] typre de valeur (varchar,string,double,integer,bool)
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getConfiguration($dbconnection,$key=NULL)
	{
		if(isset($key) && !empty($key))
		{
			$query = $dbconnection->prepare("SELECT config_key,config_value,config_description,config_type,config_visibility FROM esia_configuration WHERE config_key=?;");
			$query = Model_esia::executeStatement($query,array($key));
		}
		else
		{
			$query = $dbconnection->prepare("SELECT config_key,config_value,config_description,config_type,config_visibility FROM esia_configuration ORDER BY config_key;");
			$query = Model_esia::executeStatement($query,array());
		}
		$list_key = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$list_key[$row->config_key]['key'] = $row->config_key;

			if($row->config_type == 'user_key')
			{
				try
				{
					include_once $_SERVER['DOCUMENT_ROOT'].'../keystore/scripts/keystore.lib.php';
					$conf_key = getConfKey();
					$e_uk = getEncryptedUserKey($dbconnection);
					$d_uk = decryptUserKey($conf_key,$e_uk);
					$list_key[$row->config_key]['value'] = $d_uk;
				}
				catch (Exception $e)
				{
					//protège de l'exception de non décriptage et qui provoque une erreur 500 quand on a modifié la clé en dure.
				}

			}
			else{
				$list_key[$row->config_key]['value'] = $row->config_value;
			}

			$list_key[$row->config_key]['description'] = $row->config_description;
			$list_key[$row->config_key]['type'] = $row->config_type;
			$list_key[$row->config_key]['visibility'] = $row->config_visibility;
			/*
			$list_key[$row->config_key]['key'] = $row->config_key;
			$list_key[$row->config_key]['value'] = $row->config_value;
			$list_key[$row->config_key]['description'] = $row->config_description;
			$list_key[$row->config_key]['type'] = $row->config_type;
			$list_key[$row->config_key]['visibility'] = $row->config_visibility;*/
		}
		return $list_key;
	}
	/**
	 *  retourne l'état du démon
	 * @param PDO $dbconnection
	 * @retval integer un nombre désignant l'état du démon (1 must reload et 2 = must restart)
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getDaemonState($dbconnection)
	{
		$query = $dbconnection->prepare("SELECT config_value
				FROM esia_configuration
				WHERE config_key='SYS_RELOAD';");
		$query = Model_esia::executeStatement($query);
		$row = $query->fetch(PDO::FETCH_OBJ);
		return $row->config_value;
	}
	/**
	 * retourne les web plugins
	 * @param PDO $dbconnection
	 * @param boolean $activate (optionnelle) spécifie si se sont les plugins actifs ou tous les web plugin
	 * @param string $name (optionnelle) précise le nom d'un web plugin en particulier
	 * @return array $webplugin[]=array('id'=>$row->web_id,
						'name'=>$row->web_name,
						'dir_lang'=>$row->web_dir_lang,
						'tpl_methode'=>$row->web_tpl_method,
						'show_menu'=>$row->web_show_main_menu,
						'php_menu'=>$row->web_php_menu,
						'index_page'=>$row->web_index_page,
						'admin_page'=>$row->web_admin_page,
						'control_page'=>$row->web_page_control,
						'apache_path'=>$row->web_plugin_apache_path,
						'icon'=>$row->web_icon,
						'activate'=>$row->web_activate);
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getWebPlugin($dbconnection,$activate=TRUE,$name=NULL)
	{
		$sql = "SELECT web_id,web_name,web_dir_lang,web_tpl_method,web_php_menu,web_index_page,web_admin_page,web_plugin_apache_path,
		web_show_main_menu,web_icon, web_activate
		FROM esia_web_plugin";
		$parameter = array();
		if($activate == TRUE)
		{
			$sql .= " WHERE web_activate=true";
		}
		if($activate == TRUE && !empty($name))
		{
			$sql .= " AND web_name=?";
			$parameter[] = $name;
		}
		elseif($activate == FALSE && !empty($name))
		{
			$sql .= " WHERE web_name=?";
			$parameter[] = $name;
		}
		$sql .= " ORDER BY web_id";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$parameter);
		$webplugin = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$webplugin[] = array('id' => $row->web_id,
					'name' => $row->web_name,
					'dir_lang' => $row->web_dir_lang,
					'tpl_method' => $row->web_tpl_method,
					'show_menu' => $row->web_show_main_menu,
					'php_menu' => $row->web_php_menu,
					'index_page' => $row->web_index_page,
					'admin_page' => $row->web_admin_page,
					//'control_page'=>$row->web_page_control,
					'apache_path' => $row->web_plugin_apache_path,
					'icon' => $row->web_icon,
					'activate' => $row->web_activate,
			);
		}
		return $webplugin;
	}
	/**
	 * retourne le ou les nom des différents type de device disponnible ainsi que leur 'web plugin" si ils en ont.
	 * @param PDO $dbconnection
	 * @param string|integer $type le nom ou l'id du type de device (table esia_device_type)
	 * @param string|integer $device L'id ou le nom du device
	 * @retval array un tableau associatif:
	 *		$types[]=array('id'=>$row->type_id,
	 *             'name'=>$row->type_name,
	 *             'web_id'=>$row->type_web_id,
	 *            'admin_page'=>$row->type_admin_page,
	 *             'img_ok_path'=>$row->type_img_ok_path,
	 *             'img_warning_path'=>$row->type_img_warning_path,
	 *             'img_critical_path'=>$row->type_img_critical_path,
	 *             'img_unknown_path'=>$row->type_img_unknown_path,
	 *             'web_plugin'=>$row->web_activate,
	 *             'web_plugin_show_menu'=>$row->web_show_main_menu,
	 *             'web_plugin_name'=>$row->web_name,
	 *             'web_plugin_lang_dir'=>$row->web_dir_lang,
	 *             'web_plugin_php_menu'=>$row->web_php_menu,
	 *             'web_plugin_menu_method'=>$row->web_tpl_method,
	 *             'web_plugin_index_page'=>$row->web_index_page,
	 *             'web_plugin_admin_page'=>$row->web_admin_page,
	 *             'web_plugin_control_page'=>$row->web_page_control,
	 *             'web_plugin_list_page'=>$row->type_list_page,
	 *             'web_plugin_apache_path'=>$row->web_plugin_apache_path,
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getDeviceType($dbconnection,$type=NULL,$device=NULL)
	{
		$sql = "SELECT type_web_id,web_name,web_dir_lang,web_php_menu,web_index_page,web_admin_page,web_activate,web_tpl_method,
		web_plugin_apache_path,web_page_control,web_show_main_menu,type_id, type_name, type_web_id, type_admin_page, type_img_ok_path,
		type_img_warning_path, type_img_critical_path, type_img_unknown_path,type_list_page
		FROM esia_device_type LEFT JOIN esia_web_plugin ON web_id=type_web_id";
		$param = array();
		if(!empty($type))
		{
			if(is_numeric($type)) {
				$sql .= " WHERE type_id=?";
			} else {
				$sql .= " WHERE type_name=?";
			}
			$param[] = $type;
		}
		elseif(!empty($device))
		{
			if(is_numeric($device)) {
				$sql .= " JOIN esia_device ON device_type= type_name WHERE device_id=?";
			} else {
				$sql .= " JOIN esia_device ON device_type= type_name WHERE device_name=?";
			}
			$param[] = $device;
		}
		$sql .= " ORDER BY lower(type_name)";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param);
		$types = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$types[$row->type_name] = array('id' => $row->type_id,
					'name' => $row->type_name,
					'web_id' => $row->type_web_id,
					'admin_page' => $row->type_admin_page,
					'img_ok_path' => $row->type_img_ok_path,
					'img_warning_path' => $row->type_img_warning_path,
					'img_critical_path' => $row->type_img_critical_path,
					'img_unknown_path' => $row->type_img_unknown_path,
					'web_plugin' => $row->web_activate,
					'web_plugin_show_menu' => $row->web_show_main_menu,
					'web_plugin_name' => $row->web_name,
					'web_plugin_lang_dir' => $row->web_dir_lang,
					'web_plugin_php_menu' => $row->web_php_menu,
					'web_plugin_menu_method' => $row->web_tpl_method,
					'web_plugin_index_page' => $row->web_index_page,
					'web_plugin_admin_page' => $row->web_admin_page,
					'web_plugin_control_page' => $row->web_page_control,
					'web_plugin_list_page' => $row->type_list_page,
					'web_plugin_apache_path' => $row->web_plugin_apache_path,
			);
		}
		return $types;
	}

	/**
	 * retourne le nom de groupe auquelle l'utilisateur à le droits sur au moins un noeud (droit: device_view) si aucun utilisateur n'est spécifié il retourne tout	les noms de groupes de noeud
	 * @param PDO $dbconnection
	 * @param string|integer $u (optionnelle) le nom ou l'id de l'utilisateur
	 * @retval un tableau contenant les noms des groupes de device
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getGroupDevice($dbconnection,$group=NULL,$user=NULL)
	{
		$sql = 'SELECT device_group_id, t.device_group_name,device_group_description,device_group_parent,device_group_order,device_group_path
  				FROM esia_device_group AS t';
		$params=array();
		if($user != NULL)
		{
			$sql = preg_replace('#SELECT#','SELECT DISTINCT',$sql);
			$sql .=" JOIN esia_device_group_in ON device_group_id= group_in_device_group_id
				JOIN esia_device ON device_id = group_in_device_id
				JOIN esia_group_device_access ON  group_in_device_id = access_device_id
				JOIN esia_groups ON access_group_id = group_id
				JOIN esia_memberof ON member_group_id = group_id
				WHERE member_user_id =:id";
			$params[':id'] = $user;
		}
		if($group != NULL)
		{
			if(strpos($sql, 'WHERE')===FALSE)
			{
				$sql .=" WHERE ";
			}
			else
			{
				$sql .=" AND ";
			}
			if(is_array($group))
			{
				$sql_prepare_arr="";
				foreach($group as $ind => $id)
				{
					$params[':groupid'.$ind]=$id;
					$sql_prepare_arr.=':groupid'.$ind.',';
				}
				$sql_prepare_arr= rtrim($sql_prepare_arr,',');
				$sql .=" device_group_id IN ($sql_prepare_arr)";
			}
			elseif(is_numeric($group))
			{
				$sql .=" device_group_id=:group";
			}
			else
			{
				$sql .=" device_group_name=:group";
			}
			$params[':group'] = $group;
		}
		else
		{
			$sql .=" ORDER BY device_group_path,device_group_order";
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{

			$arr[] = array('id' => $row->device_group_id ,
				'name' => $row->device_group_name,
				'description' => $row->device_group_description,
				'parent' => $row->device_group_parent,
				'fullpath' => $row->device_group_path
			);
		}
		return $arr;
	}

	/**
	 * retourne le nom des groupes de device ainsi que les groupes enfants de ce groupes
	 * @param PDO $dbconnection
	 * @param string|integer $group le nom ou l'id du groupe
	 * @retval un tableau associatifs contenant les noms des groupes de device
	 * array('id'=>$row->device_group_id,
	 *		 'name'=> $row->device_group_name,
	 *		 'child'=> un tableau associatifs si le groupe à des enfants
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getParrentGroup($dbconnection,$group)
	{
		if(is_numeric($group))
		{
			$query = $dbconnection->prepare("SELECT t2.device_group_id, t2.device_group_name,t2.device_group_description, t2.device_group_parent
					FROM esia_device_group as t1
					JOIN esia_device_group as t2 ON t2.device_group_id = t1.device_group_parent
					WHERE t1.device_group_id=:group ;");
		}
		else
		{
			$query = $dbconnection->prepare("SELECT t2.device_group_id, t2.device_group_name,t2.device_group_description, t2.device_group_parent
					FROM esia_device_group as t1
					JOIN esia_device_group as t2 ON t2.device_group_id = t1.device_group_parent
					WHERE t1.device_group_name=:group ;");
		}
		$query = Model_esia::executeStatement($query,array(':group' => $group));
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id' => $row->device_group_id ,'name' => $row->device_group_name,'description' => $row->device_group_description,'parent' => $row->device_group_parent);
		}
		return $arr;
	}

	/**
	 * retourne le nom des groupes de device auquel l'user à droit
	 * @param PDO $dbconnection
	 * @param integer $id (optionnelle) l'id de l'utilisateur
	 * @retval un tableau associatifs contenant les noms des groupes de device
	 * array('id'=>$row->device_group_id,
	 *		 'name'=> $row->device_group_name,
	 *		 'child'=> un tableau associatifs si le groupe à des enfants
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getGroupDeviceByUser($dbconnection,$id=NULL)
	{
		$sql = "SELECT DISTINCT device_group_id, device_group_name,device_group_description, device_group_parent
		FROM esia_device_group
		JOIN esia_device_group_in ON device_group_id = group_in_device_group_id
		JOIN esia_group_device_access ON  group_in_device_id = access_device_id
		JOIN esia_groups ON access_group_id = group_id
		JOIN esia_memberof ON member_group_id = group_id";

		if(is_numeric($id))
		{
			$query = $dbconnection->prepare($sql." WHERE member_user_id =:id ");
			$query = Model_esia::executeStatement($query,array(':id' => $id));
		}
		else
		{
			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query);
		}

		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('device_group_id' => $row->device_group_id,
					'device_group_name' => $row->device_group_name,
					'device_group_description' => $row->device_group_description,
					'device_group_parent' => $row->device_group_parent);
		}

		return $arr;
	}

	/**
	 * retourne le nom des groupes de device ainsi que les groupes enfants de ce groupes
	 * @param PDO $dbconnection
	 * @param integer $user l'id de l'utilisateur
	 * @param integer $group l'id du group de noeud
	 * @retval un tableau associatifs contenant les noms des groupes de device
	 * array('id'=>$row->device_group_id,
	 *		 'name'=> $row->device_group_name,
	 *		 'child'=> un tableau associatifs si le groupe à des enfants
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getAllNodeofGroup($dbconnection,$user,$group)
	{
		$list = array();
		$sql = "SELECT esia_device.device_description,esia_device.device_type,esia_device.device_username,esia_device.device_passwd,
													esia_device.device_protocol_connection,esia_device.device_snmp_version,esia_device.device_snmp_community,
													esia_device.device_snmp3_user,esia_device.device_snmp3_sec_lvl,esia_device.device_snmp3_auth_protocole,esia_device.device_snmp3_auth_key,
													esia_device.device_snmp3_priv_protocol,esia_device.device_snmp3_priv_passphrase,esia_device.device_snmp_timer,
													device_group_id,esia_device_group.device_group_name,device_group_parent,esia_device.device_id,esia_device.device_name,esia_device.device_ip_addr,esia_device.device_ip_address,
													device_snmp_port
											FROM esia_device_group
											JOIN esia_device_group_in ON device_group_id = group_in_device_group_id
											JOIN esia_device ON device_id = group_in_device_id	";
		$param = array();
		if(!empty($group))
		{
			if(isset($user))
			{
				$param[':user'] = $user;
				$sql = $sql . "WHERE (esia_device.device_id IN ( SELECT * FROM getUserAccessNode(:user)) OR esia_device.device_id IS NULL)";
			}
			if(isset($group))
			{
				if(!isset($user))
				{
					$sql = $sql . "WHERE ";
				}
				else
				{
					$sql = $sql . "AND ";
				}
				$sql = $sql ." device_group_id IN (SELECT group_id FROM ";
				if(is_array($group))
				{
					foreach($group as $id)
					{
						if(!is_numeric($id)) {
							throw new EsiaDbException('$group array must be numeric');
						}
					}
					$arrids = implode(',', $group);
					$sql = $sql."getallchilddevicegroups('{ $arrids }')";
				}
				else
				{
					$param[':group'] = $group;
					$sql = $sql."getallchilddevicegroup(:group)";
				}
			}
			$sql = $sql.")";
			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$param);
			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				$list[] = Model_esia::instanceNodeByRowDatabase($row);
			}
		}
		return $list;
	}

	/**
	 * retourne le nom des groupes de device ainsi que les groupes enfants de ce groupes
	 * @param PDO $dbconnection
	 * @param string|integer $group (optionnelle) le nom ou l'id de l'utilisateur
	 * @retval un tableau associatifs contenant les noms des groupes de device
	 * array('id'=>$row->device_group_id,
	 *		 'name'=> $row->device_group_name,
	 *		 'child'=> un tableau associatifs si le groupe à des enfants
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getGroupDeviceIn($dbconnection,$group=NULL)
	{
		$sql = 'SELECT device_group_id, device_group_name,device_group_description,device_group_parent,device_group_path
  				FROM esia_device_group';
		$params=array();
		if($group != NULL)
		{
			if(is_numeric($group))
			{
				$sql .=" WHERE device_group_id=:group";
			}
			else
			{
				$sql .=" WHERE device_group_name=:group";
			}
			$params[':group'] = $group;
		}
		else
		{
			$sql .=" ORDER BY device_group_order";
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$arr = array();
		$query_parent = $dbconnection->prepare('SELECT device_group_id, device_group_name,device_group_description, device_group_parent,device_group_path
				FROM esia_device_group WHERE device_group_parent=:id;');
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$tmps = Model_esia::preparedtree_node_groupe($dbconnection,$row->device_group_id,$query_parent);
			$arr[] = array('id' => $row->device_group_id ,
							'name' => $row->device_group_name,
							'description' => $row->device_group_description,
							'child' => $tmps,
							'parent' => $row->device_group_parent,
							'fullpath' => $row->device_group_path
			);
		}
		//print_r($arr);
		return $arr;
	}
	/**
	 * FONCTION RECURSIVE !!!!! (utilisée par la fonction getGroupDeviceIn) retourne le nom des groupes de device ainsi que les groupes enfants de ce groupes
	 * @param PDO $dbconnection
	 * @param integer $parent_id id du groupe du groupe parent
	 * @param PDOStatement $preparedQuery (optionnelle) requete sql preprare (performance)
	 * @retval un tableau associatifs contenant les noms des groupes de device enfants
	 * array('id'=>$row->device_group_id,
	 *		 'name'=> $row->device_group_name,
	 *		 'child'=> un tableau associatifs si le groupe à des enfants
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static private function preparedtree_node_groupe($dbconnection,$parent_id,PDOStatement $preparedQuery=NULL)
	{
		$arr = NULL;
		if(!isset($preparedQuery))
		{
			$queryp = $dbconnection->prepare('SELECT device_group_id, device_group_name,device_group_description, device_group_parent
				FROM esia_device_group WHERE device_group_parent=:id;');
			$preparedQuery = $queryp;
		}
		else
		{
			$queryp = $preparedQuery;
		}
		$result = Model_esia::executeStatement($queryp,array(':id' => $parent_id));
		$srow = $result->fetchAll();
		foreach($srow as $row)
		{
			$tmps = Model_esia::preparedtree_node_groupe($dbconnection,$row['device_group_id'],$preparedQuery);
			$arr[] = array('id' => $row['device_group_id'] ,'name' => $row['device_group_name'],'description' => $row['device_group_description'],'child' => $tmps);
		}

		return $arr;
	}

	/**
	 * retourne toutes les informations sur un plugin de check en particulier si rien n'est spécifié, on retourne l'information sur tout les plugins
	 * @param PDO $dbconnection
	 * @param string|integer $id (optionnel) le nom ou l'id d'un plugin
	 * @throws Exception
	 * @retval un tableau associatif contenant les parametres du(des) plugin(s) ainsi que ses paramètres d'alertes associés
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getPlugin($dbconnection, $id=NULL)
	{
		$plugin = array();
		$sql = "SELECT plugin_id, plugin_name, plugin_path, plugin_type, plugin_trap_oid, plugin_default_parameter,
					plugin_default_interval_time, plugin_priority, plugin_description, plugin_html_template, plugin_tag, plugin_manufacturer,
					react_program_id, react_parameter, react_ok, react_warning, react_critical, react_unknown
			FROM esia_plugins
			LEFT JOIN esia_plugin_react_program ON plugin_id = react_plugin_id
			WHERE plugin_delete <> TRUE";

		$params = array();
		if(!empty($id))
		{
			$params[':plugin'] = $id;
			if(is_numeric($id))
			{
				$sql .= ' AND plugin_id=:plugin';
			}
			elseif(is_string($id))
			{
				$sql .= ' AND plugin_name=:plugin';
			}
			else
			{
				throw new Exception("bad parameter plugin must be numeric or equals plugin's name");
			}
		}
		$sql .= ' ORDER BY plugin_name;';

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$plugin[] = array(
				'id' => $row->plugin_id,
				'name' => $row->plugin_name,
				'path' => $row->plugin_path,
				'type' => $row->plugin_type,
				'trap_oid' => $row->plugin_trap_oid,
				'parameter' => $row->plugin_default_parameter,
				'interval_time' => $row->plugin_default_interval_time,
				'description' => $row->plugin_description,
				'priority' => $row->plugin_priority,
				'html_template' => $row->plugin_html_template,
				'tag' => $row->plugin_tag,
				'manufacturer' => $row->plugin_manufacturer,
				'alerting' => $row->react_program_id,
				'alerting_parameter' => $row->react_parameter,
				'ok' => $row->react_ok,
				'warning' => $row->react_warning,
				'critical' => $row->react_critical,
				'unknown' => $row->react_unknown,
			);
		}
		return $plugin;
	}

	static function getAllServices($dbconnection)
	{
		$query = $dbconnection->prepare("SELECT service_id FROM esia_services;");

		$query = Model_esia::executeStatement($query,array());
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id' => $row->service_id);
		}
		return $arr;
	}

	/**
	 * @brief retoure les paramètres d'exécution d'un service avec les keywords remplis
	 * @param PDO $dbconnection
	 * @param integer|array $service Id de service ou tableau associatif de service
	 * @param null |node $node Objet node correspondant au noeud lié au service (pour accélére l'exécution)
	 * @throws EsiaDbException
	 * @throws Exception
	 * @retval string Les paramètres du service tel que le démon Esia utiliserait pour l'exécuter.
	 */
	static function getServiceExecutableParameters($dbconnection,$service,$node = NULL)
	{
		// Get service parameters & ID from $service
		$serviceParameters = "";
		$serviceId = NULL;
		if(is_array($service))
		{
			if(isset($service['parameter']))
			{
				$serviceParameters = $service['parameter'];
			}
			if(isset($service['id']))
			{
				$serviceId = $service['id'];
			}
		}
		elseif(is_string($service))
		{
			$serviceParameters = $service;
		}
		elseif(is_numeric($service))
		{
			$serviceId = $service;
		}
		else
		{
			throw new Exception("Service parameter must be a service array or a service id");
		}

		// If parameters cannot be found but we have the service ID get parameters from DB
		if(empty($serviceParameters) && !empty($serviceId))
		{
			$service = Model_esia::getServices($dbconnection, NULL, $serviceId);
			$service = array_shift($service);
			$serviceParameters = $service['parameter'];
		}

		// Replace service ID
		if(!empty($serviceId))
		{
			$serviceParameters = str_replace('$SERID', $serviceId, $serviceParameters);
		}

		if( !is_object($node) )
		{
			$node = Model_esia::InstanceNode($dbconnection,$service['device_id']);
		}
		$serviceParameters = str_replace('$NODEID',$node->getID(),$serviceParameters);
		$serviceParameters = str_replace('$NODENAME',$node->getName(),$serviceParameters);
		$serviceParameters = str_replace('$IP',$node->getIPAddress(),$serviceParameters);

		$snmp_ver = $node->getSNMPVersion();
			$snmpReq = $node->getSNMPRequest();
        $timeout = $snmpReq->getTimeout() / 1000 /1000;
		if(!empty($snmp_ver))
		{
            if(strpos($serviceParameters,'$SNMP_PARAMETERS') !== FALSE)
            {
                $snmpparameters=" -v $snmp_ver -t ".$timeout;
			    if( $snmp_ver < 3 )
			    {
                    $snmpparameters .=" -C ".$node->getSNMPCommunity();
			    }
                else
                {
                    $snmpparameters .=" -U ".$snmpReq->getSNMP3User();
                    $snmpparameters .=" -A ".$snmpReq->getSNMP3AuthKey();
                    $prot = $snmpReq->getSNMP3AuthProtocol();
                    $privkey =$snmpReq->getPrivacyKey();
                    if(!empty($privkey))
                    {
                        $snmpparameters .=" -P $privkey";
                        $prot .=",".$snmpReq->getPrivacyProtocol();
                    }
                    $snmpparameters .=" -L $prot";
                }
                $serviceParameters = str_replace('$SNMP_PARAMETERS',$snmpparameters,$serviceParameters);
            }
            else
            {
			    $serviceParameters = str_replace('$SNMP_TIMER',$timeout,$serviceParameters);
			    $serviceParameters = str_replace('$SNMP_VERSION',$snmp_ver,$serviceParameters);
			    if( $snmp_ver < 3 )
			    {
				    $serviceParameters = str_replace('$SNMP_COM',$node->getSNMPCommunity(),$serviceParameters);
			    }
			    else
			    {
				    $serviceParameters = str_replace('$SNMP3_SECLVL',$snmpReq->getSNMP3SecLevel(),$serviceParameters);
				    $serviceParameters = str_replace('$SNMP3_USER',$snmpReq->getSNMP3User(),$serviceParameters);
				    $serviceParameters = str_replace('$SNMP3_AUTH_PROT',$snmpReq->getSNMP3AuthProtocol(),$serviceParameters);
				    $serviceParameters = str_replace('$SNMP3_AUTH_KEY',$snmpReq->getSNMP3AuthKey(),$serviceParameters);
				    $serviceParameters = str_replace('$SNMP3_PRIV_PROT',$snmpReq->getPrivacyProtocol(),$serviceParameters);
				    $serviceParameters = str_replace('$SNMP3_PRIV_KEY',$snmpReq->getPrivacyKey(),$serviceParameters);
			    }
            }
		}

		if(is_a($node,'RemoteDevice'))
		{
			$serviceParameters = str_replace('$NRPE_HOST',$node->getGESA()->getPublicIPaddress(),$serviceParameters);
			$serviceParameters = str_replace('$GESASN',$node->getGESA()->getSerialNumber(),$serviceParameters);
		}

		if( preg_match('/\$AUTH_FILE_(\w+)/',$serviceParameters,$matches) && !empty($matches[1]))
		{
			$webp = $matches[1];
			$advance_config_type = Model_esia::getAdvanceNodeConfigType($dbconnection,$webp);
			$advance_config_type = array_shift($advance_config_type);
			if( isset($advance_config_type['id']))
			{
				$sql = 'SELECT adv_plugin_keyword_value FROM esia_device_advance_config WHERE adv_device_id=? AND adv_config_type_id = ?';
				$query = $dbconnection->prepare($sql);
				Model_esia::executeStatement($query,array($node->getID(),$advance_config_type['id']));
				$advance_conf_file = $query->fetch(PDO::FETCH_COLUMN);

				$serviceParameters = str_replace($matches[0],$advance_conf_file,$serviceParameters);
			}
		}
		return $serviceParameters;
	}

	/**
	 * @brief retourne les informations sur un ou des services
	 * @param PDO $dbconnection
	 * @param null|integer|string $node identifiant d'un noeud (nom ou device_id)
	 * @param integer |integer[] $service identifiant ou un tableau d'identifiant de service
	 * @param integer |integer[] $pluginId identifiant de plugin
	 * @param boolean	FALSE par défaut, indique si on souhaite inclure les columns pattern_id et le pattern_name
	 * @throws EsiaDbException
	 * @retval number[]['string'] array('id'=>$row->service_id,
										*'name'=>$row->service_name,
										*'plugin_id'=> $row->plugin_id
										*'path'=>$row->plugin_path,
										*'type'=>$row->plugin_type,
										*'trap_oid'=>$row->plugin_trap_oid,
										*'parameter'=>$row->service_parameter,
										*'priority'=>$row->service_priority,
										*'state'=>$row->service_previous_state,
										*'interval_time'=>$row->service_interval_time,
										*'last_exec_time'=>$row->last_exec_time,
										*'added_time'=>$row->check_time,
										*'stdout'=>$row->service_last_stdout,
										*'description'=>$row->plugin_description,
										*'check_plugin_name'=>$row->plugin_name,
										*'device_name'=>$row->device_name,
										*'device_id'=>$row->device_id,
										*'alerting_program_id'=>$row->program_id,
										*'alerting_program_name'=>$row->program_name,
										*'alerting_program_path'=>$row->program_path,
										*'alerting_program_parameter'=>$row->response_parameter,
										*'alerting_program_ok'=>$row->response_ok,
										*'alerting_program_warning'=>$row->response_warning,
										*'alerting_program_critical'=>$row->response_critical,
										*'alerting_program_unknown'=>$row->response_unknown,
										*'alerting_program_ok'=>$row->response_ok,
										*'real_state'=>$row->service_previous_state,
										*'value'=>$value);
	 */

	static function getServices($dbconnection,$node=NULL,$service=NULL,$pluginId=NULL,$with_pattern_fields=FALSE)
	{
		$sql_pattern_table = $sql_pattern_col = '';
		if( $with_pattern_fields )
		{
			$sql_pattern_table = "LEFT JOIN esia_service_pattern_included ON(service_included_id = included_id)\n";
			$sql_pattern_table.= "LEFT JOIN esia_service_patterns ON(included_pattern_id = pattern_id)";

			$sql_pattern_col = "pattern_id,pattern_name,included_name,";
		}

		$sql = "SELECT plugin_id,plugin_name,plugin_path,plugin_type,plugin_trap_oid,plugin_description,$sql_pattern_col
		service_id,service_name,service_interval_time,service_parameter,service_previous_state,service_included_id,
		to_char(service_last_exec_time,'DD-MM-YYYY HH24:MI:SS') as last_exec_time,

		extract(epoch from (now() - service_last_exec_time)) as last_exec_delta,
		service_last_stdout,to_char(service_time_added,'DD-MM-YYYY HH24:MI:SS') as check_time,
		device_id,device_name,device_type,
		program_id, program_name,program_path, response_parameter,response_ok, response_warning, response_critical, response_unknown,service_priority,response_ok

		FROM esia_plugins
		JOIN esia_services ON plugin_id = service_plugin_id
		JOIN esia_device ON service_device_id = device_id
		$sql_pattern_table
		LEFT JOIN esia_service_response_program ON service_id = response_service_id
		LEFT JOIN esia_program  ON program_id = response_program_id";
		$plugin = array();
		$ret;
		$params = array();
		if(!empty($service))
		{
			if(is_numeric($service))
			{
				$query = $dbconnection->prepare($sql." WHERE service_id=? AND service_delete=FALSE;");
				$params[] = $service;
			}
			elseif(is_array($service))
			{
				foreach($service as $id)
				{
					if(!is_numeric($id)) {
						throw new EsiaDbException('$service array must be numeric');
					}
				}
				$arrids = implode(',', $service);
				$query = $dbconnection->prepare($sql." WHERE service_id IN ($arrids) AND service_delete=FALSE;");
			}
			else
			{
				throw new EsiaDbException('$service must be numeric');
			}
		}
		elseif(!empty($node))
		{
			if(is_numeric($node)) {
				$query = $dbconnection->prepare($sql." WHERE device_id=? AND service_delete=FALSE ORDER BY service_name;");
			} elseif(is_string($node)) {
				$query = $dbconnection->prepare($sql." WHERE device_name=? AND service_delete=FALSE ORDER BY service_name ;");
			}
			$params[] = $node;
		}
		elseif(!empty($pluginId))
		{
			if(is_numeric($pluginId))
			{
				$query = $dbconnection->prepare($sql." WHERE plugin_id=? AND service_delete=FALSE ORDER BY device_name;");
			}
			$params[] = $pluginId;
		}
		else
		{
			throw new EsiaDbException('$node must not be null');
		}
		$query = Model_esia::executeStatement($query,$params);
		$q_isack = $dbconnection->prepare("SELECT happen_service_id FROM esia_error_happen WHERE happen_acquitte=0 AND happen_service_id=?");
		$q_rrd = $dbconnection->prepare("SELECT rrd_type,rrd_regex,rrd_time,rrd_name,rrd_max,rrd_vertical_label,rrd_min,rrd_graph_num,rrd_calc FROM rrd_plugins WHERE rrd_plugin_id=?;");
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$status = $row->service_previous_state;
			if($status != 0 && $status != 4)
			{
				$qe_status = Model_esia::executeStatement($q_isack,array($row->service_id));
				if($qe_status->rowCount() == 0)
				{
					$status = 0;
				}
			}
			$rrd = Model_esia::executeStatement($q_rrd,array($row->plugin_id));
			$value = array();
			while($rowrrd = $rrd->fetch(PDO::FETCH_OBJ))
			{
				if(preg_match($rowrrd->rrd_regex,$row->service_last_stdout,$regs))
				{
					$tmp = $regs[1];
					if(preg_match('#([\*\/\+\-\%])([0-9]*)#',$rowrrd->rrd_calc,$calc))
					{
						switch($calc[1])
						{
							case '+': $tmp = $tmp + $calc[2]; break;
							case '-': $tmp = $tmp - $calc[2]; break;
							case '*': $tmp = $tmp * $calc[2]; break;
							case '/': $tmp = $tmp / $calc[2]; break;
							case '%': $tmp = $tmp % $calc[2]; break;
							default: break;
						}
					}
					$value[$rowrrd->rrd_name] = $tmp;
				}
				else
				{
					$value[$rowrrd->rrd_name] = NULL;
				}
			}
			$retRow = array('id' => $row->service_id,
					'name' => $row->service_name,
					'plugin_id' => $row->plugin_id,
					'service_included_id' => $row->service_included_id,
					'path' => $row->plugin_path,
					'type' => $row->plugin_type,
					'trap_oid' => $row->plugin_trap_oid,
					'parameter' => $row->service_parameter,
					'priority' => $row->service_priority,
					'state' => $status,
					'interval_time' => $row->service_interval_time,
					'last_exec_time' => $row->last_exec_time,
					'last_exec_delta' => $row->last_exec_delta,
					'added_time' => $row->check_time,
					'stdout' => $row->service_last_stdout,
					'description' => $row->plugin_description,
					'check_plugin_name' => $row->plugin_name,
					'device_name' => $row->device_name,
					'device_id' => $row->device_id,
					'device_type' => $row->device_type,
					'alerting_program_id' => $row->program_id,
					'alerting_program_name' => $row->program_name,
					'alerting_program_path' => $row->program_path,
					'alerting_program_parameter' => $row->response_parameter,
					'alerting_program_warning' => $row->response_warning,
					'alerting_program_critical' => $row->response_critical,
					'alerting_program_unknown' => $row->response_unknown,
					'alerting_program_ok' => $row->response_ok,
					'real_state' => $row->service_previous_state,
					'value' => $value
			);

			if( $with_pattern_fields )
			{
				$retRow['included_name'] = $row->included_name;
				$retRow['pattern_id'] = $row->pattern_id;
				$retRow['pattern_name'] = $row->pattern_name;
			}

			$plugin[] = $retRow;
		}
		return $plugin;
	}

	/**
	 * @brief va rechercher le noeud, le service et le message en erreur pour un plugin donné
	 * @param int : $plugin id du plugin
	 * @param string : $state type d'erreur à afficher
	 * @retval array des erreurs trouvées
	 */
	static function getPluginByCurrentState($dbconnection,$plugin,$state)
	{
		$sql = "	SELECT plugin_id,plugin_name, service_id,service_name, device_id,device_name, error_type,happen_message
				FROM esia_plugins
				JOIN esia_services ON plugin_id = service_plugin_id
				JOIN esia_device ON service_device_id = device_id
				LEFT JOIN esia_error_happen ON service_id = happen_service_id
				LEFT JOIN esia_errors ON happen_error_id = error_id
				WHERE happen_acquitte = 0
				AND plugin_id = ?
				AND error_type = ?";
		$params = array();
		$params[] = $plugin;
		$params[] = $state;

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('plugin_id' => $row->plugin_id,
					'plugin_name' => $row->plugin_name,
					'service_id' => $row->service_id,
					'service_name' => $row->service_name,
					'device_id' => $row->device_id,
					'device_name' => $row->device_name,
					'type' => $row->error_type,
					'message' => $row->happen_message);
		}

		return $arr;
	}

	/**
	 * @brief va rechercher le noeud, le service et le message en erreur pour un service donné
	 * @param int : $service id du service
	 * @param string : $state type d'erreur à afficher
	 * @retval array des erreurs trouvées
	 */
	static function getServiceByCurrentState($dbconnection,$service,$state)
	{
		$sql = "	SELECT service_id, service_name, plugin_id,plugin_name, device_id,device_name, error_type,happen_message
				FROM esia_services
				LEFT JOIN esia_plugins ON service_plugin_id = plugin_id
				LEFT JOIN esia_device ON service_device_id = device_id
				LEFT JOIN esia_error_happen ON service_id = happen_service_id
				LEFT JOIN esia_errors ON happen_error_id = error_id
				WHERE happen_acquitte = 0
				AND service_id = ?
				AND error_type = ?";
		$params = array();
		$params[] = $service;
		$params[] = $state;

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('plugin_id' => $row->plugin_id,
					'plugin_name' => $row->plugin_name,
					'service_id' => $row->service_id,
					'service' => $row->service_name,
					'device_id' => $row->device_id,
					'device' => $row->device_name,
					'type' => $row->error_type,
					'message' => $row->happen_message);
		}

		return $arr;
	}

	/**
	 * @brief traduit le service name dans la langue de l'utilisateur
	 * @param array[] $lang le tableau contenant les variables de langue
	 * @param string $servicename nom du service en base de donnée
	 * @param string $pluginname nom technique du plugin (CHECK_XXXXXX)
	 * @retval string le nom simplifié s'il existe
	 */
	static function getCommonName($lang,$servicename,$pluginname)
	{
		$common_name  = $servicename;
		$tmpname = preg_replace('#CHECK\_|GESA\_#',"",$pluginname);
		if(array_key_exists($tmpname, $lang))
		{
			$common_name = $lang[$tmpname];
			$tname = preg_replace('#'.$pluginname.'#',"",$servicename);
			$tname = ltrim($tname,'_');
		   // echo $servicename." ".$pluginname." = ".$tname."<br />";
			if(!empty($tname))
			{
				if(preg_match('#[GESA|CHECK]_SNMP_SW_INTERFACE#', $pluginname))
				{
					$pluginname = preg_replace('#_EASY#','',$pluginname);
					$tname = preg_replace('#'.$pluginname.'_#',"",$servicename);
				}
				elseif(preg_match('#[GESA|CHECK]_SNMP_INTERFACE#', $pluginname))
				{
					$pluginname = preg_replace('#_EASY#','',$pluginname);
					$tname = preg_replace('#'.$pluginname.'_#',"",$servicename);
				}
				$common_name = preg_replace('#\-\s+\$1#',"- $tname", $common_name);
			}
			else
			{
				$common_name = preg_replace('#\s+\-\s+\$1#', "", $common_name);
			}
		}
		return $common_name;
	}

	/**
	 * retourne le chemin d'accès apache à la page de contrôle du web plugin du type de device passé en paramètre
	 * @param PDO $dbconnection
	 * @param string|integer $type (optionnel) le nom ou l'id d'un plugin
	 * @retval string le chemin d'accès apache à la page de contrôle du web plugin ou NULL
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getPageControl($dbconnection,$type)
	{

		$query = $dbconnection->prepare("SELECT web_page_control,web_plugin_apache_path
				FROM  esia_web_plugin
				JOIN esia_device_type ON type_web_id=web_id
				WHERE type_name=? AND web_activate=TRUE ");
		$query = Model_esia::executeStatement($query,array($type));
		if($r = $query->fetch(PDO::FETCH_OBJ))
		{
			return $r->web_plugin_apache_path.$r->web_page_control;
		}
		else
		{
			return '/node_view.php?nodeid=:id';
		}
	}

	/**
	 * retourne les informations sur les serveur de graphique (cacti ou munin)
	 * @param PDO $dbconnection
	 * @param string|integer $id le nom ou l'id du noeud
	 * @retval $arr = array('software'=>$r->graph_software, 'url'=>$r->graph_url, 'tree'=>$r->graph_cacti_tree_id,'leaf'=>$r->graph_cacti_leaf_id); OU FALSE si il n'y a pas de graph
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getGraphParameter($dbconnection,$id)
	{
		if(is_numeric($id))
		{
			$query = $dbconnection->prepare("SELECT graph_url,graph_cacti_tree_id,graph_cacti_leaf_id,graph_software
					FROM  esia_device_graph
					WHERE graph_device_id=? ");
		}
		else
		{
			$query = $dbconnection->prepare("SELECT graph_url,graph_cacti_tree_id,graph_cacti_leaf_id,graph_software
					FROM  esia_device_graph
					JOIN esia_device ON graph_device_id = device_id
					WHERE device_name=? ");
		}
		$query = Model_esia::executeStatement($query,array($id));
		if($r = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr = array('software' => $r->graph_software, 'url' => $r->graph_url, 'tree' => $r->graph_cacti_tree_id,'leaf' => $r->graph_cacti_leaf_id);
			return $arr;
		}
		else
		{
			return FALSE;
		}
	}

	/**
	 * retourne les noeuds d'un type de noeud, ou si l'utilisateurs à accès à c'est noeud (device_view)
	 * @param PDO $dbconnection
	 * @param integer $user_id id de l'utilisateur (optionnel)
	 * @param string $type type de noeud
	 * @retval retourne un tableau d'objet Device
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getNodebyType($dbconnection,$user_id,$type)
	{
		$list = array();
		if(isset($user_id))
		{
			$query = $dbconnection->prepare("SELECT DISTINCT device_id
				FROM esia_device
				JOIN esia_group_device_access ON  device_id = access_device_id
				JOIN esia_groups ON access_group_id = group_id
				JOIN esia_memberof ON member_group_id = group_id
				WHERE member_user_id =? AND device_type=? ;");
			$query = Model_esia::executeStatement($query,array($user_id,$type));
		}
		else
		{
			$query = $dbconnection->prepare("SELECT DISTINCT device_id
					FROM esia_device
					WHERE device_type=? ;");
			$query = Model_esia::executeStatement($query,array($type));
		}
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$list[] = Model_esia::InstanceNode($dbconnection,$row->device_id);
		}
		return $list;
	}

	static function getNodeListFilteredByStatus($dbconnection,$user_id,$node_status = NULL, $service_status = NULL)
 {
		$req = $dbconnection->prepare("SELECT group_id, group_name
											   FROM esia_groups
											   JOIN esia_memberof ON member_group_id = group_id
											   WHERE member_user_id =?");
		$req = Model_esia::executeStatement($req,array($user_id));
		while($row = $req->fetch(PDO::FETCH_OBJ)){
			$group_name[] = $row->group_name;
		}
		if(in_array("root", $group_name)) { $root = TRUE;
		} else { $root = FALSE;
		}

		$sql = 'SELECT device_id,device_name,g1.device_group_name as device_group_name,device_type,device_ip_addr,count(service_id) as services,device_description';
		$sql .= ' FROM esia_device';
		$sql .= ' LEFT JOIN esia_services ON device_id = service_device_id';
		$sql .= ' LEFT JOIN esia_device_group_in ON device_id = group_in_device_id';
		$sql .= ' LEFT JOIN esia_device_group as g1 ON device_group_id=group_in_device_group_id';
		$sql .= ' WHERE device_delete=FALSE';

		$params = array();
		if($node_status !== NULL){
			$sql .= ' AND device_id NOT IN (SELECT DISTINCT service_device_id FROM esia_services WHERE service_previous_state!=?)';
			$params[] = $node_status;
		}

		if($service_status !== NULL){
			$sql .= ' AND service_previous_state = ?';
			$params[] = $service_status;
		}

		if(!$root) {
			$sql .= ' AND device_id IN (SELECT * FROM getUserAccessNode(?))';
			$params[] = $user_id;
		}
		$sql .= ' GROUP BY device_id,g1.device_group_name';
		$sql .= ' ORDER BY device_id';

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$node['id'] = $row->device_id;
			$node['name'] = $row->device_name;
			$node['type'] = $row->device_type;
			$node['group'] = $row->device_group_name;
			$node['ip'] = $row->device_ip_addr;
			$node['description'] = $row->device_description;
			$node['services'] = $row->services;
			$list[] = $node;
		}
		return $list;
	}

	static function getNodeListWithServicesStats($dbconnection,$user_id)
 {
		$req = $dbconnection->prepare("SELECT group_id, group_name
											   FROM esia_groups
											   JOIN esia_memberof ON member_group_id = group_id
											   WHERE member_user_id =?");
		$req = Model_esia::executeStatement($req,array($user_id));
		while($row = $req->fetch(PDO::FETCH_OBJ)){
			$group_name[] = $row->group_name;
		}
		if(in_array("root", $group_name)) { $root = TRUE;
		} else { $root = FALSE;
		}

		$sql = 'SELECT device_id,device_name,g1.device_group_name as device_group_name,device_type,device_ip_address,count(service_id) as services,device_description';
		$sql .= ' FROM esia_device';
		$sql .= ' LEFT JOIN esia_services ON device_id = service_device_id';
		$sql .= ' LEFT JOIN esia_device_group_in ON device_id = group_in_device_id';
		$sql .= ' LEFT JOIN esia_device_group as g1 ON device_group_id=group_in_device_group_id';
		$sql .= ' WHERE device_delete=FALSE';
		if(!$root) {
			$sql .= ' AND device_id IN (SELECT * FROM getUserAccessNode(?))';
			$params = array($user_id);
		}
		else { $params = array();
		}
		$sql .= ' GROUP BY device_id,g1.device_group_name';
		$sql .= ' ORDER BY device_id';

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$node['id'] = $row->device_id;
			$node['name'] = $row->device_name;
			$node['type'] = $row->device_type;
			$node['group'] = $row->device_group_name;
			$node['ip'] = $row->device_ip_address;
			$node['description'] = $row->device_description;
			$node['services'] = $row->services;
			$list[] = $node;
		}
		return $list;
	}

	/**
	 * retourne les noeuds d'un groupe de noeud, ou si l'utilisateurs à accès à c'est noeud (device_view), ou encore retourne tout les noeuds
	 * @param PDO $dbconnection
	 * @param integer $user_id (optionnel) id de l'utilisateur
	 * @param string $group (optionnel) nom du group de noeud
	 * @param integer $t_return Model_esia::NODE_ASSOC (optionnel) modifie la méthode de retour (NODE_INSTANCE ou NODE_ASSOC) (constante definie)
	 * @retval si $t_return == Model_esia::NODE_ASSOC alors on retourne un tableau associatif (voir Model_esia::getAllNode() ) si $t_return == Model_esia::NODE_INSTANCE retourne un tableau d'objet Device
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getNodeList($dbconnection,$user_id=NULL,$group=NULL,$t_return = Model_esia::NODE_ASSOC )
	{
		return Model_esia::newgetNodeList($dbconnection,$user_id,$group,$t_return);
	}
	/**
	 * retourne les noeuds d'un groupe de noeud, ou si l'utilisateurs à accès à c'est noeud (device_view), ou encore retourne tout les noeuds
	 * @param PDO $dbconnection
	 * @param integer $user_id (optionnel) id de l'utilisateur
	 * @param string $group (optionnel) nom du group de noeud
	 * @param integer $t_return Model_esia::NODE_ASSOC (optionnel) modifie la méthode de retour (NODE_INSTANCE ou NODE_ASSOC) (constante definie)
	 * @retval si $t_return == Model_esia::NODE_ASSOC alors on retourne un tableau associatif (voir Model_esia::getAllNode() ) si $t_return == Model_esia::NODE_INSTANCE retourne un tableau d'objet Device
	 * @exception EsiaDbException si une erreur survient
	 */
	static function newgetNodeList($dbconnection,$user_id=NULL,$group=NULL,$t_return = Model_esia::NODE_ASSOC )
	{
		$list = array();
		if($t_return == Model_esia::NODE_INSTANCE)
		{
			$sql = "SELECT device_id,device_name,device_description,device_type,device_ip_address,device_username,device_passwd,
                                                device_protocol_connection,g1.device_group_name,g1.device_group_id,device_snmp_version,device_snmp_community,
                                                device_snmp3_user,device_snmp3_sec_lvl,device_snmp3_auth_protocole,device_snmp3_auth_key,
                                                device_snmp3_priv_protocol,device_snmp3_priv_passphrase,device_snmp_timer,device_ip_address,device_snmp_port
                                        FROM esia_device
                                        LEFT JOIN esia_device_group_in ON device_id = group_in_device_id
                                        LEFT JOIN esia_device_group as g1 ON device_group_id=group_in_device_group_id";
			if(is_numeric($user_id))
			{
				if($group == 'noGroup')
				{
					$sql .= " WHERE device_id IN ( SELECT * FROM getUserAccessNode(?))
                                                                AND device_delete=FALSE
                                                                AND NOT EXISTS(SELECT group_in_device_id FROM esia_device_group_in WHERE group_in_device_id = device_id ) ;";
					$query = $dbconnection->prepare($sql);
					$query = Model_esia::executeStatement($query,array($user_id));
				}
				elseif(isset($group) && !empty($group))
				{
					$sql .= " WHERE device_id IN ( SELECT * FROM getUserAccessNode(?)) AND device_delete=FALSE";
					if(is_numeric($group))
					{
						$sql .= " AND g1.device_group_id = ?;";
					}
					else
					{
						$sql .= " AND g1.device_group_name ILIKE ?;";
					}
					$query = $dbconnection->prepare($sql);
					$query = Model_esia::executeStatement($query,array($user_id,$group));
				}
				else
				{
					$query = $dbconnection->prepare($sql." WHERE device_id IN ( SELECT * FROM getUserAccessNode(?)) AND device_delete=FALSE;");
					$query = Model_esia::executeStatement($query,array($user_id));
				}
			}
			else
			{
				if($group == 'noGroup')
				{
					$query = $dbconnection->prepare($sql." WHERE device_id NOT IN(SELECT group_in_device_id FROM esia_device_group_in WHERE group_in_device_id = device_id ) AND device_delete=FALSE ;");
					$query = Model_esia::executeStatement($query);
				}
				elseif(isset($group) && !empty($group))
				{
					$sql .= " WHERE device_delete=FALSE";
					if(is_numeric($group))
					{
						$sql .= " AND device_group_id = ?;";
					}
					else
					{
						$sql .= " AND g1.device_group_name ILIKE ?;";
					}
					$query = $dbconnection->prepare($sql);
					$query = Model_esia::executeStatement($query,array($group));
				}
				else
				{
					$query = $dbconnection->prepare($sql);
					$query = Model_esia::executeStatement($query);
				}
			}
			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				$dev = Model_esia::instanceNodeByRowDatabase($row);
				$gesa = Model_esia::getGesaLink($dbconnection, $dev->getId());
				if($gesa != NULL)
				{
					$dev = new RemoteDevice($dev, $gesa);
				}
				$list[]=$dev;
			}
		}
		else
		{
			$list = Model_esia::getAllNode($dbconnection,$user_id,$group);
		}
		return $list;
	}
	private static function instanceNodeByRowDatabase($row)
	{
		$dev = new Device();
		$dev->setId($row->device_id);
		$dev->setName($row->device_name);
		$dev->setType($row->device_type);
		$dev->setDescription($row->device_description);
		$dev->setIPAddress($row->device_ip_address);
		$dev->setSNMPVersion($row->device_snmp_version);
		$dev->getSNMPObject()->setTimeout($row->device_snmp_timer);
		$dev->getSNMPObject()->setRemotePort($row->device_snmp_port);
		if($row->device_snmp_version == 1 || $row->device_snmp_version == 2) {
			$dev->setSNMPCommunity($row->device_snmp_community);
		} else {
			$dev->setSNMP3Parameter($row->device_snmp3_user,$row->device_snmp3_sec_lvl,$row->device_snmp3_auth_protocole,$row->device_snmp3_auth_key,$row->device_snmp3_priv_protocol,$row->device_snmp3_priv_passphrase);
		}
		$dev->setGroup($row->device_group_name);
		$dev->setGroupID($row->device_group_id);
		$dev->setTypeConnection($row->device_protocol_connection);
		return $dev;
	}

	/**
	 * retourne les device_id des noeuds accessibles par un utilisateur mais filtrés sur une adresse IP (Ne vérifie pas les liens Unitys)
	 * @param PDO $dbconnection
	 * @param integer $user_id id de l'utilisateur
	 * @param string | long $address adresse IP des noeud
	 * @retval array($row->device_id); Liste des IDs des noeuds qui possédent cette adresse IP
	 * @exception EsiaDbException si une erreur survient
	 */
	public static function getNodesByAddress($dbconnection,$userid,$address)
 {
		$sql = 'SELECT device_id FROM esia_device';
		$sql .= ' WHERE device_id IN ( SELECT * FROM getUserAccessNode(:userid))';
		$sql .= ' AND device_delete=FALSE AND device_ip_addr = :longip';

		if(is_string($address)){
			$address = ip2long($address);
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':userid' => $userid,':longip' => $address));

		$ret = array();
		while($row = $query->fetch(PDO::FETCH_OBJ)){
			$ret[] = $row->device_id;
		}
		return $ret;
	}

	/**
	 * retourne les noeuds d'un groupe de noeud, ou si l'utilisateurs à accès à c'est noeud (device_view), ou encore retourne tout les noeuds
	 * @param PDO $dbconnection
	 * @param integer $user_id (optionnel) id de l'utilisateur
	 * @param string $group (optionnel) nom du group de noeud (ou id)
	 * @retval array('id'=>$row->device_id,
	 *             'name'=>$row->device_name,
	 *             'description'=>$row->device_description,
	 *             'type'=>$row->device_type,
	 *             'ip'=>long2ip($row->device_ip_addr),
	 *             'connection_protocol'=>$row->device_protocol_connection,
	 *             'username'=>$row->device_username,
	 *             'password'=>$row->device_passwd,
	 *             'group'=>$row->device_group_name,
	 *             'snmp_version'=>$row->device_snmp_version,
	 *             'snmp_community'=>$row->device_snmp_community,
	 *             'snmp3_user'=>$row->device_snmp3_user,
	 *             'snmp3_seclvl'=>$row->device_snmp3_sec_lvl,
	 *             'snmp3_auth_protocole'=>$row->device_snmp3_auth_protocole,
	 *            'snmp3_auth_key'=>$row->device_snmp3_auth_key,
	 *			);
	 * @exception EsiaDbException si une erreur survient
	 */
	static private function getAllNode($dbconnection,$user_id=NULL,$group=NULL)
	{
		$nodes = array();
		$param = array();
		$sql = "SELECT device_id,device_name,device_description,device_type,device_ip_address,device_username,device_passwd,device_protocol_connection,g1.device_group_name,device_snmp_version,device_snmp_community,device_snmp3_user,device_snmp3_sec_lvl,device_snmp3_auth_protocole,device_snmp3_auth_key,adv_param
		FROM esia_device
		LEFT JOIN esia_device_group_in ON device_id = group_in_device_id
		LEFT JOIN esia_device_group as g1 ON device_group_id=group_in_device_group_id
		LEFT JOIN esia_device_advance_config ON device_id = adv_device_id";
		if(!empty($user_id) && is_numeric($user_id))
		{
			$sql .= " WHERE device_id IN (SELECT * FROM getUserAccessNode(?))";
			$param[] = $user_id;
			if($group == 'noGroup')
			{
				$sql .= " AND g1.device_group_name IS NULL";
			}
			elseif(isset($group) && !empty($group))
			{
				$param[] = $group;
				if(!is_numeric($group))
				{
					$sql .= " AND g1.device_group_name = ?";
				}
				else
				{
					$sql .= " AND g1.device_group_id = ?";
				}
			}
		}
		$sql .= " ORDER BY device_type,device_name";
		$query = $dbconnection->prepare("$sql");
		$query = Model_esia::executeStatement($query,$param);
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$nodes[] = array('id' => $row->device_id,
					'name' => $row->device_name,
				'description' => $row->device_description,
					'type' => $row->device_type,
				'ip' => $row->device_ip_address,
					'connection_protocol' => $row->device_protocol_connection,
					'username' => $row->device_username,
					'password' => $row->device_passwd,
					'group' => $row->device_group_name,
					'snmp_version' => $row->device_snmp_version,
					'snmp_community' => $row->device_snmp_community,
					'snmp3_user' => $row->device_snmp3_user,
					'snmp3_seclvl' => $row->device_snmp3_sec_lvl,
					'snmp3_auth_protocole' => $row->device_snmp3_auth_protocole,
					'snmp3_auth_key' => $row->device_snmp3_auth_key,
					'rdp_software' => json_decode($row->adv_param,TRUE)['type'],
					'rdp_soft_id' => json_decode($row->adv_param,TRUE)['id']
			);
		}
		return $nodes;
	}
	const NODE_TYPE_FORMAT_ARRAY = 1;
	const NODE_TYPE_FORMAT_TYPE_URL = 2;
	/**
	 * retourne les informations sur le webplugin gérant le type de device donné
	 * @param PDO $dbconnection
	 * @param string|integer $type type de device id ou nom (table esia_device_type)
	 * @param string $format
	 * @retval $arr = array('name'=>$row->web_name,
	 *					 'apache_path'=>$row->web_plugin_apache_path,
	 *					 'admin_page'=>$row->type_admin_page,
	 *					 'control_page'=>$row->web_page_control,
	 *					 'info_page'=>$row->type_list_page,
	 *					 );
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getWebPluginByNodeType($dbconnection,$type=NULL,$format= Model_esia::NODE_TYPE_FORMAT_ARRAY)
	{
		$sql = "SELECT type_web_id,type_admin_page,web_name,web_plugin_apache_path,web_page_control,type_list_page,web_admin_page,type_name
				FROM esia_device_type
				JOIN esia_web_plugin ON web_id=type_web_id
				WHERE web_activate=true";
		$params = array();
		if(isset($type))
		{
			if(is_numeric($type))
			{
				$query = $dbconnection->prepare($sql." AND type_id=?");
			}
			else
			{
				$query = $dbconnection->prepare($sql." AND type_name=?");
			}
			$params[] = $type;
		}
		else
		{
			$query = $dbconnection->prepare($sql);
		}
		$query = Model_esia::executeStatement($query,$params);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if($format==Model_esia::NODE_TYPE_FORMAT_TYPE_URL)
			{
				$arr[$row->type_name] = $row->web_plugin_apache_path.$row->web_page_control;
			}
			else
			{
				$arr[] = array('id' => $row->type_web_id,'name' => $row->web_name,
					'apache_path' => $row->web_plugin_apache_path,
					'admin_page' => $row->web_admin_page,
					'control_page' => $row->web_page_control,
					'info_page' => $row->type_list_page,
					'type' => $row->type_name
				);
			}
		}
		//les 2 premier if assure la compatibilité de l'ensemble
		if(empty($arr))
		{
			return NULL;
		}
		elseif(count($arr) == 1)
		{
			return $arr[0];
		}
		else
		{
			return $arr;
		}
	}

	/**
	 * retourne l'historique des erreurs d'un noeud ou d'un service en particulire
	 * @param PDO $dbconnection
	 * @param object|integer $dev id du device ou un objet device
	 * @param integer $service (optionnel) id du service
	 * @param string $state (optionnel) si on veut filtrer sur le type d'erreur (warning, critical, unknown)
	 * @retval array('error'=>$row->error_name,
	 *		'type'=>$row->error_type,
	 *		'message'=>$row->happen_message,
	 *		'value'=>$row->happen_value,
	 *		'time'=>$row->timestamp,
	 *		'acquitte_time'=>$row->time_acquite,
	 *		'acquitte'=>$row->happen_acquitte,
	 *		'during'=>$row->during,
	 *		'service'=>$row->service_name);
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getNodeError($dbconnection,  $dev,$service=NULL,$state=NULL )
	{
		$sql = "SELECT error_name,error_type, happen_interface_id,
		happen_message,happen_acquitte, happen_value,
		service_name, to_char(happen_time_acquitte,'DD/MM/YYYY HH24:MI:SS') as time_acquite,
		to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') as  timestamp,to_char((happen_time_acquitte - happen_timestamp),'DDD HH24:MI:SS') as during, device_name,device_type,plugin_name
		FROM esia_error_happen
		JOIN esia_errors ON happen_error_id = error_id
		LEFT JOIN esia_services ON happen_service_id =service_id
		LEFT JOIN esia_plugins ON service_plugin_id = plugin_id
		LEFT JOIN esia_device ON happen_device_id = device_id
		WHERE happen_acquitte =0";
		$param = array();
		if(@get_class($dev) == 'Device')
		{
			$sql = $sql." AND happen_device_id = ? ";
			$param[] = $dev->getId();
		}
		elseif(@get_class($dev) == 'RemoteDevice')
		{
			$sql = $sql." AND happen_device_id = ? ";
			$param[] = $dev->getId();
		}
		elseif(is_array($dev))
		{
			foreach($dev as $id)
			{
				if(!is_numeric($id)) {
					throw new EsiaDbException('$dev array must be numeric');
				}
			}
			$arrids = implode(',', $dev);
			$sql = $sql." AND happen_device_id IN ($arrids) ";
		}
		else
		{
			$sql = $sql." AND happen_device_id = ? ";
			$param[] = $dev;
		}
		if(!empty($service) && is_numeric($service))
		{
			$sql .= ' AND service_id = ? ';
			$param[] = $service;
		}
		if(!empty($state) && is_string($state))
		{
			$sql .= ' AND error_type = ? ';
			$param[] = $state;
		}
		$query = $dbconnection->prepare($sql." ORDER BY happen_timestamp DESC;");
		$query = Model_esia::executeStatement($query,$param);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('error' => $row->error_name,
					'type' => $row->error_type,
					'message' => $row->happen_message,
					'value' => $row->happen_value,
					'time' => $row->timestamp,
					'acquitte_time' => $row->time_acquite,
					'acquitte' => $row->happen_acquitte,
					'during' => $row->during,
					'service' => $row->service_name,
					'device' => $row->device_name,
					'device_type' => $row->device_type,
					'plugin_name' => $row->plugin_name);
		}
		return $arr;
	}

	/**
	 * retourne l'historique des erreurs d'un noeud ou d'un service en particulire
	 * @param PDO $dbconnection
	 * @param object|integer $dev id du device ou un objet device
	 * @param integer $service (optionnel) id du service
	 * @retval array('error'=>$row->error_name,
	 *		'type'=>$row->error_type,
	 *		'message'=>$row->happen_message,
	 *		'value'=>$row->happen_value,
	 *		'time'=>$row->timestamp,
	 *		'acquitte_time'=>$row->time_acquite,
	 *		'acquitte'=>$row->happen_acquitte,
	 *		'during'=>$row->during,
	 *		'service'=>$row->service_name);
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getNodeErrorHistoric($dbconnection,  $dev,$service=NULL )
	{
		$sql = "SELECT happen_id,error_name,error_type, happen_interface_id,user_name,
		happen_message,happen_acquitte, happen_value,
		service_name, to_char(happen_time_acquitte,'DD/MM/YYYY HH24:MI:SS') as time_acquite,
		to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') as  timestamp,to_char((happen_time_acquitte - happen_timestamp),'DDD HH24:MI:SS') as during,
		plugin_name
		FROM esia_error_happen
		JOIN esia_errors ON happen_error_id = error_id
		LEFT JOIN esia_services ON happen_service_id =service_id
		LEFT JOIN esia_plugins ON plugin_id =service_plugin_id
		LEFT JOIN esia_users ON happen_user_id=user_id
		WHERE happen_device_id = ?";
		$param = array();
		if(@get_class($dev) == 'Device')
		{
			$param[] = $dev->getId();
		}
		if(@get_class($dev) == 'RemoteDevice')
		{
			$param[] = $dev->getId();
		}
		else
		{
			$param[] = $dev;
		}
		if(is_numeric($service))
		{
			$sql .= ' AND service_id = ? ';
			$param[] = $service;
		}
		$query = $dbconnection->prepare($sql." ORDER BY happen_acquitte,happen_timestamp DESC;");
		$query = Model_esia::executeStatement($query,$param);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id' => $row->happen_id,
					'error' => $row->error_name,
					'type' => $row->error_type,
					'message' => $row->happen_message,
					'value' => $row->happen_value,
					'time' => $row->timestamp,
					'acquitte_time' => $row->time_acquite,
					'acquitte' => $row->happen_acquitte,
					'during' => $row->during,
					'service' => $row->service_name,
					'user_name' => $row->user_name,
					'plugin_name' => $row->plugin_name,
					'happen_id' => $row->happen_id,

			);
		}

		return $arr;
	}
	static function getNodeErrorHistoricTableSorter($dbconnection,  $dev,$ts_filter )
	{
		$sql = "SELECT happen_id,error_name,error_type, happen_interface_id,user_name,
		happen_message,happen_acquitte, happen_value,
		service_name, to_char(happen_time_acquitte,'DD/MM/YYYY HH24:MI:SS') as time_acquite,
		to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') as  timestamp,to_char((happen_time_acquitte - happen_timestamp),'DDD HH24:MI:SS') as during,
		plugin_name
		FROM esia_error_happen
		JOIN esia_errors ON happen_error_id = error_id
		JOIN esia_services ON happen_service_id =service_id
		LEFT JOIN esia_plugins ON plugin_id =service_plugin_id
		LEFT JOIN esia_users ON happen_user_id=user_id
		WHERE happen_device_id = :devid";

		//
		//Gestion pagination
		//
		$params = array();
		$limit='';
		$page=0;
		if(!empty($ts_filter['page']))
		{
			$page = $ts_filter['page'];
		}
		if(!empty($ts_filter['size']))
		{
			$size= $ts_filter['size'];
			if(is_numeric($size) && is_numeric($page))
			{
				$limit='LIMIT '.$size.' OFFSET '.($page*$size);
			}
		}
		//
		//GESTION des filtres
		//
		$filters='';
		if(!empty($ts_filter['filter_technical']))
		{
			$serv =$ts_filter['filter_technical'];
			if(preg_match('#^\!.*#',$serv))
			{
				$serv = preg_replace('#\!#','',$serv);
				$filters.=" AND service_name NOT ILIKE :sername";
				$params['sername']='%'.$serv.'%';
			}
			elseif(preg_match('#^\:.*$#',$serv))
			{
				$serv = preg_replace('#\:#','',$serv);
				$filters.=" AND service_name = :sername";
				$params['sername']=$serv;
			}
			else
			{
				$filters.=" AND service_name ILIKE :sername";
				$params['sername']='%'.$serv.'%';
			}
		}
		//
		//Filtres Message
		//
		if(!empty($ts_filter['filter_message']))
		{
			$msg = $ts_filter['filter_message'];
			if(preg_match('#^\!.*#',$msg))
			{
				$msg = preg_replace('#\!#','',$msg);
				$filters.=" AND happen_message NOT ILIKE :msg";
			}
			else
			{
				$filters.=" AND happen_message ILIKE :msg";
			}
			$params['msg']='%'.$msg.'%';
		}
		//
		//Filtres receivedat
		//
		if(!empty($ts_filter['filter_date_start']))
		{
			$date = $ts_filter['filter_date_start'];
			if(preg_match('#^\!.*#',$date))
			{
				$date = preg_replace('#\!#','',$date);
				$filters.=" AND to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') NOT ILIKE :date";
			}
			else
			{
				$filters.=" AND to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') ILIKE :date";
			}
			$params['date']='%'.$date.'%';
		}
		//
		//Filtres receivedat
		//
		if(!empty($ts_filter['filter_date_end']))
		{
			$date = $ts_filter['filter_date_end'];
			if(preg_match('#^\!.*#',$date))
			{
				$date = preg_replace('#\!#','',$date);
				$filters.=" AND to_char(happen_acquitte,'DD/MM/YYYY HH24:MI:SS') NOT ILIKE :date_end";
			}
			else
			{
				$filters.=" AND to_char(happen_acquitte,'DD/MM/YYYY HH24:MI:SS') ILIKE :date_end";
			}
			$params['date_end']='%'.$date.'%';
		}
		//
		//Filtres receivedat
		//
		if(!empty($ts_filter['filter_type']))
		{
			$error = $ts_filter['filter_type'];
			if(preg_match('#^\!.*#',$error))
			{
				$error = preg_replace('#\!#','',$error);
				$filters.=" AND error_type NOT ILIKE :error";
			}
			else
			{
				$filters.=" AND error_type ILIKE :error";
			}
			$params['error']='%'.$error.'%';
		}
		//
		//Filtres receivedat
		//
		if(!empty($ts_filter['filter_duration']))
		{
			$during = $ts_filter['filter_duration'];
			if(preg_match('#^\!.*#',$error))
			{
				$during = preg_replace('#\!#','',$during);
				$filters.=" AND to_char((happen_time_acquitte - happen_timestamp),'DDD HH24:MI:SS') ILIKE :during";
			}
			else
			{
				$filters.=" AND to_char((happen_time_acquitte - happen_timestamp),'DDD HH24:MI:SS') ILIKE :during";
			}
			$params['during']='%'.$during.'%';
		}
		//
		//Filtres receivedat
		//
		if(!empty($ts_filter['filter_user']))
		{
			$user = $ts_filter['filter_user'];
			if(preg_match('#^\!.*#',$user))
			{
				$user = preg_replace('#\!#','',$user);
				$filters.=" AND user_name NOT ILIKE :user";
			}
			else
			{
				$filters.=" AND user_name ILIKE :user";
			}
			$params['user']='%'.$user.'%';
		}
		if(@get_class($dev) == 'Device')
		{
			$params['devid'] = $dev->getId();
		}
		if(@get_class($dev) == 'RemoteDevice')
		{
			$params['devid'] = $dev->getId();
		}
		else
		{
			$params['devid'] = $dev;
		}
		//
		//ORDER
		//
		$order="";
		if(!empty($ts_filter['order']))
		{
			switch ($ts_filter['order'])
			{
				case 'service':
					$order = 'service_name';
					break;
				case 'technical':
					$order = 'service_name';
					break;
				case 'type':
					$order = 'error_type';
					break;
				case 'date_start':
					$order = 'happen_timestamp';
					break;
				case 'date_end':
					$order = 'happen_time_acquitte';
					break;
				case 'duration':
					$order = 'during';
					break;
				case 'user':
					$order = 'user_name';
					break;
				case 'message':
					$order = 'happen_message';
					break;
			}
			if($ts_filter['orderType'] == 'ASC')
			{
				$order.=' ASC';
			}
			elseif($ts_filter['orderType'] == 'DESC')
			{
				$order.=' DESC';
			}
		}
		else
		{
			$order = "happen_acquitte,happen_timestamp DESC";
		}
		//echo $sql." $filters ORDER BY $order $limit";
		$query = $dbconnection->prepare("$sql $filters ORDER BY $order $limit;");
		$query = Model_esia::executeStatement($query,$params);
		$arr = array();
		$i=0;
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id' => $row->happen_id,
				'error' => $row->error_name,
				'type' => $row->error_type,
				'message' => $row->happen_message,
				'value' => $row->happen_value,
				'time' => $row->timestamp,
				'acquitte_time' => $row->time_acquite,
				'acquitte' => $row->happen_acquitte,
				'during' => $row->during,
				'service' => $row->service_name,
				'user_name' => $row->user_name,
				'plugin_name' => $row->plugin_name,
				'happen_id' => $row->happen_id,
			);
			$i++;
		}
		$total = $i;
		if(!empty($limit))
		{
			$query = $dbconnection->prepare("SELECT count(*) as nbr
                                    FROM esia_error_happen
									JOIN esia_errors ON happen_error_id = error_id
									JOIN esia_services ON happen_service_id =service_id
									LEFT JOIN esia_plugins ON plugin_id =service_plugin_id
									LEFT JOIN esia_users ON happen_user_id=user_id
                                    WHERE happen_device_id = :devid $filters");
			$query = Model_esia::executeStatement($query,$params);
			$row=$query->fetch(PDO::FETCH_OBJ);
			$total = $row->nbr;
		}
		$farr['data']=$arr;
		$farr['total']=$total;
		return $farr;
	}
	/**
	 * retourne toute les erreurs pour un type de noeud (appellé par la page d'index.php notamment)
	 * @param PDO $dbconnection
	 * @param integer $user (optionnelle)si spécifié les droits utilisateurs seront vérifiés
	 * @retval un tableau contenant
	 *	$arr[]['type_error'] type d'erreur (warning,critical,unknown)
	 *	$arr[]['type_device'] type de noeud
	 *	$arr[]['error']	nom de l'erreur
	 *	$arr[]['time']	date de l'erreur
	 *	$arr[]['message'] message renvoyé
	 *	$arr[]['name'] nom du noeud,
	 *	$arr[]['id'] id du noeud,
	 *	$arr[]['plugin_name'] nom du service,
	 *	$arr[]['page_control'] page de controle du noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getError($dbconnection,$user=NULL,$timestamp=FALSE,$TSParams=NULL )
	{
		$sql = "SELECT error_type,error_name,happen_message,happen_value,device_name,device_id,happen_interface_id,service_name,service_last_stdout,
					device_type,device_description,service_priority,plugin_name, TGROUP.device_group_name,TGROUP.device_group_parent,host(device_ip_address) as device_ip_address ,";
		if($timestamp == TRUE)
		{
			$sql .= "extract(epoch from happen_timestamp) AS happen_timestamp";
		}
		else
		{
			$sql .= "to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') AS happen_timestamp";
		}
		$sql .= " FROM esia_device as t2
				JOIN esia_error_happen ON device_id = happen_device_id
				JOIN esia_services ON service_id = happen_service_id
				JOIN esia_plugins ON service_plugin_id = plugin_id
				JOIN esia_errors ON error_id = happen_error_id
				LEFT JOIN
					(SELECT group_in_device_id, T1.device_group_name AS device_group_name, T2.device_group_name AS device_group_parent
						FROM esia_device_group_in
						JOIN esia_device_group AS T1 ON group_in_device_group_id=device_group_id
						LEFT JOIN esia_device_group AS T2 ON T2.device_group_id = T1.device_group_parent
					) AS TGROUP ON device_id = TGROUP.group_in_device_id";
		$param = array();
		if(isset($user))
		{
			$sql .= " WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND happen_acquitte=0";
			$param[':user'] = $user;
		}
		else
		{
			$sql .= " WHERE happen_acquitte=0";
		}
		$sql.=" ORDER BY happen_timestamp DESC;";
		//echo $sql;
		////print_r($param);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param);
		$nodetype = array();
		$arr = Model_esia::getWebPluginByNodeType($dbconnection);
		foreach($arr as $row)
		{
			$nodetype[$row['type']] = $row['apache_path'].$row['control_page'];
		}
		//print_r($param);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(array_key_exists($row->device_type,$nodetype))
			{
				$page_control = Model_esia::replacePageControlVariable($nodetype[$row->device_type],$row);
			}
			else
			{
				$page_control = '/node_view.php?nodeid='.$row->device_id;
			}
			$out = preg_replace('#\n#','<br />',$row->happen_message);
			//echo $row->error_name."<br />";
			/*On affiche value: x que si value != 0*/
			$mess = $out;
			if($row->happen_value != 0)
			{
				$mess = $out.' value: '.$row->happen_value;
			}

			$arr[] = array('type_error' => $row->error_type,
					'type_device' => $row->device_type,
					'error' => $row->error_name,
					'time' => $row->happen_timestamp,
					'message' => $mess,
					'name' => $row->device_name,
					'id' => $row->device_id,
					'ip_addr' => $row->device_ip_address,
					'plugin_name' => $row->service_name,
					'check_plugin_name' => $row->plugin_name,
					'page_control' => $page_control,
					'group' => $row->device_group_name,
					'description' => $row->device_description,
					'priority' => $row->service_priority,
					'group_parent' => $row->device_group_parent,
					'last_stdout' => $row->service_last_stdout

			);
		}
		return $arr;
	}

	/**
	 * retourne toute les erreurs pour un type de noeud (appellé par la page d'index.php notamment)
	 * @param PDO $dbconnection
	 * @param integer $user (optionnelle)si spécifié les droits utilisateurs seront vérifiés
	 * @param number $time interval
	 * @param integer $nodeId (optionnelle)si spécifié renvoie les erreurs pour ce noeud
	 * @param integer $serviceId (optionnelle)si spécifié renvoie les erreurs pour ce service
	 * @retval un tableau contenant
	 *	$arr[]['type_error'] type d'erreur (warning,critical,unknown)
	 *	$arr[]['type_device'] type de noeud
	 *	$arr[]['error']	nom de l'erreur
	 *	$arr[]['time']	date de l'erreur
	 *	$arr[]['message'] message renvoyé
	 *	$arr[]['name'] nom du noeud,
	 *	$arr[]['id'] id du noeud,
	 *	$arr[]['plugin_name'] nom du service,
	 *	$arr[]['page_control'] page de controle du noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getAllErrorByTime($dbconnection,$user=NULL,$time=1,$nodeId=NULL,$serviceId=NULL)
	{
		$sql = "SELECT error_type,error_name,extract(epoch from happen_timestamp) AS happen_timestamp,
		happen_message,happen_value,device_name,device_id,happen_interface_id,service_name,plugin_name, TGROUP.device_group_name, TGROUP.device_group_parent,
				device_type,device_description,service_priority,extract(epoch from happen_time_acquitte) AS time_acquitte, device_ip_address

		FROM esia_device as t2
		JOIN esia_error_happen ON device_id = happen_device_id
		JOIN esia_services ON service_id = happen_service_id
		JOIN esia_plugins ON service_plugin_id = plugin_id
		JOIN esia_errors ON error_id = happen_error_id
		LEFT JOIN
			(SELECT group_in_device_id, T1.device_group_name AS device_group_name, T2.device_group_name AS device_group_parent
				FROM esia_device_group_in
				JOIN esia_device_group AS T1 ON group_in_device_group_id=device_group_id
				LEFT JOIN esia_device_group AS T2 ON T2.device_group_id = T1.device_group_parent
			) AS TGROUP ON device_id = TGROUP.group_in_device_id ";
		$param = array();
		if(isset($time))
		{
			$sql_time = "AND (happen_timestamp AT TIME ZONE 'Europe/Brussels' ) > (CURRENT_TIMESTAMP - interval '$time day')";
		}
		if(isset($user))
		{
			$sql .= " WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user)) ".$sql_time;
			$param['user'] = $user;
		}
		else
		{
			$sql .= " WHERE  member_user_id=? ".$sql_time;
		}
		if(!empty($nodeId))
		{
			$sql .= " AND device_id=:device_id";
			$param[':device_id'] = $nodeId;
		}
		if(!empty($serviceId))
		{
			$sql .= " AND service_id=:service_id";
			$param[':service_id'] = $serviceId;
		}
		$query = $dbconnection->prepare($sql." ORDER BY happen_timestamp DESC;");
		$query = Model_esia::executeStatement($query,$param);
		$nodetype = array();
		$arr = Model_esia::getWebPluginByNodeType($dbconnection);
		foreach($arr as $row)
		{
			$nodetype[$row['type']] = $row['apache_path'].$row['control_page'];
		}
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(array_key_exists($row->device_type,$nodetype))
			{
				$page_control = Model_esia::replacePageControlVariable($nodetype[$row->device_type],$row);
			}
			else
			{
				$page_control = '/node_view.php?nodeid='.$row->device_id;
			}
			$out = preg_replace('#\n#','<br />',$row->happen_message);
			$arr[] = array('type_error' => $row->error_type,
					'type_device' => $row->device_type,
					'error' => $row->error_name,
					'time' => $row->happen_timestamp,
					'message' => $out.' value: '.$row->happen_value,
					'name' => $row->device_name,
					'description' => $row->device_description,
					'id' => $row->device_id,
					'ip_addr' => $row->device_ip_address,
					'plugin_name' => $row->service_name,
					'page_control' => $page_control,
					'group' => $row->device_group_name,
					'priority' => $row->service_priority,
					'time_acquitte' => $row->time_acquitte,
					'check_plugin_name' => $row->plugin_name,
					'group_parent' => $row->device_group_parent,
			);
		}
		return $arr;
	}

	/**
	 * retourne toute les erreurs pour un type de noeud (appellé par la page d'index.php notamment)
	 * @param PDO $dbconnection
	 * @param string $type type de device
	 * @param integer $user (optionnelle) si spécifié les droits utilisateurs seront vérifiés
	 * @retval un tableau contenant
	 *	$arr[]['type_error'] type d'erreur (warning,critical,unknown)
	 *	$arr[]['type_device'] type de noeud
	 *	$arr[]['error']	nom de l'erreur
	 *	$arr[]['time']	date de l'erreur
	 *	$arr[]['message'] message renvoyé
	 *	$arr[]['name'] nom du noeud,
	 *	$arr[]['id'] id du noeud,
	 *	$arr[]['plugin_name'] nom du service,
	 *	$arr[]['page_control'] page de controle du noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getErrorByNodeType($dbconnection,$type,$user=NULL )
	{
		$sql = "SELECT error_type,error_name,to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS') AS happen_timestamp,
		happen_message,happen_value,device_name,device_id,happen_interface_id,service_name,service_last_stdout
		FROM esia_device
		JOIN esia_error_happen ON device_id = happen_device_id
		JOIN esia_errors ON error_id = happen_error_id ";
		$param = array();
		if(isset($user))
		{
			$sql .= "JOIN esia_group_device_access ON  device_id = access_device_id
			JOIN esia_groups ON access_group_id = group_id
			JOIN esia_memberof ON member_group_id = group_id
			LEFT JOIN esia_services ON service_id = happen_service_id
			WHERE device_type=:type AND member_user_id=:user AND happen_acquitte=0";
			$param['user'] = $user;
		}
		else
		{
			$sql .= "LEFT JOIN esia_services ON service_id = happen_service_id
			WHERE device_type=:type AND member_user_id=? AND happen_acquitte=0";
		}
		$query = $dbconnection->prepare($sql." ORDER BY happen_timestamp DESC;");
		$param['type'] = $type;
		$query = Model_esia::executeStatement($query,$param);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$page_control = Model_esia::getPageControl($dbconnection, $type);
			if($page_control != NULL)
			{
				$page_control = Model_esia::replacePageControlVariable($page_control, $row);
				//$page_control = preg_replace('#:name#',$row->device_name,$page_control);
			}
			$out = preg_replace('#\n#','<br />',$row->happen_message);
			$arr[] = array('type_error' => $row->error_type,
					'type_device' => $type,
					'error' => $row->error_name,
					'time' => $row->happen_timestamp,
					'message' => $out.' value: '.$row->happen_value,
					'name' => $row->device_name,
					'id' => $row->device_id,
					'plugin_name' => $row->service_name,
					 'last_stdout' => $row->service_last_stdout,
					'page_control' => $page_control
			);
		}
		return $arr;
	}

	/**
	 * retourne les programmes d'alerting installé
	 * @param PDO $dbconnection
	 * @retval $arr[<id du program>]['id']=>id du programme,
	 *	   $arr[<id du program>]['name']=>nom du prpgramme
	 *	   $arr[<id du program>]['path']=>chemin d'accès
	 *	   $arr[<id du program>]['default_parameter']=>paramètre par défaut);
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getAlertingSoftware($dbconnection)
	{
		$query = $dbconnection->prepare("SELECT program_id, program_name, program_path, program_default_parameter,program_html_template
				FROM esia_program;");
		$query = Model_esia::executeStatement($query);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->program_id] = array('id' => $row->program_id,
					'name' => $row->program_name,
					'path' => $row->program_path,
					'default_parameter' => $row->program_default_parameter,
					'html' => $row->program_html_template);
		}
		return $arr;
	}

	/**
	 * retourne les mot clés utilisable pour le moteur de recherche
	 * @param PDO $dbconnection
	 * @retval $arr[<keyword>]['keyword']=>mot-clé,
	 *	   $arr[<keyword>]['description']=>description du mot clé
	 *	   $arr[<keyword>]['file']=>chemin d'accès apache pour accèder au fichier
	 *	   $arr[<keyword>]['function']=>function personnalisée de recherche);
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getSearchKeyWords($dbconnection)
	{
		$query = $dbconnection->prepare("SELECT search_keyword, search_description, search_file, search_function,search_name
				FROM esia_search;");
		$query = Model_esia::executeStatement($query);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->search_keyword] = array('keyword' => $row->search_keyword,
					'name' => $row->search_name,
					'description' => $row->search_description,
					'file' => $row->search_file,
					'function' => $row->search_function);
		}
		return $arr;
	}

	/**
	 * count le nombre de service tournant dans l'application  si le paramètre user est défini les droit d'accès (device view) est vérifié afin que le nombre de service corresponde à ce que l'utilisateur à accès
	 * @param PDO $dbconnection
	 * @param integer $user (optionnel) id de l'utilisateur
	 * @retval le nombre de service
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function countService($dbconnection,$user=NULL)
	{
		if($user == NULL)
		{
			$query = $dbconnection->prepare("SELECT count(DISTINCT service_id ) as service_total
					FROM esia_services
					WHERE service_delete=FALSE;");
			$query = Model_esia::executeStatement($query);
		}
		else
		{
			$query = $dbconnection->prepare("SELECT count(DISTINCT service_id) as service_total
					FROM esia_services
					JOIN esia_device ON service_device_id = device_id
					JOIN esia_group_device_access ON  device_id = access_device_id
					JOIN esia_groups ON access_group_id = group_id
					JOIN esia_memberof ON member_group_id = group_id
					WHERE service_delete=FALSE AND member_user_id =?;");
			$query = Model_esia::executeStatement($query,array($user));

		}
		$row = $query->fetch(PDO::FETCH_OBJ);
		return $row->service_total;
	}

	/**
	 * count le nombre de service étant dans un état différent de OK si le paramètre user est défini les droit d'accès (device view) est vérifié afin que le nombre de service corresponde à ce que l'utilisateur à accès
	 * @param PDO $dbconnection
	 * @param integer $user (optionnel) id de l'utilisateur
	 * @retval le nombre de service
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countServiceError($dbconnection,$user=NULL)
	{
		$sql = "SELECT count(DISTINCT happen_id) as service_total_error
				FROM esia_error_happen";
		$params=array();
		if($user == NULL)
		{
			$sql .=" WHERE happen_acquitte=0";

		}
		else
		{

			$sql .=" JOIN esia_device ON happen_device_id = device_id
					 WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND happen_acquitte=0";
			$params[':user']=$user;
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$row = $query->fetch(PDO::FETCH_OBJ);
		return $row->service_total_error;
	}

	/**
	 * count le nombre de service étant dans un état OK si le paramètre user est défini les droit d'accès (device view) est vérifié afin que le nombre de service corresponde à ce que l'utilisateur à accès
	 * @param PDO $dbconnection
	 * @param integer $user (optionnel) id de l'utilisateur
	 * @retval le nombre de service
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countServiceOKState($dbconnection,$user=NULL)
	{
		if($user == NULL)
		{
			$query = $dbconnection->prepare("SELECT count(DISTINCT service_id) as service_total
					FROM esia_services
					WHERE service_previous_state=0 AND service_delete=FALSE;");
			$query = Model_esia::executeStatement($query);
		}
		else
		{
			$query = $dbconnection->prepare("SELECT count(DISTINCT service_id) as service_total
					FROM esia_services
					JOIN esia_device ON service_device_id = device_id
					JOIN esia_group_device_access ON  device_id = access_device_id
					JOIN esia_groups ON access_group_id = group_id
					JOIN esia_memberof ON member_group_id = group_id
					WHERE service_previous_state=0 AND service_delete=FALSE AND member_user_id =?;");
			$query = Model_esia::executeStatement($query,array($user));

		}
		$row = $query->fetch(PDO::FETCH_OBJ);
		return $row->service_total;
	}

	/**
	 * count le nombre de service étant dans les différents états si le paramètre user est défini les droit d'accès (device view) est vérifié afin que le nombre de service corresponde à ce que l'utilisateur à accès
	 * @param PDO $dbconnection
	 * @param array|integer $plugins un plugin_id ou un array contentant une liste de plugin_id
	 * @param integer $user (optionnel) id de l'utilisateur
	 * @throws EsiaDbException
	 * @retval  $list['OK']=<nombre de service dans un état OK>
	 * 			$list["warning"]=<nombre de service dans un état warning>
	 * 			$list["critical"]=<nombre de service dans un état critical>
	 *			$list["unknown"]=<nombre de service dans un état unknown>
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countServiceAllStateByPlugins($dbconnection,$plugins,$user=NULL)
 {
		$list = array();
		$list['OK'] = 0;
		$list["warning"] = 0;
		$list["critical"] = 0;
		$list["unknown"] = 0;
		$list["pending"] = 0;

		if(is_string($plugins)) { $plugins = explode(',',$plugins);
		}

		if(is_array($plugins))
		{
			foreach($plugins as $id)
			{
				if(!is_numeric($id)) {
					throw new EsiaDbException('$plugins must be numeric');
				}
			}
			$arrids = implode(",",$plugins);
		}
		else if(is_numeric($plugins)){
			$arrids = $plugins;
		}
		else{
			throw new EsiaDbException('$plugins must be numeric');
		}

		if($user == NULL){
			$sql = "SELECT error_type,count(DISTINCT happen_id) as nbr
				FROM esia_errors
				JOIN esia_error_happen ON error_id = happen_error_id
				JOIN esia_services ON service_id = happen_service_id
				WHERE happen_acquitte=0
					AND service_plugin_id IN ($arrids)
				GROUP BY error_type
				UNION
				SELECT 'OK' as error_type,count(*) as nbr
				FROM esia_services
				WHERE service_previous_state = 0
					AND service_plugin_id IN ($arrids)
				UNION
				SELECT 'pending',count(*)
				FROM esia_services
				WHERE service_last_stdout IS NULL AND service_previous_state = 4
					AND service_plugin_id IN ($arrids);";
			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query);
		}
		else{
			$sql = "SELECT error_type,count(DISTINCT happen_id) as nbr
				FROM esia_errors
				JOIN esia_error_happen ON error_id = happen_error_id
				JOIN esia_services ON service_id = happen_service_id
				WHERE happen_acquitte=0
					AND service_plugin_id in ($arrids)
					AND service_device_id IN ( SELECT * FROM getUserAccessNode(:user))
				GROUP BY error_type
				UNION
				SELECT 'OK' as error_type,count(*) as nbr
				FROM esia_services
				WHERE service_previous_state = 0
					AND service_plugin_id IN ($arrids)
					AND service_device_id IN ( SELECT * FROM getUserAccessNode(:user))
				UNION
				SELECT 'pending',count(*)
				FROM esia_services
				WHERE service_last_stdout IS NULL AND service_previous_state = 4
					AND service_plugin_id IN ($arrids)
					AND service_device_id IN ( SELECT * FROM getUserAccessNode(:user));";
			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,array(':user' => $user));
		}
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if($row->error_type == 'maintenance' || $row->error_type == 'pending')
			{
				$list['pending'] = $row->nbr + $list['pending'];
			}
			else
			{
				$list[$row->error_type] = $row->nbr;
			}
		}
		return $list;
	}

	/**
	 * count le nombre de service étant dans les différents états si le paramètre user est défini les droit d'accès (device view) est vérifié afin que le nombre de service corresponde à ce que l'utilisateur à accès
	 * @param PDO $dbconnection
	 * @param integer $user (optionnel) id de l'utilisateur
	 * @param array|integer $ids service_id ou un array contentant une liste de service_id
	 * @throws EsiaDbException
	 * @retval $list['OK']=<nombre de service dans un état OK>
	 * 		   $list["warning"]=<nombre de service dans un état warning>
	 *		   $list["critical"]=<nombre de service dans un état critical>
	 *		   $list["unknown"]=<nombre de service dans un état unknown>
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countServiceAllState($dbconnection,$user=NULL,$ids=NULL,$separePending=FALSE)
	{
		$list = array();
		$list['OK'] = 0;
		$list["warning"] = 0;
		$list["critical"] = 0;
		$list["unknown"] = 0;
		$list["pending"] = 0;
		if($separePending){
			$list['waiting'] = 0;
		}

		if($user == NULL)
		{
			$query = $dbconnection->prepare("SELECT error_type,count(DISTINCT happen_id) as nbr
                                        FROM esia_errors
                                        JOIN esia_error_happen ON error_id = happen_error_id
                                        WHERE happen_acquitte=0
                                        GROUP BY error_type
                                        UNION
                                        SELECT 'OK',count(*)
                                        FROM esia_services
                                        WHERE service_previous_state = 0
                                        UNION
                                        SELECT 'pending',count(*)
                                        FROM esia_services
                                        WHERE service_last_stdout IS NULL AND service_previous_state = 4;");
			$query = Model_esia::executeStatement($query);
		}
		else
		{
			if(is_array($ids))
			{
				foreach($ids as $id)
				{
					if(!is_numeric($id)) {
						throw new EsiaDbException('$ids must be numeric');
					}
				}
				$arrids = implode(",",$ids);
				$query = $dbconnection->prepare("SELECT error_type,count(DISTINCT happen_id) as nbr
						FROM esia_errors
						JOIN esia_error_happen ON error_id = happen_error_id
						WHERE happen_acquitte=0
						AND happen_device_id IN ( SELECT * FROM getUserAccessNode(:user))
						AND happen_service_id IN ($arrids)
						GROUP BY error_type
						UNION
						SELECT 'OK',count(*)
						FROM esia_services
						WHERE service_previous_state = 0
						AND service_device_id IN ( SELECT * FROM getUserAccessNode(:user))
						AND service_id IN ($arrids)
						UNION
						SELECT 'pending',count(*)
						FROM esia_services
						WHERE service_last_stdout IS NULL
						AND service_previous_state = 4
						AND service_device_id IN ( SELECT * FROM getUserAccessNode(:user))
						AND service_id IN ($arrids);");
				$query = Model_esia::executeStatement($query,array(':user' => $user));
			}
			else
			{
				$query = $dbconnection->prepare("SELECT sum(ok) AS ok, sum(warning) AS warning, sum(critical) AS critical, sum(unknown) AS unknown, sum(acq) as acquitte, sum(maintenance) as maintenance FROM view_node_state  WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user));");
				$query = Model_esia::executeStatement($query,array(':user' => $user));
				$row = $query->fetch(PDO::FETCH_OBJ);
				$list['OK'] = $row->ok;
				$list['warning'] = $row->warning;
				$list['critical'] = $row->critical;
				$list['unknown'] = $row->unknown;
				$list['maintenance'] = $row->maintenance;
				$list['pending'] = $row->maintenance + $row->acquitte;
				//On va recherche les services en attente d'exécution
				$query = $dbconnection->prepare("SELECT count(*) as nbr
                                                FROM esia_services
                                                WHERE service_last_stdout IS NULL AND service_previous_state = 4 AND service_device_id IN ( SELECT * FROM getUserAccessNode(:user))");
				$query = Model_esia::executeStatement($query,array(':user' => $user));
				$row = $query->fetch(PDO::FETCH_OBJ);
				$list['waiting'] = $row->nbr;
			}
		}
		if(empty($user) || !empty($ids))
		{
			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				if($separePending && $row->error_type == 'maintenance'){
					$list['pending'] += $row->nbr; //En maintenance (historical reasons name)
				}
				else if($separePending && $row->error_type == 'pending'){
					$list['waiting'] += $row->nbr; //En attente
				}
				else if($row->error_type == 'maintenance' || $row->error_type == 'pending')
				{
					$list['pending'] += $row->nbr;
				}
				else
				{
					$list[$row->error_type] = $row->nbr;
				}
			}
		}
		return $list;
	}


	/**
	 * count le nombre de service étant dans un état OK sur le noeud donné
	 * @param PDO $dbconnection
	 * @param integer $id id du noeud
	 * @retval nombre de service dans un état OK
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countNodeOkState($dbconnection,$id)
	{
		if(is_numeric($id))
		{
			$query = $dbconnection->prepare("SELECT count(DISTINCT service_id) as service_total
					FROM esia_services
					WHERE service_previous_state=0 AND service_delete=FALSE AND service_device_id=?;");
		}
		else
		{
			$query = $dbconnection->prepare("SELECT count(DISTINCT service_id) as service_total
					FROM esia_services
					JOIN esia_device ON service_device_id = device_id
					WHERE service_previous_state=0 AND service_delete=FALSE AND device_name =?;");
		}
		$query = Model_esia::executeStatement($query,array($id));
		$row = $query->fetch(PDO::FETCH_OBJ);
		return $row->service_total;
	}
	/**
	 * @brief retourne l'état d'un (des) passé en paramètre
	 * @param PDO $dbconnection Paramètre de connexion à la base de donnée
	 * @param interger |interger[]|string $ids identifiant d'un noeud ou un tableau d'identifant ou le nom d'un noeud
	 * @throws EsiaDbException
	 * @retval hasmap[] $list['OK']=<nombre de service dans un état OK>
						$list["warning"]=<nombre de service dans un état warning>
						$list["critical"]=<nombre de service dans un état critical>
						$list["unknown"]=<nombre de service dans un état unknown>
						$list["pending"]=<nombre de service dans un état unknown>
	 */
	static function countNodeState($dbconnection,$ids)
	{
		$list = array();
		$list['OK'] = 0;
		$list["warning"] = 0;
		$list["critical"] = 0;
		$list["unknown"] = 0;
		$list["pending"] = 0;
		$list["maintenance"] = 0;
		$list["acquitte"] = 0;
		$params = array();
		if(is_numeric($ids))
		{
			$query = $dbconnection->prepare("SELECT warning, critical,unknown,ok, acq AS acquitte,maintenance, 
						(SELECT count(*) as nbr_error
						 FROM esia_services
						 WHERE service_device_id=:id AND service_last_stdout IS NULL AND service_previous_state=4
						) as pending
					FROM view_node_state
					WHERE device_id=:id;");
			$params['id'] = $ids;
		}
		elseif (is_array($ids))
		{
			foreach($ids as $id)
			{
				if(!is_numeric($id)) {
					throw new EsiaDbException('$service array must be numeric');
				}
			}
			$arrids = implode(',', $ids);
			$query = $dbconnection->prepare("
					SELECT SUM(warning) as warning,	
							SUM(critical) AS critical,
							SUM(unknown) AS unknown,
							SUM(ok) AS ok,
							SUM(acq) AS acquitte,
						(SELECT count(*) as nbr_error
						 FROM esia_services
						 WHERE service_device_id IN ($arrids) AND service_last_stdout IS NULL AND service_previous_state=4
						) as pending
					FROM view_node_state
					WHERE device_id IN ($arrids);");
		}
		else
		{
			$query = $dbconnection->prepare("
					SELECT warning, critical,unknown,ok, acq AS acquitte,maintenance, 
						(SELECT count(*) as nbr_error
						 FROM esia_services
						 WHERE service_device_id IN (SELECT device_id FROM esia_device WHERE device_name=:name )
							 AND service_last_stdout IS NULL AND service_previous_state=4
						) as pending
					FROM view_node_state
					WHERE device_id=IN (SELECT device_id FROM esia_device WHERE device_name=:name );");
			$params['name'] = $ids;
		}
		$query = Model_esia::executeStatement($query,$params);
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$list['warning'] = $row->warning;
			$list['critical'] = $row->critical;
			$list['unknown'] = $row->unknown;
			$list['OK'] = $row->ok;
			$list['acquitte'] = $row->acquitte;
			$list['maintenance'] = $row->maintenance;
			$list['pending'] = $row->pending;
		}
		return $list;
	}

	/**
	 * count le nombre de service exécuté sur le noeud donné
	 * @param PDO $dbconnection
	 * @param integer $id id du noeud
	 * @retval nombre de service
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countNodeService($dbconnection, $id )
	{
		if(is_numeric($id))
		{
			$query = $dbconnection->prepare("SELECT count(*)as nbr
					FROM esia_services
					WHERE service_device_id=?;");
		}
		else
		{
			$query = $dbconnection->prepare("SELECT count(*)as nbr
					FROM esia_device
					JOIN esia_services ON device_id = service_device_id
					WHERE device_name=?;");
		}
		$query = Model_esia::executeStatement($query,array($id));
		$r = $query->fetch(PDO::FETCH_OBJ);
		return $r->nbr;

	}

	/**
	 * compte le nombre de noeud dans chaque type (table esia_device_type), si le paramètre user est spécifié les droits d'accès s'applique le paramètre type permet de compléter la requête avec l'état global des noeuds
	 * @param PDO $dbconnection
	 * @param integer $user (optionnel) id de l'utilisateur
	 * @param integer $type (optionnel) Model_esia::NODE_COUNT_SERVICE_STATE retourne en plus l'état global du groupe (constante definie)
	 * 									Model_esia::NODE_COUNT_SERVICE compte le nombre de service exécuté si laissé à null retourne juste le nombre de device
	 * @retval un tableau associatif
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countNodeType($dbconnection,$user=NULL,$type=NULL)
	{
		$arr = array();
		$query = '';
		$ret = FALSE;
		if(!empty($user))
		{
			$query = $dbconnection->prepare("SELECT device_type,count(DISTINCT device_id)as nbr
					FROM esia_device
					JOIN esia_group_device_access ON  device_id = access_device_id
					JOIN esia_groups ON access_group_id = group_id
					JOIN esia_memberof ON member_group_id = group_id
					WHERE member_user_id=?
					GROUP BY device_type; ");
			$query = Model_esia::executeStatement($query,array($user));
		}
		else
		{
			$query = $dbconnection->prepare("SELECT device_type,count(*)as nbr
					FROM esia_device
					GROUP BY device_type; ");
			$query = Model_esia::executeStatement($query);
		}
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->device_type] = array('nbr_device' => $row->nbr,'ok' => 0,'warning' => 0,'critical' => 0,'unknown' => 0,'nbr_service' => 0);
		}
		if($type == Model_esia::NODE_COUNT_SERVICE_STATE)
		{
			if(!empty($user))
			{
				$query = $dbconnection->prepare("SELECT device_type,error_type,count(DISTINCT happen_id)as nbr_error
						FROM esia_device
						JOIN esia_group_device_access ON  device_id = access_device_id
						JOIN esia_groups ON access_group_id = group_id
						JOIN esia_memberof ON member_group_id = group_id
						JOIN esia_error_happen ON device_id = happen_device_id
						JOIN esia_errors ON error_id = happen_error_id
						WHERE happen_acquitte=0 AND member_user_id=:user
						GROUP BY device_type,error_type
						UNION
						SELECT device_type,'ok',count(DISTINCT service_id)as nbr_error
						FROM esia_device
						JOIN esia_group_device_access ON  device_id = access_device_id
						JOIN esia_groups ON access_group_id = group_id
						JOIN esia_memberof ON member_group_id = group_id
						JOIN esia_services ON service_device_id = device_id
						WHERE service_previous_state=0 AND service_delete=FALSE	 AND member_user_id=:user
						GROUP BY device_type;");
				$query = Model_esia::executeStatement($query,array(':user' => $user));
			}
			else
			{
				$query = $dbconnection->prepare("SELECT device_type,error_type,count(*)as nbr_error
						FROM esia_device
						JOIN esia_error_happen ON device_id = happen_device_id
						JOIN esia_errors ON error_id = happen_error_id
						WHERE happen_acquitte=0
						GROUP BY device_type,error_type
						UNION
						SELECT device_type,'ok',count(*)as nbr_error
						FROM esia_device
						JOIN esia_services ON service_device_id = device_id
						WHERE service_previous_state=0 AND service_delete=FALSE
						GROUP BY device_type");
				$query = Model_esia::executeStatement($query);
			}
			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				$arr[$row->device_type][$row->error_type] = $row->nbr_error;
			}
			$type = Model_esia::NODE_COUNT_SERVICE;
		}
		if($type == Model_esia::NODE_COUNT_SERVICE)
		{
			if(!empty($user))
			{
				$query = $dbconnection->prepare("SELECT device_type , count(DISTINCT service_id)as nbr
						FROM esia_device
						JOIN esia_services ON device_id = service_device_id
						JOIN esia_group_device_access ON  device_id = access_device_id
						JOIN esia_groups ON access_group_id = group_id
						JOIN esia_memberof ON member_group_id = group_id
						WHERE member_user_id=?
						GROUP BY device_type;");
				$query = Model_esia::executeStatement($query,array($user));
			}
			else
			{
				$query = $dbconnection->prepare("SELECT device_type ,  count(*)as nbr
						FROM esia_device
						JOIN esia_services ON device_id = service_device_id
						GROUP BY device_type");
				$query = Model_esia::executeStatement($query);
			}
			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				$arr[$row->device_type]['nbr_service'] = $row->nbr;
			}
		}
		return $arr;
	}

	/**
	 * retourne l'état de chaque groupe de noeud si le paramètre user est spécifié les droit d'accès seront vérifié
	 * @param PDO $dbconnection
	 * @param integer $user (optionnel) id de l'utilisateur
	 * @retval un tableau associatif
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countStateByGroupname($dbconnection,$user=NULL)
	{
		$ret = '';
		if(!empty($user))
		{
			$query = $dbconnection->prepare("SELECT lower(device_group_name) as groupname ,error_type,count(DISTINCT happen_id) as nbr
					FROM esia_errors
					JOIN esia_error_happen ON error_id = happen_error_id
					JOIN esia_device ON device_id = happen_device_id
					JOIN esia_group_device_access ON  device_id = access_device_id
					JOIN esia_groups ON access_group_id = group_id
					JOIN esia_memberof ON member_group_id = group_id
					WHERE happen_acquitte=0 AND member_user_id=:user
					GROUP BY device_group_name,error_type

					UNION

					SELECT lower(device_group_name) as groupname ,'ok',count(DISTINCT service_id) as nbr
					FROM esia_device
					JOIN esia_services ON device_id = service_device_id
					JOIN esia_group_device_access ON  device_id = access_device_id
					JOIN esia_groups ON access_group_id = group_id
					JOIN esia_memberof ON member_group_id = group_id
					WHERE service_previous_state=0 AND member_user_id=:user
					GROUP BY device_group_name

					UNION

					SELECT  lower(device_group_name) as groupname ,'ok',0 as nbr
					FROM esia_device
					JOIN esia_group_device_access ON  device_id = access_device_id
					JOIN esia_groups ON access_group_id = group_id
					JOIN esia_memberof ON member_group_id = group_id
					WHERE member_user_id=:user AND NOT EXISTS(SELECT service_device_id FROM esia_services WHERE service_device_id = device_id ) ORDER BY groupname,error_type,nbr ");
			$query = Model_esia::executeStatement($query,array(':user' => $user));
		}
		else
		{
			$query = $dbconnection->prepare("SELECT lower(device_group_name) as groupname ,error_type,count(*) as nbr
					FROM esia_errors
					JOIN esia_error_happen ON error_id = happen_error_id
					JOIN esia_device ON device_id = happen_device_id
					WHERE happen_acquitte=0
					GROUP BY device_group_name,error_type


					UNION

					SELECT lower(device_group_name) as groupname ,'ok',count(*) as nbr
					FROM esia_device
					JOIN esia_services ON device_id = service_device_id
					WHERE service_previous_state=0
					GROUP BY device_group_name

					UNION

					SELECT  device_group_name ,'ok',0 as nbr
					FROM esia_device
					WHERE  NOT EXISTS(SELECT service_device_id FROM esia_services WHERE service_device_id = device_id ) ORDER BY groupname,error_type,nbr ");
			$query = Model_esia::executeStatement($query);
		}
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->groupname][$row->error_type] = $row->nbr;
			if(!array_key_exists("ok", $arr[$row->groupname]))
			{
				$arr[$row->groupname]['ok'] = 0;
			}
			if(!array_key_exists("warning", $arr[$row->groupname]))
			{
				$arr[$row->groupname]['warning'] = 0;
			}
			if(!array_key_exists("critical", $arr[$row->groupname]))
			{
				$arr[$row->groupname]['critical'] = 0;
			}
			if(!array_key_exists("unknown", $arr[$row->groupname]))
			{
				$arr[$row->groupname]['unknown'] = 0;
			}
		}
		ksort($arr);
		return $arr;
	}

	/**
	 * retourne l'état du groupe passé en paramètre en vérifiant les droits de l'utilisateurs
	 * @param PDO $dbconnection
	 * @param string|integer $user le nom ou l'id de l'utilisateur
	 * @param string|integer $group le nom ou l'id de du groupe
	 * @param PDOStatement $preparedrequest (optionnelle) requete sql preprare (performance)
	 * @retval un tableau associatifs contenant les noms des groupes de device
	 *		$list['OK']=0;
	 *		$list["warning"]=0;
	 *		$list["critical"]=0;
	 *		$list["unknown"]=0;
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroupNodeState($dbconnection,$user,$group,$preparedrequest=NULL)
	{
		if($group == 'noGroup')
		{
			$sql = "SELECT DISTINCT error_type,count(*)as nbr_error
					FROM getDeviceInGroup(:userid,NULL) AS T2
					JOIN esia_error_happen ON T2.device_id = happen_device_id
					JOIN esia_errors ON error_id = happen_error_id
					WHERE happen_acquitte=0
					GROUP BY error_type

					UNION

					SELECT 'OK' as error_type,count(DISTINCT service_id) as nbr_error
					FROM getDeviceInGroup(:userid,NULL) AS T2
					JOIN esia_services ON service_device_id = T2.device_id
					WHERE service_previous_state=0 AND service_delete=FALSE";
			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,array('userid' => $user));
		}
		else
		{
			if(isset($preparedrequest))
			{
				$query = $preparedrequest;
			}
			else
			{
				$query = Model_esia::prepared_getGroupNodeState($dbconnection);
			}
			$query = Model_esia::executeStatement($query,array('userid' => $user,'groupid' => $group));
		}
		//Model_esia::countNodeState($dbconnection,$id);
		$list['OK'] = 0;
		$list["warning"] = 0;
		$list["critical"] = 0;
		$list["unknown"] = 0;
		$list["pending"] = 0;
		//$result = $query->fetchAll();

		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
	//		print_r($row);
			//$list[$row->error_type]= $list[$row->error_type] + $row->nbr_error;
			$list['OK'] = $row->ok;
			$list["warning"] = $row->warning;
			$list["critical"] = $row->critical;
			$list["unknown"] = $row->unknown;
			//$list["pending"]=$row->pending;
			/*switch($row->error_type)
			{
				case 'OK': $list['OK']= $list['OK'] + $row->nbr_error; break;
				case 'warning': $list['warning']= $list['warning'] + $row->nbr_error; break;
				case 'critical': $list['critical']= $list['critical'] + $row->nbr_error; break;
				case 'unknown': $list['unknown']= $list['unknown'] + $row->nbr_error; break;
				default: $list["pending"]=$list["pending"]+ $row->nbr_error; break;
			}*/

		}
		return $list;
	}
	/**
	 * retourne groupe de noeud
	 * @param PDO $dbconnection
	 * @param integer $id du group node
	 * @param string $name du group node
	 * @retval retourne list des groupes de noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	
	static function getGroupNode($dbconnection,$id=null,$name=null)
	{
		$sql = "SELECT device_group_id,device_group_name,device_group_description,device_group_parent,device_group_path
		FROM esia_device_group ";
		
		$param=array();
		
		if(isset($id))
		{
			if(is_numeric($id))
			{
				$query = $dbconnection->prepare($sql.'WHERE device_group_id = :id;');
				$param[]=$id;
			}
			elseif(is_array($id))
			{
				$column = 'device_group_id';
				$in_sql = '';
				foreach($id as $item)
				{
					$param[':'.$column.$i]=$item;
					$in_sql .= ',:'.$column.$i;
					$i++;
				}
				$in_sql = ltrim($in_sql,',');
				$query = $dbconnection->prepare($sql."WHERE device_group_id IN ($in_sql)");
			}
		}
		else
		{
			$query = $dbconnection->prepare($sql.'WHERE device_group_name= :name;');
			$param[]=$name;
		}
		$query = model_esia::executeStatement($query, $param);
		$list_node=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_node[]=array('id'=>$row->device_group_id,
				'name'=> $row->device_group_name,
				'description'=> $row->device_group_description,
				'fullpath'=>$row->device_group_path,
				'parent'=> $row->device_group_parent);
		}
		
		return $list_node;
	}
	/**
	 * retourne l'état des groupe
	 * @param PDO $dbconnection
	 * @retval resulat de la requete sql
	 * @exception EsiaDbException si une erreur survient
	 */
	function prepared_getGroupNodeState($dbconnection)
	{
		/*$sql = "SELECT DISTINCT error_type,count(*)as nbr_error
					FROM getDeviceInGroup(:userid,:groupid) AS T2
					JOIN esia_error_happen ON T2.device_id = happen_device_id
					JOIN esia_errors ON error_id = happen_error_id
					WHERE happen_acquitte=0
					GROUP BY error_type

					UNION

					SELECT 'OK' as error_type,count(DISTINCT service_id) as nbr_error
					FROM getDeviceInGroup(:userid,:groupid) AS T2
					JOIN esia_services ON service_device_id = T2.device_id
					WHERE service_previous_state=0 AND service_delete=FALSE

					UNION
					SELECT 'pending',count(*) as nbr_error
					FROM getDeviceInGroup(:userid,:groupid) AS T2
					JOIN esia_services ON service_device_id = T2.device_id
					WHERE service_last_stdout IS NULL AND service_previous_state=4";*/
		$sql = "SELECT T2.group_in_device_group_id as group_id,SUM(T2.warning) as warning,SUM(T2.critical)  as critical,SUM(T2.unknown) as unknown,SUM(t2.ok) as OK
			FROM (
					SELECT device_id,group_in_device_group_id,
							( SELECT DISTINCT count(*)as warning
							 FROM esia_error_happen
							 JOIN esia_errors ON error_id = happen_error_id
							 WHERE happen_acquitte=0 AND happen_device_id = T1.device_id AND error_type='warning'),
							( SELECT count(*)as critical
											FROM esia_error_happen
											JOIN esia_errors ON error_id = happen_error_id
											WHERE happen_acquitte=0 AND happen_device_id = T1.device_id AND error_type='critical'),
							( SELECT count(*)as unknown
											FROM esia_error_happen
											JOIN esia_errors ON error_id = happen_error_id
											WHERE happen_acquitte=0 AND happen_device_id = T1.device_id AND error_type='unknown'),
							(SELECT count(DISTINCT service_id) as OK
							FROM esia_services
							WHERE service_previous_state=0 AND service_delete=FALSE AND service_device_id = T1.device_id)

					FROM esia_device AS T1
					JOIN esia_device_group_in ON T1.device_id = group_in_device_id
					WHERE T1.device_id IN ( SELECT * FROM getUserAccessNode(:userid)) AND device_delete=FALSE AND group_in_device_group_id = :groupid
					) AS T2
				JOIN esia_device_group ON device_group_id = T2.group_in_device_group_id
				GROUP BY T2.group_in_device_group_id";

		return $dbconnection->prepare($sql);
	}
	/**
	 * retourne l'état du groupe passé en paramètre en vérifiant les droits de l'utilisateurs
	 * @param PDO $dbconnection
	 * @param string|integer $user le nom ou l'id de l'utilisateur
	 * @param string|integer $group le nom ou l'id de du groupe
	 * @param PDOStatement $preparedrequest (optionnelle) requete sql preprare (performance)
	 * @retval un tableau associatifs contenant les noms des groupes de device
	 *		$list['OK']=0;
	 *		$list["warning"]=0;
	 *		$list["critical"]=0;
	 *		$list["unknown"]=0;
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroupNodeStates($dbconnection,$user,$group,$child=TRUE,$notReturnStante=array())
	{
		if(is_numeric($group))
		{
			$sql ="";
			if($child === FALSE)
			{
				$sql = 'SELECT SUM(warning) as warning,SUM(critical) as critical,SUM(unknown) as unknown,SUM(ok) as OK, SUM(maintenance) as maintenance,SUM(acq) as acq
						FROM esia_device_group
						LEFT JOIN esia_device_group_in ON device_group_id = group_in_device_group_id
						LEFT JOIN view_node_state AS T1 ON T1.device_id = group_in_device_id
						WHERE T1.device_id IN ( SELECT * FROM getUserAccessNode(:userid))
							AND group_in_device_group_id IN (SELECT group_id FROM getAllChildDeviceGroup(:groupid))
						GROUP BY device_group_id,device_group_parent
                                                ORDER BY device_group_parent DESC';
			}
			else
			{
				$sql = 'WITH accessgroup AS (
						SELECT device_group_id,device_group_name,device_group_description,device_group_parent,device_group_path,
								SUM(warning) as warning,SUM(critical)  as critical,
					            SUM(unknown) as unknown,SUM(ok) as ok, SUM(maintenance) as maintenance,SUM(acq) as acq, 0 as pending
						 FROM esia_device_group
						 LEFT JOIN esia_device_group_in ON device_group_id = group_in_device_group_id
						 LEFT JOIN view_node_state AS T1 ON T1.device_id = group_in_device_id
						 WHERE (
							 	T1.device_id IN ( SELECT * FROM getUserAccessNode(:userid))
								OR
								T1.device_id IS NULL
						 		)
								AND device_group_id IN (SELECT group_id FROM getAllChildDeviceGroup(:groupid))
						GROUP BY device_group_id,device_group_parent
					)
					SELECT *
					FROM (
						SELECT *
						FROM accessgroup
						UNION
						SELECT device_group_id,device_group_name,device_group_description,device_group_parent,device_group_path,null,null,null,null,null,null,null
						FROM esia_device_group
						WHERE
							device_group_id IN (SELECT device_group_parent FROM accessgroup)
							AND
							device_group_id NOT  IN (SELECT device_group_id  FROM accessgroup)
						) as Tfinal
					ORDER BY Tfinal.device_group_parent DESC';
			}
			$sql =
			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,array('userid' => $user,'groupid' => $group));
		}
		else
		{
				$sql = 'WITH accessgroup AS (
						SELECT device_group_id,device_group_name,device_group_description,device_group_parent,device_group_path,
								SUM(warning) as warning,SUM(critical)  as critical,
					            SUM(unknown) as unknown,SUM(ok) as ok, SUM(maintenance) as maintenance,SUM(acq) as acq, 0 as pending
						 FROM esia_device_group
						 LEFT JOIN esia_device_group_in ON device_group_id = group_in_device_group_id
						 LEFT JOIN view_node_state AS T1 ON T1.device_id = group_in_device_id
						 WHERE 	T1.device_id IN ( SELECT * FROM getUserAccessNode(:userid))
								OR
								T1.device_id IS NULL
						GROUP BY device_group_id,device_group_parent
					)
					SELECT *
					FROM (
						SELECT *
						FROM accessgroup
						UNION
						SELECT device_group_id,device_group_name,device_group_description,device_group_parent,device_group_path,null,null,null,null,null,null,null
						FROM esia_device_group
						WHERE
							device_group_id IN (SELECT device_group_parent FROM accessgroup)
							AND
							device_group_id NOT  IN (SELECT device_group_id  FROM accessgroup)
						) as Tfinal
					ORDER BY Tfinal.device_group_parent DESC';
			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,array('userid' => $user));
		}
		//Model_esia::countNodeState($dbconnection,$id);

		$arr = array();
		$result = $query->fetchAll(PDO::FETCH_ASSOC);
		//on ne veux que l'état du groupe passé en paramètre
		if($child === FALSE)
		{
			$row = $result[0];
			$arr[$group] = array('ok' => $row['ok'],
									'warning' => $row['warning'],
									'critical' => $row['critical'],
									'unknown' => $row['unknown'],
									'maintenance' => $row['maintenance'] + $row['acq']
			);
		}
		else
		{
			//on sélectionne les fils de premier niveau
			foreach($result as $row)
			{
				if(is_numeric($group))
				{
					//on prend les fils du parents
					if($row['device_group_parent'] == $group)
					{
							//on récuère l'état du sous fils.
							$status = array();
							if(!in_array($row['device_group_parent'],$notReturnStante))
							{
								$status = model_esia::returnGroupNodeState($result, $row['device_group_id']);
							}
							else
							{
								$status['ok'] = 1;
								$status['warning'] = 0;
								$status['critical'] = 0;
								$status['unknown'] = 0;
								$status['pending'] = 0;
								$status['maintenance'] = 0;
								$status['child'] = 2;
							}
							$arr[] = array('id' => $row['device_group_id'],
								'name' => $row['device_group_name'],
								'description' => $row['device_group_description'],
								'parent' => $row['device_group_parent'],
								'fullpath'=>$row['device_group_path'],
								'ok' => $status['ok'],
								'warning' => $status['warning'],
								'critical' => $status['critical'],
								'unknown' => $status['unknown'],
								'pending' => $status['pending'],
								'maintenance' => $status['maintenance'] + $status['acq'],
								'child' => $status["child"]	);

					}
				}
				else
				{
					if(empty($row['device_group_parent']))
					{
						$state = array();
						if(!in_array($row['device_group_id'],$notReturnStante))
						{
							$state = model_esia::returnGroupNodeState($result, $row['device_group_id']);
						}
						else
						{
							$state['ok'] = 1;
							$state['warning'] = 0;
							$state['critical'] = 0;
							$state['unknown'] = 0;
							$state['pending'] = 0;
							$state['maintenance'] = 0;
							$state['child'] = 2;
						}
						$arr[] = array('id' => $row['device_group_id'],
							'name' => $row['device_group_name'],
							'description' => $row['device_group_description'],
							'fullpath'=>$row['device_group_path'],
							'parent' => $row['device_group_parent'],
							'ok' => $state['ok'],
							'warning' => $state['warning'],
							'critical' => $state['critical'],
							'unknown' => $state['unknown'],
							'pending' => $state['pending'],
							'maintenance' => $state['maintenance'] + $state['acq'],
							'child' => $state["child"]	);
					}
				}
			}
		}
		return $arr;
	}
	private static function returnGroupNodeState(&$table,$groupid)
	{
		$list['ok'] = 0;
		$list["warning"] = 0;
		$list["critical"] = 0;
		$list["unknown"] = 0;
		$list["pending"] = 0;
		$list["maintenance"] = 0;
		$list["acq"] = 0;
		$list["child"] = NULL;
		$arr = array();

		foreach($table as $key => $row)
		{
			if($row['device_group_parent'] == $groupid)
			{
				$state = model_esia::returnGroupNodeState($table, $row['device_group_id']);

				$arr[] = array('id' => $row['device_group_id'],
						'name' => $row['device_group_name'],
						'description' => $row['device_group_description'],
						'parent' => $row['device_group_parent'],
						'fullpath' => $row['device_group_path'],
						'ok' => $state['ok'],
						'warning' => $state['warning'],
						'critical' => $state['critical'],
						'unknown' => $state['unknown'],
						'maintenance' => $state['maintenance'],
						'acq' => $state['acq'],
						
						'pending' => $row['pending'] + $state['pending'],
						'child' => $state["child"]	);
				$list['ok'] += $state['ok'];
				$list['warning'] += $state['warning'];
				$list['critical'] += $state['critical'];
				$list['unknown'] += $state['unknown'];
				$list['maintenance'] += $state['maintenance'];
				$list['acq'] += $row['acq'];
				$list['pending'] += $row['pending'];
				unset($table[$key]);
			}
			elseif($row['device_group_id'] == $groupid)
			{
				$list['ok'] += $row['ok'];
				$list['warning'] += $row['warning'];
				$list['critical'] += $row['critical'];
				$list['unknown'] += $row['unknown'];
				$list['maintenance'] += $row['maintenance'];
				$list['pending'] += $row['pending'];
				unset($table[$key]);
			}
		}

		$list["child"] = $arr;
		return $list;
	}
	/**
	 * retourne l'état de chaque groupe de noeud si le paramètre user est spécifié les droit d'accès seront vérifié
	 * @param PDO $dbconnection
	 * @param integer $user id de l'utilisateur
	 * @retval un tableau associatif
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countStateAllNode($dbconnection,$user,$nodes=NULL)
	{
		$arr = array('ok' => 0,'warning' => 0,'critical' => 0,'unknown' => 0,'pending' => 0);
		if(!empty($nodes))
		{
			if(is_numeric($nodes))
			{
				$query = $dbconnection->prepare("SELECT * FROM view_node_state WHERE device_id IN ( SELECT * FROM getUserAccessNode(?)) AND device_id = ?");
				$query = Model_esia::executeStatement($query,array($user,$nodes));
			}
			elseif(is_array($nodes))
			{
				foreach($nodes as $id)
				{
					if(!is_numeric($id)) {
						throw new EsiaDbException('$nodes array must be numeric');
					}
				}
				$arrids = implode(',', $nodes);
				$query = $dbconnection->prepare("SELECT * FROM view_node_state WHERE device_id IN ( SELECT * FROM getUserAccessNode(?)) AND device_id IN ($arrids);");
				$query = Model_esia::executeStatement($query,array($user));
			}
			else
			{
				throw new EsiaDbException('$nodes must be numeric');
			}
			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
			//	$state = Model_esia::countNodeState($dbconnection,$line['id']);
				if($row->critical != 0)
				{
					$arr['critical']++;
				}
				elseif($row->warning != 0)
				{
					$arr['warning']++;
				}
				elseif($row->unknown != 0)
				{
					$arr['unknown']++;
				}
				else
				{
					$arr['ok']++;
				}
			}
		}
		else
		{
			$query = $dbconnection->prepare("SELECT * FROM getNodesState(?)");
			$query = Model_esia::executeStatement($query,array($user));
			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				$arr[$row->typerror] = $row->nbr;
			}
		}
		return $arr;
	}

	/**
	 * calcul le pourcentage de disponnibilité depuis la date d'ajout du service
	 * @param PDO $dbconnection
	 * @param object|integer $node id ou un objet Device du noeud
	 * @param integer $service id du service
	 * @throws EsiaDbException
	 * @retval le pourcentage de disponibilité
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countPourcentAvailabilityService($dbconnection,$node,$service)
	{
		if(@get_class($node) == 'Device')
		{
			$node = $node->getId();
		}
		elseif(@get_class($node) == 'RemoteDevice')
		{
				$node = $node->getId();
		}
		if(is_numeric($node) && is_numeric($service))
		{

			//récupération de la date d'ajout du service
			$query = $dbconnection->prepare("SELECT service_name,date_part('epoch', service_time_added)::int AS time_added
					FROM esia_services
					WHERE service_device_id = ? AND service_id=? ");
			$query = Model_esia::executeStatement($query,array($node,$service));
			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				$timeStart = $row->time_added;
			}
			//récupération de la date de début et de fin de chaque erreur du service
			$query = $dbconnection->prepare("SELECT date_part('epoch', happen_timestamp)::int AS time_happen,
					date_part('epoch', happen_time_acquitte)::int AS time_acquitte
					FROM esia_error_happen
					WHERE happen_device_id = ? AND happen_service_id=? ");
			$query = Model_esia::executeStatement($query,array($node,$service));
			$timeNotavailable = 0;
			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				if(empty($row->time_acquitte)) //si l'erreur n'est pas encore acquittée
				{
					$timeNotavailable = $timeNotavailable + (time() - $row->time_happen);
				}
				else
				{
					$timeNotavailable = $timeNotavailable + ($row->time_acquitte - $row->time_happen);
				}
			};
			//calcul le nombre de seconde écoulée depuis l'ajout du service
			$timeLive = time() - $timeStart;
			if($timeNotavailable != 0) {
				$pourcent = 100 - (($timeNotavailable / $timeLive) * 100);
			} else {
				$pourcent = 100;
			}
			return $pourcent;
		}
		else
		{
			throw new EsiaDbException("id is not numeric or not equals node name");
		}
	}

	/**
	 * le pourcentage de disponibilité d'un service sur les 12 dernier mois
	 * @param PDO $dbconnection
	 * @param object|integer $node id ou un objet Device du noeud
	 * @param integer $service id du service
	 * @retval $list[]['year']=année
	 *		   $list[]["month"]=le mois
	 *		   $list[]["pourcent"]=% de disponibilité sur le mois indiqué
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countPourcentAvailabilityServiceByMonth($dbconnection,$node,$service)
	{
		//récupération de la date de début et de fin de chaque erreur du service
		$query = $dbconnection->prepare("SELECT SUM (sec_in_error) as time_notavailable FROM (
				SELECT date_part('epoch',SUM(happen_time_acquitte - happen_timestamp)) as sec_in_error
				FROM esia_error_happen
				WHERE happen_service_id = :service AND happen_device_id = :node
				AND happen_timestamp > date (:year||'-'||:month||'-01')
				AND happen_timestamp < date (:year||'-'||:month||'-01') +interval '1 month'
				AND happen_time_acquitte < date (:year||'-'||:month||'-01')+interval '1 month'

				UNION
				SELECT date_part('epoch',SUM( (date (:year||'-'||:month||'-01') +interval '1 month') - happen_timestamp)) as sec_in_error
				FROM esia_error_happen
				WHERE happen_service_id = :service AND happen_device_id = :node
				AND happen_timestamp > date (:year||'-'||:month||'-01')
				AND happen_timestamp < date (:year||'-'||:month||'-01') +interval '1 month'
				AND ( happen_time_acquitte > date (:year||'-'||:month||'-01') +interval '1 month'
				OR happen_time_acquitte IS NULL )

				UNION

				SELECT date_part('epoch',SUM((date (:year||'-'||:month||'-01') +interval '1 month') - date (:year||'-'||:month||'-01') )) as sec_in_error
				FROM esia_error_happen
				WHERE happen_service_id =  :service
				AND happen_timestamp <  date (:year||'-'||:month||'-01')
				AND happen_acquitte=0

				UNION

					SELECT date_part('epoch',SUM(happen_time_acquitte - date (:year||'-'||:month||'-01'))) as sec_in_error
					FROM esia_error_happen
					WHERE happen_service_id = :service
						AND happen_timestamp < date (:year||'-'||:month||'-01')
						AND happen_time_acquitte < date (:year||'-'||:month||'-01') +interval '1 month'
						AND happen_time_acquitte > date (:year||'-'||:month||'-01')


				UNION

				SELECT date_part('epoch',SUM((date (:year||'-'||:month||'-01') +interval '1 month') - date (:year||'-'||:month||'-01') )) as sec_in_error
				FROM esia_services
				WHERE service_id =  :service
		AND (service_time_added - interval '1 month') >  date (:year||'-'||:month||'-01')) A;");
		$qseconde = $dbconnection->prepare("SELECT date_part('epoch', (date (:year||'-'||:month||'-01') + interval '1 month') - date (:year||'-'||:month||'-01')) as sec;");
		$date = new DateTime();
		$old = new DateTime( $date->format('Y-m').'-01');
		$arrmonth = array();
		for($i = 0;$i < 12;$i++)
		{
			$old->sub(new DateInterval('P1M'));
			//echo $old->format('Y').'/'.$old->format('m').'<br />';
			$qseconde = Model_esia::executeStatement($qseconde,array(':year' => $old->format('Y'),':month' => $old->format('m')));
			$secInMonth = $qseconde->fetch(PDO::FETCH_OBJ);
			$query = Model_esia::executeStatement($query,array(':service' => $service,':node' => $node,
					':year' => $old->format('Y'),
					':month' => $old->format('m')));
			$row = $query->fetch(PDO::FETCH_OBJ);
			if($row->time_notavailable != 0) {
				$pourcent = 100 - (($row->time_notavailable / $secInMonth->sec) * 100);
			} else {
				$pourcent = 100;
			}
			$arrmonth[] = array('year' => $old->format('Y'),'month' => $old->format('m'),'pourcent' => number_format($pourcent,2) );
		}
		//echo'<pre>'.print_r($arrmonth,true).'<pre>';
		return $arrmonth;

	}

	/**
	 * le pourcentage sur un mois donné du temps qu'un service a passé dans les différents états
	 * @param PDO $dbconnection
	 * @param object|integer $node id ou un objet Device du noeud
	 * @param integer $service id du service
	 * @param integer $month le mois
	 * @param integer $year l'année
	 * @retval $list[]['ok']=% de temps passé en ok
	 *		   $list[]['warning']=% de temps passé en warning
	 *		   $list[]["critical"]=% de temps passé en critical
	 *		   $list[]["unknown"]=% de temps passé en unknown
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countPourcentAvailabilityServiceByErrorType($dbconnection,$node,$service,$month,$year)
	{
		//récupération de la date de début et de fin de chaque erreur du service
		// 1 SELECT les erreurs qui ont se sont passé et qui on été acquitée courant du mois
		// 2 SELECT les erreurs qui ont se sont passé durant la pèriode et qui ont  été acquitée après la période où qui ne sont pas encore acquittée
		// 3 SELECT les erreurs qui ont se sont passé avant et qui n'ont pas été acquitée durant la période
		// 4 SELECT Si le service a été ajouté ce mois i
		$query = $dbconnection->prepare("SELECT error_type,SUM (sec_in_error) as time_notavailable FROM (

				SELECT error_type, date_part('epoch',SUM(happen_time_acquitte - happen_timestamp)) as sec_in_error
				FROM esia_error_happen
				LEFT JOIN esia_errors ON error_id = happen_error_id
				WHERE happen_service_id = :service AND happen_device_id = :node
				AND happen_timestamp > date (:year||'-'||:month||'-01')
				AND happen_timestamp < date (:year||'-'||:month||'-01') +interval '1 month'
				AND happen_time_acquitte < date (:year||'-'||:month||'-01')+interval '1 month'
				GROUP BY error_type
				UNION

				SELECT error_type,date_part('epoch',SUM( (date (:year||'-'||:month||'-01') +interval '1 month') - happen_timestamp)) as sec_in_error
				FROM esia_error_happen
				LEFT JOIN esia_errors ON error_id = happen_error_id
				WHERE happen_service_id = :service AND happen_device_id = :node
				AND happen_timestamp > date (:year||'-'||:month||'-01')
				AND happen_timestamp < date (:year||'-'||:month||'-01') +interval '1 month'
				AND ( happen_time_acquitte > date (:year||'-'||:month||'-01') +interval '1 month'
				OR happen_time_acquitte IS NULL )
				GROUP BY error_type
				UNION

				SELECT error_type, date_part('epoch',SUM((date (:year||'-'||:month||'-01') +interval '1 month') - date (:year||'-'||:month||'-01') )) as sec_in_error
				FROM esia_error_happen
				LEFT JOIN esia_errors ON error_id = happen_error_id
				WHERE happen_service_id =  :service
				AND happen_timestamp <  date (:year||'-'||:month||'-01')
				AND ( happen_acquitte=0 OR happen_time_acquitte > date (:year||'-'||:month||'-01') +interval '1 month')
				GROUP BY error_type
				UNION
				SELECT 'nopresent' as error_type,date_part('epoch',SUM((date (:year||'-'||:month||'-01') +interval '1 month') - date (:year||'-'||:month||'-01') )) as sec_in_error
				FROM esia_services
				WHERE service_id =  :service
				AND (service_time_added - interval '1 month') >  date (:year||'-'||:month||'-01')
		) A GROUP BY error_type;");
		$qseconde = $dbconnection->prepare("SELECT date_part('epoch', (date (:year||'-'||:month||'-01') + interval '1 month') - date (:year||'-'||:month||'-01')) as sec;");
		//echo $old->format('Y').'/'.$old->format('m').'<br />';
		$qseconde = Model_esia::executeStatement($qseconde,array(':year' => $year,':month' => $month));
		$secInMonth = $qseconde->fetch(PDO::FETCH_OBJ);
		$query = Model_esia::executeStatement($query,array(':service' => $service,':node' => $node,
				':year' => $year,
				':month' => $month));
		$arrmonth = array();
		$arrmonth['ok'] = 100;
		$arrmonth['warning'] = 0;
		$arrmonth['critical'] = 0;
		$arrmonth['unknown'] = 0;
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			//echo $row->error_type." ".$row->time_notavailable." = ".$secInMonth->sec."\n";
			if($row->time_notavailable != 0) {
				$pourcent = $row->time_notavailable * 100 / $secInMonth->sec;
			} else {
				$pourcent = 0;
			}
			$pourcent = number_format($pourcent,2);
			$arrmonth['ok'] = $arrmonth['ok'] - $pourcent;
			$arrmonth[$row->error_type] = $pourcent;
		}
		//print_r($arrmonth);
		return $arrmonth;

	}
	/**
	 * instancie un objet GESA selon l'id passé en paramètre
	 * @param PDO $dbconnection
	 * @param integer|string $id du noeud GESA ou son nom
	 * @param object $dev (optionnel) class Device object
	 * @throws EsiaDbException
	 * @retval un objet GESA
	 * @exception EsiaDbException si une erreur survient
	 */
	static function InstanceGESA($dbconnection, $id, $dev=NULL)
	{
		if(empty($dev) || @get_class($dev) != 'Device')
		{
			$dev = Model_esia::InstanceNode($dbconnection, $id);
		}

		$query = $dbconnection->prepare("
			SELECT gesa_device_id, gesa_max_slot, gesa_sn,gesa_version_base,gesa_version_nrpe,gesa_version_plugin,
				gesa_version_web,gesa_repo_global,gesa_repo_gesa,gesa_last_update,gesa_rsbp,gesa_local_ip,device_ip_address, gesa_macadress
			FROM gesa_info
			JOIN esia_device ON device_id = gesa_device_id
			WHERE gesa_device_id=?;
		");
		$query = Model_esia::executeStatement($query, array($dev->getId()));

		$gesa = new GESA();
		$gesa->constructDevice($dev);
		if($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$gesa->setSerialNumber($row->gesa_sn);
			$gesa->setMaxSlot($row->gesa_max_slot);
			$gesa->setVersionBase($row->gesa_version_base);
			$gesa->setVersionNRPE($row->gesa_version_nrpe);
			$gesa->setVersionPlugins($row->gesa_version_plugin);
			$gesa->setVersionWeb($row->gesa_version_web);
			$gesa->setRepositoryGlobal($row->gesa_repo_global);
			$gesa->setRepositoryGESA($row->gesa_repo_gesa);
			$gesa->setLastUpdate($row->gesa_last_update);
			$gesa->setMotherBoard($row->gesa_rsbp);
			$gesa->setLocalIPaddress($row->gesa_local_ip);
			$gesa->setPublicIPaddress($row->device_ip_address);
			$gesa->setMacAddress($row->gesa_macadress);
			$gesa->setGesaStatus(Model_esia::getGesaStatus($dbconnection, $dev->getId()));
		}
		return $gesa;
	}
	/**
	 * instancie un objet device selon l'id passé en paramètre
	 * @param PDO $dbconnection
	 * @param integer|string $id du noeud ou son nom
	 * @param string $type (optionnel) type du noeud (esia_device type)
	 * @throws EsiaDbException
	 * @retval un objet device
	 * @exception EsiaDbException si une erreur survient
	 */
	static function InstanceNode($dbconnection,$id,$type=NULL)
	{
		$sql = "SELECT device_id,device_name,device_description,device_type,device_ip_addr,device_username,device_passwd,device_ip_address,
						device_protocol_connection,g1.device_group_name,g1.device_group_id,device_snmp_version,device_snmp_community,
						device_snmp3_user,device_snmp3_sec_lvl,device_snmp3_auth_protocole,device_snmp3_auth_key,
						device_snmp3_priv_protocol,device_snmp3_priv_passphrase,device_snmp_timer,adv_param,device_snmp_port
		FROM esia_device
		LEFT JOIN esia_device_group_in ON device_id = group_in_device_id
		LEFT JOIN esia_device_group as g1 ON device_group_id=group_in_device_group_id
		LEFT JOIN esia_device_advance_config ON device_id = adv_device_id";
		if(!empty($id) && is_numeric($id))
		{
			if(!empty($type) && is_string($type))
			{
				$query = $dbconnection->prepare($sql." WHERE device_id = ? AND device_type=?");
				$query = Model_esia::executeStatement($query,array($id,$type));
			}
			else
			{
				$query = $dbconnection->prepare($sql." WHERE device_id = ?");
				$query = Model_esia::executeStatement($query,array($id));
			}
		}
		elseif(!empty($id) && is_string($id))
		{
			if(!empty($type) && is_string($type))
			{
				$query = $dbconnection->prepare($sql." WHERE device_name = ? AND device_type=?");
				$query = Model_esia::executeStatement($query,array($id,$type));
			}
			else
			{
				$query = $dbconnection->prepare($sql." WHERE device_name = ?");
				$query = Model_esia::executeStatement($query,array($id));
			}
		}
		else
		{
			throw new EsiaDbException("id is not numeric or not equals node name");
		}
		if($query->rowCount() <= 0)
		{
			throw new EsiaDbException('Node does not exist');
		}
		$row = $query->fetch(PDO::FETCH_OBJ);

		$dev = new Device();
		$dev->setId($row->device_id);
		$dev->setName($row->device_name);
		$dev->setType($row->device_type);
		$dev->setDescription($row->device_description);
		$dev->setIPAddress($row->device_ip_address);
		$dev->setSNMPVersion($row->device_snmp_version);
		$dev->setSNMPTimeout($row->device_snmp_timer);
		$dev->setSNMPPort($row->device_snmp_port);
		$rdp = json_decode($row->adv_param,TRUE);
		if( isset($rdp['type']) &&  isset($rdp['id']))
		{
			$dev->setRdpSoftware($rdp['type']);
			$dev->setRdpSoftId($rdp['id']);
		}
		if($row->device_snmp_version == 1 || $row->device_snmp_version == 2) {
			$dev->setSNMPCommunity($row->device_snmp_community);
		} else {
			$dev->setSNMP3Parameter($row->device_snmp3_user,$row->device_snmp3_sec_lvl,$row->device_snmp3_auth_protocole,$row->device_snmp3_auth_key,$row->device_snmp3_priv_protocol,$row->device_snmp3_priv_passphrase);
		}
		$dev->setGroup($row->device_group_name);
		$dev->setGroupID($row->device_group_id);
		$dev->setTypeConnection($row->device_protocol_connection);
		//ajout de la liste de groupe
		$query = $dbconnection->prepare("SELECT group_in_device_group_id FROM esia_device_group_in WHERE group_in_device_id=?");
		$query = Model_esia::executeStatement($query,array($dev->getId()));
		$grps = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$grps[] = $row->group_in_device_group_id;
		}
		$dev->setGroupList($grps);

		if($dev->getType() == 'gesa')
		{
			// The device is a gesa instance the node as gesa class
			$dev = Model_esia::InstanceGESA($dbconnection, $dev->getId(), $dev);
		}
		else
		{
			// Check if it's a remote device (behind a gesa)
			$gesa = Model_esia::getGesaLink($dbconnection, $dev->getId());
			if($gesa != NULL)
			{
				$dev = new RemoteDevice($dev, $gesa);
			}
		}

		return $dev;
	}
	/**
	 * @brief Instancie un utilisateur
	 * @detail  Instancie un utilisateur
	 * @param PDO $dbconnection connxeion à la DB
	 * @param integer|string $id l'id ou le login de l'utilisateur
	 * @retval User un objet utilisateur
	 * @exception EsiaDbException si une erreur survient
	 */
	static function InstanceUser($dbconnection,$id)
	{
		$sql = "SELECT user_id,user_name,user_password,user_lang,user_inscription_time,user_email,theme_name,user_tuto_view,home_page,user_source
				FROM esia_users
				LEFT JOIN esia_theme ON theme_id = user_theme_id";
		if(is_numeric($id))
		{
			$query = $dbconnection->prepare($sql." WHERE user_id=?");
		}
		else
		{
			$query = $dbconnection->prepare($sql." WHERE user_name=?");
		}
		$query = Model_esia::executeStatement($query,array($id));
		$row = $query->fetch(PDO::FETCH_OBJ);
		$user = new User($dbconnection);
		$user->setName($row->user_name);
		$user->setId($row->user_id);
		$user->setLang($row->user_lang);
		$user->setName($row->user_name);
		$user->setEmail($row->user_email);
		$user->setHashPassword($row->user_password);
		$user->setInscriptionTime($row->user_inscription_time);
		$user->setHomePage($row->home_page);
		$user->setAuthSource($row->user_source);
		$tuto = decbin($row->user_tuto_view);
		$tuto = str_split(strrev($tuto));
		$user->setTutorial($tuto);
		if(!empty($row->theme_name)) {
			$user->setTheme($row->theme_name);
		} else {
			$user->setTheme("");
		}
		$query = $dbconnection->prepare("SELECT group_id,group_name,privilege_name,grant_acces,privilege_site
				FROM esia_users
				JOIN esia_memberof ON user_id = member_user_id
				JOIN esia_groups ON group_id = member_group_id
				JOIN esia_privilege_grantedto ON grant_group_id=group_id
				JOIN esia_privileges ON grant_privilege_id = privilege_id
				WHERE user_id=?;");
		$query = Model_esia::executeStatement($query,array($user->getId()));
		$i = 0;
		$list = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$list[$i][0] = $row->group_id;
			$list[$i][1] = $row->group_name;
			$list[$i][2] = $row->privilege_name;
			$list[$i][3] = $row->grant_acces;
			$list[$i][4] = $row->privilege_site;
			$i++;
		}
		$user->setAccess($list);

		$query = $dbconnection->prepare("SELECT group_id
				FROM esia_users
				JOIN esia_memberof ON user_id = member_user_id
				JOIN esia_groups ON group_id = member_group_id
				WHERE user_id=?;");
		$query = Model_esia::executeStatement($query,array($user->getId()));
		$i = 0;
		$list = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$list[] = $row->group_id;
		}
		$user->setGroupUser($list);
		return $user;
	}

	/**
	 * Mise à jour des view du tuto
	 * @param PDO $dbconnection
	 * @param object $user object user
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	function updateTutoriel($dbconnection,$user)
	{
		$query = $dbconnection->prepare("UPDATE esia_users SET user_tuto_view=? WHERE user_id=?");
		$tuto = $user->getTutorial();
		//on récupère la plus grande clé
		$maxi = 0;
		foreach($tuto as $i => $value)
		{
			if($i > $maxi)
			{
				$maxi = $i;
			}
		}
		$size = $maxi;
		$i = 0;
		$binary = "";
		while($i <= $maxi)
		{
			if(!isset($tuto[$i]))
			{
				$binary .= "0";
			}
			else
			{
				$binary .= $tuto[$i];
			}
			$i++;
		}
		$binary = strrev($binary);
		$binary = bindec($binary);
		$query = Model_esia::executeStatement($query,array($binary,$user->getId()));
	}

	/**
	 * instancie un objet user selon l'id passé en paramètre
	 * @param PDO $dbconnection
	 * @param string $word une partie du nom, de la description
	 * @param integer $userid (optionnel) id de l'utilisateur ne retourne que les noeud auquel l'utilisateur à les droits d'accès
	 * @retval une liste d'objet Device
	 * @exception EsiaDbException si une erreur survient
	 */
	static function SearchNode($dbconnection,$word,$userid=NULL)
	{
		if(isset($userid) && is_numeric($userid))
		{
			$query = $dbconnection->prepare("SELECT device_id
					FROM esia_device
					JOIN esia_group_device_access ON  device_id = access_device_id
					JOIN esia_groups ON group_id = access_group_id
					JOIN esia_memberof ON group_id = member_group_id
					WHERE member_user_id=:user AND (device_name ILIKE :word OR device_description ILIKE :word);");
			$query = Model_esia::executeStatement($query,array(':user' => $userid,':word' => '%'.$word.'%'));
		}
		else
		{
			$query = $dbconnection->prepare("SELECT device_id
					FROM esia_device
					WHERE device_name ILIKE :word OR device_description ILIKE :word;");
			$query = Model_esia::executeStatement($query,array(':word' => '%'.$word.'%'));
		}
		$list = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$list[] = Model_esia::InstanceNode($dbconnection, $row->device_id);
		}
		return $list;
	}

	/**
	 * acquitteError
	 * @param PDO $dbconnection
	 * @param integer $error id de l'erreur
	 * @param integer $service (optionnel) id du service
	 * @param integer $user_id (optionnel) id de l'utilisateur
	 * @throws EsiaDbException
	 * @retval /
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function acquitteError($dbconnection,$error,$service=NULL,$user_id=NULL)
	{
		if(is_numeric($error))
		{
			if(!isset($user_id))
			{
				$query = $dbconnection->prepare("UPDATE esia_error_happen SET happen_acquitte=1,happen_time_acquitte=CURRENT_TIMESTAMP
					WHERE happen_id=?  AND happen_acquitte=0;");
				$query = Model_esia::executeStatement($query,array($error));
			}
			else
			{
				$query = $dbconnection->prepare("UPDATE esia_error_happen SET happen_acquitte=1,happen_time_acquitte=CURRENT_TIMESTAMP,happen_user_id=?
					WHERE happen_id=?  AND happen_acquitte=0;");
				$query = Model_esia::executeStatement($query,array($user_id,$error));
			}

		}
		elseif(isset($service))
		{
			if(is_array($service))
			{
				$subsql="";
				$i=0;
				$param =  array();
				foreach($service as $ser)
				{
					$subsql.='ser'.$i.',';
					$param['ser'.$i] = $ser;
					$i++;
				}
				$subsql = rtrim($subsql,',');
				$query = $dbconnection->prepare("UPDATE esia_error_happen SET happen_acquitte=1,happen_time_acquitte=CURRENT_TIMESTAMP
						WHERE happen_service_id IN ($subsql) AND happen_acquitte=0;");
				$query = Model_esia::executeStatement($query,array($param));
			}
			else
			{
				if(!isset($user_id))
				{
					$query = $dbconnection->prepare("UPDATE esia_error_happen SET happen_acquitte=1,happen_time_acquitte=CURRENT_TIMESTAMP
						WHERE happen_service_id=? AND happen_acquitte=0;");
					$query = Model_esia::executeStatement($query,array($service));
				}
				else
				{
					$query = $dbconnection->prepare("UPDATE esia_error_happen SET happen_acquitte=1,happen_time_acquitte=CURRENT_TIMESTAMP,happen_user_id=?
						WHERE happen_service_id=? AND happen_acquitte=0;");
					$query = Model_esia::executeStatement($query,array($user_id,$service));
				}
			}
		}
		else
		{
			throw new EsiaDbException("error must be numeric");
		}
	}
	/**
	 * acquitteError
	 * @param PDO $dbconnection
	 * @param integer $error id de l'erreur
	 * @param integer $service (optionnel) id du service
	 * @param integer $user_id (optionnel) id de l'utilisateur
	 * @throws EsiaDbException
	 * @retval /
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function acquitteNodeError($dbconnection,$node,$user_id)
	{
		if(is_numeric($node))
		{
			$query = $dbconnection->prepare("UPDATE esia_error_happen SET happen_acquitte=1,happen_time_acquitte=CURRENT_TIMESTAMP,happen_user_id=?
					WHERE happen_device_id=? AND happen_acquitte=0;");
			$query = Model_esia::executeStatement($query,array($user_id,$node));
		}
		else
		{
			throw new EsiaDbException("error must be numeric");
		}
	}
	/**
	 * NotacquitteError
	 * @param PDO $dbconnection
	 * @param integer $service id du service
	 * @throws EsiaDbException
	 * @retval /
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function notAcquitteError($dbconnection,$service)
	{
		if(is_numeric($service))
		{
			//On va d'abord recherche la dernière erreur qui a été acquittée pour ce service
			$query = $dbconnection->prepare("SELECT MAX(happen_id) AS happen_id FROM esia_error_happen WHERE happen_service_id=? AND happen_user_id IS NOT NULL;");
			$query = Model_esia::executeStatement($query,array($service));
			$row = $query->fetch(PDO::FETCH_OBJ);
			if(isset($row->happen_id))
			{
				$error = $row->happen_id;

				$query = $dbconnection->prepare("UPDATE esia_error_happen SET happen_acquitte=0,happen_time_acquitte=NULL
					WHERE happen_id=? AND happen_acquitte=1 AND happen_user_id IS NOT NULL;");
				$query = Model_esia::executeStatement($query,array($error));
			}
		}
	}

	/**
	 * setErrorMaintenance
	 * @param PDO $dbconnection
	 * @param integer $error id du service
	 * @throws EsiaDbException
	 * @retval /
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function setErrorMaintenance($dbconnection,$error)
	{
		if(is_numeric($error))
		{
			$query = $dbconnection->prepare("UPDATE esia_error_happen SET happen_error_id=(SELECT error_id FROM esia_errors WHERE error_name='default_maintenance') WHERE happen_id=?;");
			$query = Model_esia::executeStatement($query,array($error));
		}
	}

	static function getWaitingServices($dbconnection,$user=NULL)
	{
		$param = array();
		$sql = "SELECT service_id, service_name,device_name,device_id,service_previous_state,device_type,plugin_name,
			(SELECT device_group_name
			FROM esia_device_group_in
			JOIN esia_device_group ON device_group_id=group_in_device_group_id
			WHERE group_in_device_id = t1.device_id LIMIT 1) as device_group_name,service_last_stdout
			FROM esia_services
			JOIN esia_plugins ON plugin_id = service_plugin_id
			JOIN esia_device as t1 ON service_device_id= device_id
			WHERE service_last_exec_time IS NULL";
		if($user != NULL)
		{
			$sql .= " AND device_id IN ( SELECT * FROM getUserAccessNode(:user))";
			$param['user'] = $user;
		}
		//echo $sql;
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$page_control = Model_esia::getPageControl($dbconnection, $row->device_type);
			if($page_control != NULL)
			{
				$page_control = Model_esia::replacePageControlVariable($page_control,$row);
			}
			else
			{
				$page_control = '/node_view.php?nodeid='.$row->device_id;
			}
			$out = preg_replace('#\n#','<br />',$row->service_last_stdout);
			$errortype = 'waiting';
			$arr[] = array('type_device' => $row->device_type,
					'message' => '',
					'name' => $row->device_name,
					'id' => $row->device_id,
					'plugin_name' => $row->service_name,
					'page_control' => $page_control,
					'group' => $row->device_group_name,
					'type_error' => $errortype,
					'check_plugin_name' => $row->plugin_name
			);
		}
		return $arr;
	}

	/**
	 * acquitte l'erreur (tablea esia_error_happen)
	 * @param PDO $dbconnection
	 * @param integer $user (optionnel) id de l'erreur
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getAcquitteError($dbconnection,$user=NULL)
	{
		$param = array();
		$sql = "SELECT service_id, service_name,device_name,device_ip_address,device_id,service_previous_state,device_type,plugin_name,
			(SELECT device_group_name
			FROM esia_device_group_in
			JOIN esia_device_group ON device_group_id=group_in_device_group_id
			WHERE group_in_device_id = t1.device_id LIMIT 1) as device_group_name,service_last_stdout
			FROM esia_services
			JOIN esia_plugins ON plugin_id = service_plugin_id
			JOIN esia_device as t1 ON service_device_id= device_id
                        WHERE device_delete = false AND service_delete = false AND service_last_stdout IS NOT NULL AND service_previous_state <>0 AND service_id NOT IN (SELECT happen_service_id FROM esia_error_happen WHERE happen_acquitte =0)";
		if($user != NULL)
		{
			$sql .= " AND device_id IN ( SELECT * FROM getUserAccessNode(:user))";
			$param['user'] = $user;
		}
		//echo $sql;
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$page_control = Model_esia::getPageControl($dbconnection, $row->device_type);
			if($page_control != NULL)
			{
				$page_control = Model_esia::replacePageControlVariable($page_control,$row);
			}
			else
			{
				$page_control = '/node_view.php?nodeid='.$row->device_id;
			}
			$out = preg_replace('#\n#','<br />',$row->service_last_stdout);
			$errortype = 'unknown';
			if($row->service_previous_state == 1)
			{
				$errortype = 'warning';
			}
			elseif($row->service_previous_state == 2)
			{
				$errortype = 'critical';
			}
			$arr[] = array('type_device' => $row->device_type,
					'message' => $out,
					'name' => $row->device_name,
					'id' => $row->device_id,
					'ip_addr' => $row->device_ip_address,
					'plugin_name' => $row->service_name,
					'page_control' => $page_control,
					'group' => $row->device_group_name,
					'type_error' => $errortype,
					'check_plugin_name' => $row->plugin_name
			);
		}
		return $arr;
	}

	/**
	 * La requète préparé pour l'objet USER pour vérifier les droits d'accès
	 * @param PDO $dbconnection
	 * @retval /
	 * @exception /
	 */
	static function PrepareQueryCheckAccess($dbconnection)
	{
		$query = $dbconnection->prepare("SELECT DISTINCT access_group_id
				FROM esia_device
				JOIN esia_group_device_access ON  device_id = access_device_id
				WHERE device_name =?;");
		return $query;
	}

	/**
	 * remplace les paramètres :id,:name par les informations du noeuds
	 * @param string $pageControl la page de control avec les paramètre générique :name,:id
	 * @param object $node un objet device ou encore un objet anonyme renvoyé par PDO
	 * @retval la page de control correctement formatée
	 * @exception /
	 */
	static function replacePageControlVariable($pageControl,$node)
	{
		$page_control = $pageControl;
		$id = '';
		$name = '';
		if(@get_class($node) == 'Device')
		{
			$name = $node->getName();
			$id  = $node->getId();

		}
		elseif(@get_class($node) == 'RemoteDevice')
		{
			$name = $node->getName();
			$id  = $node->getId();

		}
		elseif(@get_class($node) == 'stdClass')
		{
			$name = $node->device_name;
			$id  = $node->device_id;
		}
		else
		{
			if(array_key_exists('name' , $node))
			{
				$name = $node['name'];
				$id = $node['id'];
			}
			else
			{
				$name = $node["\0*\0name"];
				$id = $node["\0*\0id"];
			}
		}
		$page_control = preg_replace('#:name#',$name,$page_control);
		$page_control = preg_replace('#:id#',$id,$page_control);
		return $page_control;
	}

	/**
	 * test et si il y a une gesa qui est liée elle retourne un objet device
	 * @param PDO $dbconnection
	 * @param integer $nodeid identifiant du noeud
	 * @retval NULL|GESA null si aucune gesa n'est lié sinon un objet device
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGesaLink($dbconnection,$nodeid)
	{
		$query = $dbconnection->prepare("SELECT connected_gesa_id FROM gesa_connected WHERE connected_device_id=?;");
		$query = Model_esia::executeStatement($query,array($nodeid));
		$gesa = NULL;
		if($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$gesa = Model_esia::InstanceGESA($dbconnection, $row->connected_gesa_id);
		}
		return $gesa;
	}
	/**
	* test et si il y a une gesa qui est liée elle retourne un objet device
	* @param: $dbconnection l'objet de connexion PDO
	* @param: $user_id identifiant de l'utilisateur
	* @param: $nodeid identifiant du noeud
	* @retval: une liste de noeuds connecté
	* @exception EsiaDbException si une erreur survient
	*/
	static function getConnectedTo($dbconnection,$userid,$nodeid)
	{
		$query = $dbconnection->prepare("SELECT t11.device_id,t11.device_name,t12.interface_name,t12.interface_mac_addr,t22.interface_name AS switch_interface,t22.interface_id AS switch_interface_id,t21.device_name AS switch_name,t21.device_id AS switch_id
										FROM esia_device AS t11
										JOIN esia_interface AS t12 ON device_id = interface_device_id
										JOIN esia_interface_connected_to ON conint_interface_id2 = interface_id

										JOIN esia_interface AS t22 ON t22.interface_id = conint_switch_interface_id
										JOIN esia_device AS t21 ON t21.device_id = t22.interface_device_id
										WHERE t11.device_id = :id AND t11.device_id IN ( SELECT * FROM getUserAccessNode(:userid))");
		$query = Model_esia::executeStatement($query,array(':userid' => $userid,':id' => $nodeid));
		$list = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$list[] = array('id' => $row->switch_id,'name' => $row->switch_name,'interface_name' => $row->switch_interface,'interface_id' => $row->switch_interface_id,'macnode' => $row->interface_mac_addr);
		}
		return $list;
	}
	/**
	 * @brief indique si il y a de nouvelle GESA
	 * @detail si $instance vaut true, on instancie les GESA en device si instance vaut false on retourne simplement le nombre de nouvelle GESA connectée
	 * @param PDO $dbconnection variable de connexion à la DB
	 * @param integer $userid indique l'id de l'utilisateur
	 * @param boolean $instance indique s'il faut instancier les noeuds ou nom
	 * @retval Device[]|RemoteDevice[]|integer
	 */
	static function getNewGesa($dbconnection,$userid,$instance = FALSE)
	{
		$arr = array();
		if($instance === TRUE)
		{
			$query = $dbconnection->prepare("SELECT device_id
										FROM esia_device
										WHERE device_type='gesa'
										AND device_id NOT IN (SELECT DISTINCT connected_gesa_id FROM gesa_connected)
										AND device_id IN ( SELECT * FROM getUserAccessNode(?));");
			$query = Model_esia::executeStatement($query,array($userid));
			$queryGESA = $dbconnection->prepare("SELECT gesa_device_id, gesa_max_slot, gesa_sn,gesa_version_base,gesa_version_nrpe,gesa_version_plugin,
											gesa_version_web,gesa_repo_global,gesa_repo_gesa,gesa_last_update,gesa_rsbp,gesa_local_ip,gesa_macadress
									   FROM gesa_info WHERE gesa_device_id=?");
			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				$dev  = Model_esia::InstanceNode($dbconnection, $row->device_id);
				$resultgesa = Model_esia::executeStatement($queryGESA,array($dev->getId()));
				$unity = new GESA();
				$unity->constructDevice($dev);
				if($row = $resultgesa->fetch(PDO::FETCH_OBJ))
				{
					$add= true;
					if($row->gesa_sn != $unity->getName())
					{
						if(preg_match('#[0-9]{8}([0-9]{2})#',$row->gesa_sn,$regs))
						{
							if($regs[1] == 90 || $regs[1] == 10)
							{
								$add=false;
							}
						}
					}
					if($add==true)
					{
						$unity->setSerialNumber($row->gesa_sn);
						$unity->setMaxSlot($row->gesa_max_slot);
						$unity->setVersionBase($row->gesa_version_base);
						$unity->setVersionNRPE($row->gesa_version_nrpe);
						$unity->setVersionPlugins($row->gesa_version_plugin);
						$unity->setVersionWeb($row->gesa_version_web);
						$unity->setRepositoryGlobal($row->gesa_repo_global);
						$unity->setRepositoryGESA($row->gesa_repo_gesa);
						$unity->setLastUpdate($row->gesa_last_update);
						$unity->setMotherBoard($row->gesa_rsbp);
						$unity->setLocalIPaddress($row->gesa_local_ip);
						$unity->setMacAddress($row->gesa_macadress);
						$arr[] = $unity;
					}
				}
			}
		}
		else
		{
			$query = $dbconnection->prepare("SELECT count(device_id) as nbrgesa
										FROM esia_device
										JOIN gesa_info ON gesa_device_id=device_id
										WHERE device_type='gesa'
											AND ( (gesa_sn::character varying NOT LIKE '________90%'
												 	AND gesa_sn::character varying NOT LIKE '________10%' )
												 	OR gesa_sn::character varying = device_name
												)
											AND device_id NOT IN ( SELECT DISTINCT connected_gesa_id FROM gesa_connected);");
										//AND device_id IN ( SELECT * FROM getUserAccessNode(?));");
			$query = Model_esia::executeStatement($query);
			$row = $query->fetch(PDO::FETCH_OBJ);
			$arr = $row->nbrgesa;
		}
		return $arr;
	}
	/**
	 * @brief retourne le()s dashboard (s)
	 * @detail Recupère dans la base de donnée les dashboards
	 * @param PDO $dbconnection variable de connexion à la DB
	 * @param integer $dashid (optionnel) l'identifiant du dashboard
	 * @param integer $userid (optionnel) l'identifiant de l'utilisateur
	 * @retval tableau associatif contenant les clés suivante
	 * 		list[$row->dashboard_id]=array('id'=>$row-> dashboard_id,    ID du dashboard
	 *					'name'=>$row->dashboard_name,					 Nom du dashboard
	 *					'controller'=>$row-> dashboard_controller,       identifiant du controlleur
	 *					'description'=>$row->dashboard_description,      description du dashboard
	 *					'share'=>$row->board_user_share,			     Indique si le dashboard vous appartient ou s'il est partagé
	 *		            'tile'=> array(									 Tableau contenant les tuiles lié au dashboard
	 *								   'id'=>$row->tile_id,				 identifiant des tuiles
	 *								   'position'=>$row->tile_position,  position de la tuile
	 *								   'height'=>$row->tile_height,      hauteur de la tuile
	 *								   'width'=>$row->tile_width)        largeur de la tuile
	 */
	static function getDashboards($dbconnection,$dashid = NULL, $userid = NULL)
	{
		$sql = "SELECT dashboard_id, dashboard_name, dashboard_controller, dashboard_link,dashboard_description,board_user_share,
					 tile_position, tile_height, tile_width,tile_id,dashboard_default
				FROM dashboard
				JOIN dashboard_user_access ON board_dashboard_id = dashboard_id
				LEFT JOIN dashboard_tile ON tile_dashboard_id = dashboard_id
				WHERE dashboard_default=false ";
		$param = array();
		if(is_numeric($dashid))
		{
			$sql .= " AND dashboard_id=:dashid ";
			$param[':dashid'] = $dashid;
			$separator = 'AND';
		}
		elseif(is_string($dashid))
		{
			$sql .= " AND dashboard_name=:dashid ";
			$param[':dashid'] = $dashid;
			$separator = 'AND';
		}
		if(!empty($userid))
		{
			$sql .= " AND board_user_id = :userid ";
			$param[':userid'] = $userid;
		}
		$sql .= "UNION SELECT dashboard_id, dashboard_name, dashboard_controller, dashboard_link,dashboard_description,false,
						 tile_position, tile_height, tile_width,tile_id,dashboard_default
				   		FROM dashboard
						LEFT JOIN dashboard_tile ON tile_dashboard_id = dashboard_id
						WHERE dashboard_default=true";
		if(is_numeric($dashid))
		{
			$sql .= " AND dashboard_id=:dashid2 ";
			$param[':dashid2'] = $dashid;
			$separator = 'AND';
		}
		elseif(is_string($dashid))
		{
			$sql .= " AND dashboard_name=:dashid2 ";
			$param[':dashid2'] = $dashid;
			$separator = 'AND';
		}
		$sql .= " ORDER BY dashboard_default,board_user_share,dashboard_id,tile_position;";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param );
		$list = array();
		$default_dashboard=array();
		$user_dashboard=array();
		$share_dashboard=array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!empty($dashid))
			{
				Model_esia::AddTileInDashboard($user_dashboard,$row);
			}
			else
			{
				if($row->dashboard_default ==true)
				{
					 Model_esia::AddTileInDashboard($default_dashboard,$row);
				}
				elseif($row->board_user_share==false)
				{
					Model_esia::AddTileInDashboard($user_dashboard,$row);
				}
				else
				{
					Model_esia::AddTileInDashboard($share_dashboard,$row);
				}
			}
		}
		if(isset($dashid))
		{
			return $user_dashboard;
		}
		else
		{
			return array('default'=>$default_dashboard,'personnal'=>$user_dashboard,'share'=>$share_dashboard);
		}
	}
	private static function AddTileInDashboard(&$dashboard,&$row)
	{
		if(!array_key_exists($row->dashboard_id, $dashboard))
		{
			$path = preg_replace('#\:dashid#', $row->dashboard_id, $row->dashboard_controller);
			$dashboard[$row->dashboard_id] = array('id' => $row->dashboard_id,
						'name' => htmlentities($row->dashboard_name, ENT_QUOTES),
						'controller' => $path,
						'description' => htmlentities($row->dashboard_description, ENT_QUOTES),
						'link'=>$row->dashboard_link,
						'share' => $row->board_user_share,
						'default'=> $row->dashboard_default
						);
		}
		$dashboard[$row->dashboard_id]['tile'][$row->tile_id] = array('id' => $row->tile_id,
																	  'position' => $row->tile_position,
																	'height' => $row->tile_height,
																	'width' => $row->tile_width);
	}
	/**
	 * @brief récupère le(s) tuile(s)
	 * @param PDO $dbconnection
	 * @param integer|null $tile identifiant de la tuile (optionnel)
	 * @retval mixed[][] $list[$row->tile_id]=array('id'=>$row-> tile_id,
							'position'=>$row->tile_position,
							'height'=>$row-> tile_height,
							'width'=>$row->tile_width,
							'widget'=> [$row->widget_id]'id'=>$row->widget_id,
										'name' => $row->widget_name,
										'during' => $row->widget_view_during,
										'position'=>$row->widget_position,
										'option'=>json_decode($row->widget_option),
										'description'=>$row->widget_description,
										'path'=>$path));

	 */
	static function getTiles($dbconnection,$tile = NULL)
	{
		$sql = "SELECT tile_id,tile_position, tile_height, tile_width,
				widget_id, widget_name, widget_view_during, widget_position,
      				 widget_option, widget_description,widget_id,view_file_path,view_name,widget_group_name
			   FROM dashboard_tile
			   JOIN dashboard_widget ON widget_tile_id = tile_id
			   JOIN dashboard_view ON widget_view_id = view_id
               LEFT JOIN dashboard_widget_group ON widget_group_id = view_group_id";
		$param = array();
		if(isset($tile) && is_numeric($tile))
		{
			$sql .= " WHERE tile_id = :tile ";
			$param = array('tile' => $tile);
		}
		$sql .= " ORDER BY tile_id,widget_position;";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param );
		$list = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$path = preg_replace('#\:widgetid#', $row->widget_id, $row->view_file_path);

			if(! array_key_exists($row->tile_id, $list))
			{
				$list[$row->tile_id] = array('id' => $row->tile_id,
						'position' => $row->tile_position,
						'height' => $row->tile_height,
						'width' => $row->tile_width);
			}
			$group = $row->widget_group_name;
			if(empty($group))
			{
				$group = 'OTHER';
			}
			$list[$row->tile_id]['widget'][$row->widget_id] = array('id' => $row->widget_id,
					'name' => $row->widget_name,
					'during' => $row->widget_view_during,
					'position' => $row->widget_position,
					'option' => json_decode($row->widget_option,true),
					 'description' => htmlentities($row->widget_description, ENT_QUOTES),
					'path' => $path,
					'view_name' => $row->view_name,
					 'group_name' => $group
			);
		}
		return $list;
	}
	/**
	 * @brief récupère la ou les widget(s)
	 * @param PDO $dbconnection
	 * @param integer $widgetid identifiant de la widget
	 * @param integer $user userid (depracted?)
	 * @retval array[]'id'=>$row->widget_id,
					'name' => $row->widget_name,
					'during' => $row->widget_view_during,
					'position'=>$row->widget_position,
					'option'=>json_decode($row->widget_option,true),
					'default_option'=>json_decode($row->view_default_option,true),
					'description'=>$row->widget_description,
					'path'=>$path,
					'tile'=>$row->widget_tile_id
	 */
	static function getWidget($dbconnection,$widgetid=NULL,$user=NULL)
	{
		$sql = "SELECT widget_id, widget_name, widget_view_during, widget_position,widget_tile_id,
      				 widget_option, widget_description,widget_id,view_file_path,view_default_option,view_name
			   FROM dashboard_widget
			   JOIN dashboard_view ON widget_view_id = view_id";
		$param = array();
		if(isset($widgetid) && is_numeric($widgetid))
		{
			$sql .= " WHERE widget_id = :widget ";
			$param = array('widget' => $widgetid);
		}
		$sql .= " ORDER BY widget_position;";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param );
		$list = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$path = preg_replace('#\:widgetid#', $row->widget_id, $row->view_file_path);
			$list[] = array('id' => $row->widget_id,
					'name' => $row->widget_name,
					'during' => $row->widget_view_during,
					'position' => $row->widget_position,
					'option' => json_decode($row->widget_option,TRUE),
					'default_option' => json_decode($row->view_default_option,TRUE),
					 'description' => htmlentities($row->widget_description, ENT_QUOTES),
					'path' => $path,
					'tile' => $row->widget_tile_id,
					'view_name' => $row->view_name
			);
		}
		return $list;
	}
	/**
	 * @brief met à jour une widget
	 * @param PDO $dbconnection connexion à la db
	 * @param integer $id identifiant de la db
	 * @param string $name nom de la widget
	 * @param string $description description de la widget
	 * @param integer $during  ça durée d'affichage en seconde
	 * @param array[] $parameter tableau associatif avec les parametre (qsera transformé en JSOn)
	 * @exception esiaDBException
	 */
	static function updateWidget($dbconnection,$id,$name,$description,$during,$parameter)
	{
		$sql = "UPDATE dashboard_widget SET widget_name=:name, widget_view_during=:during,widget_option=:option,
				widget_description=:description
			   WHERE widget_id = :id;";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array('id' => $id,
														   'name' => $name,
															'description' => $description,
															'during' => $during,
															'option' => json_encode($parameter)));
	}
	/**
	 * @brief retourne les vues proposées dans ESIA
	 * @param PDO $dbconnection
	 * @param integer|null $id identifiant de la vue
	 * @return mixed[][]|NULL[][]array('id'=>$row->view_id,
					'name' => $row->view_name,
					'path' => $row->view_file_path,
					'during'=>$row->view_during,
					'height'=>$row->view_default_height,
					'width'=>$row->view_default_width,
					'default_option'=>json_decode($row->view_default_option,true));
	 */
	static function getdasboardView($dbconnection,$id=NULL)
	{
		$sql = "SELECT view_id, view_name, view_file_path, view_during, view_default_height,view_default_width, view_default_option,view_group_id,widget_group_name
				FROM dashboard_view
                LEFT JOIN dashboard_widget_group ON widget_group_id = view_group_id";
		$param = array();
		if(isset($id) && is_numeric($id))
		{
			$sql .= " WHERE view_id=?";
			$param = array($id);
		}
		$sql .= " ORDER BY widget_group_order;";
		$list = array();
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param);
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$group = $row->widget_group_name;
			if(empty($group))
			{
				$group = 'OTHER';
			}
			$list[] = array('id' => $row->view_id,
					'name' => $row->view_name,
					'path' => $row->view_file_path,
					'during' => $row->view_during,
					'height' => $row->view_default_height,
					'width' => $row->view_default_width,
				'group_name' => $group,
					'default_option' => json_decode($row->view_default_option,TRUE));
		}
		return $list;
	}
	/**
	 * @brief retourne la disponnibilité sur le service demandé
	 * @param PDO $dbconnection
	 * @param integer $nodeid
	 * @param integer $serviceid
	 * @param Date $endate (optionnel) date au format ('YYYY-MM-DD') sinon la date courante est utilisée
	 * @param number $interval (optionnel) le nombre de jour à calculé (défault:30)
	 * @retval mixed[] tableau associatif [date]=disponnibilité en pourcentage.
	 */
	static function getAvailability($dbconnection,$nodeid,$serviceid,$endate=NULL,$interval=30)
	{
		$sql = " SELECT * FROM get_availability(?,?,to_date(?,'YYYY-MM-DD'),?::interval)";
		$query = $dbconnection->prepare($sql);
		$i = 0;
		$list = array();
		while($i < $interval)
		{
			$timestamp = time() - ($i * 60 * 60 * 24);
			$date = date('Y-m-d',$timestamp);
			$queryexec = Model_esia::executeStatement($query,array($nodeid,$serviceid,$date,'1day'));
			$row = $queryexec->fetchAll();
			$list[$date] = $row[0][0];
			$i++;
		}
		return $list;
	}

	static function getDayAvailability($dbconnection,$nodeid,$serviceid,$end_date_ts,$interval=24)
 {
		$sql = "SELECT * FROM get_availability(?,?,to_timestamp(?,'YYYY-MM-DD HH24')::timestamp without time zone,?::interval)";
		$query = $dbconnection->prepare($sql);
		$i = 0;
		$list = array();
		while($i < $interval){
			$timestamp = $end_date_ts - ($i * 3600);
			$date = date('Y-m-d H',$timestamp);
			$queryexec = Model_esia::executeStatement($query,array($nodeid,$serviceid,$date,'1hour'));
			$row = $queryexec->fetchAll();
			$list[$date] = $row[0][0];
			$i++;
		}
		return $list;
	}

	/**
	 * @brief effectue le remplacement d'unity
	 * @param PDO $dbconnection
	 * @param integer $idNewUnity
	 * @param integer $idOldUnity)
	 * @retval array[] tableau id tous les noeuds liés à l'unity.
	 */
	static function replaceUnity($dbconnection,$idNewUnity,$idOldUnity)
	{
		if(isset($idNewUnity) && !empty($idNewUnity) && isset($idOldUnity) && !empty($idOldUnity))
		{
			$query = $dbconnection->prepare("SELECT  gesa_sn,gesa_local_ip,device_ip_addr FROM gesa_info INNER JOIN esia_device ON (gesa_device_id=device_id) WHERE gesa_device_id = ?");
			$query = Model_esia::executeStatement($query,array($idNewUnity));

			$sn = 0;
			$row = $query->fetch(PDO::FETCH_OBJ);
			$sn = $row->gesa_sn;
			$local_ip = $row->gesa_local_ip;
			$public_ip = $row->device_ip_addr;

			$query = $dbconnection->prepare("DELETE FROM gesa_info WHERE gesa_device_id = ?");
			$query = Model_esia::executeStatement($query,array($idNewUnity));

			$query = $dbconnection->prepare("DELETE FROM esia_device WHERE device_id = ?");
			$query = Model_esia::executeStatement($query,array($idNewUnity));

			$query = $dbconnection->prepare("UPDATE gesa_info SET gesa_sn = :sn, gesa_local_ip = :local_ip WHERE gesa_device_id = :devid");
			$query = Model_esia::executeStatement($query,array(':devid' => $idOldUnity,':sn' => $sn, ':local_ip' => $local_ip));

			$query = $dbconnection->prepare("UPDATE esia_device SET device_ip_addr = :public_ip WHERE device_id = :devid");
			$query = Model_esia::executeStatement($query,array(':devid' => $idOldUnity,':public_ip' => $public_ip));

			$req = "-S ".$sn." -p 5666 -c check_gesa -t 50";
			$query = $dbconnection->prepare("UPDATE esia_services
                                                       SET service_parameter = ?
                                                       WHERE service_device_id = ?
                                                       AND service_plugin_id = (SELECT plugin_id FROM esia_plugins WHERE plugin_name = 'GESA_HEALTH')");
			$query = Model_esia::executeStatement($query,array($req,$idOldUnity));

			$query = $dbconnection->prepare("SELECT * FROM gesa_connected WHERE connected_gesa_id = ?");
			$query = Model_esia::executeStatement($query,array($idOldUnity));

			$list = array();
			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				$list[] = array('idUnity' => $row->connected_gesa_id,
					'idDevice' => $row->connected_device_id,);
			}
			return $list;
		}
	}

	/**
	 * @brief retourne la liste des services lie a un plugin
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param string integer $plugin id ou nom du plugin
	 * @retval Le vendor correspondant à la mac ou null si il n'est pas trouvé.
	 */
	static function getServiceOfPlugin($dbconnection,$plugin)
	{
		$sql = "SELECT plugin_id,plugin_name,service_id,service_name
		FROM esia_plugins
		JOIN esia_services ON plugin_id = service_plugin_id ";
		if(is_numeric($plugin))
		{
			$query = $dbconnection->prepare($sql." AND plugin_id = :id  ;");
		}
		else
		{
			$query = $dbconnection->prepare($sql." AND plugin_name = :id ;");
		}
		$query = Model_esia::executeStatement($query,array('id' => $plugin));

		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id' => $row->plugin_id,
					'name' => $row->plugin_name,
					'service_id' => $row->service_id,
					'service_name' => $row->service_name,
			);
		}
		return $arr;
	}

	/**
	 * @brief retourne la disponibilité et le nombre de seconde dans un état sur le service demandé
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param string $mac la mac adresse ou une partie de la mac recherchée
	 * @retval Le vendor correspondant à la mac ou null si il n'est pas trouvé.
	 */
	public static function searchMacVendor($dbconnection,$mac)
	{
		if(empty($mac)) { return NULL;
		}
		$mac_vendor = strtolower($mac);
		$mac_vendor = preg_replace('/[^a-z0-9]/i', '', $mac_vendor);
		$mac_vendor = substr ( $mac_vendor , 0 , 6 );

		$qvendor = $dbconnection->prepare("SELECT vendor_company
				FROM esia_mac_address_vendor
				WHERE vendor_mac_address=?;");

		$result = Model_esia::executeStatement($qvendor,array($mac_vendor));

		$vendor = NULL;
		if($rvendor = $result->fetch(PDO::FETCH_OBJ)){
			$vendor = $rvendor->vendor_company;
		}
		return $vendor;
	}

	/**
	 * @brief ajout des elements pour la prise en main à distance d'un noeud
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id l'identifiant du noeud
	 * @param integer $node_id l'identifiant du noeud
	 * @param string $logiciel le nom du logiciel utilisé pour la prise en main
	 */
	public static function getEsiaRdp($dbconnection,$id=NULL,$node_id=NULL,$logiciel=NULL)
	{
		$sql = "SELECT adv_id,adv_device_id,adv_param
				FROM esia_device_advance_config";

		$param = array();
		if(!empty($id) && $id != NULL)
		{
			$sql .= ' WHERE adv_id = ?;';
			$param[] = $id;
		}
		elseif(!empty($node_id) && $node_id != NULL)
		{
			$sql .= ' WHERE adv_device_id = ?;';
			$param[] = $node_id;
		}
		elseif(!empty($logiciel) && $logiciel != NULL)
		{
			$sql .= ' WHERE adv_param::text ILIKE \'%"type":"?"%\';';
			$param[] = $logiciel;
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param);

		$Listrdp = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$Listrdp[] = array('adv_id' => $row->adv_id,
					'adv_device_id' => $row->adv_device_id,
					'adv_param' => $row->adv_param
			);
		}
		return $Listrdp;
	}

	/**
	 * @brief retour la liste des software pour la prise en main à distance
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 */
	public static function getListLogicelPmd($dbconnection)
	{
		$query = $dbconnection->prepare("SELECT DISTINCT rdp_software FROM esia_rdp;");
		$query = Model_esia::executeStatement($query,array());

		$Listrdp = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$Listrdp[] = array('logiciel_name' => $row->rdp_software);
		}
		$Listrdp[] = array('logiciel_name' => 'none');
		return $Listrdp;
	}

	/**
	 * @brief ajout des elements pour la prise en main à distance d'un noeud
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $device_id l'identifiant du noeud
	 * @param string $logiciel le nom du logiciel utilisé pour la prise en main
	 * @param string $id_logiciel identifiant utilisé pour la connexion
	 */
	public static function addEsiaRdp($dbconnection,$device_id,$logiciel,$id_logiciel)
	{
		$query = $dbconnection->prepare("INSERT INTO esia_rdp (rdp_device_id,rdp_software,rdp_soft_id)
				    					VALUES(:device_id,:logiciel,:id_logiciel) RETURNING rdp_id");

		$query = Model_esia::executeStatement($query,array(':device_id' => $device_id,
				':logiciel' => $logiciel,
				':id_logiciel' => $id_logiciel));
		$row = $query->fetch(PDO::FETCH_OBJ);
		return $row->rdp_id;
	}

	/**
	 * @brief supprimer des elements pour la prise en main à distance d'un noeud
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id l'identifiant du device
	 */
	public static function delEsiaRdp($dbconnection,$id)
	{
		$query = $dbconnection->prepare("DELETE FROM esia_device_advance_config WHERE adv_device_id = :id AND adv_config_type_id = (SELECT device_config_type_id FROM esia_device_advance_config_type WHERE device_config_type_name='RDP');");
		$query = Model_esia::executeStatement($query,array(':id' => $id));
	}

	/**
	 * @brief Update des elements pour la prise en main à distance d'un noeud
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $device_id l'identifiant du noeud
	 * @param string $logiciel le nom du logiciel utilisé pour la prise en main
	 * @param string $id_logiciel identifiant utilisé pour la connexion
	 */
	public static function updEsiaRdp($dbconnection,$device_id,$params)
	{
		$query = $dbconnection->prepare("UPDATE esia_rdp SET rdp_device_id=:device_id , rdp_software=:logiciel , rdp_soft_id=:id_logiciel WHERE rdp_device_id = :device_id RETURNING rdp_id;");

		$query = Model_esia::executeStatement($query,array(
				':device_id' => $device_id,
				':logiciel' => $logiciel,
				':id_logiciel' => $id_logiciel));
		$row = $query->fetch(PDO::FETCH_OBJ);
		return $row->rdp_id;
	}

	/**
	 * @brief retour les plugins d'un group de noeud
	 * @param PDO $dbconnection
	 * @param integer $id l'id d'un group de noeud
	 * @param integer/stgring $pluginname l'id du plugin ou son id
	 * @retval array contenant les infos lie au group de noeud et plugin
	 * @exception une esiaDBException
	 */
	static function getPluginOfGroupNode($dbconnection,$groupnodeid=NULL,$plugin=NULL,$userid)
	{
		$sql = "SELECT plugin_id,plugin_name,service_id,service_name,device_id,device_name,device_group_id
    		   FROM esia_plugins
    		   JOIN esia_services ON plugin_id = service_plugin_id
    		   JOIN esia_device ON service_device_id = device_id
    		   JOIN esia_device_group_in ON device_id = group_in_device_id
    		   JOIN esia_device_group ON device_group_id= group_in_device_group_id
			   WHERE (esia_device.device_id IN ( SELECT * FROM getUserAccessNode(:user)) OR esia_device.device_id IS NULL) ";
		if(isset($groupnodeid) && isset($plugin))
		{
			if(is_numeric($plugin))
			{
				$query = $dbconnection->prepare($sql." AND device_group_id IN (SELECT group_id FROM getAllChildDeviceGroup(:id)) AND plugin_id = :plugin ;");
				$query = Model_esia::executeStatement($query,array('user' => $userid,'id' => $groupnodeid,'plugin' => $plugin));
			}
			elseif(is_array($plugin))
			{
				foreach($plugin as $id)
				{
					if(!is_numeric($id)) {
						throw new EsiaDbException('$plugin array must be numeric');
					}
				}
				$arrids = implode(',', $plugin);
				$query = $dbconnection->prepare($sql." AND device_group_id IN (SELECT group_id FROM getAllChildDeviceGroup(:id)) AND plugin_id IN ($arridsgetCommonName);");
				$query = Model_esia::executeStatement($query,array('user' => $userid,'id' => $groupnodeid));
			}
			else
			{
				$query = $dbconnection->prepare($sql." AND device_group_id IN (SELECT group_id FROM getAllChildDeviceGroup(:id)) AND plugin_name = :plugin ;");
				$query = Model_esia::executeStatement($query,array('user' => $userid,'id' => $groupnodeid,'plugin' => $plugin));
			}
		}
		elseif (isset($plugin))
		{
			if(is_numeric($plugin))
			{
				$query = $dbconnection->prepare($sql." AND plugin_id = :plugin ;");
				$query = Model_esia::executeStatement($query,array('user' => $userid,'plugin' => $plugin));
			}
			elseif(is_array($plugin))
			{
				foreach($plugin as $id)
				{
					if(!is_numeric($id)) {
						throw new EsiaDbException('$plugin array must be numeric');
					}
				}
				$arrids = implode(',', $plugin);
				$query = $dbconnection->prepare($sql." AND plugin_id IN ($arrids);");
				$query = Model_esia::executeStatement($query,array('user' => $userid));
			}
			else
			{
				$query = $dbconnection->prepare($sql." AND plugin_name = :plugin;");
				$query = Model_esia::executeStatement($query,array('user' => $userid,'plugin' => $plugin));
			}
		}
		elseif (isset($groupnodeid))
		{
			$query = $dbconnection->prepare($sql." AND device_group_id IN (SELECT group_id FROM getAllChildDeviceGroup(:id)) ;");
			$query = Model_esia::executeStatement($query,array('user' => $userid,'id' => $groupnodeid));
		}
		else
		{
			return NULL;
		}
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id' => $row->plugin_id,
					'name' => $row->plugin_name,
					'service_id' => $row->service_id,
					'service_name' => $row->service_name,
					'device_id' => $row->device_id,
					'device_name' => $row->device_name,
					'device_group_id' => $row->device_group_id,
			);
		}
		return $arr;
	}

	/**
	 * @brief retourne la disponibilité et le nombre de seconde dans un état sur le service demandé
	 * @param PDO $dbconnection
	 * @param integer $nodeid
	 * @param integer $serviceid
	 * @param date $endate (optionnel) date au format ('YYYY-MM-DD') sinon la date courante est utilisée
	 * @param number |date $interval (optionnel) le nombre de jour à calculé (défault:30)
	 * @retval mixed[] tableau associatif [date]=disponnibilité en pourcentage.
	 */
	static function getTimeErrorAvailability($dbconnection,$nodeid,$serviceid,$endate=NULL,$interval=30)
	{
		$startdate = $interval;
		$time = time();
		if(!isset($endate))
		{
			$endate = date('Y-m-d',$time);
		}
		if(is_numeric($interval))
		{
			$date = new DateTime($endate);
			$startdate = $date->sub(new DateInterval('P'.$interval.'D'));
			//$startdate = $time-($interval*60*60*24);
			$startdate = $startdate->format('Y-m-d');

		}
		$sql = " SELECT * FROM get_TimeError(:nodeid,:serviceid,to_date(:startdate,'YYYY-MM-DD'),to_date(:endate,'YYYY-MM-DD'))";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':nodeid' => $nodeid,
														':serviceid' => $serviceid,
														':startdate' => $startdate,
														':endate' => $endate));
		$state = array('ok' => array('nbrsec' => 0,'percent' => 0),
					   'warning' => array('nbrsec' => 0,'percent' => 0),
					   'critical' => array('nbrsec' => 0,'percent' => 0),
					   'unknown' => array('nbrsec' => 0,'percent' => 0));
		$total = 0;
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if($row->type == 'total')
			{
				$total = $row->nbrsec;
			}
			else
			{
				if(array_key_exists($row->type, $state))
				{
					$state[$row->type]['nbrsec'] = round($row->nbrsec, 2);
				}
			}
		}
		$oknbrsec = $total;
		foreach($state as $key => $value)
		{
			if($key != 'ok')
			{
				if($state[$key]['nbrsec'] != 0)
				{
					$oknbrsec = $oknbrsec - $state[$key]['nbrsec'];
					$state[$key]['percent'] = round(($state[$key]['nbrsec'] / $total) * 100, 2);
				}
			}
		}
		$state['ok']['nbrsec'] = $oknbrsec;
		$state['ok']['percent'] = round(($state['ok']['nbrsec'] / $total) * 100, 2);
		return $state;
	}

	/**
	 * @brief retourne le sn de la unity indique grace à son id
	 * @param PDO $dbconnection
	 * @param integer $id
	 * @retval int sn de la unity
	 */

	public static function getGesaSN($dbconnection,$id)
	{
		$query = $dbconnection->prepare("SELECT gesa_sn
									   FROM gesa_info
									   JOIN esia_device ON device_id = gesa_device_id
									   WHERE gesa_device_id=?");
		try{
			$query = self::executeStatement($query,array($id));
		}
		catch(Exception $ex){
			return NULL;
		}

		if($row = $query->fetch(PDO::FETCH_OBJ))
		{
			return $row->gesa_sn;
		}
		return NULL;
	}

	public static function getRrdPlugin($dbconnection,$id,$rrd_name)
	{
		$query = $dbconnection->prepare("SELECT rrd_vertical_label
									   FROM rrd_plugins
									   WHERE rrd_plugin_id =:id
									   AND rrd_name = :name");
		try{
			$query = self::executeStatement($query,array(':id' => $id,
														':name' => $rrd_name
			));
		}
		catch(Exception $ex){
			return NULL;
		}

		if($row = $query->fetch(PDO::FETCH_OBJ))
		{
			return $row->rrd_vertical_label;
		}
		return NULL;
	}
	public static function getRRDInformationByServiceId($dbconnection,$id)
	{
		$query = $dbconnection->prepare("SELECT service_id,service_name,service_parameter,service_last_stdout,device_name,plugin_name,
							rrd_type,rrd_regex,rrd_time,rrd_name,rrd_max,rrd_min,rrd_vertical_label,rrd_graph_color,rrd_graph_type,rrd_regex_threshold,rrd_graph_num,rrd_calc
						FROM esia_services
						JOIN esia_device ON device_id = service_device_id
						JOIN esia_plugins ON plugin_id = service_plugin_id
						JOIN rrd_plugins ON rrd_plugin_id=service_plugin_id WHERE service_id=? ORDER BY rrd_graph_type,rrd_name;");
		Model_esia::executeStatement($query,array($id));
		$parameter = array();
		$convertRRDLabel2C3 = array('second' => 'second',
				'ms' => 'millisecond',
				'percent' => 'percent',
				'bps' => 'bps',
				'packet/s' => 'packetps',
				'temperature' => 'Celsius',
				'connections' => 'number',
				'Bytes' => 'bytes');

		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$num = 0;
			if(!empty($row->rrd_graph_num))
			{
				$num = $row->rrd_graph_num;
			}
			$parameter[$row->rrd_name] = array('name' => $row->rrd_name,
					'unity' => $row->rrd_vertical_label,
					'calc' => $row->rrd_calc,
					'type' => self::RRD_GRAPH_TYPES_TO_C3[$row->rrd_graph_type],
					'regex' => $row->rrd_regex,
					'color' => $row->rrd_graph_color,
					'num' => $num
			);
		}
		return $parameter;
	}
	/**
	 * @brief retourne le tableau des nouveaux messages à afficher
	 * @param PDO $dbconnection
	 * @param array $userGroupId
	 * @retval mixed[] tableau contenant les nouveaux messages.
	 */

	public static function getCommunicationNewMsg($dbconnection,$user_id,$userDroits,$all=TRUE)
	{
		$sql = "SELECT message_id, type_message, getcontent, title, message, start_date, end_date, show_link,admin_message
			    FROM esia_communication_message
				JOIN esia_communication_message_link_user ON message_link_id = message_id
				WHERE start_date <= NOW() and end_date > NOW()
				AND user_link_id = :userID";

		if($all == FALSE)
		{
			$sql .= " AND show_link = FALSE;";
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':userID' => $user_id));

		$com_msg = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if($row->admin_message == TRUE)
			{
				if($userDroits)
				{
					$com_msg[] = array('id' => $row->message_id,
							'type' => $row->type_message,
							'getcontent' => $row->getcontent,
							'title' => $row->title,
							'message' => $row->message,
							'start_date' => $row->start_date,
							'end_date' => $row->end_date,
							'show' => $row->show_link,
							'admin_message' => $row->admin_message,
					);
				}
			}
			else
			{
				$com_msg[] = array('id' => $row->message_id,
						'type' => $row->type_message,
						'getcontent' => $row->getcontent,
						'title' => $row->title,
						'message' => $row->message,
						'start_date' => $row->start_date,
						'end_date' => $row->end_date,
						'show' => $row->show_link,
						'admin_message' => $row->admin_message,
				);
			}
		}
		return $com_msg;
	}

	/**
	 * @brief retourne le tableau des messages
	 * @param PDO $dbconnection
	 * @param integer $id
	 * @param string $type
	 * @param string $getcontent
	 * @retval mixed[] tableau contenant les messages.
	 */

	public static function getCommunicationMsg($dbconnection,$id=NULL,$type=NULL,$getcontent=NULL)
	{
		$sql = "SELECT message_id, type_message, getcontent, title, message, start_date, end_date,admin_message
				FROM esia_communication_message";

		$param = array();
		if(!empty($id) && $id != NULL)
		{
			$sql .= ' WHERE message_id = ?;';
			$param[] = $id;
		}
		elseif(!empty($type) && $type != NULL)
		{
			$sql .= ' WHERE type_messahge = ?;';
			$param[] = $type;
		}
		elseif(!empty($getcontent) && $getcontent != NULL)
		{
			$sql .= ' WHERE getcontent = ?;';
			$param[] = $getcontent;
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param);

		$com_msg = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$com_msg[] = array('id' => $row->message_id,
					'type' => $row->type_message,
					'getcontent' => $row->getcontent,
					'title' => $row->title,
					'message' => $row->message,
					'start_date' => $row->start_date,
					'end_date' => $row->end_date,
					'admin_message' => $row->admin_message,
			);
		}
		return $com_msg;
	}

	/**
	 * @brief suppression d'un message
	 * @param PDO $dbconnection
	 * @param integer $id
	 */

	public static function delCommunicationMsg($dbconnection,$id)
	{
		if($id != NULL)
		{
			$query = $dbconnection->prepare("DELETE FROM esia_communication_message WHERE message_id = :id");
			$query = Model_esia::executeStatement($query,array(':id' => $id));
		}
	}

	/**
	 * @brief MAJ du tuple message pour valider qu'il a été montré
	 * @param PDO $dbconnection
	 * @param string $title or integer
	 * @param boolean $bool
	 */

	public static function updateShowMsg ($dbconnection,$title,$bool)
	{
		if($title != NULL)
		{
			if(is_numeric($title))
			{
				$query = $dbconnection->prepare("UPDATE esia_communication_message SET show = :bool WHERE message_id = :id");
				$query = Model_esia::executeStatement($query,array(':id' => $title,':bool' => $bool));
			}
			else
			{
				$query = $dbconnection->prepare("UPDATE esia_communication_message SET show = :bool WHERE title = :title");
				$query = Model_esia::executeStatement($query,array(':title' => $title,':bool' => $bool));
			}
		}
	}

	/**
	 * @brief retourne le tableau des messages
	 * @param PDO $dbconnection
	 * @param integer $id
	 * @param string $type
	 * @param string $getcontent
	 * @retval mixed[] tableau contenant les messages.
	 */

	public static function reloadDateMsg($dbconnection,$title,$date)
	{
		if($title != NULL)
		{
			$query = $dbconnection->prepare("UPDATE esia_communication_message SET start_date = to_date(:date_start, 'DD/MM/YYYY') WHERE title = :title");
			$query = Model_esia::executeStatement($query,array(':title' => $title,':date_start' => $date));
		}
	}

	/**
	 * @brief MAJ du tuple de la table esia_communication_message
	 * @param PDO $dbconnection
	 * @param integer $id
	 * @param string $type
	 * @param string $getcontent
	 * @param string $title
	 * @param string $message
	 * @param date $start_date
	 * @param date $end_date
	 * @param boolean $show
	 * @param string $group_user_show en format json
	 */

	public static function updCommunicationMsg($dbconnection,$id,$type,$getcontent,$title,$message,$start_date,$end_date,$admin_msg)
	{
		$sql = "UPDATE esia_communication_message SET type_message=:type, getcontent=:getcontent, title=:title, message=:msg,start_date = to_date(:date_start, 'DD/MM/YYYY'),admin_message=:admin_msg,
				end_date = to_date(:date_end, 'DD/MM/YYYY') WHERE message_id = :id;";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':id' => $id,
				':type' => $type,
				':getcontent' => $getcontent,
				':title' => $title,
				':msg' => $message,
				':date_start' => $start_date,
				':admin_msg' => $admin_msg,
				':date_end' => $end_date));
	}

	/**
	 * @brief ADD d'un tuple de la table esia_communication_message
	 * @param PDO $dbconnection
	 * @param integer $id
	 * @param string $type
	 * @param string $getcontent
	 * @param string $title
	 * @param string $message
	 * @param date $start_date
	 * @param date $end_date
	 * @param boolean $show
	 * @param json $group_user_show
	 */

	public static function addCommunicationMsg($dbconnection,$type,$getcontent,$title,$message,$start_date,$end_date,$show,$admin_msg)
	{
		$query = $dbconnection->prepare("INSERT INTO esia_communication_message (type,getcontent,title,message,start_date,end_date)
				    					VALUES(:type,:getcontent,:title,:message,to_date(:start_date, 'DD/MM/YYYY'),to_date(:end_date, 'DD/MM/YYYY'),:admin_msg) RETURNING message_id");

		$query = Model_esia::executeStatement($query,array(':type' => $type,
				':getcontent' => $getcontent,
				':title' => $title,
				':message' => $message,
				':start_date' => $start_date,
				'admin_msg' => $admin_msg,
				':end_date' => $end_date));
		$row = $query->fetch(PDO::FETCH_OBJ);
		return $row->message_id;
	}

	/**
	 * @brief ADD d'un tuple de la table esia_communication_message
	 * @param PDO $dbconnection
	 * @param integer $id_msg identifiant message
	 * @param integer $user_id identifiant user
	 */

	public static function updateMessageUserRead($dbconnection,$id_msg,$user_id)
	{
		$query = $dbconnection->prepare("UPDATE esia_communication_message_link_user SET show_link = TRUE WHERE message_link_id=:msg_id AND user_link_id=:user_id;");
		$query = Model_esia::executeStatement($query,array(':msg_id' => $id_msg,':user_id' => $user_id));
	}

	/**
	 * @brief ADD d'un tuple de la table esia_communication_message
	 * @param PDO $dbconnection
	 * @param integer $id_msg identifiant message
	 * @param integer $user_id identifiant user
	 */

	public static function insertMessageUserRead($dbconnection,$id_msg,$user_id)
	{
		$query = $dbconnection->prepare("INSERT INTO esia_communication_message_link_user(message_link_id, user_link_id, show_link) VALUES (:msg_id, :user_id, FALSE);");
		$query = Model_esia::executeStatement($query,array(':msg_id' => $id_msg,':user_id' => $user_id));
	}

	/**
	 * brief Récupére la liste des noms hiérarchisés des groupes de noeuds d'un utilisateur et leurs ids
	 * @param PDO $dbconnection l'object db
	 * @param integer $user l'id de l'utilisateur concerné
	 * @return mixed[]	Un array de type array(id=>integer,name=>'parent_name>parent_name>group_name)
	 */
	public static function getNodeGroupHierarchyByUser($dbconnection,$user)
 {
		// Récupération des groupes
		$sql = 'SELECT device_group_id,device_group_name,device_group_parent FROM esia_device_group LEFT JOIN esia_device_group_in ON device_group_id = group_in_device_group_id';
		$sql .= ' GROUP BY device_group_id,device_group_parent ORDER BY device_group_parent DESC';
		$query = $dbconnection->prepare($sql);
		$all_groups = Model_esia::executeStatement($query);

		$indexed_groups = array();
		foreach($all_groups as $group){
			$indexed_groups[$group['device_group_id']] = $group;
		}
		unset($all_groups);

		$sql = 'SELECT device_group_id,device_group_name,device_group_parent FROM esia_device_group LEFT JOIN esia_device_group_in ON device_group_id = group_in_device_group_id WHERE group_in_device_id IN';
		$sql .= ' (SELECT * FROM getUserAccessNode(?)) AND device_group_id IN (SELECT group_id FROM getAllChildDeviceGroup(NULL))';
		$sql .= ' GROUP BY device_group_id,device_group_parent ORDER BY device_group_parent DESC';
		$query = $dbconnection->prepare($sql);
		$user_groups = Model_esia::executeStatement($query,array($user));

		$list = array();
		foreach($user_groups as $u_node){
			$grp_item['id'] = $u_node['device_group_id'];
			$grp_item['name'] = $u_node['device_group_name'];

			while($u_node['device_group_parent'] != NULL){
				$u_node = $indexed_groups[$u_node['device_group_parent']];
				$grp_item['name'] = $u_node['device_group_name'] . '>' . $grp_item['name'];
			}
			$list[] = $grp_item;
		}
		return $list;
	}
	/**
	 * retourne les informations de la licence
	 * @param PDO $dbconnection
	 * @retval array un objet contenant la licence
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getLicenceInformation($dbconnection,$type='LICENCE')
	{
		$query = $dbconnection->prepare("SELECT config_value
				FROM esia_configuration
				WHERE config_key=?;");
		$query = Model_esia::executeStatement($query,array($type));
		$string = $query->fetch(PDO::FETCH_OBJ);
		$licence = array();
		if(!empty($string->config_value))
		{
			if(preg_match('#id\:\s?(.*)\n#',$string->config_value,$regs))
			{
				$licence['id'] = $regs[1];
			}
			if(preg_match('#Owner\:\s?(.*)\n#',$string->config_value,$regs))
			{
				$licence['owner'] = $regs[1];
			}
			if(preg_match('#Node\:\s?(.*)\n#',$string->config_value,$regs))
			{
				$licence['node'] = intval($regs[1]);
			}
			if(preg_match('#Type\:\s?(.*)\n#',$string->config_value,$regs))
			{
				$licence['type'] = $regs[1];
			}
			if(preg_match('#ExpirationDate\:\s?(.*)\n#',$string->config_value,$regs))
			{
				$licence['expire'] = $regs[1];
				$licence['expire_date_formated'] = strftime('%d/%m/%Y',$regs[1]);
			}
		}
		else
		{
			throw new EsiaDbException('Licence not found');
		}
		$query = $dbconnection->prepare("SELECT config_value FROM esia_configuration WHERE config_key='LICENCE_RETEST';");
		$query = Model_esia::executeStatement($query);
		$retest = $query->fetch(PDO::FETCH_OBJ);
		if($retest->config_value != 0)
		{
			if(!is_executable('/usr/local/esia/licence/validate_licence'))
			{
				$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=3 WHERE config_key='LICENCE_RETEST';");
				$query = Model_esia::executeStatement($query);
				throw new EsiaDbException('Installation is corrupted');
			}
			$param ="";
			if($type =='LICENCE_EGIDE')
			{
				$param= '-t LICENCE_EGIDE';
			}
			exec('/usr/local/esia/licence/validate_licence '.$param,$output,$ret);
			if($ret == 0)
			{
				$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=0 WHERE config_key='LICENCE_RETEST';");
				$query = Model_esia::executeStatement($query);
			}
			else
			{
				throw new EsiaDbException($output[0]);
			}
		}
		elseif($retest->config_value == 3)
		{
			throw new EsiaDbException('Installation is corrupted: Daemon is stopped');
		}

		return $licence;
	}
	/**
	 * retourne les informations de la licence
	 * @param PDO $dbconnection
	 * @retval integer 0 si aucune erreur
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function updateLicenceInformation($dbconnection,$licence)
	{
		$flag = 0;
		$type ="";
		if(!empty($licence))
		{
			if(preg_match('#id\:(.*)\n#',$licence,$regs))
			{
				$flag++;
			}
			if(preg_match('#Owner\:(.*)\n#',$licence,$regs))
			{
				$flag++;
			}
			if(preg_match('#Node\:(.*)\n#',$licence,$regs))
			{
				$node = trim($regs[1]);
				if(is_numeric($node))
				{
					$flag++;
				}
			}
			if(preg_match('#Type\:(.*)\n#',$licence,$regs))
			{
				$valid_type = array("Galaxy","Unity","Universe","Mercury","Mercury-client","Mercury-fixed","Infinity","Egide","Egide-client");
				$type = trim($regs[1]);
				if(in_array($type,$valid_type))
				{
					$flag++;
				}
			}
			if(preg_match('#ExpirationDate\:(.*)\n#',$licence,$regs))
			{
				$time = time();
				$expi = trim($regs[1]);
				if($time < $expi)
				{
					$flag++;
				}
				else
				{
					throw new EsiaDbException('Licence file expired');
				}
			}
		}
		else
		{
			throw new EsiaDbException('Licence not found');
		}
		if($flag < 5)
		{
			throw new EsiaDbException('Bad format licence file');
		}
		if($type=='Egide'||$type=='Egide-client')
		{
			$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=? WHERE config_key='LICENCE_EGIDE';");
			$query = Model_esia::executeStatement($query, array($licence));
		}
		else
		{
			$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=? WHERE config_key='LICENCE';");
			$query = Model_esia::executeStatement($query, array($licence));
		}

		return $licence;
	}
	/**
	 * nettoie le fichier licence
	 * @param PDO $dbconnection
	 * @retval integer 0 si aucune erreur
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function cleanLicenceInformation($dbconnection,$type='LICENCE')
	{
		$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=NULL WHERE config_key=?;");
		$query = Model_esia::executeStatement($query, array($type));
	}
	/**
	 * Retourne le nombre de noeud consommé de la licence
	 * @param PDO $dbconnection
	 * @retval integer 0 si aucune erreur
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function countNodeNumberForLicence($dbconnection)
	{
		$query = $dbconnection->prepare("SELECT count(*) as nbr FROM esia_device WHERE device_type <> 'gesa';");
		$query = Model_esia::executeStatement($query);
		$numbernode = $query->fetch(PDO::FETCH_OBJ);
		$numbernode = $numbernode->nbr;
		$query = $dbconnection->prepare("SELECT SUM(conso_node) as nbr FROM esia_consumption;");
		$query = Model_esia::executeStatement($query);
		$mercurynode = $query->fetch(PDO::FETCH_OBJ);
		$numbernode = $numbernode + $mercurynode->nbr;
		//-1 pour décompter le serveur locale.
		return ($numbernode - 1);
	}
	/**
	 * Retourne les informations des clients mercurys
	 * @param PDO $dbconnection
	 * @param integer $nodeid  (optionnel) l'identifiant du client mercury
	 * @retval array un tableau contenant les informations du client mercury array('id'=>$row->device_id,'name'=>$row->device_name,'advanced_node'=>$row->conso_node,'state'=>$state,'max_node'=>$row->conso_max_node)
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getMercuryClient($dbconnection,$nodeid=NULL)
	{
		$sql = "SELECT device_id,device_name,conso_node,date_part('hour',(NOW() - conso_last_update)) as hour,conso_max_node
                                        FROM esia_device
                                        JOIN esia_consumption ON device_id = conso_device_id";
		$param = array();
		if(!empty($nodeid))
		{
			$sql .= " WHERE device_id=?";
			$param[] = $nodeid;
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$state['warning'] = 0;
			$state['unknown'] = 0;
			if($row->hour > 1)
			{
				$state['critical'] = 1;
				$state['ok'] = 0;
			}
			else
			{
				$state['critical'] = 0;
				$state['ok'] = 1;
			}
			$arr[] = array('id' => $row->device_id,'name' => $row->device_name,'advanced_node' => $row->conso_node,'state' => $state,'max_node' => $row->conso_max_node);
		}

		return $arr;
	}
	/**
	 * Met à jour la conso maximun des clients mercury.
	 * @param PDO $dbconnection
	 * @retval integer 0 si aucune erreur
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function updateMercuryClient($dbconnection,$nodeid,$nodesattribute)
	{
		$query = $dbconnection->prepare("UPDATE esia_consumption SET conso_max_node=? WHERE conso_device_id =?");
		$query = Model_esia::executeStatement($query,array($nodesattribute,$nodeid));
	}
	/**
	 * @brief Ressort le top des noeuds en erreur sur une période et un nombre donnée
	 * @param PDO $dbconnection
	 * @param integer $user identifiant de l'utilisateur
	 * @param integer $interval interval de temps en jour
	 * @param integer $typeerror type des erreurs à compter
	 * @param integer $nbr nombre de noeuds maximun à retourner
	 */
	static function getNodesTopError($dbconnection,$user=null,$interval=1,$typeerror=array('WARNING','CRITICAL'),$nbr=10)
	{
		if($nbr < 1 )
		{
			//echo "ici $nbr";
			throw new EsiaDbException('nbr must be numeric and > 0');
		}
		$arr = implode("','", $typeerror);
		$arr = strtolower($arr);
		$sql = "SELECT device_id,device_name,device_type, count(*) as nbr
        FROM esia_device
        JOIN esia_error_happen ON device_id = happen_device_id
        JOIN esia_errors ON error_id = happen_error_id
        WHERE 	happen_timestamp > (CURRENT_DATE - :day::interval) AND
	        device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND
	        error_type IN ('$arr')
        GROUP BY device_id,device_name,device_type
        ORDER BY nbr DESC
        LIMIT $nbr";
		$nodetype = array();
		$arr = Model_esia::getWebPluginByNodeType($dbconnection);
		foreach($arr as $row)
		{
			$nodetype[$row['type']]=$row['apache_path'].$row['control_page'];
		}
		$arr = array();
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':user'=>$user,':day'=>"$interval days"));
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			if(array_key_exists($row->device_type,$nodetype))
			{
				$page_control = Model_esia::replacePageControlVariable($nodetype[$row->device_type],$row);
			}
			else
			{
				$page_control = '/node_view.php?nodeid='.$row->device_id;
			}
			$arr[]=array(
				'type_device'=>$row->device_type,
				'name'=>$row->device_name,
				'page_control'=>$page_control,
				'nbr'=>$row->nbr,
				'id'=>$row->device_id);
		}
		return $arr;
	}
	/**
	 * @brief Retourne les configuration avancée mise sur un noeuds
	 * @param PDO $dbconnection
	 * @param integer $nodeid identifiant du noeuds
	 * @param integer|string $configid identifiant de la configuration avancée (su nodeid est spécifié, on peut passer le nom pour avoir directement le bon.
	 * @param integer $config_type_id identifiant de la configuration
	 * @retval array array(  'id'=> $row->adv_id,
				'node_id'=> $row->adv_device_id,
				'name'=> $row->device_config_type_name,
				'controller'=> $row->device_config_type_controller,
				'parameter'=> json_decode($row->adv_param,TRUE),
				'config_type_id'=> $row->adv_config_type_id,
				'web_plugin_id'=> $row->web_id,
				'web_plugin_path'=> $row->web_plugin_apache_path,
				'file_content'=>$authfile
	 */
	static function getAdvanceNodeConfig($dbconnection,$nodeid=null,$configid=null,$config_type_id=null)
	{
		$sql = "SELECT adv_id, adv_device_id, adv_config_type_id, adv_param, adv_authfile_crypt,adv_plugin_keyword_value, device_config_type_name,
						device_config_type_controller,web_plugin_apache_path,web_id,device_config_type_nodetab
                FROM esia_device_advance_config
                JOIN esia_device_advance_config_type ON adv_config_type_id = device_config_type_id
                LEFT JOIN esia_web_plugin ON device_config_type_webplugin_id = web_id";
		$params = array();
		if(is_numeric($nodeid) && is_string($configid))
		{
			$sql .=" WHERE adv_device_id = :id AND device_config_type_name= :name";
			$params[':id']=$nodeid;
			$params[':name']=$configid;
		}
		elseif(is_numeric($nodeid))
		{
			$sql .=" WHERE adv_device_id = :id ";
			$params[':id']=$nodeid;
		}
		elseif(is_numeric($configid))
		{
			$sql .=" WHERE adv_id = :id ";
			$params[':id']=$configid;
		}
		elseif(is_numeric($config_type_id))
		{
			$sql .=" WHERE adv_config_type_id = :id ";
			$params[':id']=$config_type_id;
		}

		$query = $dbconnection->prepare($sql);
		$query = self::executeStatement($query,$params);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$authfile="";
			if(!empty($row->adv_authfile_crypt))
			{
				$authfile=LoadSettings($dbconnection,$row->adv_id);

			}
			$arr[]= array(  'id'=> $row->adv_id,
				'node_id'=> $row->adv_device_id,
				'name'=> $row->device_config_type_name,
				'controller'=> $row->device_config_type_controller,
				'parameter'=> json_decode($row->adv_param,TRUE),
				'config_type_id'=> $row->adv_config_type_id,
				'web_plugin_id'=> $row->web_id,
				'web_plugin_path'=> $row->web_plugin_apache_path,
				'file_content'=>$authfile,
				'nodetab'=>$row->device_config_type_nodetab
			);
		}
		return $arr;
	}
	/**
	 * @brief Retourne les configuration avancée disponnible pour les noeuds
	 * @param PDO $dbconnection
	 * @param integer|string $configid identifiant de la configuration avancée
	 * @param integer|string $webplugin identifiant d'un webplugin pour avoir ceux lié
	 * @retval array $arr[]= array(  'id'=> $row->device_config_type_id,
				'name'=> $row->device_config_type_name,
				'controller'=> $row->device_config_type_controller,
				'parameter'=> json_decode($row->device_config_type_default_parameter,TRUE),
				'web_plugin_id'=> $row->web_id,
				'web_plugin_path'=> $row->web_plugin_apache_path,
				'web_plugin_activate'=> $row->web_activate
	 */
	static function getAdvanceNodeConfigType($dbconnection,$id=null,$webplugin=null)
	{
		$sql = "SELECT device_config_type_id, device_config_type_name, device_config_type_controller,device_config_type_default_parameter::text, web_id, web_plugin_apache_path,web_activate
                FROM esia_device_advance_config_type
                LEFT JOIN esia_web_plugin ON device_config_type_webplugin_id = web_id";
		$params = array();
		if(is_numeric($id))
		{
			$sql .=" WHERE device_config_type_id = :id ";
			$params[':id']=$id;
		}
		elseif(is_string($id))
		{
			$sql .=" WHERE device_config_type_name = :name ";
			$params[':name']=$id;
		}
		elseif(is_numeric($webplugin))
		{
			$sql .=" WHERE web_id = :id ";
			$params[':id']=$webplugin;
		}
		elseif(is_string($webplugin))
		{
			$sql .=" WHERE web_name = :name ";
			$params[':name']=$webplugin;
		}
		$query = $dbconnection->prepare($sql);
		$query = self::executeStatement($query,$params);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[]= array(  'id'=> $row->device_config_type_id,
				'name'=> $row->device_config_type_name,
				'controller'=> $row->device_config_type_controller,
				'parameter'=> json_decode($row->device_config_type_default_parameter,TRUE),
				'web_plugin_id'=> $row->web_id,
				'web_plugin_path'=> $row->web_plugin_apache_path,
				'web_plugin_activate'=> $row->web_activate
			);
		}
		return $arr;
	}
	/**
	 * @brief Retourne les onglets suppplémentaire d'un noeud ou d'un web plugin
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du noeud
	 * @param integer $web_id identifiant d'un webplugin pour avoir ceux lié
	 * @retval array $arr[]= array(  'id'=> $row->tab_id,
				'name'=> $row->tab_name,
				'url'=> $apache_path.'/'.$row->tab_url,
				'web_plugin_id'=> $row->web_id,
				'web_plugin_path'=> $row->web_plugin_apache_path,
			);
	 */
	static function getNodeTabs($dbconnection,$id=null,$web_id=null)
	{
		$sql = "SELECT tab_id,tab_linked_order, tab_linked_option,tab_name, tab_default_options, tab_url, tab_web_id, web_plugin_apache_path,tab_linked_device_id
				FROM device_tab_linked
				JOIN device_tab ON tab_linked_tab_id = tab_id
				JOIN esia_web_plugin ON tab_web_id = web_id
				WHERE web_activate=true";
		$params = array();
		if(!empty($id))
		{
			$sql .=' AND tab_linked_device_id = :id ';
			$params[':id']=$id;
		}
		if(!empty($web_id))
		{
			$sql .=' AND web_id = :webid ';
			$params[':webid']=$web_id;
		}
		$sql .=' ORDER BY tab_linked_order ';
		$query = $dbconnection->prepare($sql);
		$query = self::executeStatement($query,$params);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$apache_path = rtrim($row->web_plugin_apache_path,'/');

			$arr[]= array(  'id'=> $row->tab_id,
				'name'=> $row->tab_name,
				'url'=> $apache_path.'/'.$row->tab_url,
				'web_plugin_id'=> $row->tab_web_id,
				'web_plugin_path'=> $row->web_plugin_apache_path,
				'node_id' => $row->tab_linked_device_id
			);
		}
		return $arr;
	}
	/**
	 * @brief Ajoute l'onglet au noeud lié
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du noeud
	 * @param string $tabname nom de l'onglet dans la table device_tab
	 * @retval /
	 */
	static function addNodeTabs($dbconnection,$id,$tabname)
	{
		$sql = "INSERT INTO device_tab_linked(tab_linked_device_id, tab_linked_tab_id)
					VALUES (:nodeid,(SELECT tab_id FROM device_tab WHERE tab_name=:name)) ON CONFLICT DO NOTHING;";
		$query = $dbconnection->prepare($sql);
		$params[':nodeid']=$id;
		$params[':name']=$tabname;
		$query = self::executeStatement($query,$params);
	}

	/**
	  *  @brief Configure le morceau de code sql et les params pour une clause sql de type IN or NOT IN
	  * @details
	  * Cette fonction permet de générer une partie de code sql à placer dans une clause IN or NOT IN. Elle retourne ce code et configure les paramètre dans $params
	  * Exemple :
	  * $params = array();
	  * $sql_in = pdo_array_to_SQL($'device_id_key',array(4,7,12),&$params);
	  * $sql = 'SELECT * FROM esia_devices WHERE device_id IN ($sql_in)';
	  * $query = $dbconnection->prepare($sql);
	  * $query = self::executeStatement($query,$params);
	  * @params string $key Pas important mais nécessaire, doit juste être unique pour une requête sql. Il est conseillé de mettre la colonne de la table conçernée
	  * @params array $arr Doit être un tableau (non-associatif). La liste des valeurs qu'on souhaite passer dans la clause de type IN
	  * @params array $params Tableau à passer en paramètre à la fonction self::executeStatement().
	  * @retval string retourne la portion d'une requête SQL à mettre entre les parenthése d'une clause IN.
	*/
	static function pdo_array_to_SQL($key,$arr,&$params)
	{
	    $in_sql = '';

	    if(is_array($arr))
	    {
	        $i=0;
	        foreach($arr as $item)
	        {
	            $params[':'.$key.$i]=$item;
	            $in_sql .= ',:'.$key.$i;
	            $i++;
	        }

	        $in_sql = ltrim($in_sql,',');
	    }
	    else
	    {
	        throw new Exception("Parameter arr must be an array");
	    }
	    return $in_sql;
	}
	/**
	 * @brief Ajoute une tache cron à la table
	 * @param PDO $dbconnection
	 * @param integer $id identifiant de la table plugin
	 * @param string $parameter parametre à passer à la tache planifié
	 * @param int $interval d'execution en minute si champs à null ou vide alors c"est une tache oneSHOT
	 * @param timestamp $timeexec timestamp Unix pour la prochaine exécution
	 * @param string $logfile nom du fichier de log les mots clés suivant sont disponibles($DATE,$SERVICEID)
	 * @param int $retrie d'execution en minute
	 * @param int $retrie_interval d'execution en minute pour les retries
	 * @retval integer l'id db de la tache
	 */
	static function addCronTask($dbconnection,$id,$parameter,$interval,$timeexec,$logfile,$retrie,$retrie_interval)
	{
		$sql = "INSERT INTO esia_cron_plugin(
					cron_id, cron_plugin_id, cron_interval, cron_status, cron_next_exec, cron_log_file,
					cron_retries, cron_retries_interval, cron_parameter)
				VALUES (DEFAULT,:plugin,:inter,'PENDING',to_timestamp(:timeexec),:log,:retrie,
						:retrieinter, :parameter) RETURNING cron_id;";
		$query = $dbconnection->prepare($sql);

		$param[':plugin']=$id;
		$param[':parameter']=$parameter;
		$param[':timeexec']=$timeexec;
		$param[':log']=$logfile;
		$param[':retrie']=$retrie;
		$param[':retrieinter']="$retrie_interval second";

		if(empty($interval))
		{
			$param[':inter'] = '0 second';
		}
		elseif(is_int($interval))
		{
			$param[':inter'] = $interval . ' second';
		}
		elseif(preg_match('#^[0-9]*\s+[a-zA-Z]*#',$interval))
		{
			$param[':inter'] = $interval;
		}
		else
		{
			throw new EsiaDbException('Bad interval parameter: '.$interval.', '.$retrie_interval.' value must be integer > 0 or string like 1 month');
		}

		$query = self::executeStatement($query,$param);
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->cron_id;
	}
	/**
	 * @brief Retourne les plugins de type cron de la table esia_plugins (champs plugin_cron_type='cron')
	 * @param PDO $dbconnection
	 * @param integer|array $plugin identifiant de la table plugin
	 * @retval 			$arr[]= array(  'id'=> $row->plugin_id,
	 *			'name'=> $row->plugin_name,
	 *			'path'=> $row->plugin_path,
	 *			'default_parameter'=> ''.$row->plugin_default_parameter,
	 *			'default_interval_time'=> $row->plugin_default_interval_time/1000,
	 *			'type'=> $row->plugin_cron_type,
	 *			'only_flag'=> $row->plugin_cron_only_existing,
	 */
	static function getCronPlugin($dbconnection,$plugin=null)
	{
		$sql = "SELECT plugin_id, plugin_name, plugin_path, plugin_default_parameter, plugin_default_interval_time,
						plugin_cron_type, plugin_cron_only_existing
				FROM esia_plugins
				WHERE plugin_type='cron' ";
		$params=array();
		if(!empty($plugin))
		{
			if(is_array($plugin))
			{
				$tmp="";
				foreach($plugin as $id)
				{
					if(is_numeric($id))
					{
						$tmp=":plugin".$id.',';
						$params[':plugin'.$id]=$id;
					}
				}
				$tmp=rtrim($tmp,',');
				$sql .= "AND plugin_id IN ($tmp)";
			}
			elseif(is_numeric($plugin))
			{
				$sql .= "AND plugin_id= :id";
				$params[':id']=$plugin;
			}
			else
			{
				$sql .= "AND plugin_name=:name";
				$params[':name']=$plugin;
			}
		}
		$query = $dbconnection->prepare($sql);
		$query = self::executeStatement($query,$params);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[]= array(  'id'=> $row->plugin_id,
				'name'=> $row->plugin_name,
				'path'=> $row->plugin_path,
				'default_parameter'=> ''.$row->plugin_default_parameter,
				'default_interval_time'=> $row->plugin_default_interval_time/1000,
				'type'=> $row->plugin_cron_type,
				'only_flag'=> $row->plugin_cron_only_existing,
			);
		}
		return $arr;
	}
	/**
	 * @brief Retourne les taches actives
	 * @param PDO $dbconnection
	 * @param integer|array $id identifiant de la tache cron
	 * @param integer|array $plugin identifiant de la table plugin
	 * @param string $orderBy tri (next pour cron_next_exec, last pour cron_last_exec)
	 * @param boolean|null $isRepeate filtre les resultats pour retourner uniquement les taches repetables si true, non repetables si false et toutes les taches si null (pas de filtre). Une tache est repetable si son interval n'est pas null et > 0.
	 * @param integer|array $interval identifiant de la table plugin
	 * @retval 			$arr[]= array(  'id'=> $row->cron_id,
				'name'=> $row->plugin_name,
				'pluginid'=> $row->cron_plugin_id,
				'path'=> $row->plugin_path,
				'parameter'=> $row->cron_parameter,
				'interval'=> $row->cron_interval,
				'status'=> $row->cron_status,
				'last_exec'=> $row->last_exec,
				'next_exec'=> $row->next_exec,
				'logfile'=> $row->cron_log_file,
				'last_filelog'=>$filename,
				'retries'=> $row->cron_retries,
				'retries_interval'=> $row->cron_retries_interval,
				'default_parameter'=> $row->plugin_default_parameter,
				'default_interval_time'=> $row->plugin_default_interval_time/1000,
				'type'=> $row->plugin_cron_type,
				'only_flag'=> $row->plugin_cron_only_existing,
	 */
	static function getCronActiveTasks($dbconnection,$ids=null,$plugin=null,$orderBy=NULL,$isRepeate=NULL,$interval=NULL)
	{
		$sql = "SELECT cron_id, cron_plugin_id, cron_status,
						EXTRACT(EPOCH FROM cron_interval) as cron_interval,
						EXTRACT(EPOCH FROM cron_retries_interval) as cron_retries_interval,
						to_char(cron_last_exec,'DD/MM/YYYY HH24:MI:SS') as last_exec,
						to_char(cron_next_exec,'DD/MM/YYYY HH24:MI:SS') as next_exec,
						cron_log_file, cron_retries, cron_parameter,
						plugin_name, plugin_path, plugin_default_parameter, plugin_default_interval_time,
						plugin_cron_type, plugin_cron_only_existing
				FROM esia_cron_plugin
				JOIN esia_plugins ON plugin_id = cron_plugin_id
				WHERE cron_delete=false ";
		$params=array();
		if(!empty($ids))
		{
			if(is_array($ids))
			{
				$tmp="";
				foreach($ids as $id)
				{
					if(is_numeric($id))
					{
						$tmp.=":id".$id.',';
						$params[':id'.$id]=$id;
					}
				}
				$tmp=rtrim($tmp,',');
				$sql .= " AND cron_id IN ($tmp)";
			}
			else
			{
				$sql .= " AND cron_id= :id";
				$params[':id']=$ids;
			}
		}
		if(!empty($plugin))
		{
			if(is_array($plugin))
			{
				$tmp="";
				foreach($plugin as $id)
				{
					if(is_numeric($id))
					{
						$tmp.=":plugin".$id.',';
						$params[':plugin'.$id]=$id;
					}
				}
				$tmp=rtrim($tmp,',');
				$sql .= " AND plugin_id IN ($tmp)";
			}
			else
			{
				$sql .= " AND plugin_id= :plugin";
				$params[':plugin']=$plugin;
			}
		}
		if(!is_null($isRepeate))
		{
			if($isRepeate == TRUE)
			{
				$sql .= " AND cron_interval IS NOT NULL AND cron_interval > interval '0 second'";
			}
			elseif($isRepeate == FALSE)
			{
				$sql .= " AND cron_interval = interval '0 second'";
			}
		}
		if(!empty($interval))
		{
			if(is_numeric($interval))
			{
				$sql .= " AND cron_last_exec > NOW() - :day::interval";
				$params[':day']=$interval.' day';
			}
		}

		if(!empty($orderBy))
		{
			if($orderBy == 'next')
			{
				$sql .= " ORDER BY cron_next_exec ASC";
			}
			elseif($orderBy == 'last')
			{
				$sql .= " ORDER BY cron_last_exec ASC";
			}
		}

		$query = $dbconnection->prepare($sql);
		$query = self::executeStatement($query,$params);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$filename = "";
			$tmp  = $row->cron_log_file;
			$tmp = preg_replace('#\$DATE#','last',$tmp);
			$tmp = preg_replace('#\$SERVICEID#',$row->cron_id,$tmp);
			if(file_exists(RACINE_SITE.'/data/cron/'.$tmp))
			{
				$filename= RACINE_SITE.'/data/cron/'.$tmp;
			}
			$arr[]= array(  'id'=> $row->cron_id,
				'name'=> $row->plugin_name,
				'pluginid'=> $row->cron_plugin_id,
				'path'=> $row->plugin_path,
				'parameter'=> $row->cron_parameter,
				'interval'=> $row->cron_interval,
				'status'=> $row->cron_status,
				'last_exec'=> $row->last_exec,
				'next_exec'=> $row->next_exec,
				'logfile'=> $row->cron_log_file,
				'last_filelog'=>$filename,
				'retries'=> $row->cron_retries,
				'retries_interval'=> $row->cron_retries_interval,
				'default_parameter'=> $row->plugin_default_parameter,
				'default_interval_time'=> $row->plugin_default_interval_time/1000,
				'type'=> $row->plugin_cron_type,
				'only_flag'=> $row->plugin_cron_only_existing,
			);
		}
		return $arr;
	}
	/**
	 * @brief met à jour la tache cron demandé
	 * @param PDO $dbconnection
	 * @param integer $id identifiant de la table esia_cron_plugin
	 * @param string $parameter parametre à passer à la tache planifié
	 * @param int $interval d'execution en seconde
	 * @param timestamp $timeexec timestamp Unix pour la prochaine exécution
	 * @param string $logfile nom du fichier de log les mots clés suivant sont disponibles($DATE,$SERVICEID)
	 * @param int $retrie d'execution en seconde
	 * @param int $retrie_interval d'execution en minute pour les retries
	 * @retval /
	 */
	static function updateCronTask($dbconnection,$id,$parameter,$interval,$timeexec,$logfile,$retrie,$retrie_interval)
	{
		$sql = "UPDATE esia_cron_plugin
				SET cron_interval = :inter,
					cron_next_exec = to_timestamp(:timeexec),
					cron_log_file = :log,
					cron_retries = :retrie,
					cron_retries_interval  = :retrieinter,
					cron_parameter  = :parameter
				WHERE cron_id = :cron;";
		$query = $dbconnection->prepare($sql);

		$param[':cron']=$id;
		$param[':parameter']=$parameter;
		$param[':timeexec']=$timeexec;
		$param[':log']=$logfile;
		$param[':retrie']=$retrie;
		$param[':retrieinter']="$retrie_interval second";

		if(empty($interval))
		{
			$param[':inter'] = '0 second';
		}
		elseif(is_int($interval))
		{
			$param[':inter'] = $interval . ' second';
		}
		elseif(preg_match('#^[0-9]*\s+[a-zA-Z]*#',$interval))
		{
			$param[':inter'] = $interval;
		}
		else
		{
			throw new EsiaDbException('Bad interval parameter: '.$interval.', '.$retrie_interval.' value must be integer > 0 or string like 1 month');
		}

		$query = self::executeStatement($query,$param);
	}
	/**
	 * @brief supprime la tache cron demandé
	 * @param PDO $dbconnection
	 * @param integer $id identifiant de la table esia_cron_plugin
	 * @retval /
	 */
	static function deleteCronTask($dbconnection,$id)
	{
		$sql = "UPDATE esia_cron_plugin SET cron_delete=true,cron_status='DELETE' WHERE  cron_id = :cron;";

		$query = $dbconnection->prepare($sql);
		$params[':cron']=$id;
		$query = self::executeStatement($query,$params);
	}
	/**
	 * @brief retourne les erreurs de la tache passé en paramètre ou toutes
	 * @param PDO $dbconnection
	 * @param integer|array $id identifiant de la table esia_cron_plugin
	 * @retval array 	$arr[]= array(  'error_id'=> $row->cron_error_id,
	 *			'cron_id'=> $row->cron_id,
	 *			'date'=>$row->dateerror,
     *		'logfile'=> $row->cron_logfile);
	 */
	static function getCronTaskError($dbconnection,$cronids=null)
	{
		$params = array();
		$sql = "SELECT cron_error_id, cron_id, to_char(cron_date,'DD/MM/YYYY HH24:MI:SS') as dateerror, cron_logfile
				FROM esia_cron_error";
		if(is_numeric($cronids))
		{
			$sql .=" WHERE cron_id = :cron";
			$params[':cron']=$cronids;
		}
		elseif(is_array($cronids))
		{
			$tmp="";
			foreach($cronids as $id)
			{
				if(is_numeric($id))
				{
					$tmp .= ":plugin".$id.',';
					$params[':plugin'.$id]=$id;
				}
			}
			$tmp=rtrim($tmp,',');
			$sql .= " WHERE cron_id IN ($tmp)";
		}
		$query = $dbconnection->prepare($sql." ORDER BY cron_date DESC");
		$query = self::executeStatement($query,$params);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[]= array(  'error_id'=> $row->cron_error_id,
				'cron_id'=> $row->cron_id,
				'date'=>$row->dateerror,
				'logfile'=> $row->cron_logfile);
		}
		return $arr;
	}
	/**
	 * @brief supprime l'historique des erreurs
	 * @param PDO $dbconnection
	 * @param integer $cronids supprime toutes les erreurs lié au cron passé en paramètre
	 * @param integer|array $error_id supprime toutes les erreurs passé en paramètre
	 * @retval /
	 */
	static function deleteCronTaskArchive($dbconnection,$cronids=null,$error_id=null)
	{
		$params = array();
		$sql = "UPDATE esia_cron_error SET cron_delete = TRUE";
		if(is_numeric($cronids))
		{
			$sql .=" WHERE cron_id = :cron;";
			$params[':cron']=$cronids;
		}
		elseif(!empty($error_id))
		{
			if(is_numeric($error_id))
			{
				$sql .=" WHERE cron_error_id = :error;";
				$params[':error']=$error_id;
			}
			else
			{
				$tmp="";
				foreach($error_id as $id)
				{
					if(is_numeric($id))
					{
						$tmp=":plugin".$id.',';
						$params[':plugin'.$id]=$id;
					}
				}
				$tmp=rtrim($tmp,',');
				$sql .= "AND cron_error_id IN ($tmp)";
			}
		}
		else
		{
			throw new EsiaDbException('Bad parameter: delete crontask no id found');
		}
		$query = $dbconnection->prepare($sql);
		$query = self::executeStatement($query,$params);
	}
	/**
	 * @brief Get gesa connection state from GESA_HEALTH service
	 * @param PDO $dbconnection
	 * @param integer $gesaId device id of the GESA
	 * @retval integer state code of the GESA_HEALTH service on this $gesaId
	 * @exception esiaDBException
	 */
	public static function getGesaHealthState($dbconnection, $gesaId)
	{
		$sql = "
			SELECT service_previous_state
			FROM esia_services
			WHERE service_plugin_id = (SELECT plugin_id FROM esia_plugins WHERE plugin_name = 'GESA_HEALTH')
			AND service_delete = FALSE
			AND service_device_id = :device_id;
		";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, array(':device_id' => $gesaId));

		return $query->fetch(PDO::FETCH_COLUMN);
	}
	/**
	 * @brief Get gesa remote devices node ids
	 * @param PDO $dbconnection
	 * @param integer $gesaId id of the GESA
	 * @retval array node ids monitored behind the gesa
	 * @exception esiaDBException
	 */
	static function getGesaLinkedNodes($dbconnection, $gesaId)
	{
		$sql = "
			SELECT device_id
			FROM esia_device
			JOIN gesa_connected ON device_id = connected_device_id
			WHERE device_delete = FALSE
			AND connected_gesa_id = :gesa_id;
		";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, array(':gesa_id' => $gesaId));

		return $query->fetchAll(PDO::FETCH_COLUMN);
	}
	/**
	 * @brief Get gesa status (new, ok, critical, warning, unknown)
	 * @param PDO $dbconnection
	 * @param integer $gesaId id of the GESA
	 * @retval string status of the gesa
	 * @exception esiaDBException
	 */
	static function getGesaStatus($dbconnection, $gesaId)
	{
		$status = 'new';

		$remoteNodeList = Model_esia::getGesaLinkedNodes($dbconnection, $gesaId);
		if(!empty($remoteNodeList))
		{
			$stateCode = Model_esia::getGesaHealthState($dbconnection, $gesaId);
			if(isset($stateCode) && is_numeric($stateCode))
			{
				switch($stateCode)
				{
					case 0:
					case 4:
						$status = 'ok';
						break;
					case 1:
						$status = 'critical';
						break;
					case 2:
						$status = 'warning';
						break;
					default:
						$status = 'unknown';
						break;
				}
			}
			else
			{
				$status = 'unknown';
			}
		}

		return $status;
	}
	/**
	 * @brief récupère les Channel RSS
	 * @param PDO $dbconnection
	 * @param integer $channelid id du canal
 	 * @param string $type type du canal 'REMOTE' ou 'LOCAL'
  	 * @param integer $userid identifiant de l'utilisateur
   	 * @param string $sort 'ID' ou 'NAME' trie automatique sur un channel
	 * @retval array un array contenant les infos du channel
           $arr[]=array('id'=>$row->channel_id,
                        'name'=>$row->channel_name,
                        'description'=>$row->channel_description,
                        'lang'=>$row->channel_language,
                        'publish'=>$row->channel_last_publication,
                        'type'=>$row->channel_type,
           				'image'=>$row->channel_image,
           				'url'=>$row->channel_url,
           				'notdelete'=>$row->channel_notdelete,
           				'enable'=>$row->channel_enable,
           				'category'=>$row->category
	 * @exception esiaDBException
	 */
	static function getRSSChannel($dbconnection, $channelid=NULL, $type=null,$userid=null,$sort=null)
	{
        $sql ="WITH channelcategory AS (
					SELECT link_channel_id, STRING_AGG(category_name,', ') AS category
					FROM rss_link_channel_to_category
					JOIN rss_category ON link_category_id=category_id
					GROUP BY link_channel_id)

				SELECT channel_id, channel_name, channel_description, channel_language, channel_last_publication,
						channel_image, channel_type, channel_url,channel_enable, channel_notdelete, category,channel_group_link
                FROM rss_channel";
        if(is_numeric($userid))
		{
			$sql.=" JOIN rss_user_subscribe ON subscribe_channel_id=channel_id";
		}

		$sql.=" LEFT JOIN channelcategory ON link_channel_id=channel_id";
        $params=array();
        if(is_numeric($channelid))
		{
			$sql .=" WHERE channel_id = :chan";
			$params[':chan']=$channelid;
		}
		elseif(is_string($channelid))
		{
			$sql .=" WHERE channel_name = :chan";
			$params[':chan']=$channelid;
		}
		elseif(is_array($channelid))
		{
			$tmp="";
			foreach($channelid as $id)
			{
				if(is_numeric($id))
				{
					$tmp .= ":chan".$id.',';
					$params[':chan'.$id]=$id;
				}
			}
			$tmp=rtrim($tmp,',');
			$sql .= " WHERE channel_id IN ($tmp)";
		}
		if(!empty($type))
		{
			if(strpos($sql,'WHERE') === FALSE)
			{
				$sql .=" WHERE";
			}
			else
			{
				$sql .=" AND";
			}
			$sql .=" channel_type = :type";
			$params[':type']=$type;
		}
		if(!empty($userid))
		{
			if(strpos($sql,'WHERE') === FALSE)
			{
				$sql .=" WHERE";
			}
			else
			{
				$sql .=" AND";
			}
			$sql .=" subscribe_user_id = :user";
			$params[':user']=$userid;
		}
		if(!empty($sort))
		{
			if($sort=='NAME')
			{
				$sql .=" ORDER BY LOWER(channel_name)";
			}
			else
			{
				$sql .=" ORDER BY channel_id";
			}
		}
		
//		exit(0);
	    $query= $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query,$params);
        $arr=array();
	    while($row=$query->fetch(PDO::FETCH_OBJ))
        {

        	$usergroup = $row->channel_group_link;
        	if(!empty($usergroup))
        	{
        		$usergroup = explode(',', $usergroup);
        	}
           $arr[]=array('id'=>$row->channel_id,
                        'name'=>$row->channel_name,
                        'description'=>$row->channel_description,
                        'lang'=>$row->channel_language,
                        'publish'=>$row->channel_last_publication,
                        'type'=>$row->channel_type,
           				'image'=>$row->channel_image,
           				'url'=>$row->channel_url,
           				'notdelete'=>$row->channel_notdelete,
           				'enable'=>$row->channel_enable,
           				'category'=>$row->category,
           				'list_category'=>Model_esia::getRSSCategory($dbconnection,null,$row->channel_id),
           				'usergroup'=>$usergroup
                        );
        }
        return $arr;
    }
    /**
     * @brief récupère les identifiant des Channels RSS via leur categorie
     * @param PDO $dbconnection
     * @param integer $channelid id du canal
     * @retval array un array contenant les id du channel
     * @exception esiaDBException
     */
    static function getRSSChannelByCategory($dbconnection, $category,$userid=NULL)
    {
    	$params=array();
    	$linksql="WHERE";
    	$sql ="SELECT DISTINCT channel_id
               FROM rss_channel
			   JOIN rss_link_channel_to_category ON link_channel_id=channel_id
			   JOIN rss_category ON link_category_id=category_id";
    	if(is_numeric($userid))
    	{
    		$sql.=" JOIN rss_user_subscribe ON subscribe_channel_id=channel_id
					WHERE subscribe_user_id = :user";
    		$params[':user']=$userid;
    		$linksql="AND";
    	}
    	if(is_numeric($category))
    	{
    		$sql .=" $linksql category_id = :cate";
    		$params[':cate']=$category;
    	}
    	elseif(is_string($category))
    	{
    		$sql .=" $linksql category_name = :cate";
    		$params[':cate']=$category;
    	}
    	elseif(is_array($category))
    	{
    		$tmp="";
    		foreach($category as $id)
    		{
    			if(is_numeric($id))
    			{
    				$tmp .= ":cate".$id.',';
    				$params[':cate'.$id]=$id;
    			}
    		}
    		$tmp=rtrim($tmp,',');
    		$sql .= " $linksql category_id IN ($tmp)";
    	}
    	$query= $dbconnection->prepare($sql);
    	$query = Model_esia::executeStatement($query,$params);
    	$arr=array();
    	while($row=$query->fetch(PDO::FETCH_OBJ))
    	{
    		$arr[]=$row->channel_id;
    	}
    	return $arr;
    }
    /**
     * @brief récupère les Message RSS
     * @param PDO $dbconnection
     * @param integer $channelid id du canal
     * @param integer $msgid id du message
     * @retval array un array contenant les infos du message
						'channel_id'=>$row->channel_id,
                        'channel_name'=>$row->channel_name,
                        'channel_type'=>$row->channel_type,
           				'id'=>$row->message_id,
                        'title'=>$row->message_title,
           				'message'=>utf8_encode($row->message_description),
                        'lang'=>$row->channel_language,
                        'link'=>$row->message_link,
                        'author'=>$row->message_author,
           				'publish'=>$row->publish,
                        'type'=>$row->channel_type,
                        'url'=>$row->channel_url,
           				'category'=>$row->category
     * @exception esiaDBException
     */
    static function getRSSMessages($dbconnection, $channelid=NULL,$msgid=NULL,$userid=NULL,$limit=500,$readMessage=NULL)
	{
        $sql ="WITH channelcategory AS (
					SELECT link_channel_id, STRING_AGG(category_name,', ') AS category
					FROM rss_link_channel_to_category
					JOIN rss_category ON link_category_id=category_id
					GROUP BY link_channel_id)

				SELECT message_id, message_title, message_description, message_link, message_author, message_image, message_categories,
 					to_char(message_date_publish,'DD-MM-YYYY HH24:MI:SS') as publish,channel_id,channel_name,channel_image,channel_type,
					channel_url,channel_language,category
	            FROM rss_message
                JOIN rss_channel ON channel_id = message_channel_id";
        if(is_numeric($userid))
        {
        	$sql.=" JOIN rss_user_subscribe ON subscribe_channel_id=channel_id";
        }
        $sql.=" LEFT JOIN channelcategory ON link_channel_id=channel_id";
        $params=array();
        $separator = 'WHERE';
        if(is_numeric($channelid))
		{
			$sql .=" WHERE channel_id = :chan";
			$params[':chan']=$channelid;
			$separator = 'AND';
		}
		elseif(is_array($channelid))
		{
			$tmp="";
			foreach($channelid as $id)
			{
				if(is_numeric($id))
				{
					$tmp .= ":chan".$id.',';
					$params[':chan'.$id]=$id;
				}
			}
			$tmp=rtrim($tmp,',');
			$sql .= " WHERE channel_id IN ($tmp)";
			$separator = 'AND';
		}

		if(is_numeric($msgid))
		{
			$sql .=" $separator message_id = :msg";
			$params[':msg']=$msgid;
			$separator = 'AND';
		}
		elseif(is_array($msgid))
		{
			$tmp="";
			foreach($msgid as $id)
			{
				if(is_numeric($id))
				{
					$tmp .= ":msg".$id.',';
					$params[':msg'.$id]=$id;
				}
			}
			$tmp=rtrim($tmp,',');
			$sql .=" $separator message_id IN ($tmp)";
			$separator = 'AND';
		}
		if(!empty($userid))
		{
			$sql .=" $separator subscribe_user_id = :user";
			$params[':user']=$userid;
			$separator = 'AND';
		}
		if(is_array($readMessage)&& !empty($readMessage))
		{
			$tmp="";
			foreach($readMessage as $id)
			{
				if(is_numeric($id))
				{
					$tmp .= ":read".$id.',';
					$params[':read'.$id]=$id;
				}
			}
			$tmp=rtrim($tmp,',');
			$sql .=" $separator message_id NOT IN ($tmp)";
		}
        $sql .= ' ORDER BY message_date_publish DESC LIMIT :limit' ;
        $params[':limit']=$limit;
	    $query= $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query,$params);
        $arr=array();
	    while($row=$query->fetch(PDO::FETCH_OBJ))
        {
			$arr[] = array(
				'channel_id'=>$row->channel_id,
				'channel_name'=>$row->channel_name,
				'channel_image'=>$row->channel_image,
				'channel_type'=>$row->channel_type,
				'id'=>$row->message_id,
				'title'=>$row->message_title,
				'message'=>mb_convert_encoding(html_entity_decode($row->message_description), "UTF-8"),
				'lang'=>$row->channel_language,
				'link'=>$row->message_link,
				'author'=>$row->message_author,
				'image'=>$row->message_image,
				'message_categories'=>$row->message_categories,
				'publish'=>$row->publish,
				'type'=>$row->channel_type,
				'url'=>$row->channel_url,
				'category'=>$row->category,
			);
        }
        return $arr;
    }
    /**
     * @brief récupère les categorie des message RSS
     * @param PDO $dbconnection
     * @param integer $category id de la category
     * @param integer $msgid id du message
     * @retval array un array contenant les infos de la categorie
				'id'=>$row->category_id,
    			'name'=>$row->category_name,
    			'description'=>$row->category_description,
     * @exception esiaDBException
     */
    static function getRSSCategory($dbconnection,$category=null,$chanid=null)
    {
    	$sql ="SELECT category_id, category_name, category_description
				FROM rss_category";
    	$params=array();
    	if(is_numeric($category))
    	{
    		$sql .=" WHERE category_id = :cate";
    		$params[':cate']=$category;
    	}
    	elseif(is_string($category))
    	{
    		$sql .=" WHERE category_name = :cate";
    		$params[':cate']=$category;
    	}
    	elseif(is_array($category))
    	{
    		$tmp="";
    		foreach($category as $id)
    		{
    			if(is_numeric($id))
    			{
    				$tmp .= ":cate".$id.',';
    				$params[':cate'.$id]=$id;
    			}
    		}
    		$tmp=rtrim($tmp,',');
    		$sql .= " WHERE category_id IN ($tmp)";
    	}
    	else
    	{
    		if(is_numeric($chanid))
    		{
    			$sql .=" JOIN rss_link_channel_to_category ON category_id=link_category_id
						WHERE link_channel_id  = :id";
    			$params[':id']=$chanid;
    		}
    	}
    	$query= $dbconnection->prepare($sql);
    	$query = Model_esia::executeStatement($query,$params);
    	$arr=array();
    	while($row=$query->fetch(PDO::FETCH_OBJ))
    	{
    		$arr[]=array('id'=>$row->category_id,
    			'name'=>$row->category_name,
    			'description'=>$row->category_description,
    		);
    	}
    	return $arr;
    }
    /**
     * @brief récupère les id des messages lu par l'utilisateur
     * @param PDO $dbconnection
     * @param integer $userid id de l'utilisateur
     * @param integer $msgid id du message
     * @retval array un array contenant les id des message lus
     * @exception esiaDBException
     */
    static function getRSSReadMessagesID($dbconnection,$userid,$channel=null,$msgid=null)
    {
     	$sql ="SELECT read_message_id
				FROM rss_user_read";
        if(!empty($channel))
        {
            $sql.=" JOIN rss_message ON read_message_id = message_id ";
        }
    	$params=array();
    	if(is_numeric($userid))
    	{
    		$sql .=" WHERE read_user_id = :user";
    		$params[':user']=$userid;
    	}
        else
        {
            throw new EsiaDBException('user id is not numeric');
        }
        if(!empty($channel))
        {
            $sql.=" AND message_channel_id = :chan ";
            $params[':chan']=$channel;
        }
		if(!empty($msgid))
		{
            if(is_numeric($msgid))
            {
                $sql .=" AND read_message_id = :msg";
                $params[':msg']=$msgid;
            }
            elseif(is_array($msgid))
            {
			    $tmp="";
			    foreach($msgid as $id)
			    {
				    if(is_numeric($id))
				    {
					    $tmp .= ":msg".$id.',';
					    $params[':msg'.$id]=$id;
				    }
			    }
			    $tmp=rtrim($tmp,',');
			    $sql .= " AND read_message_id IN ($tmp)";
            }
		}
    	$query= $dbconnection->prepare($sql);
    	$query = Model_esia::executeStatement($query,$params);
    	$arr=array();
    	while($row=$query->fetch(PDO::FETCH_OBJ))
    	{
    		$arr[]=$row->read_message_id;
    	}
    	return $arr;
    }
    /**
     * @brief récupère les id des messages lu par l'utilisateur
     * @param PDO $dbconnection
     * @param string $state ['READ'|'NOTREAD'] indique le status du message
     * @param integer $userid id de l'utilisateur
     * @param integer|array $msgid id du message
     * @retval array un array contenant les id des message lus
     * @exception esiaDBException
     */
    static function RSSMessagesRead($dbconnection,$state='READ',$userid,$msgid)
    {
    	if(empty($msgid))
    	{
    		throw new EsiaDbException('message id is NULL');
    	}
    	if($state =='READ')
    	{
    		$sql ="INSERT INTO rss_user_read(read_user_id, read_message_id)
			   	   SELECT :user,message_id
			   	    FROM rss_message";
	    	$params=array();
	    	$params[':user']=$userid;
	    	if(is_numeric($msgid))
	    	{
	    		$sql .=" WHERE message_id = :msg";
	    		$params[':msg']=$msgid;
	    	}
	    	elseif(is_array($msgid))
	    	{
	    		$tmp="";
	    		foreach($msgid as $id)
	    		{
	    			if(is_numeric($id))
	    			{
	    				$tmp .= ":msg".$id.',';
	    				$params[':msg'.$id]=$id;
	    			}
	    		}
	    		$tmp=rtrim($tmp,',');
	    		$sql .= " WHERE message_id IN ($tmp)";
	    	}
	    	$query= $dbconnection->prepare($sql.' ON CONFLICT ON CONSTRAINT "read_primaryKey" DO NOTHING;');
	    	$query = Model_esia::executeStatement($query,$params);
    	}
    	elseif($state =='NOTREAD')
		{
			$sql ="DELETE FROM rss_user_read WHERE read_user_id=:user";
			$params=array();
			$params[':user']=$userid;
			if(is_numeric($msgid))
			{
				$sql .=" AND read_message_id = :msg";
				$params[':msg']=$msgid;
			}
			elseif(is_array($msgid))
			{
				$tmp="";
				foreach($msgid as $id)
				{
					if(is_numeric($id))
					{
						$tmp .= ":msg".$id.',';
						$params[':msg'.$id]=$id;
					}
				}
				$tmp=rtrim($tmp,',');
				$sql .= " AND read_message_id IN ($tmp)";
			}
			$query= $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$params);
		}
    }
    /**
     * @brief récupère les id des messages lu par l'utilisateur
     * @param PDO $dbconnection
     * @param string $state ['SUBSCRIBE'|'UNSUBSCRIBE'] indique le status du message
     * @param integer $userid id de l'utilisateur
     * @param integer|array $msgid id du message
     * @retval array un array contenant les id des message lus
     * @exception esiaDBException
     */
    static function RSSChannelSubscribe($dbconnection,$state='SUBSCRIBE',$userid,$channel)
    {
    	if(empty($channel) || empty($userid))
    	{
    		throw new EsiaDbException('channel or user id is NULL');
    	}
    	if($state =='SUBSCRIBE')
    	{
    		$sql = "UPDATE rss_channel SET channel_enable=true ";
    		if(is_numeric($channel))
    		{
    			$sql .=" WHERE channel_id = :chan";
    			$params[':chan']=$channel;
    		}
    		elseif(is_array($channel))
    		{
    			$tmp="";
    			foreach($channel as $id)
    			{
    				if(is_numeric($id))
    				{
    					$tmp .= ":chan".$id.',';
    					$params[':chan'.$id]=$id;
    				}
    			}
    			$tmp=rtrim($tmp,',');
    			$sql .= " WHERE channel_id IN ($tmp)";
    		}
    		$query = $dbconnection->prepare($sql." AND channel_enable=false");
    		$query = Model_esia::executeStatement($query,$params);

    		$sql ="INSERT INTO public.rss_user_subscribe(subscribe_user_id, subscribe_channel_id)
			   	   SELECT :user,channel_id
			   	    FROM rss_channel";
    		$params=array();
    		$params[':user']=$userid;
    		if(is_numeric($channel))
    		{
    			$sql .=" WHERE channel_id = :chan";
    			$params[':chan']=$channel;
    		}
    		elseif(is_array($channel))
    		{
    			$tmp="";
    			foreach($channel as $id)
    			{
    				if(is_numeric($id))
    				{
    					$tmp .= ":chan".$id.',';
    					$params[':chan'.$id]=$id;
    				}
    			}
    			$tmp=rtrim($tmp,',');
    			$sql .= " WHERE channel_id IN ($tmp)";
    		}
    		$query= $dbconnection->prepare($sql.'AND channel_enable=true ON CONFLICT ON CONSTRAINT "subscribe_primaryKey" DO NOTHING;');
    		$query = Model_esia::executeStatement($query,$params);

    	}
    	elseif($state =='UNSUBSCRIBE')
    	{
    		$sql ="DELETE FROM rss_user_subscribe WHERE subscribe_user_id=:user";
    		
    		$sqlread ="DELETE FROM rss_user_read WHERE read_user_id=:user";
    		$params=array();
    		$params[':user']=$userid;
    		if(is_numeric($channel))
    		{
    			$sql .=" AND subscribe_channel_id = :chan";
    			$sqlread.=" AND read_message_id IN (SELECT message_id FROM rss_message WHERE message_channel_id=:chan)";
    			$params[':chan']=$channel;
    		}
    		elseif(is_array($channel))
    		{
    			$tmp="";
    			foreach($channel as $id)
    			{
    				if(is_numeric($id))
    				{
    					$tmp .= ":chan".$id.',';
    					$params[':chan'.$id]=$id;
    				}
    			}
    			$tmp=rtrim($tmp,',');
    			$sql .= " AND subscribe_channel_id IN ($tmp)";
    			$sqlread.=" AND read_message_id IN (SELECT message_id FROM rss_message WHERE message_channel_id IN ($tmp)";
    		}
    		$query= $dbconnection->prepare($sqlread);
    		$query = Model_esia::executeStatement($query,$params);
    		$query= $dbconnection->prepare($sql);
    		$query = Model_esia::executeStatement($query,$params);
    		
    	}
    }
    /**
     * @brief compte le nombre de message non lu par canaux abonné
     * @param PDO $dbconnection
     * @param integer $userid id de l'utilisateur
     * @param integer|array $channel id du channel
     * @retval array un array contenant les id des channel et le count des message Abonnés
     * @exception esiaDBException
     */
    static function getUnreadMessageByChannel($dbconnection,$userid,$channel=null,$sort=null)
    {
    	if(empty($userid))
    	{
    		throw new EsiaDbException('user id is NULL');
    	}
    	$sql = "SELECT channel_id,channel_name,channel_image,channel_type,count(message_id) as total, count(read_message_id) as read
				FROM rss_channel 
				JOIN rss_user_subscribe ON subscribe_channel_id=channel_id 
				LEFT JOIN rss_message ON channel_id = message_channel_id 			 
				LEFT JOIN rss_user_read ON read_message_id=message_id
				WHERE subscribe_user_id= :user";
    	$params[':user']=$userid;
    	if(is_numeric($channel))
    	{
    		$sql .=" AND channel_id = :chan";
    		$params[':chan']=$channel;
    	}
    	elseif(is_array($channel))
    	{
    		$tmp="";
    		foreach($channel as $id)
    		{
    			if(is_numeric($id))
    			{
    				$tmp .= ":chan".$id.',';
    				$params[':chan'.$id]=$id;
    			}
    		}
    		$tmp=rtrim($tmp,',');
    		$sql .= " AND channel_id IN ($tmp)";
    	}
    	$sql .= " GROUP BY channel_id, channel_name,channel_image";
    	if(!empty($sort))
    	{
    		if($sort=='NAME')
    		{
    			$sql .=" ORDER BY LOWER(channel_name)";
    		}
    		else
    		{
    			$sql .=" ORDER BY channel_id";
    		}
    	}
    	$query = $dbconnection->prepare($sql);
    	$query = Model_esia::executeStatement($query,$params);
    	$arr['channel']=array();
    	$total=0;
    	$total_notread=0;
    	$total_read=0;
    	while($row=$query->fetch(PDO::FETCH_OBJ))
    	{
    		$total_read=$total_read+$row->read;
    		$total=$total+$row->total;
    		$notread = $row->total - $row->read;
    		$total_notread=$total_notread+$notread;
    		$arr['channel'][]=array('id'=>$row->channel_id,'name'=>$row->channel_name,'image'=>$row->channel_image,'type'=>$row->channel_type,'notread'=>$notread);
    	}
    	$arr['global']=array('total'=>$total,'notread'=>$total_notread,'read'=>$total_read);
    	return $arr;
    }
}

class EsiaDbException extends Exception{
}
?>
